
package vardate;

import com.ms.com.*;
import java.util.Date;

public class VarDateSample implements com.ms.com.IUnknown,com.ms.com.NoAutoScripting,vardate.IVarDateSample
{
  private static void printWithPeriod( String str )
  {
    System.out.println( str + "." );
  }

  private Date createNewDate()
  {
    try
    {
      // wait for ~ 2 seconds.
      printWithPeriod( "Java, waiting for ~2 seconds" );
      synchronized( this )
      {
        wait( 2000 );
      }
    }
    catch( Exception e )
    {
      e.printStackTrace();

      throw new ComFailException();
    }

    // return a newly created Date.
    return new Date();
  }

  public void ByValue(java.util.Date date)
  {
    // print time in java.
    printWithPeriod( "Java, ByValue() got: " + date );
  }

  public java.util.Date ByReferenceRetval()
  {
    Date    date;

    date = createNewDate();

    // print time in java.
    printWithPeriod( "Java, ByReferenceRetval() returning: " + date );

    return date;
  }

  public void ByReferenceIn(java.util.Date pDate)
  {
    // print time in java.
    printWithPeriod( "Java, ByReferenceIn() got: " + pDate );
  }

  public void ByReferenceOut(java.util.Date[] pDate)
  {
    // create a new date to send back to the caller.
    pDate[ 0 ] = createNewDate();

    // send the new date back.
    printWithPeriod( "Java, ByReferenceOut() \"returning\": " + pDate[ 0 ] );
  }

  public void ByReferenceInOut(java.util.Date[] pDate)
  {
    // print time in java.
    printWithPeriod( "Java, ByReferenceInOut() got: " + pDate[ 0 ] );

    // create a new date to send back to the caller.
    pDate[ 0 ] = createNewDate();

    // send the new date back.
    printWithPeriod( "Java, ByReferenceInOut() \"returning\": " + pDate[ 0 ] );
  }

  public void TestJavaToNative(vardate.IVarDateSample pIDateSampleNative)
  {
    Date    dateNew;
    Date[]  aDate;

    aDate = new Date[ 1 ];

    // create a new date.
    dateNew = createNewDate();

    // print time in java.
    printWithPeriod( "Java, before calling ByValue(): " + dateNew );

    // call the native implementation of ByValue().
    pIDateSampleNative.ByValue( dateNew );

    // call the native implementation of ByReferenceRetval().
    dateNew = pIDateSampleNative.ByReferenceRetval();

    // print time in java.
    printWithPeriod( "Java, received from ByReferenceRetval(): " + dateNew );

    // create a new date.
    dateNew = createNewDate();

    // print time in java.
    printWithPeriod( "Java, before calling ByReferenceIn(): " + dateNew );

    // call the native implementation of ByReferenceIn().
    pIDateSampleNative.ByReferenceIn( dateNew );

    // call the native implementation of ByReferenceOut().
    pIDateSampleNative.ByReferenceOut( aDate );

    // print time in java.
    printWithPeriod( "Java, received from ByReferenceOut(): " + aDate[ 0 ] );

    // create a new date.
    dateNew = createNewDate();

    // print time in java.
    printWithPeriod( "Java, before calling ByReferenceInOut(): " + dateNew );

    // call the native implementation of ByReferenceInOut().
    aDate[ 0 ] = dateNew;
    pIDateSampleNative.ByReferenceInOut( aDate );

    // print time in java.
    printWithPeriod( "Java, received from ByReferenceInOut(): " + aDate[ 0 ] );
  }
}
