/*
 *  sample.dcom.OutprocClient.java
 *
 *  (C) Copyright 1995 - 1999 Microsoft Corporation.  All rights reserved.
 *
 */
package sample.dcom;

import com.ms.com.*;
import java.util.Vector;
import java.util.Date;
/**
 * Sample for demonstrating how to use CoCreateInstance for creating
 * and using a COM object from a local server, and access its methods through 
 * IDispatch calls.  The com.ms.com.Dispatch.invokev static methods
 * are used to execute method calls on the COM object. 
 */
public class OutprocDClient { 
/**
 * Class ID for the object to create.  This is the Class ID for the
 * Hub3 object.
 */
	private final _Guid clsid = 
                new _Guid("{036CB0B3-0BCD-11d2-9B9E-00C04F8C94D0}");
/**
 * Main method for starting the sample from the command line.
 */
public static void main(String args[]){	
        com.ms.lang.SystemX.exitProcessAfterMainThreadReturns( true );

		new OutprocDClient();

        // clean up.
        System.gc();
        System.runFinalization();
	}
/**
 * Creates an instance of the Hub3 COM object using a J/Direct call to
 * CoCreateInstanceEx through sample.util.DCOMLib.CoCreateInstanceEx.
 * The resulting COM object is accessed through its default IDispatch
 * interface (using com.ms.com.Dispatch).  The COM object has its
 * addListener, sendMessage, sendObjects, and removeListener methods 
 * called.  The results from InprocClient and the COM object
 * will be printed to System.out. 
 */
	public OutprocDClient(){
		Object mess = null;
		try{
			System.out.println("OutprocDClient - About to call CoCreateInstance");
			mess = com.ms.win32.Ole32.CoCreateInstance(
                                   clsid,
                                   null,
                                   com.ms.win32.win.CLSCTX_LOCAL_SERVER,
                                   new _Guid("{00000000-0000-0000-C000-000000000046}"));
		}catch(Exception ii){
			System.out.println("OutprocDClient: Exception "+ii);
		}finally{
			System.out.println("OutprocDClient - Object: "+mess);
		}
		System.out.println("OutprocDClient: about to add this as Listener");
		Variant retVal = Dispatch.invokev(mess,Dispatch.getIDOfName(mess,"addListener"),
						 Dispatch.Method,
						 new Variant[]{new Variant(this)},
						 null);
		System.out.println("OutprocDClient: returned from dispatch method call");
		String key = (String)retVal.toString();
		System.out.println("OutprocDClient: about to send a message "+key);
		Dispatch.invokev(mess,Dispatch.getIDOfName(mess,"sendMessage"),
						 Dispatch.Method,
						 new Variant[]{new Variant("This is the first Test Message")},
						 null);
		Variant[] vars = new Variant[3];
		vars[0]=new Variant(new Integer(99));
		Vector datum = new Vector();
		datum.addElement(new Date());
		datum.addElement(new Vector());
		vars[1]=new Variant(datum);
		vars[2]=new Variant(new StringBuffer("Howdy"));
		SafeArray safe = new SafeArray(Variant.VariantVariant,3);
		safe.setVariants(0,3,vars,0);
		Dispatch.invokev(mess,Dispatch.getIDOfName(mess,"sendObjects"),
						 Dispatch.Method, new Variant[]{new Variant(safe)},null);
		Dispatch.invokev(mess,Dispatch.getIDOfName(mess,"removeListener"),
						 Dispatch.Method,
						 new Variant[]{new Variant(key)},
						 null);
		System.out.println("OutprocDClient - Done");
		}
/**
 * Implemented as part of the sample.IListener interface for receiving
 * messages from the server.
 * 
 * @param String message - Message sent by the server.
 */
	public synchronized void processMessage(String message){
		System.out.println("OutprocDClient - message received "+message);
	}
/**
 * Implemented as part of the sample.IListener interface for receiving 
 * SafeArrays form the server.
 * 
 * @param SafeArray safe - Array of objects broadcast by the server.
 */
	public synchronized void processObjects(SafeArray safe){
		System.out.println("OutprocDClient - data received : Data Length ");
		Variant[] data = safe.toVariantArray();
		for (int i=0;i<data.length;i++)
			System.out.println("OutprocDClient: Element "+i+" is "+data[i].toObject().getClass().getName()+
							   " contents "+data[i].toObject());
	}
}