/*
 * sample.client.MClient.java
 * 
 * (C) Copyright 1995 - 1999 Microsoft Corporation.  All rights reserved.
 *
 */
package sample.client;

import com.ms.com.*;
/**
 * This is a command line client that creates an instance of the COM object
 * servced by the sample.server.JCServer clas, and prints it to the screen.  As
 * the served object is marshaled by value, we should see the Java objects 
 * toString as the output.  This client can be run locally or 
 * remotely, simply change the machine name string to represent the server name
 * where the sample.server.JCServer COM Server is running.  You may also need to
 * fill in the correct security credentials for user, domain, password and change
 * the authentication level to RPC_C_AUTHN_LEVEL_CONNECT
 * 
 * The second createinstance method creates a regular proxy to a JavaCOM object, 
 * and invokes methods on it to retrieve a Java COM object by ref and by value.
 * This object is accessed through its default dispatch interface.
 * 
 * @version 1.0
 */
public class MClient{
        /**
         * User name to be used in CCIEx call.
         */
        private String user = null;
        /**
         * Domain for the user
         */
        private String domain = null;
        /**
         * Password for the user.
         */
        private String password = null;
        /**
         * Authentication level for the call.
         */
        private int authn = com.ms.win32.win.RPC_C_AUTHN_LEVEL_NONE;
        /**
         * Server name, where the COM object is being hosted.  May need to change
         * this to your local machines name if you don't have service pack 4 on NT
         */
        private String machine = "localhost";
/**
 * Creates new instance of the MClient class and calls its createObject method, 
 * to test the Marshal By Value behaviour.  
 */
    public static void main(String[] args) { 
            MClient c =new MClient();
            sample.marshal.MarshalLib.registerProxy();
            c.createObject();
            c.createObject2();
			// Force a garbage collection, so the application can free any 
			// resources and close quicker.
  			com.ms.lang.SystemX.gc();
    }
/**
 * @dll.import("OLE32")
*/
    public static native int CoCreateInstanceEx(
             com.ms.com._Guid Clsid,        //CLSID of the object to be created
             com.ms.com.IUnknown punkOuter, //If part of an aggregate, the 
                                            // controlling IUnknown
             int dwClsCtx,                  //CLSCTX values
             COSERVERINFO pServerInfo,      //Machine on which the object is to 
                                            // be instantiated
             int dwCount,                   //Number of MULTI_QI structures in 
                                            // pResults
             sample.util.CStructArray pResults);  //Array of MULTI_QI structures
/**
 * Uses the sample.marshal.MarshalLib.registerProxy() method to register a 
 * Proxy for unmarshaling Java and Java COM objects which will be marshaled by
 * value to this client.  It then Creates an instance of the served COM object.
 */
    public void createObject(){
        try{
            MULTI_QI[] multi=null;
            COSERVERINFO coServerInfo = new COSERVERINFO();
            coServerInfo.pwszName = machine;
            coServerInfo.pAuthInfo = new COAUTHINFO();
            coServerInfo.pAuthInfo.dwAuthnLevel =authn;
            coServerInfo.pAuthInfo.dwAuthnSvc = 
                   com.ms.win32.win.RPC_C_AUTHN_DEFAULT;
            coServerInfo.pAuthInfo.dwAuthzSvc =
                   com.ms.win32.win.RPC_C_AUTHZ_NONE;
            coServerInfo.pAuthInfo.dwImpersonationLevel=
                   com.ms.win32.win.RPC_C_IMP_LEVEL_IDENTIFY;
            coServerInfo.pAuthInfo.pAuthIdentityData = 
                        new COAUTHIDENTITY();
            coServerInfo.pAuthInfo.pAuthIdentityData.domain=domain;
            coServerInfo.pAuthInfo.pAuthIdentityData.user=user;
            coServerInfo.pAuthInfo.pAuthIdentityData.password=password;

            multi = (MULTI_QI[])sample.util.StructArray.newStructArray(
                                     MULTI_QI.class,
                                     1);
            multi[0].pIID=new _Guid("{00000000-0000-0000-C000-000000000046}");
           
            System.out.println("\n\n\nAbout to create instance of object "+
                                   multi+" "+multi[0]);
            CoCreateInstanceEx(
                                        sample.server.JCServer.IID_SERVEDOBJECT,
                                        null,
                                        com.ms.win32.win.CLSCTX_SERVER,
                                        // NOTE: you may need to change localhost
                                        // to your machine name until NT SP 4 is
                                        // released/installed 
                                        coServerInfo,
                                        1,
                                        sample.util.StructArray.toCStructArray(multi));
	        System.out.println("Done CCIEx "+
                                Integer.toHexString(multi[0].hr));
            System.out.println("CoCreateInstanceEx created : "+multi[0].pItf.getClass().getName());
            com.ms.com.ComLib.release(multi[0].pItf);
        // The Object that is created will NEVER have a JCW on it, it will always 
        // be the regular Java object for that class.  (We know this, because only a 
        // Java object which was serializable can be marshaled by value with this
        // technique.)  This is controled by the
        // pItf field in sample.dcom.MULTI_QI.  If the field is typed as IUnknown,
        // and the Object passed does not implement com.ms.com.IUnknown you will
        // get a com.ms.com.CUnknown as the return value.  If the field is typed
        // as IUnknown and the object implements an com.ms.com.IUnknown, the Java
        // Object will be returned.
        //
        }catch(Exception ii){
            System.out.println("Exception : "+ii);
        }
    }
/**
 * Uses the sample.marshal.MarshalLib.registerProxy() method to register a 
 * Proxy for unmarshaling Java and Java COM objects which will be marshaled by
 * value to this client.  It then Creates an instance of the served COM object.
 * 
 * This method creates a regular instance of a remote COM object (it recieves the
 * default proxy), and then calls methods on it to receive a Java COM object as 
 * the result.  The resultant COM object will either be by ref or by value 
 * depending on which methods were called.
 */
    public void createObject2(){
        try{
            MULTI_QI[] multi=null;
            multi = (MULTI_QI[])sample.util.StructArray.newStructArray(
                                     MULTI_QI.class,
                                     1);
            multi[0].pIID=new _Guid("{00000000-0000-0000-C000-000000000046}");
            COSERVERINFO coServerInfo = new COSERVERINFO();
            coServerInfo.pwszName = machine;
            coServerInfo.pAuthInfo = new COAUTHINFO();
            coServerInfo.pAuthInfo.dwAuthnLevel =authn;
            coServerInfo.pAuthInfo.dwAuthnSvc = 
                   com.ms.win32.win.RPC_C_AUTHN_DEFAULT;
            coServerInfo.pAuthInfo.dwAuthzSvc =
                   com.ms.win32.win.RPC_C_AUTHZ_NONE;
            coServerInfo.pAuthInfo.dwImpersonationLevel=
                   com.ms.win32.win.RPC_C_IMP_LEVEL_IDENTIFY;
            coServerInfo.pAuthInfo.pAuthIdentityData = 
                        new COAUTHIDENTITY();
            coServerInfo.pAuthInfo.pAuthIdentityData.domain=domain;
            coServerInfo.pAuthInfo.pAuthIdentityData.user=user;
            coServerInfo.pAuthInfo.pAuthIdentityData.password=password;

            System.out.println("\n\n\nAbout to create instance of object "+
                                   multi+" "+multi[0]);
            CoCreateInstanceEx(
                                        sample.server.JavaCOM.GUID_JavaCOM,
                                        null,
                                        com.ms.win32.win.CLSCTX_SERVER,
                                        // NOTE: you may need to change localhost
                                        // to your machine name until NT SP 4 is
                                        // released/installed 
                                        coServerInfo,
                                        1,
                                        sample.util.StructArray.toCStructArray(multi));
	        System.out.println("Done CCIEx "+
                                Integer.toHexString(multi[0].hr));
			System.out.println("CoCreateInstanceEx created : "+multi[0].pItf.getClass().getName());
            invokeMethods(multi[0].pItf);
            com.ms.com.ComLib.release(multi[0].pItf);
        // The Object that is created will NEVER have a JCW on it, it will always 
        // be the regular Java object for that class.  (We know this, because only a 
        // Java object which was serializable can be marshaled by value with this
        // technique.)  This is controled by the
        // pItf field in sample.dcom.MULTI_QI.  If the field is typed as IUnknown,
        // and the Object passed does not implement com.ms.com.IUnknown you will
        // get a com.ms.com.CUnknown as the return value.  If the field is typed
        // as IUnknown and the object implements an com.ms.com.IUnknown, the Java
        // Object will be returned.
        //
        }catch(Exception ii){
            System.out.println("Exception : "+ii);
        }
    }
/**
 * Calls the two methods from sample.server.ICustom and displays the results.
 * One method will return an object by Reference, and the other by Value.
 * 
 * @param Object object - the object to invoke the methds on.
 */
    protected void invokeMethods(Object object){
         if (object!=null){
			 System.out.println("About to cast");
			 sample.server.ICustom cc = (sample.server.ICustom)object;
			 // Need to store this object in a local variable so that we can
			 // explicitly release it before exiting.  If we don't release it, 
			 // we will get an 'OLEChannelWnd fault at 0x77b640a5
			 Object o = cc.getParamByRef();
			 try{
			 System.out.println("By Ref retval : "+o.getClass().getName());
			 com.ms.com.Dispatch.invokev(o,
									com.ms.com.Dispatch.getIDOfName(o,"ping"),
                                    com.ms.com.Dispatch.Method,
                                    new com.ms.com.Variant[]{},
                                    null).toString();
			 }catch(Exception ii){
					System.out.println("Exception when pinging ByRef "+ii);	 
			 }
			 Object o2 = cc.getParamByValue();
			 System.out.println("By Value retval : "+o2.getClass().getName());
			 try{
			 com.ms.com.Dispatch.invokev(com.ms.com.ComLib.makeProxyRef(o2),
									com.ms.com.Dispatch.getIDOfName(o2,"ping"),
                                    com.ms.com.Dispatch.Method,
                                    new com.ms.com.Variant[]{},
                                    null).toString();
			 }catch(Exception ii){
					System.out.println("Exception when pinging ByVal "+ii);	 
			 }
			 com.ms.com.ComLib.release(cc);
			 com.ms.com.ComLib.release(o);
        }
    }
}