/* 
 * sample.marshal.MarshalLib.java
 * 
 * (C) Copyright 1995 - 1999 Microsoft Corporation.  All rights reserved.
 *
 */
package sample.marshal;

/**
 * This utility class is used to wrap a Java or Java COM object so that it will be
 * marshaled by Value to the COM client expecting the object.  The wrapper is a 
 * generic Java COM object that implements IMarshal, and takes care of serializing
 * the parameter to the client, and reconstructing it using a generic Proxy object.
 * The client MUST call registerProxy() before it expects the Marshaled By Value
 * object to ensure that COM can locate the proxy required for unmarshaling.  The
 * registerProxy method should only need to be called once per process.
 * 
 * The object that is to be marshaled by value must implement java.io.Serializable
 * or java.io.Externalizable.  JCW's cannot be marshaled by value, however any 
 * Java COM object that is created inProc will automatically be unwrapped and is 
 * accessible as a regular Java object, so it CAN be marshaled by value.
 * 
 * @version 1.0
 */
public class MarshalLib { 
        /**
         * Contains the key returned from CoRegisterClassObject(...) that will
         * be requied for the CoRevokeClass(...) call in the finalize method
         * of this class.
         */
        static int[] regKey = new int[1];
        /**
         * This boolean will be used to determine if we have already registered the
         * Proxy.
         */
        static boolean registered = false;
/**
 * Registers the generic Java COM Proxy object that will be used to unmarshal 
 * all Java and Java COM objects that have been marshaled by value.  Succesive 
 * calls to this method will do nothing, if the proxy has already been registered.
 */
    public synchronized static void registerProxy(){
        if (!registered){
            System.out.println("Registering Proxy");
        regKey[0]= com.ms.win32.Ole32.CoRegisterClassObject(
                                 sample.marshal.ByValue.IID,
                                 new sample.dcom.COMServer(
                                             sample.marshal.ByValue.class,
                                             sample.marshal.ByValue.IID,
                                             false,
                                             false,
                                             true,
                                             false),
                                 com.ms.win32.win.CLSCTX_INPROC_SERVER,
                                 com.ms.win32.win.REGCLS_MULTIPLEUSE);
        com.ms.win32.Ole32.CoRegisterPSClsid(
                      sample.marshal.ByValue.IID,
                      sample.marshal.ByValue.IID);
        System.out.println("Guid for dll proxy is "+
                           com.ms.win32.Ole32.CoGetPSClsid(
                                       sample.marshal.ByValue.IID));
        registered=true;
        }else
            System.out.println("Already Registered Proxy");
    }
/**
 * Takes a Java Object or Java COM object and wraps it in a Java COM object which
 * takes care of marshaling the original by value.  The returned ByValue object
 * should be returned from the COM object method as the Object to be marshaled by
 * value.  
 * 
 * NOTE : Only Java objects which implement java.io.Serializable or 
 *      java.io.Externalizable will work here.  Also, this will not work for JCWs. 
 * 
 * @param Object o - any Java or Java COM object that is to be marshaled by value
 */
    public static synchronized  ByValue marshalByValue(Object o){
        if ((!(o instanceof java.io.Serializable))&&
            (!(o instanceof java.io.Externalizable)))
            throw new RuntimeException(
            "Cannot marshal this object by value");
        return new ByValue(o);   
    }
/**
 * This finalize method is used to call CoRevokeClassObject on the Proxy COM object
 * used for unmarshaling Java and Java COM marshal by value objects.
 */
    public void finalize(){
        com.ms.win32.Ole32.CoRevokeClassObject(regKey[0]);   
    }
}