/*
 *  sample.apartment.Client.java
 *
 *  (C) Copyright 1995 - 1999 Microsoft Corporation.  All rights reserved.
 *
 */
package sample.apartment;

import com.ms.com.*;
import java.awt.Frame;
import java.util.Vector;
import java.util.Date;
import java.awt.event.ActionListener;
/**
 * Sample for demonstrating how to use CoCreateInstance for creating
 * and using a COM object from a Local Server. 
 */
public class Client implements ActionListener{
	/**
	 * Button that will trigger a Dispatch method in the COM object
	 */
	private java.awt.Button b = new java.awt.Button("Ping the COM Object");
	/**
	 * Label that contains the count from the COM object.
	 */
	private java.awt.Label count = new java.awt.Label();
	/**
	 * Frame that holds the Client GUI
	 */
	private java.awt.Frame clientGUI = new java.awt.Frame();
	/**
 	 * Class ID for the object to create.  This is the Class ID for the
 	 * Hub2 object.
 	 */							   
	private final _Guid clsid = 
                   new _Guid("{5c06abc0-2dcd-11ce-8417-00aa00389b71}");
	/**
	 * The COM object created from a local server.
	 */
	private sample.IPing mess = null;
	/**
	 * Windows Thread ID of the message loop
	 */
	private int loopThreadID=0;
/**
 * Main method for starting the sample from the command line.
 */
	public static void main(String args[]){	
		Client c = new Client();
	}
/**
 * Handles the messages generated by the 'PING' button in the Frame.
 * 
 * @param java.awt.ActionEvent e - Event generated by the PING button
 */
	public void actionPerformed(java.awt.event.ActionEvent e){
		String t = mess.ping();
		clientGUI.setTitle(t.substring(t.indexOf(":")+1,t.length()));
		count.setText(t.substring(0,t.indexOf(":")));
		clientGUI.validate();
	}
/**
 * Creates a Frame and displays the Client GUI.
 */
	public void startClientGUI(){
		clientGUI.setSize(300,300);
		clientGUI.setVisible(true);
		b.addActionListener(this);
		clientGUI.add(b,"North");
		clientGUI.add(count,"Center");
		count.setFont(new java.awt.Font("SansSerif",140,140));
		clientGUI.addWindowListener(new java.awt.event.WindowAdapter(){
				/**
				 * Handles the window close message
				 */							
				public void windowClosing(java.awt.event.WindowEvent e){
					com.ms.win32.User32.PostThreadMessage(
									loopThreadID,
									com.ms.win32.win.WM_QUIT,
									0,
									0);
				}
			});
	}

/**
 * Creates an instance of the Hub2 COM object using a J/Direct call to
 * CoCreateInstanceEx through sample.util.DCOMLib.CoCreateInstanceEx.
 * The resulting COM object is cast to sample.IMessenger, and has its
 * addListener, sendMessage, sendObjects, and removeListener methods 
 * called.  The results from InprocClient and the IMessenger COM object
 * will be printed to System.out. 
 */
	public Client(){
		com.ms.com.ComLib.declareMessagePumpThread();
		loopThreadID = com.ms.win32.Kernel32.GetCurrentThreadId();
		int HRESULT = 0;
		try{
			System.out.println("Client :about to call CoCreateInstanceEx");
			mess = (sample.IPing)com.ms.win32.Ole32.CoCreateInstance(
                                      clsid,null,
                                      com.ms.win32.win.CLSCTX_SERVER,
                                      new _Guid("{00000000-0000-0000-C000-000000000046}"));
		}catch(Exception ii){
			System.out.println("Client: Exception "+ii);
			System.exit(1);
		}finally{
			System.out.println("Client - HResult: "+Integer.toHexString(HRESULT));
			System.out.println("Client - Object: "+mess);
		}
		startClientGUI();
		// The GUI will hang whenever a Dispatch call is made to the COM object, 
		// unless this thread (the one that created the COM Object) is declared as
		// a message pump, and handles the win32 messages.  If the object were 
		// created inproc rather than out-of-proc, the message pump would not be 
		// necessary.
		com.ms.win32.MSG msg = new com.ms.win32.MSG();
		while (com.ms.win32.User32.GetMessage(msg, 0, 0, 0)) {
			com.ms.win32.User32.TranslateMessage(msg);
			if (msg.message==com.ms.win32.win.WM_QUIT)
				break;
            com.ms.win32.User32.DispatchMessage(msg);
			}
		com.ms.com.ComLib.release(mess);
		// This timer is inserted to ensure that the ComLib.release has
		// enough time to signal the release of 'mess' before System.exit
		// is called.
		try{Thread.sleep(0);}catch(Exception ii){}
		System.exit(0);
	}
}