/*
 * sample.client.AWTClient.java
 *
 * (C) Copyright 1995 - 1999 Microsoft Corporation.  All rights reserved.
 *
 */
package sample.client;

import java.awt.*;
import java.awt.event.*;
import com.ms.com.*;
/**
 * This is a simple client application that uses only AWT components for creating
 * the GUI, which is used to connect to the Java implemented DCOM server.  The
 * GUID for the COM object is hard coded in the clsid field, however all of the
 * security information is user specified at runtime.  Error messages will be 
 * displayed in the bottom status area of the Browser, as well as information
 * on which ThreadID the COM object is running on if the call was successful.
 * 
 * This applet only uses the IDispatch interface for communicating with the COM
 * object, so there is no need to register any proxy/stub or tlb code on the 
 * client in order for this to work.
 *
 * @see sample.server.DCOMService
 *  
 * @version 1.0
 *
 */
public class AWTClient extends java.applet.Applet implements ActionListener { 
    /**
 	 * Class ID for the object to create.  This is the Class ID for the
 	 * Pinger object that is served by sample.server.DCOMService.
 	 */							   
	private final _Guid clsid = 
                  new _Guid("{BA009432-272A-11d2-B906-00C04F8C9491}");
    /**
     * The created instance of our COM object.
     */
    protected Object object = null;
    /** 
     * Text Field used to gather the user name to be used for CoCreateInstanceEx
     * security and for setting the proxy security on each interface.
     */
    protected TextField userText = new TextField(10);
    /** 
     * Text Field used to gather the user password to be used for CoCreateInstanceEx
     * security and for setting the proxy security on each interface.
     */    
    protected TextField passText = new TextField(10);
    /** 
     * Text Field used to gather the domain that the user is valid on, to be used 
     * for CoCreateInstanceEx security and for setting the proxy security on each 
     * interface.
     */
    protected TextField domainText = new TextField(10);
    /** 
     * Text Field used to gather the server name to be used for specifying which 
     * machine to create the DCOM object on.
     */
    protected TextField serverText = new TextField(10);
    /** 
     * Labels the User TextField.
     */
    protected Label userLabel = new Label("User :");
    /** 
     * Labels the Password TextField.
     */
    protected Label passLabel = new Label("Password :");
    /** 
     * Labels the Domain TextField.
     */
    protected Label domainLabel = new Label("Domain :");
    /** 
     * Labels the Server Name TextField.
     */
    protected Label serverLabel = new Label("Server :");
    /**
     * Groups the identify, and impersonate radio button options into a group.
     */
    protected CheckboxGroup area = new CheckboxGroup();
    /**
     * Radio button used to specify identify as the Impersonation level
     */
    protected Checkbox identify = new Checkbox();
    /**
     * Radio button used to specify impersonate as the Impersonation level
     */
    protected Checkbox impersonate = new Checkbox();
    /**
     * This is a Choice box that will list the Authentication Level options.
     */
    protected Choice authn = new Choice();
    /**
     * This is the main display area where the number representing the Ping count
     * is displayed.
     */
    protected Label count = new Label();
    /**
     * The button that will be used to invoke the method on the COM object through
     * IDispatch
     */                                     
    protected Button ping = new Button("Ping!");
    /**
     * Button used to connect to the COM object using the user specified 
     * credentials.
     */
    protected Button connect = new Button("Connect!");
    /**
     * Disconnects this Client from the COM object.
     */
    protected Button disconnect = new Button("Disconnect!");
    /**
     * Used to describe the 'None' Authentication Level.
     */
    protected final String NONE = "None";
    /**
     * Used to describe the 'Call' Authentication Level.
     */
    protected final String CALL = "Call";
    /**
     * Used to describe the 'Connect' Authentication Level.
     */
    protected final String CONNECT = "Connect";
    /**
     * Used to describe the 'Default' Authentication Level.
     */
    protected final String DEFAULT = "Default";
    /**
     * Used to describe the 'Packet' Authentication Level.
     */
    protected final String PACKET = "Packet";
    /**
     * Used to describe the 'Packet Integrity' Authentication Level.
     */
    protected final String PACKET_INTEGRITY = "Packet Integrity";
    /**
     * Used to describe the 'Packet Privacy' Authentication Level.
     */
    protected final String PACKET_PRIVACY = "Packet Privacy";
    /**
     * This thread will create the COM object
     */
    private Thread comObjectThread = null;
    /**
     * Flags the creation of a COM object.
     */
    private boolean creatingObject = false;
/**
 * handles all of the button events for connecting, disconnecting, and pinging the
 * COM object.
 * 
 * @param ActionEvent e - Event generated by the buttons when they are pressed.
 */
    public void actionPerformed(ActionEvent e){
        if (e.getActionCommand().equals("Ping!")) 
            this.processPing();
        if (e.getActionCommand().equals("Connect!")) 
            this.processConnect();
        if (e.getActionCommand().equals("Disconnect!")) 
            this.processDisconnect();
    }
/**
 * Used to create the Login Panel, with the duser, password, domain and servername
 * text areas.
 * 
 * @return Panel - containing the login text fields.
 */                                       
    protected Panel getLoginPanel(){
        Panel p = new Panel();
        GridLayout g = new GridLayout(4,2);
        passText.setEchoChar('*');
        p.setLayout(g);
        p.add(userLabel);
        p.add(userText);
        p.add(passLabel);
        p.add(passText);
        p.add(domainLabel);
        p.add(domainText);
        p.add(serverLabel);
        p.add(serverText);
        return p;
    }
/**
 * Worker method used to create the options panel for the display.  The options
 * consist of the CheckBox group with the impersonate and identify buttons, as well
 * as the authentication level options.
 * 
 * @return Panel - contains all of the user definable options.
 */
    protected Panel getOptionsPanel(){
        Panel p = new Panel();
        p.setLayout(new GridLayout(2,1));
        identify = new Checkbox("Identify",area,false);
        identify.setState(true);
        impersonate = new Checkbox("Impersonate",area,false);
        p.add(identify);
        p.add(impersonate);
        Panel mite = new Panel();
        mite.setLayout(new GridLayout(2,1));
        mite.add(p);
        Panel poo = new Panel();
        authn.add(NONE);
        authn.add(CALL);
        authn.add(CONNECT);
        authn.add(DEFAULT);
        authn.add(PACKET);
        authn.add(this.PACKET_INTEGRITY);
        authn.add(this.PACKET_PRIVACY);
        poo.add(this.authn);
        mite.add(poo);
        return mite;
       
    }
/**
 * Creates a panel containing the Ping, Connect and Disconnect buttons.
 * 
 * @return Panel - containing all of the buttons for the GUI.
 */
    protected Panel getButtonPanel(){
        Panel p = new Panel();
        p.setLayout(new GridLayout(1,3));
        p.add(ping);
        ping.addActionListener(this);
        p.add(connect);
        connect.addActionListener(this);
        p.add(disconnect);
        disconnect.addActionListener(this);
        return p;
    }
/**
 * Simple constructor that sets up the GUI by, calling all of the worker methods
 * for creating the components, and adding them to the main application space.
 */    
    public AWTClient(){
        Panel p = new Panel();
        p.setLayout(new GridLayout(1,2));
        p.add(getLoginPanel());
        p.add(getOptionsPanel());
        Panel top = new Panel();
        top.setLayout(new GridLayout(2,1));
        top.add(p);
        top.add(getButtonPanel());
        
        count.setFont(new java.awt.Font("SansSerif",140,140));
        count.setForeground(Color.red);
        count.setText("             ");
        
        Panel q = new Panel();
        q.setLayout(new GridLayout(2,1));
        q.add(top);
        q.add(count);
        this.add("Center",q);
    }
/**
 * This is an internal worker method that is used to transpose the option selected
 * for the authentication level from a List of items to the correct 
 * RPC_C_AUTHN_LEVEL_* flag.
 * 
 * @return - the correct RPC_C_AUTHN_LEVEL_* flag
 */
    protected int getAuthnLevel(){
        String authn = (String)this.authn.getSelectedItem();  
        if (authn.equals(NONE))
            return com.ms.win32.win.RPC_C_AUTHN_LEVEL_NONE;
        else if (authn.equals(CALL))
            return com.ms.win32.win.RPC_C_AUTHN_LEVEL_CALL;
        else if (authn.equals(CONNECT))
            return com.ms.win32.win.RPC_C_AUTHN_LEVEL_CONNECT;
        else if (authn.equals(DEFAULT))
            return com.ms.win32.win.RPC_C_AUTHN_LEVEL_DEFAULT;
        else if (authn.equals(PACKET))
            return com.ms.win32.win.RPC_C_AUTHN_LEVEL_PKT;
        else if (authn.equals(PACKET_INTEGRITY))
            return com.ms.win32.win.RPC_C_AUTHN_LEVEL_PKT_INTEGRITY;
        else if (authn.equals(PACKET_PRIVACY))
            return com.ms.win32.win.RPC_C_AUTHN_LEVEL_PKT_PRIVACY;
        else
            return com.ms.win32.win.RPC_C_AUTHN_LEVEL_NONE;
    }
/**
 * Creates the COSERVERINFO structure, populating each of the sub structures and
 * fields with the appropriate results from the user specified input.
 * 
 * @return COSERVERINFO structure that can be used in the CoCreateInstanceEx calls
 *          and inthe CoSetProxyBlanket calls.
 */
    protected sample.dcom.COSERVERINFO getCOSERVERINFO(){
        int impLevel;
        if (area.getSelectedCheckbox()==impersonate)
            impLevel=com.ms.win32.win.RPC_C_IMP_LEVEL_IMPERSONATE;
        else 
            impLevel=com.ms.win32.win.RPC_C_IMP_LEVEL_IDENTIFY;
       //NOTE: localhost and 127.0.0.1 
                                            // will NOT work here. this will be 
                                            // fixed with NT Service Pack 4.0
        sample.dcom.COSERVERINFO coServerInfo = new sample.dcom.COSERVERINFO(this.serverText.getText(),
																			 getAuthnLevel(),
																			 impLevel,
																			 userText.getText(),
																			 domainText.getText(),
																			 passText.getText());
		return coServerInfo;
    }
/**
 * This method will perform the connection to the user specified server, using the
 * user specified security credentials and settings.  It uses a J/Direct call to
 * CoCreateInstanceEx for creating the object, and then call CoSetProxyBlanket to 
 * set the security credentials on the IUnknown and IDispatch interfaces.  If a
 * call is made to this method for a connection, and an object has already been 
 * created, that object will be disconnected before the connection begins.  Once
 * the connection has completed, a processPing call is made.  The thread 
 * information (Win32 Thread ID, and Java Thread info) is displayed in the status
 * area of the browser.  The count of the ping calls made is displayed in the main
 * application area.
 */ 
    private synchronized void processConnect() {
        if (!creatingObject){
            creatingObject=true;
            if (object!=null)   
                processDisconnect();
                comObjectThread = new Thread(){
                    public void run(){
                        createInstance();
                        creatingObject=false;
                        if (object!=null)
                            processPing();
                        synchronized(this){
                            try{wait();}catch(Exception ii){
                                System.out.println("Exception waiting "+ii);
                            }
                        }
                        System.out.println("COM Create Thread stopping");
                    }//end of run
                };//End of Thread t
            com.ms.com.ComLib.startMTAThread(comObjectThread);
        }
    }
/**
 * Uses a CoCreateInstanceEx call to create an instance of the COM object that will be used
 * and then sets the appropriate security on its interfaces.
 */
    private void createInstance(){
         int HRESULT = 0;
  		try{
            sample.dcom.COSERVERINFO ser = getCOSERVERINFO();
            
            //Create the Multi_Qi structure idicating which interfaces we want
            
            MULTI_QI[] mul = (MULTI_QI[])sample.util.StructArray.newStructArray(MULTI_QI.class,1);
            mul[0].pIID = new _Guid("{00000000-0000-0000-C000-000000000046}");
            getAppletContext().showStatus("Connecting...");
            
            //Perform the actual Createinstance call.      
            sample.dcom.DCOMLib.CoCreateInstanceEx(
                                     clsid,
                                     null,
                                     com.ms.win32.win.CLSCTX_LOCAL_SERVER|
                                     com.ms.win32.win.CLSCTX_REMOTE_SERVER,
                                     ser,
                                     1,
                                     sample.util.StructArray.toCStructArray(mul));
            getAppletContext().showStatus("CoCreateInstanceEx HRESULT = "+
                           Integer.toHexString(HRESULT));
            object=mul[0].pItf;
           
           //We only begin to set the proxy security if the create was successful
            if(HRESULT==0)
                this.setSecurity(ser,object);
        }catch(Exception ii){
	        this.getAppletContext().showStatus(HRESULT+" "+ii);
        }// Handles a possible exception from CoCreateInstanceEx
	    
        com.ms.lang.SystemX.gc();//Ensure that the reference to the object held 
                                //by the MULTI_QI structure is GCd
    }
/**
 * Sets the security on the IUnknown and IDispatch interfaces for the given COM object, 
 * using the security credentials from the COSERVERINFO structure.  Typically the 
 * security credentials will be the same as those provided to the CoCreateInstanceEx call.
 * 
 * @param COSERVERINFO ser - Contains the security credentials
 * 
 * @param Object object - COM object to set security on.
 */
    private void setSecurity(sample.dcom.COSERVERINFO ser, Object object){
		int ref = com.ms.com.ComLib.unknownToPtr(object,
                      new _Guid("{00000000-0000-0000-C000-000000000046}"));
        int HRESULT = sample.dcom.DCOMLib.CoSetProxyBlanket(
												ref,
                                                ser.pAuthInfo.dwAuthnSvc,
                                                ser.pAuthInfo.dwAuthzSvc,
                                                ser.pAuthInfo.pwszServerPrincName,
                                                ser.pAuthInfo.dwAuthnLevel,
                                                ser.pAuthInfo.dwImpersonationLevel,
                                                ser.pAuthInfo.pAuthIdentity,
                                                ser.pAuthInfo.dwCapabilities);
		ComLib.ptrToUnknown(IUnknown.class,ref,false);
      	        getAppletContext().showStatus("CoSetProxyBlanket HRESULT = "+
                           Integer.toHexString(HRESULT));
            try{// This second call may throw a ClassCastException if the security
                // settings for IUnknown are insufficient.  The user needs to 
                // specify a higher level of security credentials, ie connect 
                // rather than none for authentication, or impersonate rather than
                // identify.
                int addd =   com.ms.com.ComLib.unknownToPtr(object,
                                   new _Guid("{00020400-0000-0000-C000-000000000046}"));
                HRESULT = sample.dcom.DCOMLib.CoSetProxyBlanket(
                                                addd,
                                                ser.pAuthInfo.dwAuthnSvc,
                                                ser.pAuthInfo.dwAuthzSvc,
                                                ser.pAuthInfo.pwszServerPrincName,
                                                ser.pAuthInfo.dwAuthnLevel,
                                                ser.pAuthInfo.dwImpersonationLevel,
                                                ser.pAuthInfo.pAuthIdentity,
                                                ser.pAuthInfo.dwCapabilities);
				ComLib.ptrToUnknown(IUnknown.class,addd,false);
      	        getAppletContext().showStatus("CoSetProxyBlanket HRESULT = "+
                           Integer.toHexString(HRESULT));
            }catch(ClassCastException c){
                this.getAppletContext().showStatus("Security Credentials are insufficient");
                com.ms.com.ComLib.release(object);
                object=null;
                return;
            }
    }
/**
 * Disconnects from the currently created COM object, by calling com.ms.com.ComLib
 * release method, and updating the status area.
 */
    private void processDisconnect() {
        if (object!=null)
            com.ms.com.ComLib.release(object);
        if (this.comObjectThread!=null)
            synchronized (comObjectThread){
                comObjectThread.notify();
            }
	    object=null;
        count.setText("");
        this.getAppletContext().showStatus("Disconnected...");
    }
/**
 * Invokes the Ping method through the IDispatch interface of the created COM 
 * object.  The status area is updated with the Thread information, such as the 
 * windows THread ID and the Java Thread information provided by toString().  The
 * count returned from the ping method is displayed in the large area in the 
 * application.
 */
    private synchronized void processPing() {
        if (object!=null){
            try{
       		String t = com.ms.com.Dispatch.invokev(object,
									com.ms.com.Dispatch.getIDOfName(object,"ping"),
                                    com.ms.com.Dispatch.Method,
                                    new com.ms.com.Variant[]{},
                                    null).toString();
            
            this.getAppletContext().showStatus(t.substring(t.indexOf(":")+1,t.length()));
		count.setText(t.substring(0,t.indexOf(":")));
            }catch(com.ms.com.ComFailException cc){
                this.getAppletContext().showStatus("Dispatch Error : "+Integer.toHexString(cc.getHResult()));
            }catch(com.ms.com.ComError ee){
                 this.getAppletContext().showStatus("Dispatch QI error : ");
            }
        }else{
            this.getAppletContext().showStatus(
                                "You must connect to the server first!");
        }
    }
}