/*
 * sample.client.ClientGUI.java
 *
 * (C) Copyright 1995 - 1999 Microsoft Corporation.  All rights reserved.
 *
 */
package sample.client;

import com.ms.wfc.app.*;
import com.ms.wfc.core.*;
import com.ms.wfc.ui.*;
import com.ms.wfc.html.*;
import sample.dcom.*;
import com.ms.com.*;
/**
 * This is a simple client application that uses WFC components for creating
 * the GUI, which is used to connect to the Java implemented DCOM server.  The
 * GUID for the COM object is hard coded in the clsid field, however all of the
 * security information is user specified at runtime.  Error messages will be 
 * displayed in the bottom status area, the Title of the Frame contains information
 * on which ThreadID the COM object is running on if the call was successful.
 * 
 * This application only uses the IDispatch interface for communicating with the COM
 * object, so there is no need to register any proxy/stub or tlb code on the 
 * client in order for this to work.
 *
 * @see sample.server.DCOMService
 *  
 * @version 1.0
 */
public class ClientGUI extends Form{
    /**
 	 * Class ID for the object to create.  This is the Class ID for the
 	 * Pinger object that is served by sample.server.DCOMService.
 	 */							   
	private final String clsid = "{BA009432-272A-11d2-B906-00C04F8C9491}";
    /**
     * The created instance of our COM object.
     */
    protected Object object = null;
     /**
     * Used to describe the 'None' Authentication Level.
     */
    protected final String NONE = "None";
    /**
     * Used to describe the 'Call' Authentication Level.
     */
    protected final String CALL = "Call";
    /**
     * Used to describe the 'Connect' Authentication Level.
     */
    protected final String CONNECT = "Connect";
    /**
     * Used to describe the 'Default' Authentication Level.
     */
    protected final String DEFAULT = "Default";
    /**
     * Used to describe the 'Packet' Authentication Level.
     */
    protected final String PACKET = "Packet";
    /**
     * Used to describe the 'Packet Integrity' Authentication Level.
     */
    protected final String PACKET_INTEGRITY = "Packet Integrity";
    /**
     * Used to describe the 'Packet Privacy' Authentication Level.
     */
    protected final String PACKET_PRIVACY = "Packet Privacy";
/**
 * Constructor for setting up the GUI.
 */    
    public ClientGUI(){
		super();
		// Required for Visual J++ Form Designer support
		initForm();		
        count.setFont(new com.ms.wfc.ui.Font("Times New Roman",80));
        authn.setSelectedIndex(0);
        authn.setText(NONE);
        authn.setItems(new Object[] {
                       NONE, 
                       CALL, 
                       CONNECT, 
                       DEFAULT, 
                       PACKET, 
                       PACKET_INTEGRITY, 
                       PACKET_PRIVACY});
	}
/**
 * ClientGUI overrides dispose so it can clean up the
 * component list.
 */
	public void dispose(){
		super.dispose();
		components.dispose();
        this.processDisconnect(null,null);
	}
/**
 * Invokes the Ping method through the IDispatch interface of the created COM 
 * object.  The status area is updated with the Thread information, such as the 
 * windows Thread ID and the Java Thread information provided by toString().  The
 * count returned from the ping method is displayed in the large area in the 
 * application.
 */
    private synchronized void processPing(Object source, Event e) {
        if (object!=null){
            try{
                String t = com.ms.com.Dispatch.invokev(object,
										com.ms.com.Dispatch.getIDOfName(object,"ping"),
                                        com.ms.com.Dispatch.Method,
                                        new Variant[]{},
                                        null).toString();
		        this.setText(t.substring(t.indexOf(":")+1,t.length()));
		        count.setText(t.substring(0,t.indexOf(":")));
            }catch(com.ms.com.ComFailException cc){
                status.setText("Dispatch Error : "+Integer.toHexString(cc.getHResult()));
            }catch(com.ms.com.ComError ee){
                 status.setText("Dispatch error : Access Denied");
            }

        }else{
            MessageBox.show("You must connect to the server first!","ERROR!",
                            MessageBox.ICONERROR);
        }
    }
/**
 * This is an internal worker method that is used to transpose the option selected
 * for the authentication level from a List of items to the correct 
 * RPC_C_AUTHN_LEVEL_* flag.
 * 
 * @return - the correct RPC_C_AUTHN_LEVEL_* flag
 */
    protected int getAuthnLevel(){
        String authn = (String)this.authn.getSelectedItem();  
        if (authn==null)
            return com.ms.win32.win.RPC_C_AUTHN_LEVEL_NONE;
        if (authn.equals(NONE))
            return com.ms.win32.win.RPC_C_AUTHN_LEVEL_NONE;
        else if (authn.equals(CALL))
            return com.ms.win32.win.RPC_C_AUTHN_LEVEL_CALL;
        else if (authn.equals(CONNECT))
            return com.ms.win32.win.RPC_C_AUTHN_LEVEL_CONNECT;
        else if (authn.equals(DEFAULT))
            return com.ms.win32.win.RPC_C_AUTHN_LEVEL_DEFAULT;
        else if (authn.equals(PACKET))
            return com.ms.win32.win.RPC_C_AUTHN_LEVEL_PKT;
        else if (authn.equals(PACKET_INTEGRITY))
            return com.ms.win32.win.RPC_C_AUTHN_LEVEL_PKT_INTEGRITY;
        else if (authn.equals(PACKET_PRIVACY))
            return com.ms.win32.win.RPC_C_AUTHN_LEVEL_PKT_PRIVACY;
        else
            return com.ms.win32.win.RPC_C_AUTHN_LEVEL_NONE;
    }
/**
 * Creates the COSERVERINFO structure, populating each of the sub structures and
 * fields with the appropriate results from the user specified input.
 * 
 * @return COSERVERINFO structure that can be used in the CoCreateInstanceEx calls
 *          and inthe CoSetProxyBlanket calls.
 */
    protected sample.dcom.COSERVERINFO getCOSERVERINFO(){
        int impLevel;
        if (impersonate.getChecked())
            impLevel=com.ms.win32.win.RPC_C_IMP_LEVEL_IMPERSONATE;
        else 
            impLevel=com.ms.win32.win.RPC_C_IMP_LEVEL_IDENTIFY;
       //NOTE: localhost and 127.0.0.1 
                                            // will NOT work here. this will be 
                                            // fixed with NT Service Pack 4.0
        sample.dcom.COSERVERINFO coServerInfo = new sample.dcom.COSERVERINFO(this.serverText.getText(),
																			 getAuthnLevel(),
																			 impLevel,
																			 userText.getText(),
																			 domainText.getText(),
																			 passText.getText());
		/*
        coServerInfo.pwszName = serverText.getText();
        coServerInfo.pAuthInfo = new sample.dcom.COAUTHINFO();
        coServerInfo.pAuthInfo.dwAuthnLevel = 
                         getAuthnLevel();
        coServerInfo.pAuthInfo.dwAuthnSvc = 
                    com.ms.win32.win.RPC_C_AUTHN_DEFAULT;
        coServerInfo.pAuthInfo.dwAuthzSvc = 
                    com.ms.win32.win.RPC_C_AUTHZ_NONE;
        coServerInfo.pAuthInfo.dwImpersonationLevel = 
                                                     impLevel;
        coServerInfo.pAuthInfo.pAuthIdentity = 
                          new sample.dcom.COAUTHIDENTITY();
        coServerInfo.pAuthInfo.pAuthIdentity.domain=
                                domainText.getText();
        coServerInfo.pAuthInfo.pAuthIdentity.user = 
                                userText.getText();
        coServerInfo.pAuthInfo.pAuthIdentity.password =
                                passText.getText();
		coServerInfo.pAuthInfo.pAuthIdentity.domainLength=
							domainText.getText().length();
		coServerInfo.pAuthInfo.pAuthIdentity.userLength=
							userText.getText().length();
		coServerInfo.pAuthInfo.pAuthIdentity.passwordLength =
							passText.getText().length();
		coServerInfo.pAuthInfo.pAuthIdentity.flags=2;
		*/
         return coServerInfo;
    }
/**
 * This method will perform the connection to the user specified server, using the
 * user specified security credentials and settings.  It uses a J/Direct call to
 * CoCreateInstanceEx for creating the object, and then call CoSetProxyBlanket to 
 * set the security credentials on the IUnknown and IDispatch interfaces.  If a
 * call is made to this method for a connection, and an object has already been 
 * created, that object will be disconnected before the connection begins.  Once
 * the connection has completed, a processPing call is made.  The thread 
 * information (Win32 Thread ID, and Java Thread info) is displayed in the title
 * area of the frame.  The count of the ping calls made is displayed in the main
 * application area.
 */ 
    private synchronized void processConnect(Object source, Event e) {
        if (object!=null)   
            processDisconnect(null,null);
        int HRESULT = 0;
  		try{
            sample.dcom.COSERVERINFO ser = getCOSERVERINFO();
            
            //Create the Multi_Qi structure idicating which interfaces we want
            MULTI_QI[] mul = (MULTI_QI[])sample.util.StructArray.newStructArray(MULTI_QI.class,1);
            mul[0].pIID = new _Guid("{00000000-0000-0000-C000-000000000046}");
            sample.util.CStructArray cs = sample.util.StructArray.toCStructArray(mul);
            status.setText("Connecting...");
            //Perform the actual Createinstance call.      
            HRESULT = DCOMLib.CoCreateInstanceEx(
                                     new com.ms.com._Guid(clsid),
                                     null,
                                     com.ms.win32.win.CLSCTX_LOCAL_SERVER|
                                     com.ms.win32.win.CLSCTX_REMOTE_SERVER,
                                     ser,
                                     1,
                                     cs);
            status.setText("CoCreateInstanceEx HRESULT = "+
                           Integer.toHexString(HRESULT));
           object=mul[0].pItf;
           
           //We only begin to set the proxy security if the create was successful
            if(HRESULT==0){
				int ref = com.ms.com.ComLib.unknownToPtr(object,
                            new _Guid("{00000000-0000-0000-C000-000000000046}"));
                HRESULT = DCOMLib.CoSetProxyBlanket(
												ref,
                                                ser.pAuthInfo.dwAuthnSvc,
                                                ser.pAuthInfo.dwAuthzSvc,
                                                ser.pAuthInfo.pwszServerPrincName,
                                                ser.pAuthInfo.dwAuthnLevel,
                                                ser.pAuthInfo.dwImpersonationLevel,
                                                ser.pAuthInfo.pAuthIdentity,
                                                ser.pAuthInfo.dwCapabilities);
				ComLib.ptrToUnknown(IUnknown.class,ref,false);
      	        status.setText("CoSetProxyBlanket IUnknown HRESULT = "+
                           Integer.toHexString(HRESULT));
            try{// This second call may throw a ClassCastException if the security
                // settings for IUnknown are insufficient.  The user needs to 
                // specify a higher level of security credentials, ie connect 
                // rather than none for authentication, or impersonate rather than
                // identify.
                
                int addd =   com.ms.com.ComLib.unknownToPtr(object,
                                                            new _Guid("{00020400-0000-0000-C000-000000000046}"));
                HRESULT = DCOMLib.CoSetProxyBlanket(
                                                addd,
                                                ser.pAuthInfo.dwAuthnSvc,
                                                ser.pAuthInfo.dwAuthzSvc,
                                                ser.pAuthInfo.pwszServerPrincName,
                                                ser.pAuthInfo.dwAuthnLevel,
                                                ser.pAuthInfo.dwImpersonationLevel,
                                                ser.pAuthInfo.pAuthIdentity,
                                                ser.pAuthInfo.dwCapabilities);
				ComLib.ptrToUnknown(IUnknown.class,addd,false);
                status.setText("CoSetProxyBlanket IDispatch HRESULT = "+
                           Integer.toHexString(HRESULT));
            }catch(ClassCastException c){
                status.setText("Security Credentials are insufficient");
                com.ms.com.ComLib.release(object);
                object=null;
                return;
            }
            }// End of if - End of CoSetProxyBlanket    
        }catch(Exception ii){
	        status.setText(HRESULT+" "+ii);
        }// Handles a possible exception from CoCreateInstanceEx
	    com.ms.lang.SystemX.gc();//Ensure that the reference to the object held 
                                //by the MULTI_QI structure is GCd
        if (HRESULT==0)
            processPing(null,null);
    }
/**
 * Disconnects from the currently created COM object, by calling com.ms.com.ComLib
 * release method, and updating the status area.
 */
    private void processDisconnect(Object source, Event e) {
        if (object!=null)
            com.ms.com.ComLib.release(object);
	    object=null;
        count.setText("");
        status.setText("Disconnected...");
    }

    /**
     * NOTE: The following code is required by the Visual J++ form
     * designer.  It can be modified using the form editor.  Do not
     * modify it using the code editor.
     */
    Container components = new Container();
    Panel panel1 = new Panel();
    Splitter splitter1 = new Splitter();
    Panel panel2 = new Panel();
    StatusBar status = new StatusBar();
    GroupBox groupBox1 = new GroupBox();
    RadioButton identify = new RadioButton();
    RadioButton impersonate = new RadioButton();
    Button button1 = new Button();
    ComboBox authn = new ComboBox();
    Button button2 = new Button();
    Button button3 = new Button();
    Edit userText = new Edit();
    Edit passText = new Edit();
    Edit domainText = new Edit();
    Edit serverText = new Edit();
    Label label1 = new Label();
    Label label2 = new Label();
    Label label3 = new Label();
    Label label4 = new Label();
    Label count = new Label();

    private void initForm() { 
        this.setText("ClientGUI");
        this.setAutoScaleBaseSize(new Point(5, 13));
        this.setClientSize(new Point(437, 348));

        panel1.setBackColor(Color.ACTIVEBORDER);
        panel1.setDock(ControlDock.TOP);
        panel1.setSize(new Point(437, 152));
        panel1.setTabIndex(3);
        panel1.setText("panel1");

        splitter1.setCursor(Cursor.HSPLIT);
        splitter1.setDock(ControlDock.TOP);
        splitter1.setLocation(new Point(0, 152));
        splitter1.setSize(new Point(437, 8));
        splitter1.setTabIndex(2);
        splitter1.setTabStop(false);

        panel2.setBackColor(Color.CONTROLLIGHTLIGHT);
        panel2.setDock(ControlDock.FILL);
        panel2.setLocation(new Point(0, 160));
        panel2.setSize(new Point(437, 164));
        panel2.setTabIndex(1);
        panel2.setText("panel2");

        status.setBackColor(Color.CONTROL);
        status.setLocation(new Point(0, 324));
        status.setSize(new Point(437, 24));
        status.setTabIndex(0);
        status.setText("");

        groupBox1.setLocation(new Point(304, 8));
        groupBox1.setSize(new Point(120, 64));
        groupBox1.setTabIndex(4);
        groupBox1.setText("Impersonation Level");

        identify.setLocation(new Point(16, 24));
        identify.setSize(new Point(112, 16));
        identify.setTabIndex(1);
        identify.setTabStop(true);
        identify.setText("Identify");
        identify.setChecked(true);

        impersonate.setLocation(new Point(16, 40));
        impersonate.setSize(new Point(96, 16));
        impersonate.setTabIndex(0);
        impersonate.setTabStop(true);
        impersonate.setText("Impersonate");

        button1.setLocation(new Point(264, 120));
        button1.setSize(new Point(80, 24));
        button1.setTabIndex(6);
        button1.setText("Connect!");
        button1.addOnClick(new EventHandler(this.processConnect));

        authn.setLocation(new Point(304, 88));
        authn.setSize(new Point(120, 21));
        authn.setTabIndex(5);
        authn.setText("");

        button2.setLocation(new Point(352, 120));
        button2.setSize(new Point(72, 24));
        button2.setTabIndex(8);
        button2.setText("Disconnect");
        button2.addOnClick(new EventHandler(this.processDisconnect));

        button3.setLocation(new Point(8, 112));
        button3.setSize(new Point(216, 32));
        button3.setTabIndex(7);
        button3.setText("PING!");
        button3.addOnClick(new EventHandler(this.processPing));

        userText.setLocation(new Point(96, 8));
        userText.setSize(new Point(152, 20));
        userText.setTabIndex(0);
        userText.setText("");

        passText.setLocation(new Point(96, 32));
        passText.setSize(new Point(152, 20));
        passText.setTabIndex(1);
        passText.setText("");
        passText.setPasswordChar('*');

        domainText.setLocation(new Point(96, 56));
        domainText.setSize(new Point(152, 20));
        domainText.setTabIndex(2);
        domainText.setText("");

        serverText.setLocation(new Point(96, 80));
        serverText.setSize(new Point(152, 20));
        serverText.setTabIndex(3);
        serverText.setText("");

        label1.setLocation(new Point(16, 8));
        label1.setSize(new Point(72, 16));
        label1.setTabIndex(12);
        label1.setTabStop(false);
        label1.setText("User Name:");

        label2.setLocation(new Point(16, 32));
        label2.setSize(new Point(72, 16));
        label2.setTabIndex(11);
        label2.setTabStop(false);
        label2.setText("Password:");

        label3.setLocation(new Point(16, 56));
        label3.setSize(new Point(64, 16));
        label3.setTabIndex(10);
        label3.setTabStop(false);
        label3.setText("Domain");

        label4.setLocation(new Point(16, 80));
        label4.setSize(new Point(64, 16));
        label4.setTabIndex(9);
        label4.setTabStop(false);
        label4.setText("Server Name");

        count.setAnchor(ControlAnchor.ALL);
        count.setForeColor(Color.RED);
        count.setLocation(new Point(32, 16));
        count.setSize(new Point(368, 144));
        count.setTabIndex(0);
        count.setTabStop(false);
        count.setText("");
        count.setTextAlign(HorizontalAlignment.CENTER);

        this.setNewControls(new Control[] {
                            status, 
                            panel2, 
                            splitter1, 
                            panel1});
        panel1.setNewControls(new Control[] {
                              label4, 
                              label3, 
                              label2, 
                              label1, 
                              serverText, 
                              domainText, 
                              passText, 
                              groupBox1, 
                              userText, 
                              button3, 
                              button2, 
                              button1, 
                              authn});
        panel2.setNewControls(new Control[] {
                              count});
        groupBox1.setNewControls(new Control[] {
                                 impersonate, 
                                 identify});
    }
/**
 * The main entry point for the application. 
 *
 * @param args Array of parameters passed to the application
 * via the command line.
 */
	public static void main(String args[]){
		Application.run(new ClientGUI());
	}
}
