/*
 *  sample.dcom.COAUTHIDENTITY.java
 *
 *  (C) Copyright 1995 - 1999 Microsoft Corporation.  All rights reserved.
 *
 */
package sample.dcom;

/** 
 * The SEC_WINNT_AUTH_IDENTITY structure allows you to pass a particular user 
 * name and password to the runtime library for the purpose of authentication. 
 * Note that this structure must remain valid for the lifetime of the binding 
 * handle.
 * 
 * For Windows 95, Windows 3.x and MS-DOS, the strings are ANSI; for Windows NT, 
 * the strings may be ANSI or Unicode, depending on the value you assign to the 
 * Flags field. For Windows NT, Windows 95, and Macintosh, the values for 
 * UserLength; DomainLength, and PasswordLength are the length of the 
 * corresponding string without the terminating 0X0000.
 * 
 * This class will also act as a custom marshaler, it should be used whenever 
 * the COAUTHIDENTITY struct is embedded within another structure.  When used as a 
 * field in another struct, it must have a dll.structmap directive like this:
 * 
 * **
 * * dll.structmap([customMarshal="sample.dcom.COAUTHIDENTITY", type=CUSTOM]) 
 * *
 *  public COAUTHIDENTITY foo;
 * 
 * @see sample.dcom.COAUTHINFO 
 * @see sample.dcom.DCOMLib
 * 
 * @version 1.0
 *  
 * @dll.struct(unicode) 
 */
public final class COAUTHIDENTITY implements Cloneable{
    /**
     * The Java VM will read this field to determine how many bytes the custom
     * marshaled object takes up in the native C.
     * 
     * NOTE: static struct fields are NOT considered part of the native C
     *          structure!
     */
    public static int cbByValSize = 4;
/** 
 * String containing the user name. 
 */
    public String user;
/** 
 * Number of characters in the user name, not counting the terminator.  This
 * is exactly the same as the length of a java String.
 */
	public int userLength;
/**
 * String containing the domain name or the workgroup name. 
 */
	public String domain;
/** 
 * Number of characters in the domain, not counting the terminator.  This
 * is exactly the same as the length of a java String.
 */
	public int domainLength;
/** 
 * String containing the user's password in the domain or workgroup. 
 */
	public String password;
/** 
 * Number of characters in the password, not counting the terminator.  This
 * is exactly the same as the length of a java String.
 */
	public int passwordLength;
/** 
 * Long value specifying that the strings are ANSI (0x1) or Unicode (0x2).
 */
	public int flags;
    
/**
 * Simple constructor for creating an emptu COAUTHIDENTITY struct
 */
    public COAUTHIDENTITY(){}
/**
 * Constructer that automatically sets the length of each string, and the flags to
 * 2 - for UNICODE.
 */
    public COAUTHIDENTITY(String user,
                          String domain,
                          String password){
        this.user=user;
        if (user!=null)
            this.userLength=user.length();
        this.domain=domain;
        if (domain!=null)
            this.domainLength=domain.length();
        this.password=password;
        if (password!=null)
            this.passwordLength=password.length();
        this.flags=2;
    }
/**
 * Initializes all values in the struct.
 */
    public COAUTHIDENTITY(String user, 
                          int userLength,
                          String domain,
                          int domainLength,
                          String password, 
                          int passwordLength,
                          int flags){
        this.user =user;
        this.userLength=userLength;
        this.domain=domain;
        this.domainLength=domainLength;
        this.password = password;
        this.passwordLength=passwordLength;
        this.flags=flags;
    }
 /**
  * This overrides the default clone method, creating a new instance of this
  * struct with exactly the same values.
  * 
  * @return Object - the new struct with identical values.
  */
    public Object clone(){
        return new COAUTHIDENTITY(user,userLength,domain,domainLength,
                                  password,passwordLength,flags);   
    }
/**
 * The Java VM will call this method to marshal the COAUTHIDENTITY structure from
 * C back to Java.
 * 
 * @param ppStruct - This is a pointer to a memory location where the C structure is.
 * @param Flags - any flags that have been specified by the Java Object representing
 *          the C structure. [Normally this is unused].
 * 
 * @return COAUTHIDENTITY - this is a Java representation of the C structure. This
 *          value will be placed where the custom marshaled field is.
 */    
    public static COAUTHIDENTITY toJava(int ppStruct, int Flags){  
        //This causes a propogate on all of the fields in the COAUTHIDENTITY struct.
        return (COAUTHIDENTITY)com.ms.dll.DllLib.ptrToStruct(COAUTHIDENTITY.class,
                                                com.ms.dll.DllLib.read4(ppStruct));  
    }
/**
 * The Java VM will call this method to marshal the field specified with this 
 * custom marshaler to native C.  This is accomplished by propogating any Java
 * objects to native C, and setting the ppStruct value to point to a pinned 
 * COAUTHIDENTITY struct.
 * 
 * @param COAUTHIDENTITY struct - the struct to be marshaled to native C.
 * @param int Flags - Any flags specified in the dll.structmap declaration for the
 *          field containing the COAUTHIDENTITY object.
 * @param int ppStruct - this is the memory location of where to write the address
 *          of the marshaled COAUTHIDENTITY struct.
 */    
    public static void toExternal(COAUTHIDENTITY struct, int Flags, int ppStruct ) { 
        com.ms.dll.DllLib.propagateStructFields(struct,false);
        int handle = com.ms.dll.Root.alloc(struct);
        com.ms.dll.DllLib.write4(ppStruct,com.ms.dll.DllLib.addrOf(handle));
        com.ms.dll.Root.free(handle);
    }
}