/*
 *  sample.dcom.COSERVERINFO.java
 *
 *  (C) Copyright 1995 - 1999 Microsoft Corporation.  All rights reserved.
 *
 */
package sample.dcom;

/** 
 * The COSERVERINFO structure is used primarily to identify a remote system in 
 * object creation functions. Machine resources are named using the naming 
 * scheme of the network transport. By default, all UNC ("\\server" or "server") 
 * and DNS names ("server.com", "www.foo.com", or "135.5.33.19") names are 
 * allowed. 
 * 
 * If you are using the NTLMSSP security package, the default case, the pAuthinfo 
 * parameter should be set to zero. If you are a vendor supporting another 
 * security package, refer to COAUTHINFO. The mechanism described there is 
 * intended to allow DCOM activations to work correctly with security providers 
 * other than NTLMSSP, or to specify additional security information used during 
 * remote activations for interoperability with alternate implementations of 
 * DCOM. If pAuthInfo is set, those values will be used to specify the 
 * authentication settings for the remote call. These settings will be passed to 
 * RpcBindingSetAuthInfoEx. 
 * 
 * If the pAuthInfo field is not specified, any values in the AppID section of 
 * the registry will be used to override the following default authentication 
 * settings:
 * 
 *			dwAuthnSvc				RPC_C_AUTHN_WINNT 
 *			dwAuthzSvc				RPC_C_AUTHZ_NONE 
 *			pszServerPrincName		NULL 
 *			dwAuthnLevel			RPC_C_AUTHN_LEVEL_CONNECT 
 *			dwImpersonationLevel	RPC_C_IMP_LEVEL_IMPERSONATE 
 *			pvAuthIdentityData		NULL 
 *			dwCapabilities			RPC_C_QOS_CAPABILITIES_DEFAULT 
 * 
 * This class will also act as a custom marshaler, it should be used whenever 
 * the COSERVERINFO struct is embedded within another structure.  When used as a 
 * field in another struct, it must have a dll.structmap directive like this:
 * 
 * **
 * * dll.structmap([customMarshal="sample.dcom.COSERVERINFO", type=CUSTOM]) 
 * *
 *  public COSERVERINFO foo;

 * 
 * @see sample.dcom.COAUTHIDENTITY
 * @see sample.dcom.COAUTHINFO
 * @see sample.dcom.DCOMLib
 * 
 * @version 1.0a
 *
 * @dll.struct(unicode) 
 */
public final class COSERVERINFO implements Cloneable{
    /**
     * The Java VM will read this field to determine how many bytes the custom
     * marshaled object takes up in the native C.
     * 
     * NOTE: static struct fields are NOT considered part of the native C
     *          structure!
     */
    public static int cbByValSize = 4;
/**
 * This reserved word must always be null (0)
 */
  public int dwReserved1;
/**
 * Pointer to the name of the machine to be used. 
 * 
 *  NOTE: localhost and 127.0.0.1 will NOT work here. this will be fixed with 
 *          NT Service Pack 4.0
 */
  public String pwszName;
/**
 * When using NTLMSSP, this value must be set to zero. A non-zero value, which 
 * is a pointer to a COAUTHINFO structure, would only be used when a security 
 * package other than NTLMSSP is being used. 
 *
 * @dll.structmap([customMarshal="sample.dcom.COAUTHINFO", type=CUSTOM]) 
 */
  public COAUTHINFO pAuthInfo;
/**
 * Reserved for future use. Must be 0. 
 */
  public int dwReserved2;
/**
 * Creates an empty COSERVERINFO
 */
  public COSERVERINFO(){}
/**
 * Creates a COSERVERINFO with the machine name and COAUTHINFO struct set.
 */
  public COSERVERINFO(String machineName,
                      COAUTHINFO auth){
      this.pwszName=machineName;
      this.pAuthInfo=auth;
  }
/**
 * Creates a COSERVERINFO structure, initializing the relevant fields in the 
 * COSERVERINFO, COAUTHINFO and COAUTHIDENITY structures.  This will cover all 
 * the necessary fields for passing this struct in a CoCreateInstnaceEx call.
 * dwReserved1, and dwReserved2 will be set to 0.
 */
  public COSERVERINFO(String machineName,
                      int dwAuthnLevel,
                      int dwImpersonationLevel,
                      String user,
                      String domain,
                      String password){
      this.pwszName=machineName;
      this.pAuthInfo=new COAUTHINFO(dwAuthnLevel,
                                    dwImpersonationLevel,
                                    new COAUTHIDENTITY(user,
                                                       domain,
                                                       password));
  
  }
/**
 * This overrides the default clone method, creating a new instance of this
 * struct with exactly the same values.
 * 
 * @return Object - the new struct with identical values.
 */
    public Object clone(){
        return new COSERVERINFO(pwszName,pAuthInfo);   
    }
/**
 * The Java VM will call this method to marshal the COSERVERINFO structure from
 * C back to Java.
 * 
 * @param ppStruct - This is a pointer to a memory location where the C structure is.
 * @param Flags - any flags that have been specified by the Java Object representing
 *          the C structure. [Normally this is unused].
 * 
 * @return COSERVERINFO - this is a Java representation of the C structure. This
 *          value will be placed where the custom marshaled field is.
 */    
    public static COSERVERINFO toJava(int ppStruct, int Flags){  
        //This causes a propogate on all of the fields in the COSERVERINFO struct.
        return (COSERVERINFO)com.ms.dll.DllLib.ptrToStruct(COSERVERINFO.class,
                                                com.ms.dll.DllLib.read4(ppStruct));  
    }
/**
 * The Java VM will call this method to marshal the field specified with this 
 * custom marshaler to native C.  This is accomplished by propogating any Java
 * objects to native C, and setting the ppStruct value to point to a pinned 
 * COSERVERINFO struct.
 * 
 * @param COSERVERINFO struct - the struct to be marshaled to native C.
 * @param int Flags - Any flags specified in the dll.structmap declaration for the
 *          field containing the COSERVERINFO object.
 * @param int ppStruct - this is the memory location of where to write the address
 *          of the marshaled COSERVERINFO struct.
 */    
    public static void toExternal(COSERVERINFO struct, int Flags, int ppStruct ) { 
        com.ms.dll.DllLib.propagateStructFields(struct,false);
        int handle = com.ms.dll.Root.alloc(struct);
        com.ms.dll.DllLib.write4(ppStruct,
              com.ms.dll.DllLib.addrOf(handle));
        com.ms.dll.Root.free(handle);
    }
 }