/*
 *  sample.dcom.DCOMLib.java
 *
 *  Copyright (c) 1995 - 1999, Microsoft Corporation
 *
 */
package sample.dcom;

import com.ms.com.IUnknown;
import com.ms.dll.DllLib;
/**
 * This class contains methods for helping implement DCOM objects easier.  It
 * leverages the power of J/Direct for importing the required COM functions
 * and mapping the necessary C structures for creating COM objects.    
 * 
 * @version 1.0
 * 
 * @dll.import("OLE32")
*/
public class DCOMLib {
/** 
 * Creates an instance of a specific class on a specific machine.
 * 
 * CoCreateInstanceEx creates a single uninitialized object associated with the 
 * given CLSID on a specified remote machine. This is an extension of the function 
 * CoCreateInstance, which creates an object on the local machine only. In addition,
 * rather than requesting a single interface and obtaining a single pointer to that 
 * interface, CoCreateInstanceEx makes it possible to specify an array of 
 * structures, each pointing to an interface identifier (IID) on input, and, on 
 * return, containing (if available) a pointer to the requested interface and the 
 * return value of the QueryInterface call for that interface. This permits fewer 
 * round trips between machines.
 * 
 * The CoCreateInstanceEx helper function encapsulates three calls: first, to 
 * CoGetClassObject to connect to the class object associated with the specified 
 * CLSID, specifying the machine location of the class; second, to 
 * IClassFactory::CreateInstance to create an uninitialized instance, and finally, 
 * to IClassFactory::Release, to release the class object. 
 * 
 * The object so created must still be initialized through a call to one of the 
 * initialization interfaces (such as IPersistStorage:::Load). The two helper 
 * functions, CoGetInstanceFromFile and CoGetInstanceFromIStorage encapsulate both 
 * the instance creation and initialization from the obvious sources.
 * 
 * 
 * @param com.ms.com._Guid rclsid - CLSID of the object to be created. 
 * 
 * @param com.ms.com.IUnknown punkOuter - When non-NULL, indicates the instance is 
 *          being created as part of an aggregate, and punkOuter is to be used as 
 *          the new instance's controlling IUnknown. Aggregation is currently not 
 *          supported cross-process or cross-machine. When instantiating an object 
 *          out of process, CLASS_E_NOAGGREGATION will be returned if punkOuter is 
 *          non-NULL. 
 * 
 * @param int dwClsCtx - Values taken from the CLSCTX enumeration. 
 * 
 * @param COSERVERINFO pServerInfo - Machine on which to instantiate the object. 
 *          May be NULL, in which case the object is instantiated on the current 
 *          machine or at the machine specified in the registry under the class's 
 *          RemoteServerName named-value, according to the interpretation of the 
 *          dwClsCtx parameter. See the CLSCTX documentation for details). 
 * 
 * @param int dwCount - Number of MULTI_QI structures in pResults. Must be greater 
 *          than zero. 
 * 
 * @param Object pResults - Array of MULTI_QI structures. Each structure has three 
 *          members: the identifier for a requested interface (pIID), the location 
 *          to return the interface pointer (pItf) and the return value of the call 
 *          to QueryInterface (hr). 
 * 
 * @return int - This function supports the standard return value E_INVALIDARG, as 
 *      well as the following: 
 * 
 *          S_OK                        Indicates success. 
 * 
 *          CO_S_NOTALLINTERFACES       At least one, but not all of the interfaces 
 *                                      requested in the pResults array were 
 *                                      successfully retrieved. The hr field of 
 *                                      each of the MULTI_QI structures in pResults 
 *                                      indicates with S_OK or E_NOINTERFACE whether
 *                                      the specific interface was returned. 
 * 
 *          E_NOINTERFACE               None of the interfaces requested in the 
 *                                      pResults array were successfully retrieved. 
 */
public static native int CoCreateInstanceEx(
             com.ms.com._Guid Clsid,        //CLSID of the object to be created
             com.ms.com.IUnknown punkOuter, //If part of an aggregate, the 
                                            // controlling IUnknown
             int dwClsCtx,                  //CLSCTX values
             sample.dcom.COSERVERINFO pServerInfo,      //Machine on which the object is to 
                                            // be instantiated
             int dwCount,                   //Number of MULTI_QI structures in 
                                            // pResults
             sample.util.CStructArray pResults);  //Array of MULTI_QI structures
/**
 * This method can NOT be reliably called from Java.  It sets the security PROCESS
 * WIDE, and if it is NOT called by the client application COM will call it for 
 * the application.  This method can only be called ONCE per process.
 * 
 * Registers security and sets the default security values for the process. This 
 * function is called exactly once per process, either explicitly or implicitly. It 
 * can be called by the client or the server, or both. For legacy applications and 
 * other applications that do not explicitly call CoInitializeSecurity, COM calls 
 * this function implicitly with values from the registry. If you set process-wide 
 * security using the registry and then call CoInitializeSecurity, the AppID 
 * registry values will be ignored, and the CoInitializeSecurity values will be 
 * used.
 *  
 * The CoInitializeSecurity function initializes the security layer and sets the 
 * specified values as the security default. If the process does not call 
 * CoInitializeSecurity, COM calls it automatically the first time an interface is 
 * marshaled or unmarshaled, registering the system default security. No default 
 * security packages are registered until then.
 * 
 * When the EOAC_APPID flag is set in dwCapabilities, pVoid points to an AppID and 
 * all other parameters to CoInitializeSecurity are ignored (and must be zero). 
 * CoInitializeSecurity looks for the authentication level under the AppID key in 
 * the registry and uses it to determine the default security. If pVoid is NULL, 
 * CoInitializeSecurity will look up the application .exe name in the registry and 
 * use the AppID stored there. This gives the same security settings as if the 
 * process had not called CoInitializeSecurity. Before NT 4 SP 4, 
 * CoInitializeSecurity returned an error if it did not find the specified AppID in 
 * the registry. For more information on how the AppID key is used to set security, 
 * see Setting Process-wide Security Through the Registry and the 
 * EOLE_AUTHENTICATION_CAPABILITIES enumeration.
 * 
 * If the EOAC_ACCESS_CONTROL flag is set, pVoid is a pointer to an IAccessControl 
 * object, which determines who can call the process. DCOM will AddRef the 
 * IAccessControl and will Release it when CoUninitialize is called. The state of 
 * the IAccessControl object should not be changed. If EOAC_ACCESS_CONTROL is 
 * specified, the dwAuthnLevel cannot be none.
 * 
 * The CoInitializeSecurity function returns an error if both the EOAC_APPID and 
 * EOAC_ACCESS_CONTROL flags are set.
 * 
 * If neither the EOAC_APPID nor the EOAC_ACCESS_CONTROL flag is specified in 
 * dwCapabilities, pVoid must be a pointer to a Win32 SECURITY_DESCRIPTOR. A 
 * security descriptor contains two ACLs: the discretionary ACL (DACL) indicates who
 * is (and who is not) allowed to call the process; the system ACL (SACL) contains 
 * audit information. DCOM looks for the COM_RIGHTS_EXECUTE flag in the DACL to find
 * out which callers are permitted to connect to the process's objects. Until DCOM 
 * supports auditing, the SACL must be NULL. A DACL with no ACEs allows no access. A
 * NULL DACL will allow calls from anyone. For more information on ACLs and ACEs, 
 * seeAccess Control Model. 
 * 
 * If pVoid is a security descriptor, the owner and group of the SECURITY_DESCRIPTOR
 * must be set  applications should call AccessCheck (not IsValidSecurityDescriptor)
 * to ensure that their security descriptor is correctly formed prior to calling 
 * CoInitializeSecurity. DCOM will copy the specified security descriptor. If the 
 * application passes a NULL security descriptor, COM will construct one that 
 * allows calls from anyone. 
 * 
 * When a CAPI handle is specified for the SSL entry in pAuthList, the CAPI handle 
 * must not be freed until CoUninitialize is called. If the list is specified but 
 * there is no entry for SSL, DCOM will try to find a default identity the first 
 * time DCOM negotiates SSL as the authentication service. If that succeeds, DCOM 
 * will save that identity as the default. Otherwise, the client will be anonymous 
 * for SSL. 
 * 
 * 
 * @param Object[] pVoid - Defines the access permissions. This 
 *      parameter is used when the process calling CoInitializeSecurity acts as a 
 *      server. Its value can be NULL or a pointer to one of three types: either an 
 *      AppID, an IAccessControl object, or a Win32 security descriptor. The type 
 *      of data that pVoid points to is indicated by a flag set in the 
 *      dwCapabilities parameter. If EOAC_APPID is specified, pVoid must be a 
 *      pointer to a GUID that specifies the AppID of the process. In this case, 
 *      all other parameters of the call are ignored, and registry values are used 
 *      for security checks. If EOAC_ACCESS_CONTROL is specified, pVoid must be a 
 *      pointer to an IAccessControl object, which determines who can call the 
 *      process. If the capability flags (dwCapabilities) do not include either 
 *      EOAC_APPID or EOAC_ACCESS_CONTROL, pVoid must be a pointer to a Win32 
 *      security descriptor. If pVoid is NULL, no ACL checking will be done. If it 
 *      is not NULL, COM will check ACLs on calls and dwAuthnLevel cannot be 
 *      RPC_C_AUTHN_LEVEL_NONE. See the Remarks section for more information. 
 * 
 * @param int cAuthSvc - Count of entries in asAuthSvc. Zero means register no 
 *      authentication services. If you specify zero, you cannot receive secure 
 *      calls. A value of -1 tells COM to choose which authentication services to 
 *      register, and if this is the case, the asAuthSvc parameter must be zero. 
 *      This value is used by COM when the process acts as a server. 
 * 
 * @param Object[] asAuthSvc - Array of authentication service information used by 
 *      COM to choose the authentication service for the process when it acts as a 
 *      server. These values are control which security providers are used for 
 *      incoming calls. Outgoing calls may use any security provider installed on 
 *      the machine. The entries for NTLMSSP, Kerberos, and Snego must have the 
 *      pPrincipalName member set to NULL. The SSL entry should have the 
 *      pPrincipalName member set to a CAPI (Crypto API) handle that specifies a 
 *      private certificate to be used as the application's server certificate. 
 * 
 * @param int pReserved1 - Reserved for future use; this value must be NULL. 
 * 
 * @param int dwAuthnLevel - The default authentication level for the process. COM 
 *      will fail calls that arrive with a lower authentication level. By default, 
 *      all proxies will use at least this authentication level. This value should 
 *      contain one of the flags from the RPC_C_AUTHN_LEVEL_xxx enumeration. By 
 *      default, all calls to IUnknown are made at this level. 
 * 
 * @param int dwImpLevel - The default impersonation level for proxies. The value of
 *      this parameter applies when the process is the client. It should be a value 
 *      from the RPC_C_IMP_LEVEL_xxx enumeration. Outgoing calls from the client 
 *      always use the impersonation level as specified (it is not negotiated). 
 *      Incoming calls to the client can be at any impersonation level. By default, 
 *      all IUnknown calls are made with this impersonation level so even 
 *      security-aware applications should set this level carefully. To determine 
 *      which impersonation levels each authentication service supports, see the 
 *      description of the authentication services in COM and Security Packages. 
 *      For more information about impersonation levels, see Impersonation. 
 * 
 * @param Object[] pAuthList - This value must be NULL on NT 4. On NT 5, this 
 *      parameter is a pointer to a SOLE_AUTHENTICATION_LIST, which is an array of 
 *      SOLE_AUTHENTICATION_INFO structures. This list indicates the default 
 *      authentication information to use for each authentication service. It 
 *      applies only to clients. When DCOM negotiates the default authentication 
 *      service for a proxy, it picks the default authentication information from 
 *      this list. If the pAuthInfo member of the SOLE_AUTHENTICATION_INFO structure
 *      representing the chosen authentication service is NULL, DCOM will use the 
 *      process identity to represent the client. If both pAuthList and one of the 
 *      cloaking flags are set, CoInitializeSecurity will return an error. 
 * 
 *      The NTLMSSP and Kerberos entries in the list should each contain a pointer 
 *      to a SEC_WINNT_AUTH_IDENTITY_W or to a SEC_WINNT_AUTH_IDENTITY_EXW structure
 *      containing the user name and password. 
 * 
 *      For the SSL entry, the pAuthInfo member of the SOLE_AUTHENTICATION_INFO 
 *      structure should be a CAPI handle to a private certificate. 
 * 
 *      For Snego, the pAuthInfo member should be NULL or point to a 
 *      SEC_WINNT_AUTH_IDENTITY_EXW structure. If non-NULL, the structure's 
 *      PackageList member must point to a string containing a comma-separated list 
 *      of authentication service names, and the PackageListLength member must give 
 *      the number of bytes in the PackageList string. If pAuthInfo is NULL, Snego 
 *      will pick a list of authentication services to try based on those available 
 *      on the client machine. 
 * 
 * @param int dwCapabilities - Additional capabilities of the client or server. If a
 *      flag other than one of the following EOLE_AUTHENTICATION_CAPABILITIESflags 
 *      is specified, CoInitializeSecurity will return an error: EOAC_APPID, 
 *      EOAC_ACCESS_CONTROL, EOAC_STATIC_CLOAKING, EOAC_DYNAMIC_CLOAKING, 
 *      EOAC_AUTO_IMPERSONATION, EOAC_SECURE_REFS, EOAC_REQUIRE_FULLSIC, 
 *      EOAC_MAKE_FULLSIC, EOAC_ANY_AUTHORITY, or EOAC_MUTUAL_AUTH. Some of these 
 *      flags cannot be set simultaneously. For more information on these flags, 
 *      see the EOLE_AUTHENTICATION_CAPABILITIES enumeration and the Remarks section.
 * 
 * @param int pReserved3 - Reserved for future use; it must be set to NULL. 
 * 
 * @return int - This function supports the standard return value E_INVALIDARG, as 
 * well as the following: 
 * 
 *      S_OK                    Indicates success. 
 * 
 *      RPC_E_TOO_LATE          CoInitializeSecurity has already been called. 
 * 
 *      RPC_E_NO_GOOD_SECURITY_PACKAGES asAuthSvc was not NULL and none of the 
 *                              authentication services in the list could be 
 *                              registered. Check the results saved in asAuthSvc for 
 *                              authentication service specific error codes. 
 * 
 *      E_OUT_OF_MEMORY
 */
public static native int CoInitializeSecurity(
        Object[] pVoid,    //Points to security descriptor
        int cAuthSvc,                   //Count of entries in asAuthSvc
        Object[] asAuthSvc,             //SOLE_AUTHENTICATION_SERVICE
                                        //Array of names to register
        Object[] pReserved1,            //Reserved for future use
        int dwAuthnLevel,               //The default authentication level 
                                        // for proxies
        int dwImpLevel,                 //The default impersonation level 
                                        // for proxies
        Object[] pAuthList,             //SOLE_AUTHENTICATION_LIST
                                        //Authentication information for 
                                        // each authentication service
        int dwCapabilities,             //Additional client and/or 
                                        // server-side capabilities
        Object[] pReserved3             //Reserved for future use);
);
/**
 * Retrieves a list of the authentication services registered when the process 
 * called CoInitializeSecurity.
 * 
 * CoQueryAuthenticationServices retrieves a list of the authentication services 
 * currently registered. If the process calls CoInitializeSecurity, these are the 
 * services registered through that call. If the application does not call it, 
 * CoInitializeSecurity is called automatically by COM, registering the default 
 * security package, the first time an interface is marshaled or unmarshaled. 
 * 
 * This function returns only the authentication services registered with 
 * CoInitializeSecurity. It does not return all of the authentication services 
 * installed on the machine, butEnumerateSecurityPackages does. 
 * CoQueryAuthenticationServices is primarily useful for custom marshalers, to 
 * determine which principal names an application can use.
 * 
 * Different authentication services support different levels of security. For 
 * example, NTLMSSP does not support delegation or mutual authentication while 
 * Kerberos does. The application is responsible only for registering authentication
 * services that provide the features the application needs. This function provides 
 * a way to find out which services have been registered with CoInitializeSecurity.
 * 
 * 
 * @param int[] pcAuthSvc - Pointer to return the number of entries returned in the 
 *      rgAuthSvc array. May not be NULL. 
 * 
 * @param int[] asAuthSvc - Pointer to an array of SOLE_AUTHENTICATION_SERVICE 
 *      structures. The list is allocated through a call to CoTaskMemAlloc. The 
 *      caller must free the list when finished with it by calling CoTaskMemFree.
 * 
 * @return int - This function supports the standard return values E_INVALIDARG and 
 *      E_OUTOFMEMORY, as well as the following: 
 * 
 *      S_OK                Indicates success. 
 */
public static native int CoQueryAuthenticationServices(
        int[] pcAuthSvc,    //Pointer to the number of entries returned in 
        int[] asAuthSvc     // the array  SOLE_AUTHENTICATION_SERVICE** asAuthSvc
                            //Pointer to an array of structures                                            
);
/**
 * Sets the authentication information that will be used to make calls on the 
 * specified proxy. This is a helper function for IClientSecurity::SetBlanket.
 * 
 * Sets the authentication information that will be used to make calls on the 
 * specified proxy. The IClientSecurity::SetBlanket method and CoSetProxyBlanket 
 * function return an error if you set any of the following flags in the 
 * capabilities: EOAC_SECURE_REFS, EOAC_ACCESS_CONTROL, or EOAC_APPID.
 * 
 * This function encapsulates the following sequence of common calls 
 * (error handling excluded):
 * 
 *      pProxy->QueryInterface(IID_IClientSecurity, (void**)&pcs);
 *          pcs->SetBlanket(pProxy, dwAuthnSvc, dwAuthzSvc, pServerPrincName, 
 *                          dwAuthnLevel, dwImpLevel, pAuthInfo, dwCapabilities);
 * pcs->Release();
 * 
 * For more information about how to use this function, see the Remarks section 
 * of the IClientSecurity::SetBlanket method.
 * 
 * @param int pProxy - Indicates the proxy to set. 
 * 
 * @param int dwAuthnSvc - A single DWORD value from the list of RPC_C_AUTHN_xxx 
 *          constants indicating the authentication service to use. It may be 
 *          RPC_C_AUTHN_NONE if no authentication is required. If 
 *          RPC_C_AUTHN_DEFAULT is specified, DCOM will pick an authentication 
 *          service following its normal security blanket negotiation algorithm. 
 * 
 * @param int dwAuthzSvc - A single DWORD value from the list of RPC_C_AUTHZ_xxx 
 *          constants indicating the authorization service to use. If 
 *          RPC_C_AUTHZ_DEFAULT is specified, DCOM will pick an authorization 
 *          service following its normal security blanket negotiation algorithm. If 
 *          you are using NTLMSSP, Kerberos, or SSL for the authentication service, 
 *          use RPC_C_AUTHZ_NONE for the authorization service. 
 * 
 * @param String pServerPrincName - Pointer to a WCHAR string that indicates the 
 *          server principal name to use with the authentication service. If 
 *          COLE_DEFAULT_PRINCIPAL is specified, DCOM will pick a principal name 
 *          using its security blanket negotiation algorithm. If using Kerberos, 
 *          this value must not be NULL. It must be the correct principal name of 
 *          the server or the call will fail. For SSL, this value must be one of 
 *          the msstd or fullsic forms described in the SSL section of COM and 
 *          Security Packages or NULL if you do not want mutual authentication.. 
 * 
 * @param int dwAuthnLevel - A single DWORD value from the list of 
 *          RPC_C_AUTHN_LEVEL_xxx constants indicating the authentication level to 
 *          use. If RPC_C_AUTHN_LEVEL_DEFAULT is specified, DCOM will pick an 
 *          authentication level following its normal security blanket negotiation 
 *          algorithm. If this value is none, the authentication service must also
 *          be none. SSL converts all authentication levels (except none) to 
 *          encrypt. 
 * 
 * @param int dwImpLevel - A single DWORD value from the list of RPC_C_IMP_LEVEL_xxx
 *          constants indicating the impersonation level to use. If 
 *          RPC_C_IMP_LEVEL_DEFAULT is specified, DCOM will pick an impersonation 
 *          level following its normal security blanket negotiation algorithm. If 
 *          you are using NTLMSSP, this value must be RPC_C_IMP_LEVEL_IMPERSONATE 
 *          or RPC_C_IMP_LEVEL_IDENTIFY. In NT 5, NTLMSSP supports delegate-level 
 *          impersonation on the same machine (RPC_C_IMP_LEVEL_DELEGATE). When 
 *          using SSL, this value must be RPC_C_IMP_LEVEL_IMPERSONATE. 
 * 
 * @param Object pAuthInfo - Pointer to an RPC_AUTH_IDENTITY_HANDLE value that 
 *          establishes the identity of the client. This parameter is not used for 
 *          calls on the same machine. It is authentication-service specific. If 
 *          NULL is specified, DCOM uses the current proxy identity (which is 
 *          either the process token or the impersonation token). If the handle 
 *          refers to a structure, that identity is used. The format of the 
 *          structure referred to by the handle depends on the provider of the 
 *          authentication service. For NTLMSSP or Kerberos, the structure is a 
 *          SEC_WINNT_AUTH_IDENTITY_W or SEC_WINNT_AUTH_IDENTITY_EXW 
 *          (on Windows NT 5) structure. The client must insure that the memory 
 *          remains valid and unchanged until a different identity is set on th 
 *          eproxy or all proxies on the object are released. For SSL, this value 
 *          may be NULL or a CAPI (Crypto API) handle to a private certificate. 
 *          If NULL is specified, the client is anonymous. If a CAPI handle is 
 *          specified, the caller must not free it as long as any proxy to the 
 *          object exists in the current apartment. If COLE_DEFAULT_AUTHINFO is 
 *          specified, DCOM will pick the authentication information following its 
 *          normal security blanket negotiation algorithm. CoSetProxyBlanket will 
 *          return an error if both pAuthInfo is set and one of the cloaking flags 
 *          is set in dwCapabilities. 
 * 
 * @param int dwCapabilities - A DWORD defining flags that specify the capabilities 
 *          of this proxy. Capability flags are defined in the 
 *          EOLE_AUTHENTICATION_CAPABILITIES enumeration. EOAC_MUTUAL_AUTH is 
 *          accepted, but has no effect. EOAC_ANY_AUTHORITY means DCOM will trust 
 *          an SSL server certificate if the top level certificate authority is not 
 *          trusted. EOAC_DEFAULT means DCOM will pick the capabilities using its 
 *          normal security blanket negotation algorithm. EOAC_NONE indicates no 
 *          capability flags are set. Either EOAC_STATIC_CLOAKING or 
 *          EOAC_DYNAMIC_CLOAKING can be set if pAuthInfo is not set. If any 
 *          capability flags other than those mentioned here are set, 
 *          CoSetProxyBlanket will return an error. 
 * 
 * @return int - 
 * 
 *          S_OK                        Success.
 * 
 *          E_INVALIDARG                One or more arguments is invalid. 
 * 
 */
public static native int CoSetProxyBlanket(
    int pProxy,                 //Indicates the proxy to set
    int dwAuthnSvc,             //Authentication service to use
    int dwAuthzSvc,             //Authorization service to use
    String pServerPrincName,    //Server principal name to use with 
                                // the authentication service
    int dwAuthnLevel,           //Authentication level to use
    int dwImpLevel,             //Impersonation level to use
    Object   pAuthInfo,         //Identity of the client
    int dwCapabilities          //Capability flags
      );
/**
 * Disconnects all remote process connections being maintained on behalf of all 
 * the interface pointers that point to a specified object. Only the process that 
 * actually manages the object should call CoDisconnectObject. 
 * 
 * The CoDisconnectObject function enables a server to correctly disconnect all 
 * external clients to the object specified by pUnk.
 * 
 * The CoDisconnectObject function performs the following tasks: 
 * 
 * Checks to see if the object to be disconnected implements the IMarshal 
 *          interface. If so, it gets the pointer to that interface; if not, it 
 *          gets a pointer to the standard marshaler's (i.e., COM's) IMarshal 
 *          implementation. 
 * 
 * Using whichever IMarshal interface pointer it has acquired, the function then 
 *          calls IMarshal::DisconnectObject to disconnect all out-of-process 
 *          clients. 
 * 
 * An object's client does not call CoDisconnectObject to disconnect itself from 
 *          the server (clients should use IUnknown::Release for this purpose). 
 *          Rather, an OLE server calls CoDisconnectObject to forcibly disconnect 
 *          an object's clients, usually in response to a user closing the server 
 *          application.
 * 
 * Similarly, an OLE container that supports external links to its embedded 
 *          objects can call CoDisconnectObject to destroy those links. Again, 
 *          this call is normally made in response to a user closing the 
 *          application. The container should first call IOleObject::Close for all 
 *          its OLE objects, each of which should send IAdviseSink::OnClose 
 *          notifications to their various clients. Then the container can safely 
 *          call CoDisconnectObject to close any existing connections.
 * 
 * CoDisconnectObject does not necessarily disconnect out-of-process clients 
 * immediately. If any marshaled calls are pending on the server object, 
 * CoDisconnectObject disconnects the object only when those calls have returned. 
 * In the meantime, CoDisconnectObject sets a flag that causes any new marshaled 
 * calls to return CO_E_OBJECTNOTCONNECTED
 * 
 * 
 * @param com.ms.com.IUnknown - pUnk [in] - Pointer to any IUnknown-derived 
 *          interface on the object to be disconnected. 
 * 
 * @param int dwReserved [in] - Reserved for future use; must be zero. 
 * 
 * @return int - Return Values
 * 
 * 
 *          S_OK     All connections to remote processes were successfully deleted. 
 */
public static native int CoDisconnectObject(
  IUnknown  pUnk,  //Pointer to the interface on the object
  int dwReserved);  //Reserved for future use
}