/*
 * sample.marshal.ByValue.java
 *
 * (C) Copyright 1995 - 1999 Microsoft Corporation.  All rights reserved.
 *
 */
package sample.marshal;

import com.ms.com.*;
/**
 * This is a generic Java COM object that can be used to marshal by value and Java
 * object, or Java implemented COM object.  It also contains the necessary 
 * functionality for unmarshaling any Java/Java COM object.  Use the 
 * MarshalLib.registerProxy() method to initialize this proxy in the client for
 * unmarshaling the java/JCOM objects.
 * 
 * @see sample.marshal.MarshalLib
 * 
 * @version 1.0
 */
public class ByValue implements com.ms.com.IMarshal{ 
        /**
         * GUID for this class, which represents the PRoxy for unmarshaling Java
         * and Java COM objects.
         */
        public static final com.ms.com._Guid IID = 
                   new com.ms.com._Guid("{0F1D8BE2-63EA-11d2-B933-00C04F8C9491}");
        /**
         * The Java Object that is to be marshaled by value.
         */
        Object comObject = null;
/**
 * Empty constructor that is required for COM to be able to create an instance
 * of this class as a COM object.
 */        
        public ByValue(){}
/**
 * Constructor used by MarshalLib to wrap an object for marshaling by value.
 * 
 * @param Object client - the Java object to be marshaled by value.
 */
        ByValue(Object client){
            comObject = client;   
        }
/**
 * This is the first method that will be called once it is determined that 
 * this object requires custom marshaling.  It is called to obtain the CLSID
 * of the proxy Object.  This method is similar to IStdMarshalInfo::GetClassForHandler.
 */
    public com.ms.com._Guid GetUnmarshalClass(com.ms.com._Guid riid,
                                  IUnknown pv,
                                  int dwDestContext,
                                  int pvDestContext,
                                  int mshlflags){
            System.out.println("Inside GetUnmarshalClass "+riid+" "+pv);
            return IID;
      }
/**
 * This is the second method that will be called by COM to determine what
 * size packet the interface will need for marshaling this object.  COM will then
 * allocate the memory and create apointer to a Stream object that refers to the
 * allocated memory.
 */    
    public int GetMarshalSizeMax(com.ms.com._Guid riid,
                                  IUnknown pv,
                                  int dwDestContext,
                                  int pvDestContext,
                                  int mshlflags){
        System.out.println("Inside GetMarshalSizeMax "+pv);
        try{
            java.io.ByteArrayOutputStream buf = new java.io.ByteArrayOutputStream();
            java.io.ObjectOutputStream oo = new java.io.ObjectOutputStream(
                                               buf);
            oo.writeObject(this.comObject);
            System.out.println("Object size is "+buf.size());
            return buf.size();
        }catch(java.io.IOException ii){
            throw new com.ms.com.ComFailException(com.ms.win32.win.E_FAIL);   
        }
     }
/**
 * The third method that will be called during the marshaling process, and is 
 * used to actually marshal the interface.  This method will fill the IStream with
 * the data that represents this interface. The IStream will be sent to the Client
 * for the Proxy object to use in recreating the object.
 */
    int i=0;
    public void MarshalInterface(com.ms.com.IStream pstm,
                                 com.ms.com._Guid riid,
                                 IUnknown pv,
                                 int dwDestContext,
                                 int pvDestContext,
                                 int mshlflags){
        System.out.println((i++)+"Inside MarshalInterface "+pstm+" "+riid+" "+comObject);
        try{
            java.io.ObjectOutputStream oo = 
        new java.io.ObjectOutputStream(new com.ms.activeX.ActiveXOutputStream(pstm));
            oo.writeObject(comObject);
            oo.close();
        }catch(java.io.IOException ii){
            System.out.println("Error writing object "+ii);
            throw new com.ms.com.ComFailException(com.ms.win32.win.E_FAIL);
        }
        // pstm is valid only for this method, realease the Java hold now!
        com.ms.com.ComLib.release( pstm );
        System.out.println("Done MarshalInterface");
}
/**
 * This method should ONLY be called in the proxy, not here.  This method will be
 * called by the proxy once the data has been recieved from the real COM object and
 * will be used to initialize the interface proxy.
 * 
 * From the IMarshal::UnmarshalInterface MSDN documentation:
 * 
 * "your implementation of UnmarshalInterface is free to 
 * create a different object and, if necessary, return a pointer to it." 
 */
    public com.ms.com.IUnknown UnmarshalInterface(com.ms.com.IStream pstm,
                                 com.ms.com._Guid riid){
        System.out.println("Inside UnMarshalInterface "+pstm+" "+riid);
        try{
        java.io.ObjectInputStream oo = new java.io.ObjectInputStream(
                           new com.ms.activeX.ActiveXInputStream(pstm));
        Object o = oo.readObject();
        oo.close();
        System.out.println("Read object ! "+o);
        // We would want to have ppv be an int[], and set ppv[0] to the result
        // of com.ms.com.ComLib.unknownToPtr(com.ms.com.ComLib.makeProxyRef(o),
        //                                  riid);
        // If we anticipate this this Proxy will be used by C implemented Clients
        // which may specify an interface other than IUnknown to be Created in 
        // their CoCreateInstance(Ex) call.
        
        System.out.println("PDone UnmarshalInterface");
        // pstm is valid only for this method, realease the Java hold now!
        com.ms.com.ComLib.release( pstm );
        return (IUnknown)com.ms.com.ComLib.makeProxyRef(o);
        }catch(Exception ii){
            System.out.println("Error writing object "+ii);
            throw new com.ms.com.ComFailException(com.ms.win32.win.E_FAIL);
        }
     }
/**
 * From the MSDN documentation on IMarshal::ReleasemarshalData: 
 * 
 * "If your implementation stores state information about marshaled data packets, 
 * you can use this method to release the state information associated with the 
 * data packet represented by pStm. Your implementation should also position the 
 * seek pointer in the stream past the last byte of data."
 * 
 * We don't need to do anything here. 
 */    
    public void ReleaseMarshalData(com.ms.com.IStream pstm){
        System.out.println("Inside ReleaseMarshalData");
        // pstm is valid only for this method, realease the Java hold now!
        com.ms.com.ComLib.release( pstm );
    }
/**
 * This method allows us to take steps to notify its proxy that it is about to 
 * shut down.
 * 
 * As part of its normal shutdown code, a server should call the CoDisconnectObject 
 * function, which in turn calls IMarshal::DisconnectObject, on each of its 
 * running objects that implements IMarshal.
 * 
 * We don't need to do anything here.
 */
    public void DisconnectObject(int dwReserved){
        System.out.println("Inside DisconnectObject");
        throw new com.ms.com.ComFailException(com.ms.win32.win.CO_E_OBJNOTCONNECTED);
    }

}