/*
 * sample.ntservice.Advapi32.java
 *
 * (C) Copyright 1995 - 1999 Microsoft Corporation.  All rights reserved.
 *
 */
package sample.ntservice;

/**
 * This class contains the J/Direct methods used to expose the NT Service functions
 * required to fully implement an NT Service in Java.  The documentation related to
 * each function has been provided for convenience.
 * 
 * @version 1.0 
 * 
 * @dll.import("advapi32.dll", setLastError)
 */
public class Advapi32{
	
	public static native  boolean StartServiceCtrlDispatcher(
									sample.util.CStructArray lpServiceStartTable);
	
/**
 * A Win32-based service calls the RegisterServiceCtrlHandler function to register a
 * function to handle its service control requests. 
 *  
 * The ServiceMain function of a new service should immediately call the 
 * RegisterServiceCtrlHandler function to register a control handler function with 
 * the control dispatcher. This enables the control dispatcher to invoke the 
 * specified function when it receives control requests for this service. The 
 * threads of the calling process can use the service status handle returned by this
 * function to identify the service in subsequent calls to the SetServiceStatus 
 * function. 
 * 
 * This function must be called before the first SetServiceStatus call because it 
 * returns a service status handle for the caller to use so that no other service 
 * can inadvertently set this service status. In addition, the control handler must 
 * be in place to field control requests by the time the service specifies the 
 * controls it accepts through the SetServiceStatus function. 
 * 
 * When the control handler function is invoked with a control request, it must call
 * SetServiceStatus to notify the service control manager of its current status, 
 * regardless of whether the status of the service has changed. 
 * 
 * The service status handle does not have to be closed. 
 * 
 * 
 * @param String lpServiceName - Pointer to a null-terminated string that names the 
 *          service run by the calling thread. This is the service name that the 
 *          service control program specified in the CreateService function when 
 *          creating the service. 
 * 
 * @param int lpHandlerProc - Pointer to the handler function to be registered. For 
 *          more information, see Handler. 
 * 
 * @return int - If the function succeeds, the return value is a service status 
 *          handle.  If the function fails, the return value is zero. To get 
 *          extended error information, call GetLastError. 
 * 
 *          The following error codes can be set by the service control manager. 
 *          Other error codes can be set by the registry functions that are called 
 *          by the service control manager. 
 * 
 *          Value                       Meaning 
 * 
 *          ERROR_INVALID_NAME          The specified service name is invalid. 
 * 
 *          ERROR_SERVICE_DOES_NOT_EXIST The specified service does not exist. 
 * 
 */
	public static native SERVICE_STATUS_HANDLE RegisterServiceCtrlHandler(
									String lpServiceName,
									int lpHandlerProc);
/**
 * The SetServiceStatus function updates the service control manager's status 
 * information for the calling service. 
 * 
 * A ServiceMain function first calls the RegisterServiceCtrlHandler function to get
 * the service's SERVICE_STATUS_HANDLE. Then it immediately calls the 
 * SetServiceStatus function to notify the service control manager of its 
 * SERVICE_START_PENDING status. 
 * 
 * When a service receives a control request, the service's Handler function must 
 * call SetServiceStatus, even if the service's status did not change. A service can
 * also use this function at any time and by any thread of the service to notify 
 * the service control manager of status changes. Examples of such unsolicited 
 * status updates include: 
 * 
 *      Checkpoint updates that occur when the service is in transition from one 
 *      state to another (that is, SERVICE_START_PENDING). 
 * 
 *      Fatal error updates that occur when the service must stop due to a 
 *      recoverable error. 
 * 
 * A service can call this function only after it has called 
 * RegisterServiceCtrlHandler to get a service status handle.
 * 
 * 
 * @param SERVICE_STATUS_HANDLE hServiceStatus - Specifies a handle to the service 
 *          control manager's status information structure for the current service. 
 *          This handle is returned by the RegisterServiceCtrlHandler function. 
 * 
 * @param SERVICE_STATUS lpServiceStatus - Pointer to the SERVICE_STATUS structure 
 *          the contains the latest status information for the calling service. 
 * 
 * @return int - If the function succeeds, the return value is nonzero.  If the 
 *          function fails, the return value is zero. To get extended error 
 *          information, call GetLastError. 
 * 
 *          The following error codes can be set by the service control manager. 
 *          Other error codes can be set by the registry functions that are called 
 *          by the service control manager. 
 * 
 *          Value                       Meaning 
 * 
 *          ERROR_INVALID_HANDLE        The specified handle is invalid. 
 * 
 *          ERROR_INVALID_DATA          The specified service status structure is 
 *                                      invalid. 
 * 
 */
	public static native boolean SetServiceStatus(
                                     SERVICE_STATUS_HANDLE hServiceStatus,
									 SERVICE_STATUS lpServiceStatus);
/**
 * The RegisterEventSource function returns a registered handle to an event log. 
 * 
 * If the source name cannot be found, the event logging service uses the 
 * Application logfile; it does not create a new source. Events are reported for the
 * source, however, there are no message and category message files specified for 
 * looking up descriptions of the event identifiers for the source.
 * 
 * To close the handle to the event log, call the DeregisterEventSource function. 
 * 
 * @param String lpUNCServerName - Pointer to a null-terminated string that 
 *          specifies the Universal Naming Convention (UNC) name of the server on 
 *          which this operation is to be performed. If this parameter is NULL, the 
 *          operation is performed on the local computer. 
 * 
 * @param String lpSourceName - Pointer to a null-terminated string that specifies 
 *          the name of the source referenced by the returned handle. The source 
 *          name must be a subkey of a logfile entry under the EventLog key in the 
 *          registry. For example, WinApp is a valid source name if the registry has
 *          the following key: 
 * 
 *              HKEY_LOCAL_MACHINE 
 *                  System 
 *                      CurrentControlSet 
 *                          Services 
 *                              EventLog 
 *                                  Application 
 *                                      WinApp 
 *                                  Security 
 *                                  System 
 * 
 * @return int - If the function succeeds, the return value is a handle that can be 
 *          used with the ReportEvent function.
 * 
 *          If the function fails, the return value is NULL. To get extended error 
 *          information, call com.ms.dll.DllLib.getLastWin32Error(). 
 */
	public static native int RegisterEventSource(String lpUNCServerName,
												 String lpSourceName);
/**
 * The ReportEvent function writes an entry at the end of the specified event log. 
 * This function is used to log an event. The entry is written to the end of the 
 * configured logfile for the source identified by the hEventLog parameter. The 
 * ReportEvent function adds the time, the entry's length, and the offsets before 
 * storing the entry in the log. To enable the function to add the username, you 
 * must supply the user's SID in the lpUserSid parameter. 
 * 
 * *NOTE The UNICODE version of the function is used (ReportEventW) rather than the
 * ANSI version (ReportEvent or ReportEventA) because there is no way to specicy
 * a String[] as ANSI or UNICODE with J/Direct currently
 * 
 * @param int hEventLog - Handle to the event log. This handle is returned by the 
 *							RegisterEventSource function. 
 * @param short wType - Specifies the type of event being logged. This parameter 
 *						can be one of the following values: Value Meaning 
 * 
 *					EVENTLOG_ERROR_TYPE Error event 
 *					EVENTLOG_WARNING_TYPE Warning event 
 *					EVENTLOG_INFORMATION_TYPE Information event 
 *					EVENTLOG_AUDIT_SUCCESS Success Audit event 
 *					EVENTLOG_AUDIT_FAILURE Failure Audit event 
 * 
 * @param short wCategory - Specifies the event category. This is source-specific 
 *							information; the category can have any value. 
 * 
 * @param short dwEventID - Specifies the event. The event identifier specifies the 
 *							message that goes with this event as an entry in the 
 *							message file associated with the event source. 
 * 
 * @param Object lpUserSid - Pointer to the current user's security identifier. 
 *							This parameter can be NULL if the security identifier 
 *							is not required. 
 * 
 * @param int wNumStrings - Specifies the number of strings in the array 
 *								pointed to by the lpStrings parameter. A value of 
 *							zero indicates that no strings are present. 
 * 
 * @param int dwDataSize - Specifies the number of bytes of event-specific raw 
 *							(binary) data to write to the log. If this parameter is 
 *							zero, no event-specific data is present. 
 * 
 * @param String[] lpStrings - Pointer to a buffer containing an array of 
 *							null-terminated strings that are merged into the 
 *							message from the message file before Event Viewer 
 *							displays the string to the user. This parameter must be 
 *							a valid pointer (or NULL), even if wNumStrings is zero. 
 *							Each string has a limit of 32K bytes. 
 * 
 * @param int lpRawData - Pointer to the buffer containing the binary data. This 
 *						parameter must be a valid pointer (or NULL), even if the 
 *						dwDataSize parameter is zero. 
 * 
 * @reutrn boolean - If the function succeeds, the return value is true, 
 *					indicating that the entry was written to the log.  If the 
 *					function fails, the return value is zero. To get extended error 
 *					information, call GetLastError.  
 */
	public static native boolean  ReportEventW(
					int hEventLog,    // handle returned by RegisterEventSource
					short wType,          // event type to log
					short wCategory,      // event category
					int dwEventID,     // event identifier
					Object lpUserSid,      // user security identifier (optional)
					short wNumStrings,    // number of strings to merge with message
					int dwDataSize,    // size of binary data, in bytes
					String[] lpStrings,  // array of strings to merge with message
					Object lpRawData     // address of binary data
					);
/**
 * The DeregisterEventSource function closes a handle to the specified event log. 
 * 
 * @param int hEventLog - Handle to the event log. This handle is returned by the 
 *          RegisterEventSource function. 
 * 
 * @return boolean - If the function succeeds, the return value is nonzero.  If the 
 *          function fails, the return value is zero. To get extended error 
 *          information, call com.ms.dll.DllLib.getLastWin32Error(). 
 */
	public static native boolean DeregisterEventSource(int hEventLog);
/**
 * The OpenSCManager function establishes a connection to the service control 
 * manager on the specified computer and opens the specified service control manager
 * database. 
 * 
 * @param int lpMachineName - Pointer to a null-terminated string that names the 
 *							target computer. If the pointer is NULL or points to an 
 *							empty string, the function connects to the service 
 *							control manager on the local computer. 
 * 
 * @param int lpDatabaseName - Pointer to a null-terminated string that names the 
 *							service control manager database to open. This parameter
 *							should be set to SERVICES_ACTIVE_DATABASE. If it is NULL,
 *							the SERVICES_ACTIVE_DATABASE database is opened by 
 *							default. 
 * 
 * @param int dwDesiredAccess - Specifies the access to the service control manager.
 *								Before granting the requested access, the system 
 *								checks the access token of the calling process 
 *								against the discretionary access-control list of the
 *								security descriptor associated with the service 
 *								control manager. The SC_MANAGER_CONNECT access type 
 *								is implicitly specified by calling this function. In 
 *								addition, any or all of the following service 
 *								control manager object access types can be 
 *								specified: Type Description 
 * 
 * SC_MANAGER_ALL_ACCESS	Includes STANDARD_RIGHTS_REQUIRED, in addition to all 
 *							of the access types listed in this table. 
 * 
 * SC_MANAGER_CONNECT		Enables connecting to the service control manager. 
 * 
 * SC_MANAGER_CREATE_SERVICE	Enables calling of the CreateService function to 
 *								create a service object and add it to the database.
 * 
 * SC_MANAGER_ENUMERATE_SERVICE	Enables calling of the EnumServicesStatus function 
 *								to list the services that are in the database. 
 * 
 * SC_MANAGER_LOCK			Enables calling of the LockServiceDatabase function to 
 *							acquire a lock on the database. 
 * 
 * SC_MANAGER_QUERY_LOCK_STATUS		Enables calling of the QueryServiceLockStatus 
 *									function to retrieve the lock status information
 *									for the database. 
 * 
 * The dwDesiredAccess parameter can specify any or all of the following generic 
 * access types: Generic access Service manager access GENERIC_READ Combines the 
 * following access types: STANDARD_RIGHTS_READ, SC_MANAGER_ENUMERATE_SERVICE, and 
 * SC_MANAGER_QUERY_LOCK_STATUS. 
 * 
 * GENERIC_WRITE Combines the following access types: STANDARD_RIGHTS_WRITE and 
 * SC_MANAGER_CREATE_SERVICE. 
 * 
 * GENERIC_EXECUTE Combines the following access types: STANDARD_RIGHTS_EXECUTE, 
 * SC_MANAGER_CONNECT, and SC_MANAGER_LOCK. 
 * 
 * @return int - If the function succeeds, the return value is a handle to the 
 *				specified service control manager database.
 * 
 * If the function fails, the return value is NULL. To get extended error 
 * information, call GetLastError. 
 * 
 * Errors
 * The following error codes can be set by the SCM. Other error codes can be set by 
 * the registry functions that are called by the SCM. 
 * 
 * Error code Meaning 
 * ERROR_ACCESS_DENIED The requested access was denied. 
 * ERROR_DATABASE_DOES_NOT_EXIST The specified database does not exist. 
 * ERROR_INVALID_PARAMETER A specified parameter is invalid. 
 * 
 * Remarks
 * 
 * When a process uses the OpenSCManager function to open a handle to a service 
 * control manager database, the system performs a security check before granting 
 * the requested access. All processes are permitted SC_MANAGER_CONNECT, 
 * SC_MANAGER_ENUMERATE_SERVICE, and SC_MANAGER_QUERY_LOCK_STATUS access to all 
 * service control manager databases. This enables any process to open a service 
 * control manager database handle that it can use in the OpenService, 
 * EnumServicesStatus, and QueryServiceLockStatus functions. Only processes with 
 * Administrator privileges are able to open a database handle used by the 
 * CreateService and LockServiceDatabase functions. 
 * 
 * The returned handle is only valid for the process that called the OpenSCManager 
 * function. It can be closed by calling the CloseServiceHandle function. 
 */
	public static native SC_HANDLE OpenSCManager(String lpMachineName, 
										   String lpDatabaseName,
										   int dwDesiredAccess);
/**
 * The OpenService function opens a handle to an existing service. 
 * 
 * The returned handle is only valid for the process that called OpenService. It 
 * can be closed by calling the CloseServiceHandle function. 
 * 
 * 
 * @param SC_HANDLE hSCManager - Handle to the service control manager database. The
 *          OpenSCManager function returns this handle. 
 * 
 * @param String lpServiceName - Pointer to a null-terminated string that names the 
 *          service to open. The maximum string length is 256 characters. The 
 *          service control manager database preserves the case of the characters, 
 *          but service name comparisons are always case insensitive. Forward-slash 
 *          (/) and backslash (\) are invalid service name characters. 
 * 
 * @param int dwDesiredAccess - Specifies the access to the service. Before granting
 *          the requested access, the system checks the access token of the calling 
 *          process against the discretionary access-control list of the security 
 *          descriptor associated with the service object. 
 * 
 *          The STANDARD_RIGHTS_REQUIRED constant enables the following service 
 *          object access types: 
 * 
 *          Standard rights                     Description 
 * 
 *          DELETE                  Enables calling of the DeleteService function to
 *                                  delete the service. 
 * 
 *          READ_CONTROL            Enables calling of the 
 *                                  QueryServiceObjectSecurity function to query the
 *                                  security descriptor of the service object. 
 * 
 *          WRITE_DAC|WRITE_OWNER   Enables calling of the SetServiceObjectSecurity 
 *                                  function to modify the security descriptor of 
 *                                  the service object. 
 * 
 *          You can specify any or all of the following service object access types:
 *          
 *          Access                  Description 
 * 
 *          SERVICE_ALL_ACCESS      Includes STANDARD_RIGHTS_REQUIRED in addition to
 *                                  all of the access types listed in this table. 
 * 
 *          SERVICE_CHANGE_CONFIG   Enables calling of the ChangeServiceConfig 
 *                                  function to change the service configuration. 
 * 
 *          SERVICE_ENUMERATE_DEPENDENTS    Enables calling of the 
 *                                  EnumDependentServices function to enumerate all 
 *                                  the services dependent on the service. 
 * 
 *          SERVICE_INTERROGATE     Enables calling of the ControlService function 
 *                                  to ask the service to report its status 
 *                                  immediately. 
 * 
 *          SERVICE_PAUSE_CONTINUE  Enables calling of the ControlService function 
 *                                  to pause or continue the service. 
 * 
 *          SERVICE_QUERY_CONFIG    Enables calling of the QueryServiceConfig 
 *                                  function to query the service configuration. 
 * 
 *          SERVICE_QUERY_STATUS    Enables calling of the QueryServiceStatus 
 *                                  function to ask the service control manager 
 *                                  about the status of the service. 
 * 
 *          SERVICE_START           Enables calling of the StartService function to 
 *                                  start the service. 
 * 
 *          SERVICE_STOP            Enables calling of the ControlService function 
 *                                  to stop the service. 
 * 
 *          SERVICE_USER_DEFINED_CONTROL Enables calling of the ControlService 
 *                                  function to specify a user-defined control code.
 * 
 * 
 *          You can specify any of the following generic access types: 
 * 
 *          Generic access              Service access 
 * 
 *          GENERIC_READ            Combines the following access types: 
 *                                      STANDARD_RIGHTS_READ, 
 *                                      SERVICE_QUERY_CONFIG, 
 *                                      SERVICE_QUERY_STATUS, 
 *                                      and SERVICE_ENUMERATE_DEPENDENTS. 
 * 
 *          GENERIC_WRITE           Combines the following access types: 
 *                                      STANDARD_RIGHTS_WRITE 
 *                                      and SERVICE_CHANGE_CONFIG.
 * 
 *          GENERIC_EXECUTE         Combines the following access types: 
 *                                      STANDARD_RIGHTS_EXECUTE, 
 *                                      SERVICE_START, 
 *                                      SERVICE_STOP, 
 *                                      SERVICE_PAUSE_CONTINUE, 
 *                                      SERVICE_INTERROGATE, 
 *                                      and SERVICE_USER_DEFINED_CONTROL. 
 * 
 * @return int - If the function succeeds, the return value is a handle to the 
 *          service.  If the function fails, the return value is NULL. To get 
 *          extended error information, call com.ms.dll.DllLib.getLastWin32Error(). 
 * 
 *          The following error codes can be set by the service control manager. 
 *          Others can be set by the registry functions that are called by the 
 *          service control manager. 
 * 
 *          Error code                  Meaning 
 * 
 *          ERROR_ACCESS_DENIED         The specified service control manager 
 *                                      database handle does not have access to the 
 *                                      service.
 * 
 *          ERROR_INVALID_HANDLE        The specified handle is invalid.
 * 
 *          ERROR_INVALID_NAME          The specified service name is invalid. 
 * 
 *          ERROR_SERVICE_DOES_NOT_EXIST    The specified service does not exist. 
 */
	public static native SC_HANDLE OpenService(SC_HANDLE hSCManager,
										 String lpServiceName,
										 int dwDesiredAccess);
/**
 * The CreateService function creates a service object and adds it to the specified 
 * service control manager database. 
 * 
 * The CreateService function creates a service object and installs it in the 
 * service control manager database by creating a key with the same name as the 
 * service under the following registry key:
 * 
 *      HKEY_LOCAL_MACHINE\System\CurrentControlSet\Services 
 * 
 * Information specified for this function is saved as values under this key. Setup 
 * programs and the service itself can create subkeys under this key for any service
 * specific information. 
 * 
 * The returned handle is only valid for the process that called CreateService. It 
 * can be closed by calling the CloseServiceHandle function. 
 * 
 * @param SC_HANDLE hSCManager - Handle to the service control manager database. 
 *          This handle is returned by the OpenSCManager function and must have 
 *          SC_MANAGER_CREATE_SERVICE access. 
 * 
 * @param String lpServiceName - Pointer to a null-terminated string that names the 
 *          service to install. The maximum string length is 256 characters. The 
 *          service control manager database preserves the case of the characters, 
 *          but service name comparisons are always case insensitive. Forward-slash 
 *          (/) and back-slash (\) are invalid service name characters. 
 * 
 * @param String lpDisplayName - Pointer to a null-terminated string that is to be 
 *          used by user interface programs to identify the service. This string has 
 *          a maximum length of 256 characters. The name is case-preserved in the 
 *          service control manager. display name comparisons are always 
 *          case-insensitive. 
 * 
 * @param int dwDesiredAccess - Specifies the access to the service. Before granting 
 *          the requested access, the system checks the access token of the calling 
 *          process. 
 * 
 *          The STANDARD_RIGHTS_REQUIRED constant enables the following service 
 *          object access types: 
 * 
 *          Standard rights                     Description 
 * 
 *          DELETE                      Enables calling of the DeleteService 
 *                                      function to delete the service. 
 * 
 *          READ_CONTROL                Enables calling of the 
 *                                      QueryServiceObjectSecurity function to query
 *                                      the security descriptor of the service 
 *                                      object. 
 * 
 *          WRITE_DAC|WRITE_OWNER       Enables calling of the 
 *                                      SetServiceObjectSecurity function to modify 
 *                                      the security descriptor of the service 
 *                                      object. 
 * 
 *          You can specify any or all of the following service object access types:
 * 
 *          Access                              Description 
 * 
 *          SERVICE_ALL_ACCESS          Includes STANDARD_RIGHTS_REQUIRED in 
 *                                      addition to all of the access types listed 
 *                                      in this table. 
 * 
 *          SERVICE_CHANGE_CONFIG       Enables calling of the ChangeServiceConfig 
 *                                      function to change the service configuration.
 * 
 *          SERVICE_ENUMERATE_DEPENDENTS Enables calling of the 
 *                                      EnumDependentServices function to enumerate 
 *                                      all the services dependent on the service. 
 * 
 *          SERVICE_INTERROGATE         Enables calling of the ControlService 
 *                                      function to ask the service to report its 
 *                                      status immediately. 
 * 
 *          SERVICE_PAUSE_CONTINUE      Enables calling of the ControlService 
 *                                      function to pause or continue the service. 
 * 
 *          SERVICE_QUERY_CONFIG        Enables calling of the QueryServiceConfig 
 *                                      function to query the service configuration.
 * 
 *          SERVICE_QUERY_STATUS        Enables calling of the QueryServiceStatus 
 *                                      function to ask the service control manager 
 *                                      about the status of the service.
 * 
 *          SERVICE_START               Enables calling of the StartService function
 *                                      to start the service. 
 * 
 *          SERVICE_STOP                Enables calling of the ControlService 
 *                                      function to stop the service. 
 * 
 *          SERVICE_USER_DEFINED_CONTROL Enables calling of the ControlService 
 *                                      function to specify a user-defined control 
 *                                      code.
 * 
 *          You can specify any of the following generic access types: 
 * 
 *          Generic access              Service access 
 * 
 *          GENERIC_READ                Combines the following access types: 
 *                                          STANDARD_RIGHTS_READ, 
 *                                          SERVICE_QUERY_CONFIG, 
 *                                          SERVICE_QUERY_STATUS, 
 *                                          and SERVICE_ENUMERATE_DEPENDENTS. 
 * 
 *          GENERIC_WRITE               Combines the following access types: 
 *                                          STANDARD_RIGHTS_WRITE 
 *                                          and SERVICE_CHANGE_CONFIG. 
 * 
 *          GENERIC_EXECUTE             Combines the following access types: 
 *                                          STANDARD_RIGHTS_EXECUTE, 
 *                                          SERVICE_START, 
 *                                          SERVICE_STOP, 
 *                                          SERVICE_PAUSE_CONTINUE, 
 *                                          SERVICE_INTERROGATE, 
 *                                          and SERVICE_USER_DEFINED_CONTROL. 
 * 
 * @param int dwServiceType - A set of bit flags that specify the type of service. 
 *          You must specify one of the following service types. 
 * 
 *          Value                           Meaning
 * 
 *          SERVICE_WIN32_OWN_PROCESS       Specifies a Win32-based service that 
 *                                          runs in its own process. 
 * 
 *          SERVICE_WIN32_SHARE_PROCESS     Specifies a Win32-based service that 
 *                                          shares a process with other services. 
 * 
 *          SERVICE_KERNEL_DRIVER           Specifies a driver service. 
 * 
 *          SERVICE_FILE_SYSTEM_DRIVER      Specifies a file system driver service. 
 * 
 *          If you specify either SERVICE_WIN32_OWN_PROCESS or 
 *          SERVICE_WIN32_SHARE_PROCESS, you can also specify the following flag. 
 *          
 *          Value                           Meaning 
 * 
 *          SERVICE_INTERACTIVE_PROCESS     Enables a Win32-based service process to
 *                                          interact with the desktop. 
 * 
 * @param int dwStartType - Specifies when to start the service. You must specify 
 *          one of the following start types. 
 * 
 *          Value                           Meaning
 * 
 *          SERVICE_BOOT_START              Specifies a device driver started by the
 *                                          system loader. This value is valid only 
 *                                          for driver services. 
 * 
 *          SERVICE_SYSTEM_START            Specifies a device driver started by the 
 *                                          IoInitSystem function. This value is 
 *                                          valid only for driver services. 
 * 
 *          SERVICE_AUTO_START              Specifies a service to be started 
 *                                          automatically by the service control 
 *                                          manager during system startup. 
 * 
 *          SERVICE_DEMAND_START            Specifies a service to be started by the 
 *                                          service control manager when a process 
 *                                          calls the StartService function. 
 * 
 *          SERVICE_DISABLED                Specifies a service that can no longer 
 *                                          be started. 
 * 
 * @param int dwErrorControl - Specifies the severity of the error if this service 
 *          fails to start during startup, and determines the action taken by the 
 *          startup program if failure occurs. You must specify one of the following 
 *          error control flags.
 * 
 *          Value                           Meaning
 * 
 *          SERVICE_ERROR_IGNORE            The startup program logs the error but 
 *                                          continues the startup operation. 
 * 
 *          SERVICE_ERROR_NORMAL            The startup program logs the error and 
 *                                          puts up a message box pop-up but 
 *                                          continues the startup operation. 
 * 
 *          SERVICE_ERROR_SEVERE            The startup program logs the error. If 
 *                                          the last-known-good configuration is 
 *                                          being started, the startup operation 
 *                                          continues. Otherwise, the system is 
 *                                          restarted with the last-known-good 
 *                                          configuration. 
 * 
 *          SERVICE_ERROR_CRITICAL          The startup program logs the error, if 
 *                                          possible. If the last-known-good 
 *                                          configuration is being started, the 
 *                                          startup operation fails. Otherwise, the 
 *                                          system is restarted with the last-known 
 *                                          good configuration. 
 * 
 * @param String lpBinaryPathName - Pointer to a null-terminated string that 
 *          contains the fully qualified path to the service binary file. If the 
 *          path contains a space, it must be quoted so that it is correctly 
 *          interpreted. For example, "d:\\my share\\myservice.exe" should be 
 *          specified as "\"d:\\my share\\myservice.exe\"". 
 * 
 * @param String lpLoadOrderGroup - Pointer to a null-terminated string that names 
 *          the load ordering group of which this service is a member. Specify NULL 
 *          or an empty string if the service does not belong to a group. 
 * 
 * @param int lpdwTagId - Pointer to a DWORD variable that receives a tag value that
 *          is unique in the group specified in the lpLoadOrderGroup parameter. 
 *          Specify NULL if you are not changing the existing tag. 
 * 
 *          You can use a tag for ordering service startup within a load ordering 
 *          group by specifying a tag order vector in the GroupOrderList value of 
 *          the following registry key: 
 * 
 *          HKEY_LOCAL_MACHINE\System\CurrentControlSet\Control 
 * 
 *          Tags are only evaluated for driver services that have SERVICE_BOOT_START
 *          or SERVICE_SYSTEM_START start types. 
 * 
 * @param int lpDependencies - Pointer to a double null-terminated array of 
 *          null-separated names of services or load ordering groups that the system 
 *          must start before this service. Specify NULL or an empty string if the 
 *          service has no dependencies. Dependency on a group means that this 
 *          service can run if at least one member of the group is running after an 
 *          attempt to start all members of the group. 
 * 
 *          You must prefix group names with SC_GROUP_IDENTIFIER so that they can be
 *          distinguished from a service name, because services and service groups 
 *          share the same name space. 
 * 
 * @param String lpServiceStartName - Pointer to a null-terminated string that names
 *          the service. If the service type is SERVICE_WIN32_OWN_PROCESS, use an 
 *          account name in the form DomainName\UserName. The service process will 
 *          be logged on as this user. If the account belongs to the built-in domain,
 *          you can specify .\UserName. If the service type is 
 *          SERVICE_WIN32_SHARE_PROCESS you must specify the LocalSystem account. 
 *          If you specify NULL, CreateService uses the LocalSystem account. 
 * 
 *          If the service type is SERVICE_KERNEL_DRIVER or 
 *          SERVICE_FILE_SYSTEM_DRIVER, the name is the driver object name that the 
 *          system uses to load the device driver. Specify NULL if the driver is to 
 *          use a default object name created by the I/O system. 
 * 
 * @param String lpPassword - Pointer to a null-terminated string that contains the 
 *          password to the account name specified by the lpServiceStartName 
 *          parameter. If the pointer is NULL or if it points to an empty string, 
 *          the service has no password. Passwords are ignored for driver services. 
 *          If lpServiceStartName is LocalSystem, the password must be NULL.
 * 
 * @return int - If the function succeeds, the return value is a handle to the 
 *          service.  If the function fails, the return value is NULL. To get 
 *          extended error information, call com.ms.dll.DllLib.getLastWin32Error(). 
 * 
 *          The following error codes can be set by the service control manager. 
 *          Other error codes can be set by the registry functions that are called 
 *          by the service control manager. 
 * 
 *          Value                       Meaning 
 * 
 *          ERROR_ACCESS_DENIED         The handle to the specified service control 
 *                                      manager database does not have 
 *                                      SC_MANAGER_CREATE_SERVICE access. 
 * 
 *          ERROR_CIRCULAR_DEPENDENCY   A circular service dependency was specified.
 * 
 *          ERROR_DUP_NAME              The display name already exists in the 
 *                                      service control manager database either as a 
 *                                      service name or as another display name. 
 * 
 *          ERROR_INVALID_HANDLE        The handle to the specified service control 
 *                                      manager database is invalid. 
 * 
 *          ERROR_INVALID_NAME          The specified service name is invalid.
 * 
 *          ERROR_INVALID_PARAMETER     A parameter that was specified is invalid. 
 * 
 *          ERROR_INVALID_SERVICE_ACCOUNT The user account name specified in the 
 *                                      lpServiceStartName parameter does not exist.
 * 
 *          ERROR_SERVICE_EXISTS        The specified service already exists in this
 *                                      database. 
 */
	public static native SC_HANDLE CreateService(
			SC_HANDLE hSCManager,  // handle to service control manager 
			    		             // database
			String lpServiceName, // pointer to name of service to start
			String lpDisplayName, // pointer to display name
			int dwDesiredAccess, // type of access to service
			int dwServiceType,   // type of service
			int dwStartType,     // when to start service
			int dwErrorControl,  // severity if service fails to start
			String lpBinaryPathName,  // pointer to name of binary file
			String lpLoadOrderGroup,  // pointer to name of load ordering 
						                 // group
			String lpdwTagId,     // pointer to variable to get tag identifier
			Object lpDependencies,  // pointer to array of dependency names
			String lpServiceStartName,
					                // pointer to account name of service
			String lpPassword       // pointer to password for service account
			);
/**
 * The StartService function starts a service. 
 * 
 * When a driver service is started, the StartService function does not return until
 * the device driver has finished initializing. 
 * 
 * When a Win32-based service is started, the service control manager spawns the 
 * service process, if necessary. If the specified service shares a process with 
 * other services, the required process may already exist. The StartService function 
 * does not wait for the first status update from the new service, because it can 
 * take a while. Instead, it returns when the service control manager receives 
 * notification from the service control dispatcher that the ServiceMain thread for 
 * this service was created successfully. 
 * 
 * The service control manager sets the following default status values before 
 * returning from StartService: 
 * 
 *          Current state of the service is set to SERVICE_START_PENDING. 
 * 
 *          Controls accepted is set to none (zero). 
 * 
 *          The CheckPoint value is set to zero. 
 * 
 *          The WaitHint time is set to 2 seconds. 
 * 
 * The calling process can determine if the new service has finished its 
 * initialization by calling the QueryServiceStatus function periodically to query 
 * the service's status. 
 * 
 * A service cannot call StartService during initialization. The reason is that the 
 * service control manager locks the service control database during initialization,
 * so a call to StartService will block. Once the service reports to the service 
 * control manager that it has successfully started, it can call StartService. 
 * 
 * @param SC_HANDLE hService - Handle to the service. This handle is returned by the
 *          OpenService or CreateService function, and it must have SERVICE_START
 *          access. 
 * 
 * @param int dwNumServiceArgs - Specifies the number of argument strings in the 
 *          lpServiceArgVectors array. If lpServiceArgVectors is NULL, this 
 *          parameter can be zero. 
 * 
 * @param int lpServiceArgVectors  - Pointer to an array of pointers that point to 
 *          null-terminated argument strings passed to a service. Driver services 
 *          do not receive these arguments. If no arguments are passed to the 
 *          service being started, this parameter can be NULL. The service accesses 
 *          these arguments through its ServiceMain function. The first argument 
 *          (argv[0]) is the name of the service by default, followed by the 
 *          arguments, if any, in the lpServiceArgVectors array. 
 * 
 * @return int - If the function succeeds, the return value is nonzero.  If the 
 *          function fails, the return value is zero. To get extended error 
 *          information, call com.ms.dll.DllLib.getLastWin32Error(). 
 * 
 *          The following error codes can be set by the service control manager. 
 *          Others can be set by the registry functions that are called by the 
 *          service control manager. 
 * 
 *          Value                   Meaning 
 * 
 *          ERROR_ACCESS_DENIED     The specified handle was not opened with 
 *                                  SERVICE_START access. 
 * 
 *          ERROR_INVALID_HANDLE    The specified handle is invalid. 
 * 
 *          ERROR_PATH_NOT_FOUND    The service binary file could not be found. 
 * 
 *          ERROR_SERVICE_ALREADY_RUNNING An instance of the service is already 
 *                                  running. 
 * 
 *          ERROR_SERVICE_DATABASE_LOCKED The database is locked. 
 * 
 *          ERROR_SERVICE_DEPENDENCY_DELETED The service depends on a service that 
 *                                  does not exist or has been marked for deletion. 
 * 
 *          ERROR_SERVICE_DEPENDENCY_FAIL The service depends on another service 
 *                                  that has failed to start. 
 * 
 *          ERROR_SERVICE_DISABLED  The service has been disabled. 
 * 
 *          ERROR_SERVICE_LOGON_FAILED The service could not be logged on.
 * 
 *          ERROR_SERVICE_MARKED_FOR_DELETE The service has been marked for deletion.
 * 
 *          ERROR_SERVICE_NO_THREAD A thread could not be created for the service. 
 * 
 *          ERROR_SERVICE_REQUEST_TIMEOUT The service did not respond to the start 
 *                                  request in a timely fashion. 
 */
	public static native boolean StartService(
				SC_HANDLE hService,            // handle of service
				int dwNumServiceArgs,        // number of arguments
				String[] lpServiceArgVectors   // array of argument strings 
											// string pointers
			);
/**
 * The ControlService function sends a control code to a Win32-based service. 
 * 
 * The ControlService function asks the service control manager to send the 
 * requested control code to the service. The service control manager sends the code
 * if the service accepts the control and if the service is in a state in which a 
 * control can be sent to it. You cannot stop and start a service unless the 
 * security descriptor allows you to. The default security descriptor allows 
 * LocalSystem, Administrators, and Power Users to stop and start services. To 
 * change the security descriptor of a service, use SetServiceObjectSecurity. 
 * 
 * The QueryServiceStatus or function returns a SERVICE_STATUS structure whose 
 * dwCurrentState and dwControlsAccepted members indicate the current state and 
 * controls accepted by a running service. All running services accept the 
 * SERVICE_CONTROL_INTERROGATE control code by default. Each service specifies the 
 * other control codes that it accepts when it calls the SetServiceStatus function 
 * to report its status. A service should always accept these codes when it is 
 * running, no matter what it is doing. 
 * 
 * The following table shows the action of the service control manager in each of 
 * the possible service states:
 * 
 *          Service state       Stop        Other controls 
 * 
 *          STOPPED             (c)         (c)  
 *          STOP_PENDING        (b)         (b)  
 *          START_PENDING       (a)         (b)  
 *          RUNNING             (a)         (a)
 *          CONTINUE_PENDING    (a)         (a)
 *          PAUSE_PENDING       (a)         (a)
 *          PAUSED              (a)         (a)
 * 
 * (a) If the service accepts this control code, send the request to the service; 
 *          otherwise, ControlService returns zero and GetLastError returns 
 *          ERROR_INVALID_SERVICE_CONTROL.  
 * 
 * (b) The service is not in a state in which a control can be sent to it, so 
 *          ControlService returns zero and GetLastError returns 
 *          ERROR_SERVICE_CANNOT_ACCEPT_CTRL.  
 * 
 * (c) The service is not active, so ControlService returns zero and GetLastError 
 *          returns ERROR_SERVICE_NOT_ACTIVE.  
 * 
 * @param SC_HANDLE hService - Handle to the service. This handle is returned by the
 *          OpenService or CreateService function. The access required for this 
 *          handle depends on the dwControl code requested. 
 * 
 * @param int dwControl - Specifies the requested control code. This value can be 
 *          one of the standard control codes in the following table: 
 * 
 *          Value                   Meaning 
 * 
 *          SERVICE_CONTROL_STOP    Requests the service to stop. The hService 
 *                                  handle must have SERVICE_STOP access. 
 * 
 *          SERVICE_CONTROL_PAUSE   Requests the service to pause. The hService 
 *                                  handle must have SERVICE_PAUSE_CONTINUE access.
 * 
 *          SERVICE_CONTROL_CONTINUE Requests the paused service to resume. The 
 *                                  hService handle must have SERVICE_PAUSE_CONTINUE
 *                                  access. 
 * 
 *          SERVICE_CONTROL_INTERROGATE Requests the service to update immediately 
 *                                  its current status information to the service 
 *                                  control manager. The hService handle must have 
 *                                  SERVICE_INTERROGATE access. 
 * 
 *          SERVICE_CONTROL_SHUTDOWN The ControlService function fails if this 
 *                                  control code is specified. 
 * 
 *          This value can also be a user-defined control code, as described in the 
 *          following table: 
 * 
 *          Value                       Meaning
 * 
 *          Range 128 to 255.           The service defines the action associated 
 *                                      with the control code. The hService handle 
 *                                      must have SERVICE_USER_DEFINED_CONTROL 
 *                                      access.  
 * 
 * @param SERVICE_STATUS lpServiceStatus - Pointer to a SERVICE_STATUS structure to 
 *          receive the latest service status information. The information returned 
 *          reflects the most recent status that the service reported to the service
 *          control manager. 
 * 
 * @return int - If the function succeeds, the return value is nonzero.  If the 
 *          function fails, the return value is zero. To get extended error 
 *          information, call GetLastError. 
 * 
 *          The following error codes can be set by the service control manager. 
 *          Other error codes can be set by the registry functions that are called 
 *          by the service control manager. 
 * 
 *          Value                   Meaning
 * 
 *          ERROR_ACCESS_DENIED     The specified handle was not opened with the 
 *                                  necessary access. 
 * 
 *          ERROR_DEPENDENT_SERVICES_RUNNING The service cannot be stopped because 
 *                                  other running services are dependent on it. 
 * 
 *          ERROR_INVALID_SERVICE_CONTROL The requested control code is not valid, 
 *                                  or it is unacceptable to the service. 
 * 
 *          ERROR_SERVICE_CANNOT_ACCEPT_CTRL The requested control code cannot be 
 *                                  sent to the service because the state of the 
 *                                  service is SERVICE_STOPPED, 
 *                                  SERVICE_START_PENDING, or SERVICE_STOP_PENDING. 
 * 
 *          ERROR_SERVICE_NOT_ACTIVE    The service has not been started. 
 * 
 *          ERROR_SERVICE_REQUEST_TIMEOUT  The service did not respond to the start 
 *                                  request in a timely fashion. 
 * 
 */
	public static native boolean ControlService(
				SC_HANDLE hService,  // handle to service
				int dwControl,     // control code
				SERVICE_STATUS lpServiceStatus 
									// pointer to service status structure
);
/**
 * The QueryServiceStatus function retrieves the current status of the specified 
 * service. 
 * 
 * The QueryServiceStatus function returns the most recent service status 
 * information reported to the service control manager. If the service just changed 
 * its status, it may not have updated the service control manager yet. Applications
 * can find out the current service status by interrogating the service directly 
 * using the ControlService function with the SERVICE_CONTROL_INTERROGATE control 
 * code. 
 * 
 * 
 * @param SC_HANDLE hService - Handle to the service. This handle is returned by the
 *          OpenService or the CreateService function, and it must have 
 *          SERVICE_QUERY_STATUS access. 
 * 
 * @param SERVICE_STATUS lpServiceStatus - Pointer to a SERVICE_STATUS structure in 
 *          which the status information is returned. 
 * 
 * @return int - If the function succeeds, the return value is nonzero.  If the 
 *          function fails, the return value is zero. To get extended error 
 *          information, call GetLastError. 
 * 
 *          The following error codes can be set by the service control manager. 
 *          Other error codes can be set by the registry functions that are called 
 *          by the service control manager. 
 * 
 *          Value                       Meaning
 * 
 *          ERROR_ACCESS_DENIED         The specified handle was not opened with 
 *                                      SERVICE_QUERY_STATUS access. 
 * 
 *          ERROR_INVALID_HANDLE        The specified handle is invalid. 
 */
	public static native boolean QueryServiceStatus(
		SC_HANDLE hService,  // handle of service
		SERVICE_STATUS lpServiceStatus 
                       // address of service status structure
		);
/**
 * The CloseServiceHandle function closes the following types of handles: 
 * 
 *  - a handle to a service control manager object as returned by the OpenSCManager 
 *          function 
 * 
 *  - a handle to a service object as returned by either the OpenService or 
 *          CreateService function 
 * 
 * The CloseServiceHandle function does not destroy the service control manager 
 * object referred to by the handle. A service control manager object cannot be 
 * destroyed. A service object can be destroyed by calling the DeleteService 
 * function. 
 * 
 * 
 * @param SC_HANDLE hSCObject - Handle to the service control manager object or the 
 *          service object to close. 
 * 
 * @return boolean - If the function succeeds, the return value is nonzero.  If the 
 *          function fails, the return value is zero. To get extended error 
 *          information, call GetLastError. 
 * 
 *          The following error code can be set by the service control manager. 
 *          Other error codes can be set by registry functions that are called by 
 *          the service control manager. 
 * 
 *          Value                   Meaning
 * 
 *          ERROR_INVALID_HANDLE    The specified handle is invalid. 
 */
	public static native boolean CloseServiceHandle(
				SC_HANDLE hSCObject   // handle to service or service control 
								        // manager database
				);
/**
 * The ChangeServiceConfig function changes the configuration parameters of a 
 * service. 
 * 
 * The ChangeServiceConfig function changes the configuration information for the 
 * specified service in the service control manager database. You can obtain the 
 * current configuration information by using the QueryServiceConfig function. 
 * 
 * If the configuration is changed for a service that is running, with the exception
 * of lpDisplayName, the changes do not take effect until the service is stopped.
 * 
 * The startup program uses load ordering groups to load groups of services in a 
 * specified order with respect to the other groups in the list. The list of load 
 * ordering groups is contained in the ServiceGroupOrder value of the following 
 * registry key: 
 * 
 *          HKEY_LOCAL_MACHINE\System\CurrentControlSet\Control 
 * 
 * 
 * @param HC_HANDLE hService - Handle to the service. This handle is returned by the
 *          OpenService or CreateService function and must have 
 *          SERVICE_CHANGE_CONFIG access. 
 * 
 * @param int dwServiceType - A set of bit flags that specify the type of service. 
 *          Specify SERVICE_NO_CHANGE if you are not changing the existing service 
 *          type; otherwise, specify one of the following flags to indicate the 
 *          service type. 
 * 
 *          Value                   Meaning 
 * 
 *          SERVICE_WIN32_OWN_PROCESS Specifies a Win32-based service that runs in 
 *                                  its own process. 
 * 
 *          SERVICE_WIN32_SHARE_PROCESS Specifies a Win32-based service that shares 
 *                                  a process with other services. 
 * 
 *          SERVICE_KERNEL_DRIVER   Specifies a driver service. 
 * 
 *          SERVICE_FILE_SYSTEM_DRIVER Specifies a file system driver service. 
 * 
 * 
 *          If you specify either SERVICE_WIN32_OWN_PROCESS or 
 *          SERVICE_WIN32_SHARE_PROCESS, you can also specify the following flag. 
 * 
 *          Value                           Meaning 
 * 
 *          SERVICE_INTERACTIVE_PROCESS     Enables a Win32-based service process to
 *                                          interact with the desktop. 
 * 
 * @param int dwStartType - Specifies when to start the service. Specify 
 *          SERVICE_NO_CHANGE if you are not changing the existing start type; 
 *          otherwise, specify one of the following flags to indicate the start type.
 *  
 *          Value                           Meaning 
 * 
 *          SERVICE_BOOT_START              Specifies a device driver started by the
 *                                          system loader. This value is valid only 
 *                                          for driver services. 
 * 
 *          SERVICE_SYSTEM_START            Specifies a device driver started by the 
 *                                          IoInitSystem function. This value is 
 *                                          valid only for driver services. 
 * 
 *          SERVICE_AUTO_START              Specifies a service to be started 
 *                                          automatically by the service control 
 *                                          manager during system startup. 
 * 
 *          SERVICE_DEMAND_START            Specifies a service to be started by the 
 *                                          service control manager when a process 
 *                                          calls the StartService function. 
 * 
 *          SERVICE_DISABLED                Specifies a service that can no longer 
 *                                          be started. 
 * 
 * @param int dwErrorControl - Specifies the severity of the error if this service 
 *          fails to start during startup, and determines the action taken by the 
 *          startup program if failure occurs. Specify SERVICE_NO_CHANGE if you are 
 *          not changing the existing error control; otherwise, specify one of the 
 *          following flags to indicate the error control. 
 * 
 *          Value                           Meaning 
 * 
 *          SERVICE_ERROR_IGNORE            The startup program logs the error but 
 *                                          continues the startup operation. 
 * 
 *          SERVICE_ERROR_NORMAL            The startup program logs the error and 
 *                                          puts up a message box pop-up but 
 *                                          continues the startup operation. 
 * 
 *          SERVICE_ERROR_SEVERE            The startup program logs the error. If 
 *                                          the last-known-good configuration is 
 *                                          being started, the startup operation 
 *                                          continues. Otherwise, the system is 
 *                                          restarted with the last-known-good 
 *                                          configuration. 
 * 
 *          SERVICE_ERROR_CRITICAL          The startup program logs the error, if 
 *                                          possible. If the last-known-good 
 *                                          configuration is being started, the 
 *                                          startup operation fails. Otherwise, the 
 *                                          system is restarted with the last-known 
 *                                          good configuration. 
 * 
 * @param String lpBinaryPathName - Pointer to a null-terminated string that 
 *          contains the fully qualified path to the service binary file. Specify 
 *          NULL if you are not changing the existing path. If the path contains a 
 *          space, it must be quoted so that it is correctly interpreted. For 
 *          example, "d:\\my share\\myservice.exe" should be specified as 
 *          "\"d:\\my share\\myservice.exe\"". 
 * 
 * @param String lpLoadOrderGroup - Pointer to a null-terminated string that names 
 *          the load ordering group of which this service is a member. Specify NULL 
 *          if you are not changing the existing group. Specify an empty string if 
 *          the service does not belong to a group. 
 * 
 * @param int lpdwTagId - Pointer to a DWORD variable that receives a tag value that
 *          is unique in the group specified in the lpLoadOrderGroup parameter. 
 *          Specify NULL if you are not changing the existing tag. 
 * 
 *          You can use a tag for ordering service startup within a load ordering 
 *          group by specifying a tag order vector in the GroupOrderList value of 
 *          the following registry key: 
 * 
 *          HKEY_LOCAL_MACHINE\System\CurrentControlSet\Control 
 * 
 *          Tags are only evaluated for driver services that have SERVICE_BOOT_START
 *          or SERVICE_SYSTEM_START start types. 
 * 
 * @param String lpDependencies - Pointer to a double null-terminated array of 
 *          null-separated names of services or load ordering groups that the system 
 *          must start before this service can be started. (Dependency on a group 
 *          means that this service can run if at least one member of the group is 
 *          running after an attempt to start all members of the group.) Specify 
 *          NULL if you are not changing the existing dependencies. Specify an 
 *          empty string if the service has no dependencies. 
 * 
 *          You must prefix group names with SC_GROUP_IDENTIFIER so that they can be
 *          distinguished from a service name, because services and service groups 
 *          share the same name space. 
 * 
 * @param String lpServiceStartName - Pointer to a null-terminated string that names
 *          the service. Specify NULL if you are not changing the existing name. If 
 *          the service type is SERVICE_WIN32_OWN_PROCESS, use an account name in 
 *          the form DomainName\UserName. The service process will be logged on as 
 *          this user. If the account belongs to the built-in domain, you can 
 *          specify .\UserName. If the service type is SERVICE_WIN32_SHARE_PROCESS 
 *          you must specify the LocalSystem account. 
 * 
 *          If the service type is SERVICE_KERNEL_DRIVER or 
 *          SERVICE_FILE_SYSTEM_DRIVER, the name is the driver object name that the 
 *          system uses to load the device driver. Specify NULL if the driver is to 
 *          use a default object name created by the I/O system. 
 * 
 * @param String lpPassword - Pointer to a null-terminated string that contains the 
 *          password to the account name specified by the lpServiceStartName 
 *          parameter. Specify NULL if you are not changing the password. Specify an
 *          empty string if the service has no password. 
 * 
 *          Passwords are ignored for driver services. 
 * 
 * @param String lpDisplayName - Pointer to a null-terminated string that is to be 
 *          used by applications to identify the service for its users. This string 
 *          has a maximum length of 256 characters. The name is case-preserved in 
 *          the service control manager. Display name comparisons are always 
 *          case-insensitive. 
 * 
 * @return int - If the function succeeds, the return value is nonzero.  If the 
 *          function fails, the return value is zero. To get extended error 
 *          information, call GetLastError. 
 * 
 *          The following error codes may be set by the service control manager. 
 *          Other error codes may be set by the registry functions that are called 
 *          by the service control manager. 
 * 
 *          Value                       Meaning
 * 
 *          ERROR_ACCESS_DENIED         The specified handle was not opened with 
 *                                      SERVICE_CHANGE_CONFIG access. 
 * 
 *          ERROR_CIRCULAR_DEPENDENCY   A circular service dependency was specified.
 * 
 *          ERROR_DUP_NAME              The display name already exists in the 
 *                                      service controller manager database, either 
 *                                      as a service name or as another display name.
 * 
 *          ERROR_INVALID_HANDLE        The specified handle is invalid. 
 * 
 *          ERROR_INVALID_PARAMETER     A parameter that was specified is invalid.
 * 
 *          ERROR_INVALID_SERVICE_ACCOUNT The account name does not exist, or a 
 *                                      service is specified to share the same 
 *                                      binary file as an already installed service 
 *                                      but with an account name that is not the 
 *                                      same as the installed service. 
 * 
 *          ERROR_SERVICE_MARKED_FOR_DELETE The service has been marked for deletion.
 */
	public static native boolean ChangeServiceConfig(
					SC_HANDLE hService,     // handle to service
					int dwServiceType,   // type of service
					int dwStartType,     // when to start service
					int dwErrorControl,  // severity if service fails to start
					String lpBinaryPathName,  // pointer to service binary file name
					String lpLoadOrderGroup,  // pointer to load ordering group name
					int[] lpdwTagId,     // pointer to variable to get tag identifier
					String lpDependencies,    // pointer to array of dependency names
					String[] lpServiceStartName,
										// pointer to account name of service
					String[] lpPassword,   // pointer to password for service account
					String[] lpDisplayName      // pointer to display name
					);
/**
 * The DeleteService function marks the specified service for deletion from the 
 * service control manager database. 
 *  
 * The DeleteService function marks a service for deletion from the service control 
 * manager database. The database entry is not removed until all open handles to the
 * service have been closed by calls to the CloseServiceHandle function, and the 
 * service is not running. A running service is stopped by a call to the 
 * ControlService function with the SERVICE_CONTROL_STOP control code. If the 
 * service cannot be stopped, the database entry is removed when the system is 
 * restarted. 
 * 
 * The service control manager deletes the service by deleting the service key and 
 * its subkeys from the registry. 
 * 
 * 
 * @param SC_HANDLE hService - Handle to the service. This handle is returned by the
 *          OpenService or CreateService function, and it must have DELETE access. 
 * 
 * @return boolean - If the function succeeds, the return value is nonzero.  If the 
 *          function fails, the return value is zero. To get extended error 
 *          information, call com.ms.dll.DllLib.getLastWin32Error(). 
 * 
 *          The following error codes may be set by the service control manager. 
 *          Others may be set by the registry functions that are called by the 
 *          service control manager. 
 * 
 *          Value                   Meaning 
 * 
 *          ERROR_ACCESS_DENIED     The specified handle was not opened with DELETE 
 *                                  access.
 * 
 *          ERROR_INVALID_HANDLE    The specified handle is invalid. 
 * 
 *          ERROR_SERVICE_MARKED_FOR_DELETE     The specified service has already 
 *                                              been marked for deletion.
 */
	public static native boolean DeleteService(
                SC_HANDLE hService   // handle to service
                );
/**
 * The SetConsoleCtrlHandler function adds or removes an application-defined 
 * HandlerRoutine function from the list of handler functions for the calling 
 * process. 
 * 
 * Each console process has its own list of application-defined HandlerRoutine 
 * functions that handle ctrl+c and ctrl+break signals. The handler functions also 
 * handle signals generated by the system when the user closes the console, logs off,
 * or shuts down the system. Initially, the handler list for each process contains 
 * only a default handler function that calls the ExitProcess function. A console 
 * process adds or removes additional handler functions by calling the 
 * SetConsoleCtrlHandler function, which does not affect the list of handler 
 * functions for other processes. When a console process receives any of the 
 * control signals, its handler functions are called on a last-registered, 
 * first-called basis until one of the handlers returns TRUE. If none of the 
 * handlers returns TRUE, the default handler is called. 
 * 
 * For console processes, the ctrl+c and ctrl+break key combinations are typically 
 * treated as signals (CTRL_C_EVENT and CTRL_C_BREAK_EVENT). When a console window 
 * with the keyboard focus receives ctrl+c or ctrl+break, the signal is typically 
 * passed to all processes sharing that console. 
 * 
 * ctrl+break is always treated as a signal, but typical ctrl+c behavior can be 
 * changed in three ways that prevent the handler functions from being called: 
 * 
 *      The SetConsoleMode function can disable the ENABLE_PROCESSED_INPUT mode for 
 *      a console's input buffer, so ctrl+c is reported as keyboard input rather 
 *      than as a signal. 
 * 
 *      Windows NT: Calling SetConsoleCtrlHandler with the NULL and TRUE arguments 
 *      causes the calling process to ignore ctrl+c signals. This attribute is 
 *      inherited by child processes, but it can be enabled or disabled by any 
 *      process without affecting existing processes. 
 * 
 *      If a console process is being debugged and ctrl+c signals have not been 
 *      disabled, the kernel generates a DBG_CONTROL_C exception. This exception is 
 *      raised only for the benefit of the debugger, and an application should never
 *      use an exception handler to deal with it. If the debugger handles the 
 *      exception, an application will not notice the ctrl+c, with one exception: 
 *          alertable waits will terminate. If the debugger passes the exception on 
 *          unhandled, ctrl+c is passed to the console process and treated as a 
 *          signal, as previously discussed. 
 * 
 *      A console process can use the GenerateConsoleCtrlEvent function to send a 
 *      ctrl+c or ctrl+break signal to a console process group. 
 * 
 * The system generates CTRL_CLOSE_EVENT, CTRL_LOGOFF_EVENT, and CTRL_SHUTDOWN_EVENT
 * signals when the user closes the console, logs off, or shuts down the system so 
 * that the process has an opportunity to clean up before termination. Console 
 * functions, or any C run-time functions that call console functions, may not work 
 * reliably during processing of any of the three signals mentioned previously. The 
 * reason is that some or all of the internal console cleanup routines may have been
 * called before executing the process signal handler. 
 * 
 * 
 * Windows NT: If no handler function is specified, the function sets an inheritable
 * attribute that determines whether the calling process ignores ctrl+c signals. 
 * 
 * @param int HandlerRoutine - Pointer to the application-defined HandlerRoutine 
 *          function to add or remove. 
 * 
 *          Windows NT: This parameter can be NULL. 
 * 
 * @param boolean Add - Specifies whether to add or remove the function pointed to 
 *          by the HandlerRoutine parameter from the handler list. If this parameter
 *          is TRUE, the handler is added; if it is FALSE, the handler is removed.
 * 
 *          Windows NT: If the HandlerRoutine parameter is NULL, a TRUE value causes
 *          the calling process to ignore ctrl+c input, and a FALSE value restores 
 *          normal processing of ctrl+c input. This attribute of ignoring or 
 *          processing ctrl+c is inherited by child processes. 
 * 
 * @return boolean - If the function succeeds, the return value is nonzero.  If the 
 *          function fails, the return value is zero. To get extended error 
 *          information, call com.ms.dll.DllLib.getLastWin32Error(). 
 * 
 * @dll.import("KERNEL32.DLL", setLastError)
 */
    public static native boolean SetConsoleCtrlHandler(
            int HandlerRoutine,  // address of handler function
            boolean Add          // handler to add or remove
            );
}