/*
 * sample.ntservice.pSC_HANDLE.java
 *
 * (C) Copyright 1995 - 1999 Microsoft Corporation.  All rights reserved.
 *
 */
package sample.ntservice;
/**
 * This structure represents the C type SC_HANDLE that is used in win32 calls to 
 * functions related to NT Services.  It is provided to make the method calls to 
 * J/Direct more readable, although passing/receiving an integer instead of this 
 * structure would also work.  A nice feature that comes with using a structure, is
 * that if the return value is 0, the structure will be null.
 * 
 * This custom marshaler should be used whenever the SC_HANDLE struct is 
 * embedded within another structure.  When used as a field in another struct, 
 * it must have a dll.structmap directive like this:
 * 
 * **
 * * dll.structmap([customMarshal="sample.dcom.SC_HANDLEMarshal", type=CUSTOM]) 
 * *
 *  public SC_HANDLE foo;

 * 
 * @version 1.0
 * 
 * @dll.struct()
 */
public class SC_HANDLE implements Cloneable{
    /**
     * This is the actual value of the handle.
     */
	int value;
    /**
     * The Java VM will read this field to determine how many bytes the custom
     * marshaled object takes up in the native C.
     * 
     * NOTE: static fields are NOT marshaled to C !
     */
    public static int cbByValSize = 4;
/**
 * Retrieves the internal value of this SC_HANDLE.  
 * 
 * @return int - integer representing the SC_HANDLE.
 */
    public int getValue(){
        return value;
    }
/**
 * Overrides the default Object implementation of equals to provide a method 
 * capable of comparing two instances of SC_HANDLE.  This method will return true
 * only when the values contained in both SC_HANDLE objects are identical, NOT if
 * they are both pointers to the same Java object.
 * 
 * @param Object o - Object to compare ourselves to.
 * 
 * @return boolean - true if we have the same value as the parameter.
 */
    public boolean equals(Object o){
        if (o==null) 
            return false;
        if (!(o instanceof SC_HANDLE))
            return false;
        if (((SC_HANDLE)o).getValue()==value)
            return true;
        return false;
    }
 /**
  * This overrides the default clone method, creating a new instance of this
  * struct with exactly the same values.
  * 
  * @return Object - the new struct with identical values.
  */
    public Object clone(){
        SC_HANDLE s = new SC_HANDLE();
        s.value=value;
        return s;
    }
/**
 * The Java VM will call this method to marshal the SC_HANDLE structure from
 * C back to Java.
 * 
 * @param ppStruct - This is a pointer to a memory location where the C structure is.
 * @param Flags - any flags that have been specified by the Java Object representing
 *          the C structure. [Normally this is unused].
 * 
 * @return SC_HANDLE - this is a Java representation of the C structure. This
 *          value will be placed where the custom marshaled field is.
 */    
    public static SC_HANDLE toJava(int ppStruct, int Flags){  
        //This causes a propogate on all of the fields in the SC_HANDLE struct.
        return (SC_HANDLE)com.ms.dll.DllLib.ptrToStruct(SC_HANDLE.class,
                                                com.ms.dll.DllLib.read4(ppStruct));  
    }
/**
 * The Java VM will call this method to marshal the field specified with this 
 * custom marshaler to native C.  This is accomplished by propogating any Java
 * objects to native C, and setting the ppStruct value to point to a pinned 
 * SC_HANDLE struct.
 * 
 * @param SC_HANDLE struct - the struct to be marshaled to native C.
 * @param int Flags - Any flags specified in the dll.structmap declaration for the
 *          field containing the SC_HANDLE object.
 * @param int ppStruct - this is the memory location of where to write the address
 *          of the marshaled SC_HANDLE struct.
 */    
    public static void toExternal(SC_HANDLE struct, int Flags, int ppStruct ) { 
        com.ms.dll.DllLib.propagateStructFields(struct,false);
        com.ms.dll.DllLib.write4(ppStruct,
              com.ms.dll.DllLib.addrOf(com.ms.dll.Root.alloc(struct)));
    }
}