/*
 * sample.ntservice.SERVICE_STATUS.java
 *
 * (C) Copyright 1995 - 1999 Microsoft Corporation.  All rights reserved.
 *
 */
package sample.ntservice;
/**
 * The SERVICE_STATUS structure contains information about a service. The 
 * ControlService, EnumDependentServices, EnumServicesStatus, and QueryServiceStatus
 * functions use this structure to return information about a service. A service 
 * uses this structure in the SetServiceStatus function to report its current 
 * status to the service control manager. 
 *  
 * This custom marshaler should be used whenever the SERVICE_STATUS struct is 
 * embedded within another structure.  When used as a field in another struct, 
 * it must have a dll.structmap directive like this:
 * 
 * **
 * * dll.structmap([customMarshal="sample.dcom.SERVICE_STATUS", type=CUSTOM]) 
 * *
 *  public SERVICE_STATUS foo;
 * 
 * @version 1.0
 * 
 * @dll.struct()
 */
public class SERVICE_STATUS implements Cloneable{
    /**
     * The value returned includes one of the following service type flags to 
     * indicate the type of service. In addition, for a SERVICE_WIN32 service, the 
     * SERVICE_INTERACTIVE_PROCESS flag might be set, indicating that the service 
     * process can interact with the desktop. 
     * 
     *  Value                           Meaning 
     * 
     *  SERVICE_WIN32_OWN_PROCESS       A service type flag that indicates a Win32 
     *                                  service that runs in its own process. 
     * 
     *  SERVICE_WIN32_SHARE_PROCESS     A service type flag that indicates a Win32 
     *                                  service that shares a process with other 
     *                                  services. 
     * 
     *  SERVICE_KERNEL_DRIVER           A service type flag that indicates a device 
     *                                  driver. 
     * 
     *  SERVICE_FILE_SYSTEM_DRIVER      A service type flag that indicates a file 
     *                                  system driver. 
     * 
     *  SERVICE_INTERACTIVE_PROCESS     A flag that indicates a Win32 service 
     *                                  process that can interact with the desktop. 
     */
	public int dwServiceType;
    /**
     * Indicates the current state of the service. One of the following values is 
     * specified: 
     * 
     *      Value                       Meaning
     * 
     *      SERVICE_STOPPED             The service is not running. 
     * 
     *      SERVICE_START_PENDING       The service is starting. 
     * 
     *      SERVICE_STOP_PENDING        The service is stopping. 
     * 
     *      SERVICE_RUNNING             The service is running. 
     * 
     *      SERVICE_CONTINUE_PENDING    The service continue is pending. 
     * 
     *      SERVICE_PAUSE_PENDING       The service pause is pending. 
     * 
     *      SERVICE_PAUSED              The service is paused. 
     */
	public int dwCurrentState;
    /**
     * Specifies the control codes that the service will accept and process. A user 
     * interface process can control a service by specifying a control command in 
     * the ControlService function. By default, all services accept the 
     * SERVICE_CONTROL_INTERROGATE value. Any or all of the following flags can be 
     * specified to enable the other control codes. 
     * 
     *      Value                       Meaning 
     * 
     *      SERVICE_ACCEPT_STOP         The service can be stopped. This enables the
     *                                  SERVICE_CONTROL_STOP value. 
     * 
     *      SERVICE_ACCEPT_PAUSE_CONTINUE The service can be paused and continued. 
     *                                  This enables the SERVICE_CONTROL_PAUSE and 
     *                                  SERVICE_CONTROL_CONTINUE values. 
     * 
     *      SERVICE_ACCEPT_SHUTDOWN     The service is notified when system shutdown
     *                                  occurs. This enables the system to send a 
     *                                  SERVICE_CONTROL_SHUTDOWN value to the 
     *                                  service. The ControlService function cannot 
     *                                  send this control code. 
     */
	public int dwControlsAccepted;
    /**
     * Specifies an Win32 error code that the service uses to report an error that 
     * occurs when it is starting or stopping. To return an error code specific to 
     * the service, the service must set this value to ERROR_SERVICE_SPECIFIC_ERROR 
     * to indicate that the dwServiceSpecificExitCode member contains the error code.
     * The service should set this value to NO_ERROR when it is running and on 
     * normal termination. 
     */
	public int dwWin32ExitCode;
    /**
     * Specifies a service specific error code that the service returns when an 
     * error occurs while the service is starting or stopping. This value is ignored
     * unless the dwWin32ExitCode member is set to ERROR_SERVICE_SPECIFIC_ERROR. 
     */
	public int dwServiceSpecificExitCode;
    /**
     * Specifies a value that the service increments periodically to report its 
     * progress during a lengthy start, stop, pause, or continue operation. For 
     * example, the service should increment this value as it completes each step 
     * of its initialization when it is starting up. The user interface program that
     * invoked the operation on the service uses this value to track the progress of
     * the service during a lengthy operation. This value is not valid and should be
     * zero when the service does not have a start, stop, pause, or continue 
     * operation pending. 
     */
	public int dwCheckPoint;
    /**
     * Specifies an estimate of the amount of time, in milliseconds, that the 
     * service expects a pending start, stop, pause, or continue operation to take 
     * before the service makes its next call to the SetServiceStatus function with
     * either an incremented dwCheckPoint value or a change in dwCurrentState. If 
     * the amount of time specified by dwWaitHint passes, and dwCheckPoint has not 
     * been incremented, or dwCurrentState has not changed, the service control 
     * manager or service control program can assume that an error has occurred.
     */
	public int dwWaitHint;
    /**
     * The Java VM will read this field to determine how many bytes the custom
     * marshaled object takes up in the native C.
     * 
     * NOTE: static fields are NOT marshaled to C !
     */
    public static int cbByValSize = 4;
/**
 * Overrides the default Object implementation of equals to provide a method 
 * capable of comparing two instances of SERVICE_STATUS.  This method will return 
 * true only when the values contained in both SERVICE_STATUS objects are identical,
 * NOT if they are both pointers to the same Java object.
 * 
 * @param Object o - Object to compare ourselves to.
 * 
 * @return boolean - true if we have the same value as the parameter.
 */
    public boolean equals(Object o){
        if (o==null) 
            return false;
        if (!(o instanceof SERVICE_STATUS))
            return false;
        if ((((SERVICE_STATUS)o).dwCurrentState==dwCurrentState)&&
            (((SERVICE_STATUS)o).dwServiceType==dwServiceType)&&
            (((SERVICE_STATUS)o).dwControlsAccepted==dwControlsAccepted)&&
            (((SERVICE_STATUS)o).dwWin32ExitCode==dwWin32ExitCode)&&
            (((SERVICE_STATUS)o).dwServiceSpecificExitCode==
                    dwServiceSpecificExitCode)&&
            (((SERVICE_STATUS)o).dwCheckPoint==dwCheckPoint)&&
            (((SERVICE_STATUS)o).dwWaitHint==dwWaitHint))
                return true;
        return false;
    }
 /**
  * This overrides the default clone method, creating a new instance of this
  * struct with exactly the same values.
  * 
  * @return Object - the new struct with identical values.
  */
    public Object clone(){
        SERVICE_STATUS s = new SERVICE_STATUS();
        s.dwCheckPoint=dwCheckPoint;
        s.dwControlsAccepted = dwControlsAccepted;
        s.dwCurrentState = dwCurrentState;
        s.dwServiceSpecificExitCode = dwServiceSpecificExitCode;
        s.dwServiceType = dwServiceType;
        s.dwWaitHint = dwWaitHint;
        s.dwWin32ExitCode = dwWin32ExitCode;
        return s;
    }
/**
 * The Java VM will call this method to marshal the SERVICE_STATUS structure from
 * C back to Java.
 * 
 * @param ppStruct - This is a pointer to a memory location where the C structure is.
 * @param Flags - any flags that have been specified by the Java Object representing
 *          the C structure. [Normally this is unused].
 * 
 * @return SERVICE_STATUS - this is a Java representation of the C structure. This
 *          value will be placed where the custom marshaled field is.
 */    
    public static SERVICE_STATUS toJava(int ppStruct, int Flags){  
        //This causes a propogate on all of the fields in the SERVICE_STATUS struct.
        return (SERVICE_STATUS)com.ms.dll.DllLib.ptrToStruct(SERVICE_STATUS.class,
                                                com.ms.dll.DllLib.read4(ppStruct));  
    }
/**
 * The Java VM will call this method to marshal the field specified with this 
 * custom marshaler to native C.  This is accomplished by propogating any Java
 * objects to native C, and setting the ppStruct value to point to a pinned 
 * SERVICE_STATUS struct.
 * 
 * @param SERVICE_STATUS struct - the struct to be marshaled to native C.
 * @param int Flags - Any flags specified in the dll.structmap declaration for the
 *          field containing the SERVICE_STATUS object.
 * @param int ppStruct - this is the memory location of where to write the address
 *          of the marshaled SERVICE_STATUS struct.
 */    
    public static void toExternal(SERVICE_STATUS struct, int Flags, int ppStruct ) { 
        com.ms.dll.DllLib.propagateStructFields(struct,false);
        com.ms.dll.DllLib.write4(ppStruct,
              com.ms.dll.DllLib.addrOf(com.ms.dll.Root.alloc(struct)));
    }
}