/*
 * sample.ntservice.SERVICE_STATUS_HANDLE.java
 *
 * (C) Copyright 1995 - 1999 Microsoft Corporation.  All rights reserved.
 *
 */
package sample.ntservice;
/**
 * Handle to a service status value.
 * 
 * This custom marshaler should be used whenever the SERVICE_STATUS_HANDLE struct is 
 * embedded within another structure.  When used as a field in another struct, 
 * it must have a dll.structmap directive like this:
 * 
 * **
 * * dll.structmap([customMarshal="sample.dcom.SERVICE_STATUS_HANDLE", type=CUSTOM]) 
 * *
 *  public SERVICE_STATUS_HANDLE foo;
 * 
 * @version 1.0
 * 
 * @dll.struct()
 */
public class SERVICE_STATUS_HANDLE implements Cloneable{
    /**
     * This is the actual value of the handle. 
     */
	int handle;
    /**
     * The Java VM will read this field to determine how many bytes the custom
     * marshaled object takes up in the native C.
     * 
     * NOTE: static fields are NOT marshaled to C !
     */
    public static int cbByValSize = 4;
/**
 * Retrieves the internal value of this SERVICE_STATUS.  
 * 
 * @return int - integer representing the SERVICE_STATUS.
 */
    public int getValue(){
        return handle;
    }
/**
 * Overrides the default Object implementation of equals to provide a method 
 * capable of comparing two instances of SERVICE_STATUS.  This method will return 
 * true only when the values contained in both SERVICE_STATUS objects are identical,
 * NOT if they are both pointers to the same Java object.
 * 
 * @param Object o - Object to compare ourselves to.
 * 
 * @return boolean - true if we have the same value as the parameter.
 */
    public boolean equals(Object o){
        if (o==null) 
            return false;
        if (!(o instanceof SERVICE_STATUS_HANDLE))
            return false;
        if (((SERVICE_STATUS_HANDLE)o).getValue()==handle)
            return true;
        return false;
    }
 /**
  * This overrides the default clone method, creating a new instance of this
  * struct with exactly the same values.
  * 
  * @return Object - the new struct with identical values.
  */
    public Object clone(){
        SERVICE_STATUS_HANDLE q = new SERVICE_STATUS_HANDLE();
        q.handle=handle;
        return q;
    }
/**
 * The Java VM will call this method to marshal the SERVICE_STATUS_HANDLE structure from
 * C back to Java.
 * 
 * @param ppStruct - This is a pointer to a memory location where the C structure is.
 * @param Flags - any flags that have been specified by the Java Object representing
 *          the C structure. [Normally this is unused].
 * 
 * @return SERVICE_STATUS_HANDLE - this is a Java representation of the C structure. This
 *          value will be placed where the custom marshaled field is.
 */    
    public static SERVICE_STATUS_HANDLE toJava(int ppStruct, int Flags){  
        //This causes a propogate on all of the fields in the SERVICE_STATUS_HANDLE struct.
        return (SERVICE_STATUS_HANDLE)com.ms.dll.DllLib.ptrToStruct(SERVICE_STATUS_HANDLE.class,
                                                com.ms.dll.DllLib.read4(ppStruct));  
    }
/**
 * The Java VM will call this method to marshal the field specified with this 
 * custom marshaler to native C.  This is accomplished by propogating any Java
 * objects to native C, and setting the ppStruct value to point to a pinned 
 * SERVICE_STATUS_HANDLE struct.
 * 
 * @param SERVICE_STATUS_HANDLE struct - the struct to be marshaled to native C.
 * @param int Flags - Any flags specified in the dll.structmap declaration for the
 *          field containing the SERVICE_STATUS_HANDLE object.
 * @param int ppStruct - this is the memory location of where to write the address
 *          of the marshaled SERVICE_STATUS_HANDLE struct.
 */    
    public static void toExternal(SERVICE_STATUS_HANDLE struct, int Flags, int ppStruct ) { 
        com.ms.dll.DllLib.propagateStructFields(struct,false);
        com.ms.dll.DllLib.write4(ppStruct,
              com.ms.dll.DllLib.addrOf(com.ms.dll.Root.alloc(struct)));
    }
}