/*
 * sample.ntservice.SERVICE_TABLE_ENTRY.java
 *
 * (C) Copyright 1995 - 1999 Microsoft Corporation.  All rights reserved.
 *
 */
package sample.ntservice;
/**
 * The SERVICE_TABLE_ENTRY structure is used by the StartServiceCtrlDispatcher 
 * function to specify the ServiceMain function for a Win32 service that can run in 
 * the calling process. 
 * 
 * This custom marshaler should be used whenever the SERVICE_TABLE_ENTRY struct is 
 * embedded within another structure.  When used as a field in another struct, 
 * it must have a dll.structmap directive like this:
 * 
 * **
 * * dll.structmap([customMarshal="sample.dcom.SERVICE_TABLE_ENTRY", type=CUSTOM]) 
 * *
 *  public SERVICE_TABLE_ENTRY foo;
 * 
 * @version 1.0
 * 
 * @dll.struct()
 */
public class SERVICE_TABLE_ENTRY implements Cloneable{
 /**
  * String that names a service that can run in this service process. This string 
  * is ignored if the service is installed in the service control manager database 
  * as a SERVICE_WIN32_OWN_PROCESS service type. For a SERVICE_WIN32_SHARE_PROCESS 
  * service process, this string names the service that uses the ServiceMain 
  * function pointed to by the lpServiceProc member. 
  */
	public String lpServiceName;
   /**
    * Pointer to a ServiceMain function. 
    * 
    * @see sample.ntservice.NTService.ServiceMain
    */
	public int lpServiceProc;
    /**
     * The Java VM will read this field to determine how many bytes the custom
     * marshaled object takes up in the native C.
     * 
     * NOTE: static fields are NOT marshaled to C !
     */
    public static int cbByValSize = 4;

/**
 * Overrides the default Object implementation of equals to provide a method 
 * capable of comparing two instances of pSERVICE_TABLE_ENTRY.  This method will 
 * return true only when the values contained in both pSERVICE_TABLE_ENTRY objects 
 * are identical, NOT if they are both pointers to the same Java object.
 * 
 * @param Object o - Object to compare ourselves to.
 * 
 * @return boolean - true if we have the same value as the parameter.
 */
    public boolean equals(Object o){
        if (o==null)
            return false;
        if (!(o instanceof SERVICE_TABLE_ENTRY))
            return false;
        if((((SERVICE_TABLE_ENTRY)o).lpServiceName==lpServiceName)&&
            (((SERVICE_TABLE_ENTRY)o).lpServiceProc==lpServiceProc))
            return true;
        else 
            return false;
    }
 /**
  * This overrides the default clone method, creating a new instance of this
  * struct with exactly the same values.
  * 
  * @return Object - the new struct with identical values.
  */
    public Object clone(){
        SERVICE_TABLE_ENTRY q = new SERVICE_TABLE_ENTRY();
        q.lpServiceName=lpServiceName;
        q.lpServiceProc = lpServiceProc;
        return q;
    }
/**
 * The Java VM will call this method to marshal the SERVICE_TABLE_ENTRY structure from
 * C back to Java.
 * 
 * @param ppStruct - This is a pointer to a memory location where the C structure is.
 * @param Flags - any flags that have been specified by the Java Object representing
 *          the C structure. [Normally this is unused].
 * 
 * @return SERVICE_TABLE_ENTRY - this is a Java representation of the C structure. This
 *          value will be placed where the custom marshaled field is.
 */    
    public static SERVICE_TABLE_ENTRY toJava(int ppStruct, int Flags){  
        //This causes a propogate on all of the fields in the SERVICE_TABLE_ENTRY struct.
        return (SERVICE_TABLE_ENTRY)com.ms.dll.DllLib.ptrToStruct(SERVICE_TABLE_ENTRY.class,
                                                com.ms.dll.DllLib.read4(ppStruct));  
    }
/**
 * The Java VM will call this method to marshal the field specified with this 
 * custom marshaler to native C.  This is accomplished by propogating any Java
 * objects to native C, and setting the ppStruct value to point to a pinned 
 * SERVICE_TABLE_ENTRY struct.
 * 
 * @param SERVICE_TABLE_ENTRY struct - the struct to be marshaled to native C.
 * @param int Flags - Any flags specified in the dll.structmap declaration for the
 *          field containing the SERVICE_TABLE_ENTRY object.
 * @param int ppStruct - this is the memory location of where to write the address
 *          of the marshaled SERVICE_TABLE_ENTRY struct.
 */    
    public static void toExternal(SERVICE_TABLE_ENTRY struct, int Flags, int ppStruct ) { 
        com.ms.dll.DllLib.propagateStructFields(struct,false);
        com.ms.dll.DllLib.write4(ppStruct,
              com.ms.dll.DllLib.addrOf(com.ms.dll.Root.alloc(struct)));
    }
}