/*
 * sample.server.DCOMService.java
 *
 * (C) Copyright 1995 - 1999 Microsoft Corporation.  All rights reserved.
 *
 */
package sample.server;

import sample.dcom.*;
/**
 * Implementation of a simple NT Service that will be a COM Server for a Java
 * implemented COM object.  This class serves as the main method for registering
 * the NTService, eventlog message information, and the appropriate registry
 * settings for the COM Server to run as an NT Service.
 * 
 * @see sample.ntservice.NTService
 * @see sample.dcom.COMServer
 * @see sample.server.Pinger
 * 
 * @version 1.0
 */
public class DCOMService extends sample.ntservice.NTService{
        /**
         * This is the classfactory that hosts the COM Singleton object
         */
        private COMServer object = null;
        /**
         * CLSID that we will be registering our classfactory with.
         */
        private com.ms.com._Guid clsid = Pinger.CLSID_Pinger;
/**
 * This method is used to insert any registry keys that are applicable for 
 * the COM Server to operate.  This overrides the sample.ntservice.NTService
 * method for inserting registrykeys.
 * 
 *  HKEY_CLASSES_ROOT\CLSID\<GUID>\
 * 
 *  HKEY_CLASSES_ROOT\AppID\<GUID>\
 *                                     
 *  
 * @param String eventPath - full pathname to the exe or dll which contains the 
 *          message resource required for formating event log messages.
 * 
 * @param eventTypes - value used for the TypesSupported value.
 * 
 * @param String categoryPath - full pathname to the exe or dll which contains the
 *          category resource information.  Typically this would be the same as 
 *          the event message file.
 * 
 * @param int catCount - the number of catagories defined for this event log.  
 *          These categories MUST be number from 0 to this number -1 in the
 *          message file.
 * 
 * @exception com.ms.lang.RegKeyException - thrown if there was a problem dealing
 *          with these keys in the registry.
 */
    protected void insertRegistryKeys(String eventPath,
                                      int eventTypes,
                                      String categoryPath,
                                      int catCount) throws com.ms.lang.RegKeyException{
        try{
        System.out.println("Adding clsid");
        com.ms.lang.RegKey myKey = new com.ms.lang.RegKey(
                            com.ms.lang.RegKey.CLASSES_ROOT,
                            "CLSID\\"+clsid.toString(),
                            com.ms.lang.RegKey.KEYOPEN_CREATE);
        System.out.println(" Key is "+myKey);
            myKey.setValue("",this.getServiceDisplayName());
            myKey.setValue("AppId",clsid.toString());
        }catch(Exception i){
            System.out.println("Exception adding CLSID "+i);   
        }
        try{
        System.out.println("adding appid");
            com.ms.lang.RegKey myKey = new com.ms.lang.RegKey(
                            com.ms.lang.RegKey.CLASSES_ROOT,
                            "AppId\\"+clsid.toString(),
                            com.ms.lang.RegKey.KEYOPEN_CREATE);
            myKey.setValue("",this.getServiceDisplayName());
            myKey.setValue("LocalService",this.getServiceName());
        }catch(Exception i){
            System.out.println("Exception adding AppId "+i);
        }
        super.insertRegistryKeys(eventPath,eventTypes,categoryPath,catCount);
    }
/**
 * Removes all registry keys that deal with the eventlog.  These are under:
 * 
 *  HKEY_CLASSES_ROOT\CLSID\<GUID>\
 * 
 *  HKEY_CLASSES_ROOT\AppID\<GUID>\
 *                                     
 * 
 * @exception com.ms.lang.RegKeyException - thrown if there was a problem dealing
 *          with these keys in the registry.
 */
    protected void removeRegistryKeys()throws com.ms.lang.RegKeyException{
        try{   
            System.out.println("Removing clsid");
        com.ms.lang.RegKey myKey = new com.ms.lang.RegKey(
                            com.ms.lang.RegKey.CLASSES_ROOT,
                            "CLSID\\",
                            com.ms.lang.RegKey.KEYOPEN_WRITE);
            myKey.deleteSubKey(clsid.toString());
        }catch(Exception i){
            System.out.println("Exception removing CLSID "+i);
        }
        try{
            System.out.println("Removing AppID");
           com.ms.lang.RegKey myKey = new com.ms.lang.RegKey(
                            com.ms.lang.RegKey.CLASSES_ROOT,
                            "AppId\\",
                            com.ms.lang.RegKey.KEYOPEN_WRITE);
            myKey.deleteSubKey(clsid.toString());
        }catch(Exception i){
            System.out.println("Exception removing AppId "+i);
        }
            super.removeRegistryKeys();
    }
/**
 * This performs the 'Pause' operation by calling suspend on our COMServer.
 */
    protected void servicePause(){
        if (object!=null)
            object.suspend();
    }
/**
 * This performs the 'Continue' operation by calling resume on our COMServer.
 */
    protected void serviceContinue(){
        if (object!=null)
            object.resume();
    }
/**
 * Shuts down our COMServer, by calling its shutdown method, and stopping it
 * (as it is a Thread).  An entry to the eventlog is written specifying that a 
 * shutdown is about to occur, and when it has completed.
 */
    protected void serviceStop(){
        this.logEvent(null,new String[]{"Stopping the COM Server"},
                          (short)com.ms.win32.win.EVENTLOG_INFORMATION_TYPE,
                          null,2000,2);
            object.shutdown();// Perform any COM cleanup required
            object.stop();
            this.logEvent(null,new String[]{"Stop completed successfully."},
                          (short)com.ms.win32.win.EVENTLOG_INFORMATION_TYPE,
                          null,2000,1);
    }
/**
 * This method starts the actual COM Server that will be running as our
 * NT Service.  It simply creates an instance of COMServer, specifying
 * sample.server.Pinger.class as the COM OBject, our clsid as its CLSID, 
 * and that it should be a singleton in an STA.
 * 
 * @param int dwArgc - This is the number of command line arguments that the
 *          NT Service has, which were specified when this service was started.
 * 
 * @param int spszArgv - This is the C form of the actual command line 
 *          parameters.
 */
    protected void serviceStart(int dwArgc, int spszArgv){
            object = new COMServer(Pinger.class,clsid,true,false,true);
            this.logEvent(null,new String[]{"About to start service!"},
                          (short)com.ms.win32.win.EVENTLOG_INFORMATION_TYPE,
                          null,2000,2);
            com.ms.com.ComLib.startMTAThread(object);
            this.logEvent(null,new String[]{"Service Started!"},
                          (short)com.ms.win32.win.EVENTLOG_INFORMATION_TYPE,
                          null,2000,1);
    }
/**
 * This will specify our service as an interactive process.
 * 
 * @return int - flag specifying this service as an interactive process.
 */
    protected int getProcessType(){
        return super.getProcessType()| com.ms.win32.win.SERVICE_INTERACTIVE_PROCESS;
    }
/**
 * Specifies that starting our program will take 3 seconds, and if we don't start 
 * within that time - something went wrong!.
 * 
 * @return int - represents 3 milliseconds.
 */    
    protected int getServiceStartTimeout(){
        return 3000;
    }
/**
 * Specifies that stopping our program will take 3 seconds, and if we don't stop 
 * within that time - something went wrong!
 * 
 * @return int - represents 3 milliseconds.
 */
    protected int getServiceStopTimeout(){
        return 3000;
    }
/**
 * String which represents our service name.
 * 
 * @return String - our service name.
 */
    protected String getServiceName(){
        return "JavaService";
    }
/**
 * String that will be displayed in the service manager, which will describe our
 * service.
 * 
 * @return String - description of our service.
 */
    protected String getServiceDisplayName(){
        return "A Java Service!";
    }    
/**
 * This is the command line processor for our NT Service.  This needs to handle
 * the installation and registration of our service, its removal as well as 
 * when the SCM attempts to start our service.  Conditional compilation statements
 * will determine whether the actual service .exe contains the message resources, 
 * or if they are stored in a seperate .dll file.  
 * 
 * NOTE : the .dll file is ASSUMED to be in the same directory as the .exe file.
 * 
 * @param String[] args - These are the command line arguments 
 */
    protected void init(String[] args)throws Exception{
        if (args.length==1){
            if (args[0].equals("-install")){
                try{
                    #if MAKEDLL
                        // 
                        // This will register the DLL as the EventLog 
                        // Message file.
                        System.out.println("Building the DLL version");
                        String modName = this.getModuleFileName();
                        modName=modName.substring(0,modName.indexOf(".exe"))+".dll";
                    #else
                        //
                        // This will register the EXE as the Eventlog
                        // Message file.
                        System.out.println("Building the EXE version");
                        String modName = this.getModuleFileName();
                    #endif
                    System.out.println("Message file is "+modName);
                    this.insertRegistryKeys(modName,3,modName,3);
                }catch(com.ms.lang.RegKeyException ii){
                    System.out.println(" Error adding reg keys "+ii);
                }
                register();
            }else if (args[0].equals("-remove")){
                try{
                this.removeRegistryKeys();
                }catch(com.ms.lang.RegKeyException ii){
                    System.out.println("Problem dealing with the registry "+ii);
                }
                remove();
            }else if (args[0].equals("-debug")){
                CmdDebugService(args);
            }
        }else{
            super.init(args);
        }
     }
/**
 * This is a simple main method that starts our service, and calls the init method.
 * 
 * @param String[] args - Any command line arguments that were specified.
 */ 
    public static void main(String args[]){
        DCOMService dcom = new DCOMService();
        try{
           dcom.init(args);
        }catch(Exception ii){
            dcom.logEvent(null,new String[]{"Exception initializing service : "+ii},
                          (short)com.ms.win32.win.EVENTLOG_ERROR_TYPE,
                          null,2000,3);
            System.out.println("Exception initializing service :"+ii);
        }
      }
}