/*
 * sample.safevar.Client.java
 *
 * (C) Copyright 1995 - 1999 Microsoft Corporation.  All rights reserved.
 *
 */
package sample.safevar;

import sample.safevariant.*;
import com.ms.com.ComLib;
import com.ms.com.Variant;
import com.ms.com.SafeArray;
import com.ms.com.Dispatch;
/**
 * This sample illustrates the use of SafeArray and variant for communicating
 * data between a Java implemented COM client to a VB implemented COM object.
 * 
 * @version 1.0
 */
public class Client{
	/**
	 * This is the VB implemented COM object instance, created through the Java 
	 * COM wrapper generated from the Jactivex tool on the COM dll.
	 */	
	private _VBCOMObject comObject =(_VBCOMObject)new VBCOMObject();
/**
 * This method calls each of the test methods in the VB COM object.
 */
protected void go(){
	System.out.println("About to start calling methods on the COM object.");
	processStringByRef();
	processStringByVal();
	processObjectByRef();
	processObjectByRefNull();
	processObjectByVal();
	processStringArray();
	processObjectArray();
	System.out.println("Done.");
}
/**
 * This will test the processStringByRef Function
 */
private void processStringByRef(){
	System.out.println("---------------------------------------------------------");
	Variant param = new Variant("test");
	System.out.println("\nAbout to call processStringByRef with param ["+param+"]");
	Variant psr = comObject.processStringByRef(param);
	System.out.println("processStringByRef param is ["+param+"] result is ["+
					   psr+"]");
}
/**
 * This will test the processStringByVal Function
 */
private void processStringByVal(){
	System.out.println("---------------------------------------------------------");
	Variant param2 = new Variant("test");
	System.out.println("\nAbout to call processStringByVal with param ["+param2+"]");
	Variant result  = comObject.processStringByVal(param2);
	System.out.println("processStringByVal param is ["+param2+
					   "] result is ["+result+"]");
}
/**
 * This will test the processObjectByRef Function by passing a NULL Variant
 */
private void processObjectByRefNull(){	
	System.out.println("---------------------------------------------------------");
	Variant param3 = new Variant();
	param3.changeType(Variant.VariantNull);
	System.out.println("\nAbout to call processObjectByRef with param ["+param3+"]");
	Variant psr3 = comObject.processObjectByRef(param3);
	System.out.println("processObjectByRef param is ["+param3+"] result is ["+
					   psr3+"]");
}
/**
 * This will test the processObjectByRef Function by passing a Hashtable to the 
 * function.
 */
private void processObjectByRef(){	
	System.out.println("---------------------------------------------------------");
	Variant param3 = new Variant(new java.util.Hashtable());
	System.out.println("\nAbout to call processObjectByRef with param ["+param3+"]");
	Variant psr3 = comObject.processObjectByRef(param3);
	System.out.println("processObjectByRef param is ["+param3+"] result is ["+
					   psr3+"]");
}

/**
 * This will test the processObjectByVal Function that gives the VB COM object a 
 * java Vector containing other Java Objects.
 */
private void processObjectByVal(){
	System.out.println("---------------------------------------------------------");
	java.util.Vector vec = new java.util.Vector();
	vec.addElement(new java.util.Date());
	vec.addElement(new String("String in a Vector!"));
	Variant param4 = new Variant(vec);
	System.out.println("\nAbout to call processObjectByVal with param ["+param4+"]");
	Variant psr4 = comObject.processObjectByVal(param4);
	System.out.println("processObjectByVal param is ["+param4+"] result is ["+
					   psr4+"]");
}
/**
 * This will test the processObjectArray Function by passing a SafeArray of Variant
 * wrapped java objects.
 *
 * NOTE: the type for all of the SafeArray contents MUST be Variant.VariantVariant
 *		or VB will complain that it is receiving an unknown automation type.
 */
private void processObjectArray(){
	System.out.println("---------------------------------------------------------");
	SafeArray param6 = new SafeArray(Variant.VariantVariant,4);
	param6.setVariant(0,new Variant(new java.util.Vector()));
	param6.setVariant(1,new Variant(new java.util.Hashtable()));
	param6.setVariant(2,new Variant(new java.util.Date()));
	param6.setVariant(3,new Variant(new Boolean(true)));
	System.out.println("\nAbout to call processObjectArray with param :\n");
	for (int i=param6.getLBound();i<param6.getUBound();i++)
		System.out.println("index ["+i+"] value ["+param6.getVariant(i)+"]");
	Variant psr6 = comObject.processObjectArray(param6);
	System.out.println("\nContents of parameter after call :\n");
	for (int i=param6.getLBound();i<param6.getUBound();i++)
		System.out.println("index ["+i+"] value ["+param6.getVariant(i)+"]");
	System.out.println("\nResult {type="+psr6.toSafeArray().getvt()+"} :\n");
	for (int i=psr6.toSafeArray().getLBound();i<psr6.toSafeArray().getUBound();i++)
		System.out.println("index ["+i+"] value ["+
						   psr6.toSafeArray().getVariant(i)+"]");
}
/**
 * This will test the processStringArray Function by passing a SafeArray of Strings, 
 * and receiving a Safearray of integers.
 * 
 * NOTE: the type for all of the SafeArray contents MUST be Variant.VariantVariant
 *		or VB will complain that it is receiving an unknown automation type.
 */
private void processStringArray(){
	System.out.println("---------------------------------------------------------");
	SafeArray param7 = new SafeArray(Variant.VariantVariant,4);
	param7.fromStringArray(new String[]{"start",
									 "of",
									 "the",
									 "String",
									 "Array"});
	System.out.println("\nAbout to call processStringArray with param :\n");
	for (int i=param7.getLBound();i<param7.getUBound();i++)
		System.out.println("index ["+i+"] value ["+param7.getVariant(i)+"]");
	Variant psr7 = comObject.processStringArray(param7);
	System.out.println("\nContents of parameter after call :\n");
	for (int i=param7.getLBound();i<param7.getUBound();i++)
		System.out.println("index ["+i+"] value ["+param7.getVariant(i)+"]");
	System.out.println("\nResult {type="+psr7.toSafeArray().getvt()+"} :\n");
	for (int i=psr7.toSafeArray().getLBound();i<psr7.toSafeArray().getUBound();i++)
		System.out.println("index ["+i+"] value ["+
						   psr7.toSafeArray().getVariant(i)+"]");
}
/**
 * Main method for running this client from the command line.  It simply creates an
 * instance and calls the go method.
 */	
public static void main(String args[]){
	com.ms.com.ComLib.declareMessagePumpThread();
	new Client().go();
}
}