//
//  client.cpp
//
//  Copyright (c) 1999, Microsoft Corporation.  All rights reserved.
//
//  This source files demonstrates activating the Microsoft VM through the Java
//  moniker and using interface methods that use iid_is and size_is attributes.
//

#include <stdio.h>
#include <windows.h>
#include "sample.h"
#include "sample_i.c"

#define ARRAYSIZE(a)        (sizeof(a)/sizeof((a)[0]))

//
//  BindToObject
//
//  Binds to the supplied display name using monikers.  Equivalent to the
//  CoGetObject API added in later versions of COM.
//

HRESULT BindToObject(LPOLESTR pwszDisplayName, REFIID riid, LPVOID *ppunk)
{
    HRESULT hr;
    IBindCtx *pbc;
    ULONG chEaten;
    IMoniker *pmk;

    *ppunk = NULL;

    hr = CreateBindCtx(0, &pbc);

    if (hr == S_OK) {

        hr = MkParseDisplayName(pbc, pwszDisplayName, &chEaten, &pmk);

        if (hr == S_OK) {
            hr = pmk->BindToObject(pbc, NULL, riid, ppunk);
            pmk->Release();
        }

        pbc->Release();
    }

    return hr;
}

void main(int argc, char *argv[])
{
    HRESULT hr;
    ISampleInterface *psample;
    IUnknown *punk;
    int aiElements[10];
    BSTR abstrElements[5];
    int index;

    hr = CoInitialize(NULL);

    if (hr == S_OK) {

        hr = BindToObject(L"java:SampleObject", IID_ISampleInterface, (LPVOID *)
            &psample);

        if (hr == S_OK) {

            //  Attempt to bind to the IID_IDispatch interface.  This should
            //  succeed since the Java side returns an object that has an auto-
            //  dispatch interface.
            hr = psample->BindToObject(IID_IDispatch, &punk);

            if (hr == S_OK) {
                printf("successfully bound to IID_IDispatch\n");
                punk->Release();
            } else {
                printf("failed to bind to IID_IDispatch (hr=%08x)\n", hr);
            }

            //  Attempt to bind to the IID_IDispatch interface.  This should
            //  fail since the Java side returns an object that doesn't support
            //  the interface.
            hr = psample->BindToObject(IID_IErrorInfo, &punk);

            if (hr == S_OK) {
                printf("successfully bound to IID_IErrorInfo\n");
                punk->Release();
            } else {
                printf("failed to bind to IID_IErrorInfo (hr=%08x)\n", hr);
            }

            for (index = 0; index < ARRAYSIZE(aiElements); index++) {
                aiElements[index] = index * index;
            }

            printf("before FlipIntArray call: ");
            for (index = 0; index < ARRAYSIZE(aiElements); index++) {
                printf("%d  ", aiElements[index]);
            }
            printf("\n");

            //  Call a Java method to flip the sign of the supplied integers.
            hr = psample->FlipIntArray(ARRAYSIZE(aiElements), aiElements);

            if (hr == S_OK) {
                printf("after FlipIntArray call: ");
                for (index = 0; index < ARRAYSIZE(aiElements); index++) {
                    printf("%d  ", aiElements[index]);
                }
                printf("\n");
            } else {
                printf("failed call to FlipIntArray (hr=%08x)\n", hr);
            }

            //  Call a Java method to fill the supplied array with BSTRs.
            hr = psample->FillBSTRArray(ARRAYSIZE(abstrElements), abstrElements);

            if (hr == S_OK) {
                printf("after FillBSTRArray call: ");
                for (index = 0; index < ARRAYSIZE(abstrElements); index++) {
                    if (abstrElements[index] != NULL) {
                        printf("%ws ", abstrElements[index]);
                        SysFreeString(abstrElements[index]);
                        abstrElements[index] = NULL;
                    }
                }
                printf("\n");
            } else {
                printf("failed call to FillBSTRArray (hr=%08x)\n", hr);
            }

            psample->Release();

        } else {
            printf("failed to bind to java object (hr=%08x)\n", hr);
        }

        CoUninitialize();
    }
}
