import com.ms.wfc.app.*;
import com.ms.wfc.core.*;
import com.ms.wfc.ui.*;
import com.ms.wfc.html.*;
import com.ms.wfc.io.*;

/**
 * This class can take a variable number of parameters on the command
 * line. Program execution begins with the main() method. The class
 * constructor is not invoked unless an object of type 'formMediaPlayer' is
 * created in the main() method.
 */
public class formMediaPlayer extends Form
{
	int			currentMediaIndex		= 5;	// Currently selected Media (or the filter from the Open FileDialog)
	int			prevMediaIndex			= -1;   // Previously selected Media 
	Form		formAVI;						// To refer to form displaying AVI
	Form		formHelp;						// To refer to form displaying help file
	int			currentTrack			= 1;	// Current track on cd audio
	int			totalTracks;					// Total tracks on CD audio
	String		mediaFileName;					// Name of Media file selected
	boolean		firstTime				= true; // Flag used while getting total tracks on cd audio
	boolean		helpDisplayed			= false;// Flag to indicate if help is diplayed 
	
	// Friendly names for wav, midi, cd audio, avi files. Correspond to possible filters in the 
	// OpenFileDialog box
	public static final int MEDIA_SOUND			    = 1;  
	public static final int MEDIA_MIDI_SEQUENCER    = 2;
	public static final int MEDIA_CD_AUDIO		    = 3;
	public static final int MEDIA_VIDEO_FOR_WINDOWS = 4;
	public static final int MEDIA_ALL				= 5;
	
	public static final int NUM_DRIVES				= 26;
		
	static MCI_OPEN_PARMS	mciOpenParms	= new MCI_OPEN_PARMS();	
	static int				wDeviceID		= 0;					// Device ID of the device being played
    static boolean			MediaPlaying	= false;				// Flags to indicate whether Media is  
	static boolean			MediaPaused		= false;				// playing/paused or not
	
	public formMediaPlayer()
	{
		// Required for Visual J++ Form Designer support
		initForm();	

		// TODO: Add any constructor code after initForm call
		
		this.setStartPosition(0); // Manual
				
		// Disable menu item Close and also the buttons. Enable only when media is selected.
		menuClose.setEnabled(false);
		btnPlay.setEnabled(false);
		btnStop.setEnabled(false);
		btnSkip.setEnabled(false);
				
		// Set Media Player display to approx. centre of screen
		Rectangle rect = Screen.getBounds(this);
		int centreX = (rect.width - rect.x) / 2;
		int fractionX = Math.round((float) (0.7 * (centreX - rect.x)));
		int centreY = (rect.height - rect.y) / 2;
		int fractionY = Math.round((float) (0.6 * (centreY - rect.y))); 
		this.setLeft(rect.x + fractionX) ;
		this.setTop(rect.y + fractionY);
		
		// Initialize OpenFileDialog attributes. Add filters (*.wav,*.avi,etc)
		openFileDlgOpen.reset();
		openFileDlgOpen.setInitialDir(System.getProperty("com.ms.windir") + "\\MEDIA\\");
		openFileDlgOpen.setFilter("Sound (*.wav)|*.wav|MIDI Sequencer (*.rmi;*.mid)|*.rmi;*.mid|CD Audio (*.cda)|*.cda|Video for Windows (*.avi)|*.avi|All files (*.*)|*.*");
		openFileDlgOpen.addOnFileOk(new CancelEventHandler(this.formMediaPlayer_fileOK));
		
		// Initialize new form for siplaying AVI. Place it below the MediaPlayer display
		formAVI = new Form();
		formAVI.setVisible(false);
	}
	
		
	/**
	 * formMediaPlayer overrides dispose so it can clean up the
	 * component list.
	 */
	public void dispose()
	{
		super.dispose();
		components.dispose();
	}

	// Play button click event handler 
	private void btnPlay_click(Object source, Event e)
	{
		int returnVal = 0;
		
		timerMediaPlayed.setEnabled(true);
		
		// Alternate display between "Play" and "Pause". Call appropriate routines.
		if ("Play".equals(btnPlay.getText()))
		{
			if (MediaPaused == false){
				// Play Media
				returnVal = PlayMedia(currentMediaIndex,mediaFileName,currentTrack,formAVI,this.getLeft(),this.getTop()+this.getHeight());
			}
			 else
				// Resume paused Media 
			 	ResumeMedia(currentMediaIndex); 
			 if (returnVal >= 0 )
			 {
			    labelStatus.setText(File.getName(mediaFileName) + " (Playing)");
				btnPlay.setText("Pause");
				btnStop.setEnabled(true);
				
				// Enable skip button only when media selected is cd audio
				if (currentMediaIndex == MEDIA_CD_AUDIO) 
					btnSkip.setEnabled(true);
				else
					btnSkip.setEnabled(false);
			 }
		}
		else
		{
			// Set Media Player caption to the media file selected
			labelStatus.setText(File.getName(mediaFileName) + " (Paused)");		 
			PauseMedia(currentMediaIndex);
			btnPlay.setText("Play");
			btnStop.setEnabled(false);
			
			// Enable skip button only when media is cd audio
			if (currentMediaIndex == MEDIA_CD_AUDIO) 
				 btnSkip.setEnabled(true);
			 else
				 btnSkip.setEnabled(false);
		}
	}

	// Stop button click event handler
	private void btnStop_click(Object source, Event e)
	{
		// Stop playing media. Restore buttons to initial state.
		timerMediaPlayed.setEnabled(false);		
		btnPlay.setText("Play");
		btnStop.setEnabled(false);
		labelStatus.setText(File.getName(mediaFileName) + " (Stopped)");		
		StopMedia(currentMediaIndex);
		
	}

	// Skip button click event handler. Skip button used for CD Audio to skip tracks
	private void buttonSkip_click(Object source, Event e)
	{
		int returnVal;
		
		
		if (firstTime) 
		{
			// Get the current track
			currentTrack = GetCurrentTrack();
			firstTime = false;
			// Get total tracks
			totalTracks = GetTotalTracks();
		}
				
		// If current track equals total track , skip to track 1 else increment track
		if (currentTrack == totalTracks)
			currentTrack = 1;
		else
			currentTrack = currentTrack + 1;
		
		// Display current track 
		labelTrackDisplay.setText(" " + currentTrack + " ");
		mediaFileName = "Track " + currentTrack;
		
		// Display current track on MediaPlayer caption
		labelStatus.setText(File.getName(mediaFileName) + " (Stopped)");
		StopMedia(currentMediaIndex);
		if ("Pause".equals(btnPlay.getText())) 
		{
			// Start Playing next track
			returnVal = PlayMedia(currentMediaIndex,mediaFileName,currentTrack,null,0,0);
			// Set MediaPlayer caption to indicate media has started playing
			labelStatus.setText(File.getName(mediaFileName) + " (Playing)");
		}
		
	}
		
	// Open menu click event handler
	private void menuOpen_click(Object source, Event e)
	{
		// Set filter to current media (default = MEDIA_ALL)
		openFileDlgOpen.setFilterIndex(currentMediaIndex);
		openFileDlgOpen.showDialog();
	}

	
	private void menuClose_click(Object source, Event e)
	{
		// If Media Playing then stop it.
		if (MediaPlaying) 
		{
			StopMedia(currentMediaIndex);	
			btnPlay.setText("Play");
			MediaPlaying = false;
		}
		labelStatus.setText(" ");
		
		// Disable everything
		timerMediaPlayed.setEnabled(false);
		
		// Disable all buttons, close menu and formAVI.
		menuClose.setEnabled(false);
		btnPlay.setEnabled(false);
		btnStop.setEnabled(false);
		btnSkip.setEnabled(false);
		
		labelTrackDisplay.setText("");
		formAVI.setVisible(false);
		openFileDlgOpen.setFileName("");		
		uncheckMedia(currentMediaIndex);
			
	}

	// Exit menu click event handler
	private void menuExit_click(Object source, Event e)
	{
		ExitMedia(currentMediaIndex);
		
		// Exit application
		Application.exit();
	}
	
	// Sound menu click event handler
	private void menuSound_click(Object source, Event e)
	{
		// Open dialog box with filter set to (*.wav)		
		openFileDlgOpen.setFilterIndex(1);
		openFileDlgOpen.showDialog();
	}
	
	// MIDI menu click event handler
	private void menuMIDI_click(Object source, Event e)
	{
		// Open dialog box with filter set to (*.rmi;*.mid)		
		openFileDlgOpen.setFilterIndex(2);
		openFileDlgOpen.showDialog();
	}

	// CD Audio menu click event handler
	private void menuCDAudio_click(Object source, Event e)
	{
		
		
		// Do not pop up an open file dialog in this case
		menuCDAudio.setChecked(true);
		menuClose.setEnabled(true);
		btnPlay.setEnabled(true);
		btnSkip.setEnabled(true);
		formAVI.setVisible(false);
		prevMediaIndex = currentMediaIndex;
		currentMediaIndex = MEDIA_CD_AUDIO;
		checkCurrentMedia();
		if ((prevMediaIndex != -1) && (currentMediaIndex != prevMediaIndex)) uncheckMedia(prevMediaIndex);
		
		// Pop up an error
		if ((wDeviceID = FindDevice()) < 0) {
             MessageBox.show("Error: CD-ROM device is either not installed or is being used by another application.","Error Message",MessageBox.OKCANCEL);
			Application.exit();
        }
        CloseDevice();
		labelTrackDisplay.setText(" " + currentTrack + " ");
		labelStatus.setText("Track " + currentTrack + " (Stopped)");
		mediaFileName = "Track " + currentTrack;
	}
	
	// Video for Windows menu click event handler
	private void menuVideoForWindows_click(Object source, Event e)
	{
		// Open dialog box with filter set to (*.avi)
		openFileDlgOpen.setFilterIndex(4);
		openFileDlgOpen.showDialog();
	}

	
	// OpenFileDialog OK button click event handler
	void formMediaPlayer_fileOK(Object sender, CancelEvent e)
	{
		String fn;
		String ext;
			
		// Make sure to stop currently playing media before opening another one
		if (MediaPlaying)
		{
			btnPlay.setText("Play");
			btnStop.setEnabled(false);
			btnSkip.setEnabled(false);
			labelStatus.setText(File.getName(mediaFileName) + " (Stopped)");		
			StopMedia(currentMediaIndex);
		}	
		
		mediaFileName = openFileDlgOpen.getFileName();
		openFileDlgOpen.setFileName("");
		
		
		menuClose.setEnabled(true);
		btnPlay.setEnabled(true);
		
		formAVI.setVisible(false);
		
		prevMediaIndex = currentMediaIndex;
		
		// Make current media as whatever has been selected
		if (openFileDlgOpen.getFilterIndex() == MEDIA_ALL)
		{
			fn = mediaFileName;
			ext = fn.substring(fn.length() - 3);
			if  (ext.equals("wav") || ext.equals("WAV")) currentMediaIndex = MEDIA_SOUND;
			else
			if  (ext.equals("rmi") || ext.equals("RMI") || ext.equals("mid") || ext.equals("MID")) currentMediaIndex = MEDIA_MIDI_SEQUENCER;
			else
			if  (ext.equals("avi") || ext.equals("AVI")) currentMediaIndex = MEDIA_VIDEO_FOR_WINDOWS;
			else 
			if  (ext.equals("cda") || ext.equals("CDA")) currentMediaIndex = MEDIA_CD_AUDIO;
		}
		else
		{
			fn = mediaFileName;
			ext = fn.substring(fn.length() - 3);
			int tmpMediaIndex = -1;
			if  (ext.equals("wav") || ext.equals("WAV")) tmpMediaIndex = MEDIA_SOUND;
			else
			if  (ext.equals("rmi") || ext.equals("RMI") || ext.equals("mid") || ext.equals("MID")) tmpMediaIndex = MEDIA_MIDI_SEQUENCER;
			else
			if  (ext.equals("avi") || ext.equals("AVI")) tmpMediaIndex = MEDIA_VIDEO_FOR_WINDOWS;
			else 
			if  (ext.equals("cda") || ext.equals("CDA")) tmpMediaIndex = MEDIA_CD_AUDIO;
			currentMediaIndex = openFileDlgOpen.getFilterIndex();
			if (tmpMediaIndex != currentMediaIndex)
			{
				currentMediaIndex = tmpMediaIndex;
			}
		}
		
		// Enable skip button when media selected is cd audio
		if (currentMediaIndex == MEDIA_CD_AUDIO) 
		{
			btnSkip.setEnabled(true);
		}
		else
		{
			btnSkip.setEnabled(false);
			labelTrackDisplay.setText("");
		}	
		
		// Check the currently selected media menu item
		checkCurrentMedia();
		if ((prevMediaIndex != -1) && (currentMediaIndex != prevMediaIndex)) uncheckMedia(prevMediaIndex);
		
		if (currentMediaIndex == MEDIA_CD_AUDIO)
		{
			if ((wDeviceID = FindDevice()) < 0) {
				MessageBox.show("Error: CD-ROM device is either not installed or is being used by another application.","Error Message",MessageBox.OKCANCEL);
				Application.exit();
			}
			CloseDevice(); 
			
			// update label
			labelTrackDisplay.setText(" " + currentTrack + " ");	
			mediaFileName = "Track " + currentTrack;
		}
		labelStatus.setText(File.getName(mediaFileName) + " (Stopped)"); 
		
	}
	
	
	// To check currently selected media menu item	
	private void checkCurrentMedia()
	{
		switch (currentMediaIndex)
		{
			case MEDIA_SOUND			:	menuSound.setChecked(true);
											break;
								
			case MEDIA_MIDI_SEQUENCER	:	menuMIDI.setChecked(true);
											break;
								
			case MEDIA_CD_AUDIO			:	menuCDAudio.setChecked(true);
											break;	
											
			case MEDIA_VIDEO_FOR_WINDOWS:	menuVideoForWindows.setChecked(true);
											break;
											
			default						:	break;
		}
	}
	
	// To uncheck a media menu item
	private void uncheckMedia(int mediaIndex)
	{
		switch (mediaIndex)
		{
			case MEDIA_SOUND			:	menuSound.setChecked(false);
											break;
								
			case MEDIA_MIDI_SEQUENCER	:	menuMIDI.setChecked(false);
											break;
								
			case MEDIA_CD_AUDIO			:	menuCDAudio.setChecked(false);
											break;	
											
			case MEDIA_VIDEO_FOR_WINDOWS:	menuVideoForWindows.setChecked(false);
											break;
											
			default						:	break;
		}
	}

	
	// Timer used to find out when the media finished playing
	private void timerMediaPlayed_timer(Object source, Event e)
	{
		int mciError;
		MCI_STATUS_PARMS mciStatusParms = new MCI_STATUS_PARMS();
		int length, position;
		
		if (MediaPlaying) {
			
			if (currentMediaIndex == MEDIA_CD_AUDIO)
			{
				int tmpTrack;
				
				// Get the current track on cd audio
				tmpTrack = GetCurrentTrack();
				
				if (tmpTrack != currentTrack) {
					mediaFileName = "Track " + tmpTrack;
					
					// The last track on CD is being played, so stop when over
					if (currentTrack == totalTracks)
					{
						timerMediaPlayed.setEnabled(false);
						currentTrack = 1;	
						btnPlay.setText("Play");
						btnStop.setEnabled(false);
						labelStatus.setText(File.getName(mediaFileName) + " (Stopped)");
						labelTrackDisplay.setText(" " + currentTrack + " ");
						StopMedia(currentMediaIndex);
					}
					else
					{
						currentTrack = tmpTrack;
						labelStatus.setText(File.getName(mediaFileName) + " (Playing)");
						labelTrackDisplay.setText(" " + tmpTrack + " ");
					}
				}
			}
			else
			{
			
			// Find out length of media being played
			mciStatusParms.dwItem = MCI_STATUS_LENGTH;
			mciError = mciSendCommand(wDeviceID, MCI_STATUS,MCI_STATUS_ITEM, mciStatusParms);
			if (mciError != 0) {
				printMCIError(mciError);
				CloseDevice();
			}
			length = mciStatusParms.dwReturn;
			
			// Find out current position of the media being played
			mciStatusParms.dwItem = MCI_STATUS_POSITION;
			mciError = mciSendCommand(wDeviceID, MCI_STATUS,MCI_STATUS_ITEM, mciStatusParms);
			if (mciError != 0) {
				printMCIError(mciError);
				CloseDevice();
			}
			position = mciStatusParms.dwReturn;
			
			// Media has finished playing
			if (position == length)
			{
				timerMediaPlayed.setEnabled(false);
				btnPlay.setText("Play");
				btnStop.setEnabled(false);
				btnSkip.setEnabled(false);
				labelStatus.setText(File.getName(mediaFileName) + " (Stopped)");		
				StopMedia(currentMediaIndex);
			}
			}
			}
	}
	
	void aboutForm_close(Object sender,CancelEvent e)
	{
		// Enable other forms on closing formAbout
		this.setEnabled(true);	
		formAVI.setEnabled(true);
	}

	// The About Media Player click event handler
	private void menuAbout_click(Object source, Event e)
	{
		Form formAbout;
		Label labelAbout;
		
		// Create a form with a label within the form
		// Display information in the label
		
		formAbout = new Form();
		formAbout.addOnClosing(new CancelEventHandler(this.aboutForm_close));
		formAbout.setText("About Media Player");
		formAbout.setStartPosition(0);		// Manual
		formAbout.setLocation(this.getLeft()+5,this.getTop()+5);
		formAbout.setSize(this.getWidth()-10,this.getHeight()-20);
		formAbout.setMaximizeBox(false);
		formAbout.setMinimizeBox(false);
		formAbout.setBorderStyle(this.getBorderStyle());
		labelAbout = new Label();
		labelAbout.setAutoSize(false);
		labelAbout.setBorderStyle(2);
		labelAbout.setLocation(labelAbout.getLeft()+5,labelAbout.getTop()+5);
		Rectangle r = formAbout.getBounds();
		labelAbout.setSize(r.width-20,r.height-40);
		labelAbout.setBackColor(Color.LIGHTGRAY);
		labelAbout.setForeColor(Color.BLACK);
		labelAbout.setTextAlign(2);
		labelAbout.setText("\r\nMicrosoft (R) Media Player\r\n");
		labelAbout.setText(labelAbout.getText() + "Version 1.0\r\n");
		labelAbout.setText(labelAbout.getText() + "Copyright (C) Microsoft Corp.");
		
		formAbout.add(labelAbout);
		labelAbout.setVisible(true);
		formAbout.setVisible(true);
		
		// Disable other forms until formAbout is closed 
		this.setEnabled(false);
		formAVI.setEnabled(false);
	}

	
	void helpForm_close(Object sender,CancelEvent e)
	{
		helpDisplayed = false;
	}
	
	
	// Help information
	private void menuHelpTopics_click(Object source, Event e)
	{
		Label labelHelp;
		
		// Create a form with a label within the form
		// Display information in the label
		if (helpDisplayed == false) {
			formHelp = new Form();
			formHelp.addOnClosing(new CancelEventHandler(this.helpForm_close));
			formHelp.setText("Help");
			formHelp.setStartPosition(0);		// Manual
			formHelp.setLocation(this.getLeft()-50,this.getTop());
			formHelp.setSize(this.getWidth()+200,this.getHeight()+250);
			formHelp.setMaximizeBox(false);
			formHelp.setMinimizeBox(false);
			formHelp.setBorderStyle(this.getBorderStyle());
			labelHelp = new Label();
			labelHelp.setAutoSize(false);
			labelHelp.setBorderStyle(2);
			labelHelp.setLocation(labelHelp.getLeft()+5,labelHelp.getTop()+5);
			Rectangle r = formHelp.getBounds();
			labelHelp.setSize(r.width-20,r.height-40);
			labelHelp.setBackColor(Color.LIGHTGRAY);
			labelHelp.setForeColor(Color.BLACK);
			labelHelp.setTextAlign(0);
			labelHelp.setText("MediaPlayer\r\n\r\nThe MediaPlayer is a classic example provided to play different kinds of media viz. Waveform (.WAV) files, MIDI files (.MID,.RMI), CD Audio (.CDA) and Video for Windows (.AVI) files.");
			labelHelp.setText(labelHelp.getText() + "\r\n\r\nTo play .wav,.mid,.rmi,.avi files : \r\n --  Click on Device \r\n --  Click on the required media type \r\n --  Select a file from the Dialog Box and click OK \r\n --  Click on Play button \r\n --  To pause, click on Pause button \r\n --  To stop, click on Stop button");
			labelHelp.setText(labelHelp.getText() + "\r\n\r\nTo play a track from CD : \r\n --  Click on Device \r\n --  Click on CD Audio \r\n --  Click on Play button. The media starts playing from track 1 \r\n --  To pause, click on Pause button \r\n --  To stop, click on Stop button \r\n --  To skip a track, click on Skip button. Pressing the skip button while last track is playing results in a skip to track 1.\r\n");
			labelHelp.setText(labelHelp.getText() + "\r\n\r\n Alternately, the media can also be opened using the Open menu item of the File menu."); 
			formHelp.add(labelHelp);
			labelHelp.setVisible(true);
			formHelp.setVisible(true);
			helpDisplayed = true;
		}
		else
		{
			// Do not launch another help if help already present
			// Just change focus to the original help window
			formHelp.select();
		}
	}

	
	/**
	 * NOTE: The following code is required by the Visual J++ form
	 * designer.  It can be modified using the form editor.  Do not
	 * modify it using the code editor.
	 */
	Container components = new Container();
	MainMenu mainMenuMediaPlayer = new MainMenu();
	MenuItem menuHelp = new MenuItem();
	MenuItem menuDevice = new MenuItem();
	MenuItem menuFile = new MenuItem();
	MenuItem menuAbout = new MenuItem();
	MenuItem menuSeparatorHelpAbout = new MenuItem();
	MenuItem menuHelpTopics = new MenuItem();
	MenuItem menuCDAudio = new MenuItem();
	MenuItem menuMIDI = new MenuItem();
	MenuItem menuSound = new MenuItem();
	MenuItem menuVideoForWindows = new MenuItem();
	MenuItem menuExit = new MenuItem();
	MenuItem menuSeparatorCloseExit = new MenuItem();
	MenuItem menuClose = new MenuItem();
	MenuItem menuOpen = new MenuItem();
	Button btnPlay = new Button();
	Button btnStop = new Button();
	OpenFileDialog openFileDlgOpen = new OpenFileDialog();
	Button btnSkip = new Button();
	Label labelTrack = new Label();
	Label labelTrackDisplay = new Label();
	Label labelDisplay = new Label();
	Timer timerMediaPlayed = new Timer(components);
	Label labelStatus = new Label();

	private void initForm()
	{
		menuAbout.setText("&About Media Player");
		menuAbout.addOnClick(new EventHandler(this.menuAbout_click));

		menuSeparatorHelpAbout.setText("-");

		menuHelpTopics.setText("&Help Topics");
		menuHelpTopics.addOnClick(new EventHandler(this.menuHelpTopics_click));

		menuHelp.setMenuItems(new MenuItem[] {
							  menuHelpTopics, 
							  menuSeparatorHelpAbout, 
							  menuAbout});
		menuHelp.setText("&Help");

		menuCDAudio.setText("&3 CD Audio");
		menuCDAudio.addOnClick(new EventHandler(this.menuCDAudio_click));

		menuMIDI.setText("&2 MIDI Sequencer...");
		menuMIDI.addOnClick(new EventHandler(this.menuMIDI_click));

		menuSound.setText("&1 Sound...");
		menuSound.addOnClick(new EventHandler(this.menuSound_click));

		menuVideoForWindows.setText("&4 Video for Windows...");
		menuVideoForWindows.addOnClick(new EventHandler(this.menuVideoForWindows_click));

		menuDevice.setMenuItems(new MenuItem[] {
								menuSound, 
								menuMIDI, 
								menuCDAudio, 
								menuVideoForWindows});
		menuDevice.setText("&Device");

		menuExit.setText("E&xit");
		menuExit.addOnClick(new EventHandler(this.menuExit_click));

		menuSeparatorCloseExit.setText("-");

		menuClose.setText("&Close");
		menuClose.addOnClick(new EventHandler(this.menuClose_click));

		menuOpen.setText("&Open");
		menuOpen.addOnClick(new EventHandler(this.menuOpen_click));

		menuFile.setMenuItems(new MenuItem[] {
							  menuOpen, 
							  menuClose, 
							  menuSeparatorCloseExit, 
							  menuExit});
		menuFile.setText("&File");

		mainMenuMediaPlayer.setMenuItems(new MenuItem[] {
										 menuFile, 
										 menuDevice, 
										 menuHelp});
		/* @designTimeOnly mainMenuMediaPlayer.setLocation(new Point(120, -16)); */

		this.setBackColor(Color.LIGHTGRAY);
		this.setFont(new Font("MS Sans Serif", 8.0f, FontSize.POINTS, FontWeight.NORMAL, false, false, false, CharacterSet.DEFAULT, 0));
		this.setForeColor(Color.BLACK);
		this.setText("Media Player");
		this.setAutoScale(false);
		this.setAutoScaleBaseSize(13);
		this.setBorderStyle(FormBorderStyle.FIXED_SINGLE);
		this.setClientSize(new Point(258, 124));
		this.setMaximizeBox(false);
		this.setMenu(mainMenuMediaPlayer);
		this.setStartPosition(FormStartPosition.MANUAL);

		btnPlay.setEnabled(false);
		btnPlay.setLocation(new Point(8, 56));
		btnPlay.setSize(new Point(48, 24));
		btnPlay.setTabIndex(0);
		btnPlay.setText("Play");
		btnPlay.addOnClick(new EventHandler(this.btnPlay_click));

		btnStop.setEnabled(false);
		btnStop.setLocation(new Point(56, 56));
		btnStop.setSize(new Point(48, 24));
		btnStop.setTabIndex(1);
		btnStop.setText("Stop");
		btnStop.addOnClick(new EventHandler(this.btnStop_click));

		openFileDlgOpen.setInitialDir("c:\\");
		openFileDlgOpen.setTitle("Open");
		/* @designTimeOnly openFileDlgOpen.setLocation(new Point(264, -8)); */

		btnSkip.setEnabled(false);
		btnSkip.setLocation(new Point(152, 56));
		btnSkip.setSize(new Point(48, 24));
		btnSkip.setTabIndex(2);
		btnSkip.setText("Skip");
		btnSkip.addOnClick(new EventHandler(this.buttonSkip_click));

		labelTrack.setFont(new Font("MS Sans Serif", 8.0f, FontSize.POINTS, FontWeight.BOLD, false, false, false, CharacterSet.DEFAULT, 0));
		labelTrack.setForeColor(Color.DARKGRAY);
		labelTrack.setLocation(new Point(208, 40));
		labelTrack.setSize(new Point(48, 13));
		labelTrack.setTabIndex(3);
		labelTrack.setTabStop(false);
		labelTrack.setText("Track");
		labelTrack.setAutoSize(true);

		labelTrackDisplay.setBackColor(Color.BLACK);
		labelTrackDisplay.setFont(new Font("MS Sans Serif", 12.0f, FontSize.POINTS, FontWeight.BOLD, false, false, false, CharacterSet.DEFAULT, 0));
		labelTrackDisplay.setForeColor(Color.WHITE);
		labelTrackDisplay.setLocation(new Point(200, 56));
		labelTrackDisplay.setSize(new Point(48, 24));
		labelTrackDisplay.setTabIndex(9);
		labelTrackDisplay.setTabStop(false);
		labelTrackDisplay.setText("");
		labelTrackDisplay.setTextAlign(HorizontalAlignment.CENTER);

		labelDisplay.setBackColor(Color.LIGHTGRAY);
		labelDisplay.setFont(new Font("Verdana", 18.0f, FontSize.POINTS, FontWeight.NORMAL, true, false, false, CharacterSet.DEFAULT, 0));
		labelDisplay.setForeColor(new Color(128, 0, 0));
		labelDisplay.setLocation(new Point(16, 8));
		labelDisplay.setSize(new Point(216, 29));
		labelDisplay.setTabIndex(8);
		labelDisplay.setTabStop(false);
		labelDisplay.setText("Media Player");
		labelDisplay.setAutoSize(true);
		labelDisplay.setTextAlign(HorizontalAlignment.CENTER);

		timerMediaPlayed.setInterval(400);
		timerMediaPlayed.addOnTimer(new EventHandler(this.timerMediaPlayed_timer));
		/* @designTimeOnly timerMediaPlayed.setLocation(new Point(176, -16)); */

		labelStatus.setBackColor(Color.LIGHTGRAY);
		labelStatus.setFont(new Font("MS Sans Serif", 8.0f, FontSize.POINTS, FontWeight.NORMAL, false, false, false, CharacterSet.DEFAULT, 0));
		labelStatus.setForeColor(Color.DARKGRAY);
		labelStatus.setLocation(new Point(8, 104));
		labelStatus.setSize(new Point(240, 16));
		labelStatus.setTabIndex(7);
		labelStatus.setTabStop(false);
		labelStatus.setText("");
		labelStatus.setBorderStyle(BorderStyle.FIXED_3D);

		this.setNewControls(new Control[] {
							labelStatus, 
							labelDisplay, 
							labelTrackDisplay, 
							labelTrack, 
							btnSkip, 
							btnStop, 
							btnPlay});
	}

	/**
	 * The main entry point for the application. 
	 *
	 * @param args Array of parameters passed to the application
	 * via the command line.
	 */
	public static void main(String args[])
	{
		Application.run(new formMediaPlayer());
	}
	
	// Find CD audio device
    public static int FindDevice() {
        int iDrive, mciError;

        mciOpenParms.lpstrDeviceType = "cdaudio";
        mciOpenParms.lpstrAlias = "myalias";
        for (iDrive=0; iDrive < NUM_DRIVES; ++iDrive) {
            Character tChar = new Character((char)('A' + iDrive));
            mciOpenParms.wDeviceID = 0;
            mciOpenParms.lpstrElementName = tChar.toString() + ":";

            mciError = mciSendCommand(0, MCI_OPEN, MCI_OPEN_TYPE|MCI_OPEN_SHAREABLE|MCI_OPEN_ELEMENT|MCI_OPEN_ALIAS, mciOpenParms);
            if (mciError == 0) {
                return(mciOpenParms.wDeviceID);
            }
        }
        return (-1);
    }

	// Used for printing errors generated by mci commands
    public static void printMCIError(int mciError) {
        StringBuffer sbuf = new StringBuffer(512);
        boolean flag;
		

        flag = mciGetErrorString(mciError, sbuf, sbuf.capacity()+1);
        if (flag) 
			 MessageBox.show(sbuf.toString(),"Error Message",MessageBox.OKCANCEL);
        else 
			 MessageBox.show("Call to mciGetErrorString() failed","Error Message",MessageBox.OKCANCEL);
    }

    public static boolean SetTimeFormat() {
        int mciError;
        MCI_SET_PARMS mciSetParms = new MCI_SET_PARMS();
        
        mciSetParms.dwTimeFormat = MCI_FORMAT_TMSF;
        
        mciError = mciSendCommand(wDeviceID, MCI_SET, MCI_SET_TIME_FORMAT, mciSetParms);

        if (mciError != 0) {
            printMCIError(mciError);
            return(false);
        }
        return(true);
    }

    public static void ExitMedia(int mediaType) {
        if (MediaPlaying) {
            StopMedia(mediaType);
        }
    }

    public static void CloseDevice() {
        int mciError;
				
        mciError = mciSendCommand(wDeviceID, MCI_CLOSE, 0, 0);

        if (mciError != 0) {
            printMCIError(mciError);
        }
	}

    public static void OpenDevice(int mediaType) {
        int mciError;
		int mciFlags;
		
		if (mediaType == MEDIA_CD_AUDIO)
			mciFlags = MCI_OPEN_TYPE|MCI_OPEN_SHAREABLE|MCI_OPEN_ELEMENT|MCI_OPEN_ALIAS;
		else
			mciFlags = MCI_OPEN_TYPE|MCI_OPEN_ELEMENT;
        mciError = mciSendCommand(0, MCI_OPEN, mciFlags, mciOpenParms);
        if (mediaType == MEDIA_CD_AUDIO) SetTimeFormat();

        if (mciError != 0) {
            printMCIError(mciError);
        }
    }

	// Gets total number of tracks on CD Audio
    public static int GetTotalTracks() {
        int mciError, dwFlags;
        MCI_STATUS_PARMS mciStatusParms = new MCI_STATUS_PARMS();

        OpenDevice(MEDIA_CD_AUDIO);
        dwFlags = MCI_STATUS_ITEM;
        mciStatusParms.dwItem = MCI_STATUS_NUMBER_OF_TRACKS;

        mciError = mciSendCommand(wDeviceID, MCI_STATUS, dwFlags, mciStatusParms);

        if (mciError != 0) {
            printMCIError(mciError);
            CloseDevice();
            return(0);
        }

        CloseDevice();
        return(mciStatusParms.dwReturn);
    }

	// Gets the current track being played on CD Audio
    public static int GetCurrentTrack() {
        int mciError, dwFlags;
        MCI_STATUS_PARMS mciStatusParms = new MCI_STATUS_PARMS();

        OpenDevice(MEDIA_CD_AUDIO);
        dwFlags = MCI_STATUS_ITEM;
        mciStatusParms.dwItem = MCI_STATUS_CURRENT_TRACK;

        mciError = mciSendCommand(wDeviceID, MCI_STATUS, dwFlags, mciStatusParms);

        if (mciError != 0) {
            printMCIError(mciError);
            CloseDevice();
            return(0);
        }

        CloseDevice();
        return(mciStatusParms.dwReturn);
    }

    
	public static int PlayMedia(int mediaType, String fileName,int track,Form f, int x, int y)
	{
		int mciError;
		MCI_PLAY_PARMS mciPlayParms = new MCI_PLAY_PARMS();
		MCI_STATUS_PARMS mciStatusParms = new MCI_STATUS_PARMS();
		MCI_OVLY_WINDOW_PARMS mciWindowParms = new MCI_OVLY_WINDOW_PARMS();
		MCI_OVLY_RECT_PARMS mciRectParms = new MCI_OVLY_RECT_PARMS();
		
		switch (mediaType)
		{
		
			case MEDIA_SOUND			:	mciOpenParms.lpstrDeviceType = "waveaudio";
											mciOpenParms.lpstrElementName = fileName;
											OpenDevice(MEDIA_SOUND);
											wDeviceID = mciOpenParms.wDeviceID;
											
											// Start playing
    										mciError = mciSendCommand(wDeviceID, MCI_PLAY, MCI_NOTIFY,mciPlayParms);
											if (mciError != 0) {
												printMCIError(mciError);
												CloseDevice();
												return -1;
											}
											
											MediaPlaying = true;
											break;
								
			case MEDIA_MIDI_SEQUENCER	:	
											mciOpenParms.lpstrDeviceType = "sequencer";
											mciOpenParms.lpstrElementName = fileName;
											mciOpenParms.lpstrAlias = "myalias1";
											OpenDevice(MEDIA_MIDI_SEQUENCER);		
											// The device opened successfully; get the device ID.
											wDeviceID = mciOpenParms.wDeviceID;
											
											// Check if output port is MIDI mapper
											mciStatusParms.dwItem = MCI_SEQ_STATUS_PORT;
											mciError = mciSendCommand(wDeviceID, MCI_STATUS,MCI_STATUS_ITEM, mciStatusParms);
											if (mciError != 0) {
												printMCIError(mciError);
												CloseDevice();
												return -1;
											}    
											
											// Start playing
											mciError = mciSendCommand(wDeviceID, MCI_PLAY, MCI_NOTIFY,mciPlayParms);
											if (mciError != 0) {
												printMCIError(mciError);
												CloseDevice();
												return -1;
											}
											MediaPlaying = true;
											
											break;
								
			case MEDIA_CD_AUDIO			:	OpenDevice(MEDIA_CD_AUDIO);
											mciPlayParms.dwFrom = track;
											
											// Start playing
											mciError = mciSendCommand(wDeviceID, MCI_PLAY, MCI_FROM, mciPlayParms);
											if (mciError != 0) {
												 printMCIError(mciError);
												 CloseDevice();
												 return -1;
											}
											MediaPlaying = true;
											CloseDevice();
											break;	
											
			case MEDIA_VIDEO_FOR_WINDOWS:	
											mciOpenParms.lpstrDeviceType = "avivideo";
											mciOpenParms.lpstrElementName = fileName;
											OpenDevice(MEDIA_VIDEO_FOR_WINDOWS);
											wDeviceID = mciOpenParms.wDeviceID;
											
											f.setText(File.getName(fileName));
											f.setWidth(0);
											f.setHeight(0);
											mciWindowParms.hWnd = f.getHandle();
											
											// Associate a window to play the AVI file
											mciError = mciSendCommand(wDeviceID,MCI_WINDOW,MCI_OVLY_WINDOW_HWND,mciWindowParms);
											if (mciError != 0) {
												 printMCIError(mciError);
												 CloseDevice();
												 return -1;
											}
											
											// Get coordinates of AVI
											mciError = mciSendCommand(wDeviceID,MCI_WHERE,MCI_OVLY_RECT|MCI_OVLY_WHERE_DESTINATION,mciRectParms);
											if (mciError != 0) {
												 printMCIError(mciError);
												 CloseDevice();
												 return -1;
											}
											
											// Resize the window to the size of the AVI
											f.setLeft(mciRectParms.rc.left + x);
											f.setTop(mciRectParms.rc.top + y);
											f.setWidth(mciRectParms.rc.right - mciRectParms.rc.left + 8); // Border width = 8
											f.setHeight(f.getHeight() + mciRectParms.rc.bottom - mciRectParms.rc.top);
											
															
											// Make AVI form visible
											f.setVisible(true);	
											
											// Start playing
											mciError = mciSendCommand(wDeviceID, MCI_PLAY, MCI_NOTIFY,mciPlayParms);
											if (mciError != 0) {
												printMCIError(mciError);
												CloseDevice();
												return -1;
											}
											MediaPlaying = true;
											
											break;
											
			default						:	break;
		}
		
		return 1;
	}

    public static void StopMedia (int mediaType) {
        int mciError;
        MCI_GENERIC_PARMS mciGenericParms = new MCI_GENERIC_PARMS();

        if (mediaType == MEDIA_CD_AUDIO) 
			OpenDevice(mediaType);
		
		// Stop Playing
		mciError = mciSendCommand(wDeviceID, MCI_STOP, 0, mciGenericParms);

        if (mciError != 0) {
            printMCIError(mciError);
		}
        MediaPlaying = false;
		MediaPaused = false;
		
		CloseDevice();
    }
	
			
	public static void PauseMedia(int mediaType)
	{
		int mciError;
		MCI_PLAY_PARMS mciPlayParms = new MCI_PLAY_PARMS();
		
		if (mediaType == MEDIA_CD_AUDIO)
			OpenDevice(mediaType);
		
		// Pause Playing
		mciError = mciSendCommand(wDeviceID, MCI_PAUSE, MCI_NOTIFY,mciPlayParms);
		if (mciError != 0) {
            printMCIError(mciError);
        }
		MediaPaused = true;
		MediaPlaying = false;
		
		if (mediaType == MEDIA_CD_AUDIO)
			CloseDevice();
     }
	
	
	public static void ResumeMedia(int mediaType)
	{
		int mciError;
		MCI_PLAY_PARMS mciPlayParms = new MCI_PLAY_PARMS();
		
		if (mediaType == MEDIA_CD_AUDIO)
			OpenDevice(mediaType);
		
		// Resume Playing
		if (mediaType == MEDIA_MIDI_SEQUENCER)
			mciError = mciSendCommand(wDeviceID, MCI_PLAY, MCI_NOTIFY,mciPlayParms);
		else
			mciError = mciSendCommand(wDeviceID, MCI_RESUME, MCI_NOTIFY,mciPlayParms);
		if (mciError != 0) {
            printMCIError(mciError);
        }
        MediaPaused = false;
		MediaPlaying = true;
		if (mediaType == MEDIA_CD_AUDIO)
			CloseDevice();
     }
	

	
	// Win32 functions  : Added using the J/Direct Call Builder
	
	public static final int MCI_OPEN = 0x0803;
	public static final int MCI_SET_TIME_FORMAT = 0x00000400;
	public static final int MCI_CLOSE = 0x0804;
	public static final int MCI_PLAY = 0x0806;
	public static final int MCI_STOP = 0x0808;
	public static final int MCI_SET = 0x080D;
	public static final int MCI_STATUS = 0x0814;
	public static final int MCI_FROM = 0x00000004;
	public static final int MCI_OPEN_ALIAS = 0x00000400;
	public static final int MCI_OPEN_ELEMENT = 0x00000200;
	public static final int MCI_OPEN_TYPE = 0x00002000;
	public static final int MCI_STATUS_ITEM = 0x00000100;
	public static final int MCI_TRACK = 0x00000010;
	public static final int MCI_CDA_STATUS_TYPE_TRACK = 0x00004001;
	public static final int MCI_STATUS_CURRENT_TRACK = 0x00000008;
	public static final int MCI_STATUS_MEDIA_PRESENT = 0x00000005;
	public static final int MCI_STATUS_NUMBER_OF_TRACKS = 0x00000003;
	public static final int MCI_FORMAT_TMSF = 10;
	public static final int MCI_OPEN_SHAREABLE = 0x00000100;
	public static final int MCI_SEQ_STATUS_PORT = 0x00004003;
	public static final int MCI_NOTIFY = 0x00000001;
	public static final int MCI_PAUSE = 0x0809;
	public static final int MCI_RESUME = 0x0855;
	public static final int MCI_OVLY_WINDOW_HWND = 0x00010000;
	public static final int MCI_WINDOW = 0x0841;
	public static final int MCI_OVLY_PUT_DESTINATION = 0x00040000;
	public static final int MCI_OVLY_RECT = 0x00010000;
	public static final int MCI_PUT = 0x0842;
	public static final int MCI_OVLY_WHERE_DESTINATION = 0x00040000;
	public static final int MCI_WHERE = 0x0843;
	public static final int MCI_STATUS_LENGTH = 0x00000001;
	public static final int MCI_STATUS_POSITION = 0x00000002;
	
	/**F
	 * @dll.struct(pack=1) 
	 */
	public static class MCI_GENERIC_PARMS
	{
		public int dwCallback;
	}

	/**
	 * @dll.struct(pack=1, auto) 
	 */
	public static class MCI_OPEN_PARMS
	{
		public int dwCallback;
		public int wDeviceID;
		public String lpstrDeviceType;
		public String lpstrElementName;
		public String lpstrAlias;
	}

	/**
	 * @dll.struct(pack=1) 
	 */
	public static class MCI_PLAY_PARMS
	{
		public int dwCallback;
		public int dwFrom;
		public int dwTo;
	}

	/**
	 * @dll.struct(pack=1) 
	 */
	public static class MCI_SET_PARMS
	{
		public int dwCallback;
		public int dwTimeFormat;
		public int dwAudio;
	}

	/**
	 * @dll.struct(pack=1) 
	 */
	public static class MCI_STATUS_PARMS
	{
		public int dwCallback;
		public int dwReturn;
		public int dwItem;
		public int dwTrack;
	}

	/**
	 * @dll.struct(pack=1) 
	 */
	public static class MCI_SEQ_SET_PARMS
	{
		public int dwCallback;
		public int dwTimeFormat;
		public int dwAudio;
		public int dwTempo;
		public int dwPort;
		public int dwSlave;
		public int dwMaster;
		public int dwOffset;
	}
	
	/**
	 * @dll.struct(pack=1, auto) 
	 */
	public static class MCI_OVLY_WINDOW_PARMS
	{
		public int dwCallback;
		public int hWnd;
		public int nCmdShow;
		public String lpstrText;
	}
	
	
	/**
	 * @dll.struct(pack=1) 
	 */
	public static class MCI_OVLY_RECT_PARMS
	{
		public int dwCallback;
		public com.ms.win32.RECT rc;
	}
		
	/**
	 * @dll.import("WINMM",auto) 
	 */
	public static native int mciSendCommand(int mciId, int uMsg, int dwParam1, int dwParam2);

	/**
	 * @dll.import("WINMM",auto) 
	 */
	public static native int mciSendCommand(int mciId, int uMsg, int dwParam1, Object dwParam2);

	/**
	 * @dll.import("WINMM", auto) 
	 */
	public static native boolean mciGetErrorString(int mcierr, StringBuffer pszText, int cchText);

	
}
