
/*

(C) Copyright 1995 - 1999 Microsoft Corporation.  All rights reserved.

*/

#include <windows.h>
#include <native.h>
#include "JMain.h"
#include "JFoo.h"
#include "JThread1.h"
#include "JThread2.h"

////////////////////////////////////////////////////////////////////////////////
//
//	RNIGetCompatibleVersion
//
//	Called by the virtual machine to determine what level of RNI compatibility
//	this DLL offers.
//
////
DWORD __cdecl RNIGetCompatibleVersion()
{
	return RNIVER;
}

////////////////////////////////////////////////////////////////////////////////
//
//	JMain_Square()
//
//	returns the square of its parameter;  simple callout from java to
//	native code to perform this operation.
//
////
__declspec(dllexport) long __cdecl JMain_Square(struct HJMain *pThis, long lVal)
{
	return lVal * lVal ;
}

////////////////////////////////////////////////////////////////////////////////
//
//	JMain_DoubleInstanceVar()
//
//	executes a java-implemented method to retrieve the value of an
//	instance variable;  returns twice the retrieved value.
//
////
__declspec(dllexport) void __cdecl JMain_DoubleInstanceVar(struct HJMain *pThis)
{
	long lVal ;

	lVal = execute_java_dynamic_method(0, (HObject*) pThis, "GetiVar1", "()I") ;

	pThis->iVar1 = lVal << 1 ;
}

////////////////////////////////////////////////////////////////////////////////
//
//	Jmain_showMemberArray()
//
//	executes a method which returns a private member array of
//	ints (java); loops through and displays the array members.
//
////
__declspec(dllexport) void __cdecl JMain_showMemberArray(struct HJMain *pThis)
{
	ClassArrayOfInt *paiVar1 ;
	long	*plVar2 ;
	int	cIndex ;

	// get a pointer to an instance array of int, and display it.
	paiVar1 = (ClassArrayOfInt *) execute_java_dynamic_method(0, (HObject *) pThis, "GetiCVar2", "()[I") ;
	plVar2 = paiVar1->body ;

	for (cIndex = 0; cIndex < paiVar1->length; cIndex++)
	{
		printf("%5d", *plVar2++) ;
	}
	printf("\n") ;
}

////////////////////////////////////////////////////////////////////////////////
//
//	JMain_Various()
//
//	finds JFoo class; executes a static method in that class
//
////
__declspec(dllexport) void __cdecl JMain_Various(struct HJMain *pThis)
{
	ClassClass *pcls = FindClass(NULL, "JFoo", TRUE) ;

	if (pcls)
	{
		// exec the method which expects an argument of type long in Java,
		// so we must pass a value of type int64_t, from here in native code 
		execute_java_static_method(NULL, pcls, "StaticMethod", "(J)V", (int64_t)10) ;
	}
}

////////////////////////////////////////////////////////////////////////////////
//
//	w32CreateEvent()
//
//	wrapper for win32 API CreateEvent()
//
////
__declspec(dllexport) long __cdecl JMain_w32CreateEvent(struct HJMain *pThis, long bInit)
{
	BOOL	bInitialState ;
	HANDLE	hEvent ;

	// determine if initial state is signaled or non-signaled.
	if (bInit)
	{
		bInitialState = TRUE ;
	}
	else
	{
		bInitialState = FALSE ;
	}

	// create Event object and return handle
	return (long) CreateEvent(NULL, FALSE, bInitialState, NULL) ;
}

////////////////////////////////////////////////////////////////////////////////
//
//	w32PulseEvent()
//
//	wrapper for win32 API PulseEvent()
//
////
__declspec(dllexport) void __cdecl JThread1_w32PulseEvent(struct HJThread1 *pThis,long hEvent)
{
	PulseEvent((HANDLE) hEvent) ;
}

////////////////////////////////////////////////////////////////////////////////
//
//	w32CloseHandle()
//
//	wrapper for win32 API CloseHandle()
//
////
__declspec(dllexport) long __cdecl JThread2_w32CloseHandle(struct HJThread2 *pThis, long hObject)
{
	return (long) CloseHandle((HANDLE) hObject) ;
}

////////////////////////////////////////////////////////////////////////////////
//
//	GCSafeNative()
//
//	method demonstrates GC safe way of performing GC-sensitive operations
//	in native code.
//
////
__declspec(dllexport) void __cdecl JThread2_GCSafeNative(struct HJThread2 *pThis, long hEvent)
{
	struct
	{
		HJFoo *pJFoo1, *pJFoo2 ;
	} gcSafe ;				// struct holds out pointers to JObjects
	GCFrame gcf ;

	// GCFramePush() creates storage for GC-sensitive data (JObjects in this case)
	GCFramePush(&gcf, &gcSafe, sizeof(gcSafe)) ;

	// set our member variables to point to two JObjects
	gcSafe.pJFoo1 = (HJFoo *) execute_java_constructor(0, "JFoo", 0, "(I)", 10) ;
	gcSafe.pJFoo2 = (HJFoo *) execute_java_constructor(0, "JFoo", 0, "()") ;

	// call GCEnable() - we don't know how long we might be blocked
	GCEnable() ;

	// block
	WaitForSingleObject((HANDLE) hEvent, INFINITE) ;

	// display instance variable values for above-created JObjects.
	printf("gcSafe.pJFoo1->m_iVal = %d   gcSafe.pJFoo2->m_iVal = %d\n", gcSafe.pJFoo1->m_iVal, gcSafe.pJFoo2->m_iVal) ;

	// disable GC and pop our storage
	GCDisable() ;
	GCFramePop(&gcf) ;
}

////////////////////////////////////////////////////////////////////////////////
//
//	JMain_Make2dArray()
//
//	creates square-shaped 2d array, the dimension being the length of the second
//		function argument (an array of longs); and 
//  makes that function argument be the nth element of the 2D array
//		(n being the final argument to this function)
//
////
__declspec(dllexport) struct HArrayOfArray * __cdecl JMain_Make2dArray (struct HJMain *pThis, struct HArrayOfLong *plArray, long nth)
{
	struct {
		HArrayOfLong  *plArray;
		HArrayOfArray *p2dArray;
	} gcSafe ;
	GCFrame gcf;
	HObject  *pFiller;	// objects to fill up the 2d array
	long i;				// index to the 2d array (matches function argument "nth")
	unsigned long dim = obj_length(plArray);	// matches struct HArrayOfLong's field

	// protect the argument from future calls we make to RNI functions 
	//	which may cause gc to happen
	GCFramePush(&gcf, &gcSafe, sizeof(gcSafe)) ;
	gcSafe.plArray = plArray;

	// create an array of objects, the object type being array of long
	gcSafe.p2dArray = \
		(HArrayOfArray*)ClassArrayAlloc(T_CLASS, dim, "[J");

	if (gcSafe.p2dArray)
	{
		// fill out the 2D array
		for (i = 0; i < dim; i++)
		{
			if (i == nth)
			{
				// let the function argument be the nth elem of the 2D array
				pFiller = (HObject*)gcSafe.plArray;
			} 
			else
			{
				// let a new array of longs fill in the 2D array
				pFiller = ArrayAlloc(T_LONG, dim);
				if (pFiller == NULL)
				{
					// cannot alloc a full-bodied 2D array
					// let garbage collector reclaim any & all
					//   arrays we've previously made in this function
					GCFramePop(&gcf);
					return NULL;
				}
			}

			// this RNI function supports the so-called write barrier
			//	(see SDK documentation)
			GCSetObjectReferenceForObject((gcSafe.p2dArray)->body + i, pFiller);
		}

	}

	GCFramePop(&gcf);
	return gcSafe.p2dArray;
}


////////////////////////////////////////////////////////////////////////////////
//
//	JMain_getMaximum()
//
//	returns the biggest value in the nth array in the given 2D array
//
////
__declspec(dllexport) int64_t __cdecl JMain_getMaximum (struct HJMain *pThis, struct HArrayOfArray *p2dArray, long nth)
{
	int64_t maxval;
	unsigned long i;

	// note that the 'const' is required (due to the so-called write barrier;
	// see SDK documentation) ... this means a C compiler (but not a C++ compiler)
	// will disallow any code prior to initialization of the constant var,
	// such as error-checking code for valid array index
	HArrayOfLong* const nth_array = (HArrayOfLong*)((p2dArray)->body[nth]);

	// if the 2D array is not filled in at our spot
	if (nth_array == NULL)
	{
		SignalError(NULL, "java/lang/NullPointerException", NULL);
	}
	// if nth_array has no elements
	else if (obj_length(nth_array) == 0)
	{
		SignalError(NULL, "java/lang/ArrayIndexOutOfBoundsException", NULL);
	}
	else
	{
		maxval = (nth_array)->body[0]; 
		for (i = 1; i < obj_length(nth_array); i++)
		{
			if (maxval < (nth_array)->body[i])	
				maxval = (nth_array)->body[i];
		}
	}

	return maxval;
}
