// objlist.cpp
//
// Created 01/11/99
//
// (C)Copyright 1999 Microsoft Corporation, All rights reserved.
//

#include "pch.hpp"
#pragma hdrstop

#include "objlist.hpp"

#include "hpmonmgr.hpp"


LRESULT CALLBACK ObjectListViewer::WndProc (HWND wnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
    switch (msg)
    {
    case WM_OLVEVENT_OBJ_STATUS_CHANGE:
        {
            OBJID obj = (OBJID)lParam;

            HTREEITEM htiobj = FindNextSiblingForID(TreeView_GetRoot(m_htree), obj);

            if (obj->flags & OID_FL_DEAD)
            {
                if (htiobj != NULL && htiobj == TreeView_GetSelection(m_htree))
                    ClearText();
                        
                TVITEM tvi;
                tvi.hItem = htiobj;
                tvi.mask = TVIF_PARAM | TVIF_IMAGE;
                tvi.lParam = (LPARAM)obj;
                tvi.iImage = HVTII_DEAD_OBJID;
                
                TreeView_SetItem(m_htree, &tvi);
            }
            else
            {
                if (obj->flags & m_oidflagsmask)
                {
                    if (htiobj == NULL)
                        AddTreeItemForID(TVI_ROOT, obj);
                }
                else
                {
                    if (htiobj != NULL)
                    {
                        if (htiobj == TreeView_GetSelection(m_htree))
                            ClearText();
                        DeleteTreeItem(htiobj);
                    }
                }
            }
        }
        return 0;
    }

    return ObjectViewer::WndProc(wnd, msg, wParam, lParam);
}


//static
int ObjectListViewer::PopulateTreeCB (OBJID obj, PVOID token)
{
    ObjectListViewer *olv = (ObjectListViewer*)token;

    if (obj->flags & olv->m_oidflagsmask)
        olv->AddTreeItemForID(TVI_ROOT, obj);

    return 1;
}


BOOL ObjectListViewer::ShouldDeleteTransientObject (HTREEITEM hti, OBJID obj)
{
    return TreeView_GetParent(m_htree, hti) || !(obj->flags & m_oidflagsmask);
}


BOOL ObjectListViewer::Initialize (HeapMonitorManager *mgr, PCSTR title, DWORD oidflagsmask)
{
    HeapMonitorClientRegistrationInfo reginfo;
    ZeroMemory(&reginfo, sizeof(reginfo));

    reginfo.StoppedEventMask = HMC_OBJECT_EVENTS;

    BOOL result = ObjectViewer::OVInitialize(mgr, title, &reginfo);

    if (result)
    {
        m_oidflagsmask = oidflagsmask;
        
        mgr->IterateSpecialObjects(&PopulateTreeCB, this);
    }

    return result;
}

