// refsview.cpp
//
// Created 10/08/98
//
// (C)Copyright 1998-1999 Microsoft Corporation, All rights reserved.
//

#include "pch.hpp"
#pragma hdrstop

#include "refsview.hpp"
#include "hpmonmgr.hpp"


LRESULT CALLBACK ObjectReferencesViewer::WndProc (HWND wnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
    switch (msg)
    {
    case WM_NOTIFY:
        {
            LPNMHDR pnmh = (NMHDR*)lParam; 
            switch (pnmh->code)
            {
            case TVN_ITEMEXPANDING:
            case TVN_ITEMEXPANDED:
                {
                    NMTREEVIEW *pnmtv = (NMTREEVIEW*)lParam;
                    TV_ITEM *ptvi = &pnmtv->itemNew;
                    if (ptvi->hItem == m_htiroot)
                        return BaseWndProc(wnd, msg, wParam, lParam);
                }
                break;
            }
        }
        break;
    }

    return ObjectViewer::WndProc(wnd, msg, wParam, lParam);
}


BOOL ObjectReferencesViewer::GetItemText (HTREEITEM hti, PSTR &psztext, int &cchmaxtext)
{
    BOOL result = ObjectViewer::GetItemText(hti, psztext, cchmaxtext);

    if (result && hti == m_htiroot)
    {
        int len = strlen(psztext);
        static CHAR prefix[] = "References to ";
        int prefixlen = ARRAY_ELEMENTS(prefix)-1;
        if (len+prefixlen < cchmaxtext)
        {
            MoveMemory(psztext+prefixlen, psztext, len+1);
            CopyMemory(psztext, prefix, prefixlen);
        }
    }

    return result;
}


//static
int ObjectReferencesViewer::PopulateReferencesCB (ID id, ObjectID vmid, PVOID token)
{
    ObjectReferencesViewer *orv = (ObjectReferencesViewer*)token;

    if (!id)
    {
        ASSERT(vmid);
        id = (OBJID)orv->m_mgr->LookupID(vmid);
    }

    BOOL result;

    if (id)
        result = (orv->AddTreeItemForReferencedID(orv->m_htiroot, id) != NULL);
    else
        result = (orv->AddTreeItemForObject(orv->m_htiroot, vmid) != NULL);

    if (result)
        TreeView_Expand(orv->m_htree, orv->m_htiroot, TVE_EXPAND);

    return 1;
}


BOOL ObjectReferencesViewer::Initialize (HeapMonitorManager *mgr, ObjectID vmid)
{
    BOOL result;

    result = OVInitialize(mgr, "Object References");
    if (result)
    {
        OBJID id = (OBJID)m_mgr->LookupID(vmid);
        
        if (id)
            result = (m_htiroot = AddTreeItemForID(TVI_ROOT, id)) != NULL;
        else
            result = (m_htiroot = AddTreeItemForObject(TVI_ROOT, vmid)) != NULL;
    }

    if (result)
    {
        result = mgr->FindObjectReferences(vmid, &PopulateReferencesCB, this);
    }

    return result;
}

