/*
 *
 * sample.appletViewer.ClassViewer.java
 *
 * (C) Copyright 1995 - 1999 Microsoft Corporation.  All rights reserved.
 *
*/
package sample.appletViewer;

import sample.decoder.*;
import java.util.Hashtable;
import java.util.Vector;
import java.util.Enumeration;
import java.awt.FileDialog;
import com.ms.ui.UIFrame;
import com.ms.ui.event.IUIActionListener;
import com.ms.ui.event.IUIItemListener;
import com.ms.ui.UITree;
import com.ms.ui.UIScrollViewer;
import com.ms.ui.UIPanel;
import com.ms.ui.UIList;
import com.ms.ui.event.UIItemEvent;
import com.ms.ui.event.UIActionEvent;
import com.ms.ui.UIGridLayout;
import com.ms.ui.UISplitViewer;
import com.ms.ui.UIBand;
import com.ms.ui.UIMenuList;
import com.ms.ui.UIMenuButton;
import com.ms.ui.UILine;
import com.ms.ui.UIBandBox;
import com.ms.ui.UIColumnHeader;
import com.ms.ui.UIRow;
import com.ms.ui.UIColumnViewer;
/**
 * This class servers as a viewer into the decoded .class file.  It leverages
 * the information gathered from the ClassObject and all attribute, field
 * method, and constant objects contained within.  This Frame was created from
 * some java.awt components as well as com.ms.ui.AwtUI components, occasionaly
 * there is some static from the event propogation between the components.
 *
 * @see sample.decoder.DecodeDisplayInterface
 * @see sample.decoder.ClassObject
 *
 * @date	April 28,1998
 * @version	1.2
*/
public class ClassViewer extends com.ms.ui.UIApplet implements IUIActionListener,IUIItemListener{
/**
 * The initial value for the root node which represents the base for all .class
 * attributes.
*/
	public  String ROOT_NAME = "";
/**
 * Label for the 'file' menu in the main menu bar.
 */
	public static final String FILE_MENU = "    File    ";
/**
 * Label for the 'help' menu in the main menu bar.
 */
	public static final String HELP_MENU = "    Help    ";
/**
 * This is the root node for the tree view.  It will be replaced each time a new class
 * is decoded.
*/
	private UITree root =new UITree(ROOT_NAME);
/**
 * This panel holds the two list objects which represent the table in the right side of
 * the GUI.  A ScrollPane is not needed as the List objects use scrillbars whenever 
 * required for their content.
*/
	private UIPanel table = new UIPanel();
/** 
 * Used to hold the tree, and contains the scrollbars required for when the tree is 
 * larger than the view.
*/
	private UIPanel treeView = new UIPanel();
/**
 * Keeps track of the currently displayed table to reduce screen flicker.
 */
	private transient Object currentTable=null;
/**
 * Name of the file menu load attribute that will be displayed in the main menu
*/
	private final String FILEMENULOADNAME = "    Load    ";
/**
 * Display string used in the file menu to hold the exit label item name.
*/
	private final String FILEMENUEXITNAME = "    Exit    ";
/**
 * This holds the name for the about menu item in the help menu.
*/
	private final String HELPMENUABOUT = "    About    ";
/**
 * Holds all information to be displayed, it is the decoded class to be viewed.
*/
	private ClassObject classObject = null;
/**
 * This is a lookup table for mapping a tree node to the DecodeDisplayInterface object
 * that it represents.
*/
	private Hashtable treeLookup = new Hashtable();
/**
 * Main UIPanel which contains the tree and the table
 */
	private UISplitViewer mainPanel = null;
/**
 * Contains the UIColumnViewer which is the actual table.
 */
	private UIColumnViewer tableObject=null;
/**
 * This constructor initializes the view and sets up the components that will
 * hold the data from the .class decode.  There is a tree view in the left 
 * side of a split pane and two lists on the right.  Each side has its own 
 * scroll pane to accomodate data outside the viewable area.
*/
public ClassViewer(){
	this.add("North",getMenu());
	treeView.setLayout(new UIGridLayout(1,1));
	mainPanel = new UISplitViewer(treeView, getTable());
	this.add("Center",mainPanel);
	this.validate();
	this.repaint();
	this.setSize(500,500);
	this.setVisible(true);
}
/**
 * Creates the initial table view that will house the two lists containing
 * name/value pairs for each node selected in the tree view.
 *
 * @returns UIScrollViewer -  this holds two empty AwtUIList objects.
*/
private UIPanel getTable(){
	table.setLayout(new UIGridLayout(1,1));
	table.setVisible(true);
	return table;
}
/**
 * This erases the old root node for the tree view and sets the current root 
 * to the node passed as a parameter, and refreshes the view.
 *
 * @param UITree node - this is the root node for the new tree view.
*/
private void updateTree(UITree node){
	treeView.setVisible(false);
	treeView.removeAll();
	UIScrollViewer s = new UIScrollViewer(node);
	treeView.add(s);
	root=node;
	treeView.validate();
	repaint();
	treeView.setVisible(true);
}
/**
 * Item Event handler which listens for selected tree nodes and populates the
 * right side table view with the attributes for the object represented.
 *
 * @param UIItemEvent a - Event generated by the tree node indicating that it
 *						has been selected.
*/
public synchronized void itemStateChanged(UIItemEvent a){
	if (a.getStateChange()==UIItemEvent.SELECTED){
		if (a.getItem() instanceof UITree){
			Object o = treeLookup.get(new Integer(
				((UITree)a.getItem()).getID()));
			if (currentTable!=o)
				if (o instanceof DecodeDisplayInterface){
					updateTable((DecodeDisplayInterface)o);
				}else {
				table.removeAll();
			}
			currentTable=o;
		}
	}
	treeView.requestFocus();
}
/**
 * This updates the table with the Contents information from the 
 * DecodeDisplayInterface that is passed as a parameter.  If a null is given
 * no changes are made to the table.
 *
 * @param DecodeDisplayInterface data - Object with data to be displayed.
*/
private void updateTable(DecodeDisplayInterface data){
	table.setVisible(false);
	table.removeAll();
	UIColumnHeader[] headers = new UIColumnHeader[2];
	headers[0]=new UIColumnHeader("Name");
	headers[1]=new UIColumnHeader("Value");
	UIList value = new UIList();
	if (data!=null){
		Vector contents = data.getEntries();
		table.removeAll();
		int nameLength=0;
		int valueLength=0;
		Vector sorted = contents;
		for (Enumeration i=sorted.elements();i.hasMoreElements();){
			Entry key = (Entry)i.nextElement();
			Object[] things = new Object[2];
			things[0]=key.getDescription();
			if (key.getDescription().length()>nameLength)nameLength=
												key.getDescription().length();
			things[1]=key.getValue();
			if (((String)things[1]).length()>valueLength)
				valueLength=((String)things[1]).length();
			UIRow row = new UIRow(things);
			value.add(row);
		}	
		tableObject = new UIColumnViewer(headers,value){
			public synchronized void paint(com.ms.fx.FxGraphics g){
				int w1=getWidth(0);
				int w2=getWidth(1);
				if (w1+w2<mainPanel.getChildSize(table).width){
					setWidth(0,posI(w1,w2));
					setWidth(1,mainPanel.getChildSize(table).width-posI(w1,w2)-1);
				}
				super.paint(g);	
			}
			private int posI(int w1,int w2){
				float ratio  = new Float(w1).floatValue()
							   /new Float(w1+w2).floatValue();
				ratio= new Float(mainPanel.getChildSize(table).width).floatValue()*ratio;
				if (new Float(ratio).intValue()>0) 
					return new Float(ratio).intValue();
				else return 0;
			}
		};
		tableObject.setWidth(0,nameLength*7);
		tableObject.setWidth(1,valueLength*7);
		table.add(tableObject);
		table.validate();
		repaint();
		table.setVisible(true);
	}
}
/**
 * Sorts the elements in the String/String hashtable by alphabetic comparisons
 * 
 * @param Hashtable - contains keys and elements which are Strings, sorted by key.
 * 
 * @return Vector - Vector contains the sorted keys.
 */
private Vector sort(Hashtable h){
	Vector data = new Vector();
	for (Enumeration i=h.keys();i.hasMoreElements();){
		String key = (String)i.nextElement();
		int j;
		for (j=0;j<data.size();j++)
			if (((String)data.elementAt(j)).compareTo(key)>=0)
				break;
		data.insertElementAt(key,j);	
	}
	return data;
}
/**
 * Creates the menu bar that is used in the main frame, setting the load 
 * and exit entries.
 *
 * @returns UIBandBox - this menubar contains all application controls.
*/
private UIPanel getMenu(){
	UIBandBox menuHolder = new UIBandBox();
	menuHolder.setBackground(java.awt.Color.lightGray);
	UIBand menuBar = new UIBand();
	UIMenuList fileList = new UIMenuList();
	fileList.addActionListener(this);
	fileList.add(FILEMENULOADNAME);
	fileList.add(new UILine());
	fileList.add(FILEMENUEXITNAME);
	UIMenuList helpList = new UIMenuList();
	helpList.addActionListener(this);
	helpList.add(HELPMENUABOUT);
	UIMenuButton ff = new UIMenuButton(FILE_MENU,fileList);
	menuBar.add(ff);
	menuBar.add(new UIMenuButton(HELP_MENU,helpList));
	menuHolder.add(menuBar);
	return menuHolder;
}
/**
 * This is the action event handler which listens for the load command from 
 * the menu, and launches the File Dialog and initiates the decoding of the 
 * file.  It also listens for an exit event from the window and exits the
 * program.
 *
 * @param UIActionEvent a - this is the event generated by the menu.
*/
public void actionPerformed(UIActionEvent a){
	if (a.getActionCommand().equals(FILEMENULOADNAME)){
			FileDialog d = new FileDialog(new java.awt.Frame());
			d.setMode(FileDialog.LOAD );
			d.show();
			try{
				classObject = new ClassObject(d.getDirectory()+d.getFile());
			}catch(Exception ii){
				log("ClassViewer: Exception decoding class "+ii);
			}
			root=null;
			initTree();
			}
	if (a.getActionCommand().equals(FILEMENUEXITNAME)){
	}
	if (a.getActionCommand().equals(HELPMENUABOUT)){
		table.setVisible(false);
	table.removeAll();
	UIColumnHeader[] headers = new UIColumnHeader[2];
	headers[0]=new UIColumnHeader("Name");
	headers[1]=new UIColumnHeader("Value");
	UIList value = new UIList();
		Object[] things = new Object[2];
		things[0]="Version";
		things[1]="1.2b";
		UIRow row = new UIRow(things);
		value.add(row);		
		tableObject = new UIColumnViewer(headers,value){
			public synchronized void paint(com.ms.fx.FxGraphics g){
				int w1=getWidth(0);
				int w2=getWidth(1);
				if (w1+w2<mainPanel.getChildSize(table).width){
					setWidth(0,posI(w1,w2));
					setWidth(1,mainPanel.getChildSize(table).width-posI(w1,w2)-1);
				}
				super.paint(g);	
			}
			private int posI(int w1,int w2){
				float ratio  = new Float(w1).floatValue()
							   /new Float(w1+w2).floatValue();
				ratio= new Float(mainPanel.getChildSize(table).width).floatValue()*ratio;
				if (new Float(ratio).intValue()>0) 
					return new Float(ratio).intValue();
				else return 0;
			}
		};
		tableObject.setWidth(0,25);
		tableObject.setWidth(1,100);
		table.add(tableObject);
		table.validate();
		repaint();
		table.setVisible(true);
		}
}
/**
 * This is the method which drives this application.  It simply creates a new
 * instance of the ClassViewer.
 *
 * @param String args[] - all parameters are ignored.
*/
public static void main(String args[]){
	UIFrame t = new UIFrame();
	t.add(new ClassViewer());
	t.setSize(500,500);
	t.setTitle("Class File Decoder");
	t.addWindowListener(new com.ms.ui.event.UIWindowAdapter(){
		public void windowClosing(com.ms.ui.event.UIWindowEvent e){
			System.exit(0);
		}
	});
	t.setVisible(true);
}
/**
 * Once the class file has been decoded, this method is called which traverses
 * the ClassObject creating the appropriate nodes in the tree.  Each object
 * (including the ClassObject) is cast to the DecodeDisplayInterface, and the
 * DecodeDisplayInterface.getChildren() method is used to determine which 
 * sub-nodes are created.
 *
 * @see sample.decoder.DecodeDisplayInterface
*/
private void initTree(){
	if (classObject!=null){
		updateTree(buildTree(new UITree((
			(DecodeDisplayInterface)classObject).getName()),classObject));
	}
}
/**
 * This is a recursive method that creates subnodes within the tree view.  It
 * also generates a unique ID for each tree node, storing this id in a 
 * hashtable with the original DecodeDisplayInterface Object.  Later the 
 * ID can be used from the ItemEvent to retrieve the original object, and
 * update the table with the contents information.
 *
 * @param UITree node - The last node object that was created.
 * @param Object data - this is the Object containing the information for the
 *						node.
 *
 * @returns UITree - returns the node it was passed, but with the subobjects
 *						filled in.
*/
private UITree buildTree(UITree node, Object data){
	int id = (new Integer((new Double(Math.random()*1000000)).intValue()).intValue());
	treeLookup.put(new Integer(id),data);
	node.setID(id);
	node.addItemListener(this);
	if (data instanceof Vector){
		for (int j=0;j<((Vector)data).size();j++){
			if (((Vector)data).elementAt(j) instanceof DecodeDisplayInterface)
			node.add(buildTree(new UITree(((DecodeDisplayInterface)(
				(Vector)data).elementAt(j)).getName()),((Vector)data).elementAt(j)));
			else
				node.add(new UITree((String)((Vector)data).elementAt(j)));
		}
	}
	if (data instanceof DecodeDisplayInterface){
		for(Enumeration i=((DecodeDisplayInterface)data).getChildren().keys();
		i.hasMoreElements();){
			String key = (String)i.nextElement();
			node.add(buildTree(new UITree(key),
				((DecodeDisplayInterface)data).getChildren().get(key)));
		}
	}
	return node;
}
/**
 * Generic method for writting debug information to the screen.  By having all
 * methods in this class send their log information here, there is a central
 * point of change for turning debuggin off, or sending it to a differnet 
 * location.
 *
 * @param String data - message to be displayed.
*/
public void log(String data){
//	System.out.println(data);
}
}