/*
 *
 * sample.decoder.CCustomMethodStructure.java
 *
 * (C) Copyright 1995 - 1999 Microsoft Corporation.  All rights reserved.
 *
*/
package sample.decoder;

import java.util.Vector;
import java.util.Hashtable;
/**
 * This structure is used to describe a method that is custom marshalled.  It is
 * attached to the COM_MethodPool Attribute, which is applided to classes.
 * 
 * @see sample.decoder.COM_MethodPoolAttribute
 * 
 * @date	April 28,1998
 * @version	1.1
*/
public class CCustomMethodStructure extends DecodeObject implements DecodeDisplayInterface,
		java.io.Serializable{
	protected byte[] iIDIndex = new byte[2];
	protected byte[] vtblIndex = new byte[2];
	protected byte[] hookclsIndex = new byte[2];
	protected byte[] j2cmethodIndex = new byte[2];
	protected byte[] c2jmethodIndex = new byte[2];
	protected byte[] cbUserData = new byte[2];
	protected byte[] reserved = new byte[4];
	protected byte[] bRetFlavor = new byte[1];
	protected byte[] cbSpecialComArgs = new byte[1];
	protected byte[] userData = null;
	protected Vector specialArgs = new Vector();
	protected ConstantClassObject hook = null;
	protected ConstantUtfObject javatoc = null;
	protected ConstantUtfObject ctojava = null;
/**
 * This method is a member of the DecodeDisplayInterface and is used to 
 * return the object name and the attribute name.  Subclasses of this object
 * should over ride this method to provide more detailed information.
 *
 * @see com.ms.tools.decode.DecodeDisplayInterface
 *
 * @returns String name representing this object and its attribute type.
*/
public String getName(){
	return "CCustomMethod";
}
/**
 * Required by the DecodeDisplayInterface, returns any Vectors, or subobjects
 * which will need to be displayed as a subobject in a GUI.  The Hashtable is
 * indexed by a String that is to be used as a display name, the contents
 * will be either a Vector of Strings/Objects or a Decode Object.
 *
 * @returns Hashtable of subobjects/structures to be displayed as seperate
 *				nodes in the GUI.
*/
public Hashtable getChildren(){
	Hashtable val = new Hashtable();
	val.put("CSpecialArgumentEntry ",specialArgs);
	return val;
}
/**
 * After the initial decoding phase is complete, where all of the bytes have
 * been moved to their appropriate places, this method retrieves any 
 * referenced Constant Pool objects and stores the information with the 
 * object containing the refernece.
 *
 * @param Vector data - These are the ConstantObjects that were created from the 
 *						constant pool structures in the class file.
*/
public void link(Vector data){
	hook = (ConstantClassObject)data.elementAt(Decode.getIntegerValue(hookclsIndex));
	javatoc=(ConstantUtfObject)data.elementAt(Decode.getIntegerValue(j2cmethodIndex));
	ctojava=(ConstantUtfObject)data.elementAt(Decode.getIntegerValue(c2jmethodIndex));
}
/**
 * This is the workhorse for the object, it decodes an instance of itself starting at the
 * specified byte index into the stream provided.  This was created as a static method
 * rather than a constructor, as there may be times when this method returns a more
 * specialized object which is a subclass of itself.
 *
 * @param byte[] stream -  entire bytes of the class that is being decoded
 * @param int index - this is the index into the stream for the starting byte.
 * @param Vector constantPool - these are the constantPool objects, any index into the
 *								constant pool will be resolved by retrieving the object 
 *								at the required instance from this Vector.
 *
 * @returns Object - This is the decoded object which is a populated instance of this 
 *					class.
*/
public static Object getInstance(byte[] stream, int index,Vector constantPool){
        CCustomMethodStructure xconst = new CCustomMethodStructure();
		xconst.setByteStartIndex(index);
		xconst.iIDIndex[0]=stream[index++];
		xconst.iIDIndex[1]=stream[index++];
		xconst.addEntry("Interface ID Index",index-2,2,
			""+Decode.getIntegerValue(xconst.iIDIndex),
			xconst.iIDIndex);
		xconst.vtblIndex[0]=stream[index++];
		xconst.vtblIndex[1]=stream[index++];
		xconst.addEntry("VTable Index",index-2,2,
			""+Decode.getIntegerValue(xconst.vtblIndex),
			xconst.vtblIndex);
		xconst.hookclsIndex[0]=stream[index++];
		xconst.hookclsIndex[1]=stream[index++];
		xconst.addEntry("Hook Class Index",index-2,2,
			""+Decode.getIntegerValue(xconst.hookclsIndex),
			xconst.hookclsIndex);
		xconst.j2cmethodIndex[0]=stream[index++];
		xconst.j2cmethodIndex[1]=stream[index++];
		xconst.addEntry("Java to COM Method Index",index-2,2,
			""+Decode.getIntegerValue(xconst.j2cmethodIndex),
			xconst.j2cmethodIndex);
		xconst.c2jmethodIndex[0]=stream[index++];
		xconst.c2jmethodIndex[1]=stream[index++];
		xconst.addEntry("COM to Java Method Index",index-2,2,
			""+Decode.getIntegerValue(xconst.c2jmethodIndex),
			xconst.c2jmethodIndex);
		xconst.cbUserData[0]=stream[index++];
		xconst.cbUserData[1]=stream[index++];
		xconst.addEntry("CB User Data",index-2,2,
			""+Decode.getIntegerValue(xconst.cbUserData),
			xconst.cbUserData);
		xconst.reserved[0]=stream[index++];
		xconst.reserved[1]=stream[index++];
		xconst.reserved[2]=stream[index++];
		xconst.reserved[3]=stream[index++];
		xconst.addEntry("Interface ID Index",index-4,4,
			""+Decode.getIntegerValue(xconst.reserved),
			xconst.reserved);
		xconst.bRetFlavor[0]=stream[index++];
		xconst.bRetFlavor[1]=stream[index++];
		xconst.addEntry("B Return Flavor",index-2,2,
			""+Decode.getIntegerValue(xconst.bRetFlavor),
			xconst.bRetFlavor);
		xconst.cbSpecialComArgs[0]=stream[index++];
		xconst.cbSpecialComArgs[1]=stream[index++];
		xconst.addEntry("CB Special COM Args",index-2,2,
			""+Decode.getIntegerValue(xconst.cbSpecialComArgs),
			xconst.cbSpecialComArgs);
		for (int j=0;j<Decode.getIntegerValue(xconst.cbSpecialComArgs);j++){
			xconst.specialArgs.addElement(CSpecialArgumentEntryStructure.getInstance(stream,index,constantPool));
			index=((DecodeObject)xconst.specialArgs.lastElement()).getByteStopIndex()+1;	
		}
		xconst.userData = new byte[Decode.getIntegerValue(xconst.cbUserData)];
		for (int i=0;i<Decode.getIntegerValue(xconst.cbUserData);i++)
			xconst.userData[i]=stream[index++];
		xconst.addEntry("User Data",index-xconst.userData.length,xconst.userData.length,
			""+Decode.getIntegerValue(xconst.userData),
			xconst.userData);
		xconst.setByteStopIndex(index-1);
		xconst.link(constantPool);
        return xconst;
}
/**
 * Generic method for writting debug information to the screen.  By having all
 * methods in this class send their log information here, there is a central
 * point of change for turning debuggin off, or sending it to a differnet 
 * location.
 *
 * @param String data - message to be displayed.
*/
public synchronized void log(String data){
//	System.out.println(data);
}
}