/*
 *
 * sample.decoder.CExposedAsStructure.java
 *
 * (C) Copyright 1995 - 1999 Microsoft Corporation.  All rights reserved.
 *
*/
package sample.decoder;

import java.util.Vector;
import java.util.Hashtable;
/**
 * This structure is used to expose a Java method as a COM method. It is contained
 * within the COM_ExposedAs_Group attribute which is found at the class level.
 *
 * @see sample.decoder.COM_ExposedAs_GroupAttribute
 * 
 * @date	April 28,1998
 * @version	1.1
*/
public class CExposedAsStructure extends DecodeObject implements DecodeDisplayInterface,
		java.io.Serializable{
/**
 * These are the bytes which represent the flags for the CExposedAs Structure.
 */
	protected byte[] flags = new byte[2];
/**
 * These bytes contain a valid index into the method pool.
 */
	protected byte[] methodPoolIndex = new byte[2];
/**
 * This method is a member of the DecodeDisplayInterface and is used to 
 * return the object name and the attribute name.  Subclasses of this object
 * should over ride this method to provide more detailed information.
 *
 * @see com.ms.tools.decode.DecodeDisplayInterface
 *
 * @returns String name representing this object and its attribute type.
*/
public String getName(){
	return "CExposedAs";
}
/**
 * Required by the DecodeDisplayInterface, returns any Vectors, or subobjects
 * which will need to be displayed as a subobject in a GUI.  The Hashtable is
 * indexed by a String that is to be used as a display name, the contents
 * will be either a Vector of Strings/Objects or a Decode Object.
 * 
 * @returns Hashtable of subobjects/structures to be displayed as seperate
 *				nodes in the GUI.
*/
public Hashtable getChildren(){
	return new Hashtable();
}
/**
 * After the initial decoding phase is complete, where all of the bytes have
 * been moved to their appropriate places, this method retrieves any 
 * referenced Constant Pool objects and stores the information with the 
 * object containing the refernece.
 * 
 * @param Vector data - These are the ConstantObjects that were created from the 
 *						constant pool structures in the class file.
*/
public void link(Vector data){
}
/**
 * This is the workhorse for the object, it decodes an instance of itself starting at the
 * specified byte index into the stream provided.  This was created as a static method
 * rather than a constructor, as there may be times when this method returns a more
 * specialized object which is a subclass of itself.
 *
 * @param byte[] stream -  entire bytes of the class that is being decoded
 * @param int index - this is the index into the stream for the starting byte.
 * @param Vector constantPool - these are the constantPool objects, any index into the
 *								constant pool will be resolved by retrieving the object 
 *								at the required instance from this Vector.
 *
 * @returns Object - This is the decoded object which is a populated instance of this 
 *					class.
*/
public static Object getInstance(byte[] stream, int index,Vector constantPool){
        CExposedAsStructure xconst = new CExposedAsStructure();
		xconst.setByteStartIndex(index);
		xconst.flags[0]=stream[index++];
		xconst.flags[1]=stream[index++];
		xconst.addEntry("Flags",index-2,2,""+Decode.getIntegerValue(xconst.flags),
			xconst.flags);
		xconst.methodPoolIndex[0]=stream[index++];
		xconst.methodPoolIndex[1]=stream[index++];
		xconst.addEntry("Method Pool Index",index-2,2,""+Decode.getIntegerValue(xconst.methodPoolIndex),
			xconst.methodPoolIndex);
		xconst.setByteStopIndex(index-1);
        return xconst;
}
/**
 * Generic method for writting debug information to the screen.  By having all
 * methods in this class send their log information here, there is a central
 * point of change for turning debuggin off, or sending it to a differnet 
 * location.
 *
 * @param String data - message to be displayed.
*/
public synchronized void log(String data){
	//System.out.println(data);
}
}