/*
 *
 * sample.decoder.CFuncDescStructure.java
 *
 * (C) Copyright 1995 - 1999 Microsoft Corporation.  All rights reserved.
 *
*/
package sample.decoder;

import java.util.Vector;
import java.util.Hashtable;
/**
 * A general structure used to describe a method of any type.  This structure is 
 * contained within the COM_MethodPool attribute which is found at the class level.
 * 
 * @see sample.decoder.COM_MethodPoolAttribute
 * 
 * @date	April 28,1998
 * @version	1.1
*/
public class CFuncDescStructure extends DecodeObject implements DecodeDisplayInterface,
		java.io.Serializable{
	/**
	 * These are the bytes which represent the flags for the CFuncDesc Structure.
	 */
	protected byte[] flags = new byte[2];
	/**
	 * The bytes which contain the code block size.
	 */
	protected byte[] cbSize = new byte[2];
	/**
	 * Depending on the value of the flags, this field will contain a different structure.
	 *	
	 *			flags=8			CCustomMethodStructure
	 *			flags=1			CDispMethodStructure
	 *			default			CVtlbMethodStructure
	 */
	protected DecodeObject decodeO = null;
/**
 * This method is a member of the DecodeDisplayInterface and is used to 
 * return the object name and the attribute name.  Subclasses of this object
 * should over ride this method to provide more detailed information.
 *
 * @see com.ms.tools.decode.DecodeDisplayInterface
 *
 * @returns String name representing this object and its attribute type.
*/
public String getName(){
	return "CFuncDesc";
}
/**
 * Required by the DecodeDisplayInterface, returns any Vectors, or subobjects
 * which will need to be displayed as a subobject in a GUI.  The Hashtable is
 * indexed by a String that is to be used as a display name, the contents
 * will be either a Vector of Strings/Objects or a Decode Object.
 * 
 * @returns Hashtable of subobjects/structures to be displayed as seperate
 *				nodes in the GUI.
*/
public Hashtable getChildren(){
	Hashtable val = new Hashtable();
	val.put(((DecodeDisplayInterface)decodeO).getName(),
			decodeO);
	return val;
}
/**
 * This is the workhorse for the object, it decodes an instance of itself starting at the
 * specified byte index into the stream provided.  This was created as a static method
 * rather than a constructor, as there may be times when this method returns a more
 * specialized object which is a subclass of itself.
 *
 * @param byte[] stream -  entire bytes of the class that is being decoded
 * @param int index - this is the index into the stream for the starting byte.
 * @param Vector constantPool - these are the constantPool objects, any index into the
 *								constant pool will be resolved by retrieving the object 
 *								at the required instance from this Vector.
 *
 * @returns Object - This is the decoded object which is a populated instance of this 
 *					class.
*/
public static Object getInstance(byte[] stream, int index,Vector constantPool){
        CFuncDescStructure xconst = new CFuncDescStructure();
		xconst.setByteStartIndex(index);
		xconst.cbSize[0]=stream[index++];
		xconst.cbSize[1]=stream[index++];
		xconst.addEntry("CB Size",index-2,2,""+Decode.getIntegerValue(xconst.cbSize),
			xconst.cbSize);
		xconst.flags[0]=stream[index++];
		xconst.flags[1]=stream[index++];
		if ((xconst.flags[1]&8)==8){
		xconst.decodeO = (DecodeObject)CCustomMethodStructure.getInstance
				(stream,index,constantPool);
		}else if ((xconst.flags[1]&1)==1){
			xconst.decodeO = (DecodeObject)CDispMethodStructure.getInstance
				(stream,index,constantPool);
		}else{
			xconst.decodeO = (DecodeObject)CVtblMethodStructure.getInstance
				(stream,index,constantPool);
		}
		xconst.addEntry("Flags",xconst.getByteStartIndex()+2,2,xconst.decodeO.toString(),
			xconst.flags);
		index=xconst.decodeO.getByteStopIndex()+1;
		xconst.setByteStopIndex(xconst.getByteStartIndex()+
								Decode.getIntegerValue(xconst.cbSize)-1);
        return xconst;
}
/**
 * Generic method for writting debug information to the screen.  By having all
 * methods in this class send their log information here, there is a central
 * point of change for turning debuggin off, or sending it to a differnet 
 * location.
 *
 * @param String data - message to be displayed.
*/
public synchronized void log(String data){
	//System.out.println(data);
}
}