/*
 *
 * sample.decoder.COM_CustDataAttribute.java
 *
 * (C) Copyright 1995 - 1999 Microsoft Corporation.  All rights reserved.
 *
 */
package sample.decoder;

import java.util.Vector;
import java.util.Hashtable;
/**
 * Attribute Name:  COM_CustData
 * Attribute Level: Class
 *
 * Restrictions:
 *   The class access flags must be a subset of:
 *      ACC_PUBLIC
 *      ACC_FINAL
 *      ACC_INTERFACE
 *      ACC_ABSTRACT
 *
 * This attribute exposes type library CUSTDATA elements.  Callers may
 * query a classes type library at runtime (such as through
 * IProvideClassInfo) for these attributes using methods like
 * ITypeInfo2::GetCustData.
 *
 * Attribute elements:
 *
 *      WORD nCustDatas
 *
 *          This field indicates the number of aCustData's defined in this
 *          table.
 *
 *      WORD aCustData[].GuidIndex
 *
 *          Index into the COM_GuidPool table.  Represents the GUID
 *          identity for this custom data element.
 *
 *      WORD aCustData[].DataIndex
 *
 *          Index into this classes constant pool.  Only certain types of
 *          constant pool entries are supported (currently CP_Utf8 and
 *          CP_Integer).
 *
 *========================================================================
 *
 * typedef struct {
 *   WORD        nCustDatas;     // # of elements in table
 *   // {
 *   //      WORD GuidIndex;
 *   //      WORD DataIndex;
 *   // }        aCustData[nCustDatas];
 * } MCCDP_Attr;
 *
 * support for attribute COM_CustData (struct MCCDP_Attr)
 * 
 * SCOPE : Class
 */
public class COM_CustDataAttribute extends AttributeObject
    implements DecodeDisplayInterface, java.io.Serializable{

	private int                 m_nCustDatas;
    private ConstantObject[]    m_aData;

    // unresolved information
    private int[]   m_aGuidIndices;
    private int[]   m_aDataIndices;
/**
 * After the initial decoding phase is complete, where all of the bytes have
 * been moved to their appropriate places, this method retrieves any 
 * referenced Constant Pool objects and stores the information with the 
 * object containing the refernece.
 *
 * @param Vector data - These are the ConstantObjects that were created from the 
 *						constant pool structures in the class file.
 */
    public void link( Vector data ){
        super.link( data );
        if( m_nCustDatas > 0 ){
            m_aData = new ConstantObject[ m_nCustDatas ];
            for( int index = 0; index < m_nCustDatas; index++ )
                m_aData[ index ] = (ConstantObject)(data.elementAt( m_aDataIndices[ index ] ));
            m_aDataIndices = null;
        }
	}
/**
 * This method is a member of the DecodeDisplayInterface and is used to 
 * return the object name and the attribute name.  Subclasses of this object
 * should over ride this method to provide more detailed information.
 *
 * @see sample.decoder.DecodeDisplayInterface
 *
 * @returns String name representing this object and its attribute type.
 */
    public String getName(){
	    return "COM_CustData";
    }
/**
 * Default method for determining a display String which adequatly 
 * represents this object and its contents.  Subclasses of this object 
 * should override this method to provide more detailed information.
 *
 * @returns String representing this object.
 */
    public String toString(){
	    return getName();
    }
/**
 *         typedef struct {
 *           WORD        nCustDatas;     // # of elements in table
 *           // {
 *           //      WORD GuidIndex;
 *           //      WORD DataIndex;
 *           // }        aCustData[nCustDatas];
 *       } MCCDP_Attr;
 */
    int readAttribute( byte[] stream, int index, Vector constantPool ){
        // first word is number of datas in table
        m_nCustDatas = (stream[ index++ ] << 8) | stream[ index++ ];
		addEntry("Custom Data Structures Count",index-4,4,""+m_nCustDatas,
				 new byte[]{new Integer(m_nCustDatas).byteValue()});
        if( m_nCustDatas > 0 ){
            m_aGuidIndices = new int[ m_nCustDatas ];
            m_aDataIndices = new int[ m_nCustDatas ];
            // now read datas
            for( int iEntry = 0; iEntry < m_nCustDatas; iEntry++ ){
                // first the guid index
                m_aGuidIndices[ iEntry ] = (stream[ index++ ] << 8) |
                                                        stream[ index++ ];
			addEntry("GUID Index",index-4,4,""+m_aGuidIndices,
				 new byte[]{new Integer(m_aGuidIndices[iEntry]).byteValue()});
                // then the data index
                m_aDataIndices[ iEntry ] = (stream[ index++ ] << 8) |
                                                        stream[ index++ ];
			addEntry("DataIndex",index-4,4,""+m_aDataIndices,
				 new byte[]{new Integer(m_aDataIndices[iEntry]).byteValue()});
            }
        }
        return index;
    }
/**
 * This is the workhorse for the object, it decodes an instance of itself starting at the
 * specified byte index into the stream provided.  This was created as a static method
 * rather than a constructor, as there may be times when this method returns a more
 * specialized object which is a subclass of itself.
 *
 * @param byte[] stream -  entire bytes of the class that is being decoded
 * @param int index - this is the index into the stream for the starting byte.
 * @param Vector constantPool - these are the constantPool objects, any index into the
 *								constant pool will be resolved by retrieving the object 
 *								at the required instance from this Vector.
 *
 * @returns Object - This is the decoded object which is a populated instance of this 
 *					class.
 */
    public static Object getInstance( byte[] stream, int index, Vector constantPool){
        COM_Class_TypeAttribute attr = new COM_Class_TypeAttribute();
        // do the real decoding for this attribute.
        index = attr.readAttribute( stream, index, constantPool );
        return attr;
    }
}