/*
 *
 * sample.decoder.COM_ExposedAs_GroupAttribute.java
 *
 * (C) Copyright 1995 - 1999 Microsoft Corporation.  All rights reserved.
 *
*/
package sample.decoder;

import java.util.Vector;
import java.util.Hashtable;
/**
 * The COM Exposed As Group attribute exposes a Java method to COM and prescribes the 
 * external interface (v-table or dispatch) of a COM-Callable wrapper created for a
 * java object.  This attribute is generated from the @com.method directive and is
 * part of the Java/COM integration subsystem.
 * 
 * SCOPE : Class
 * 
 * @see sample.decoder.CExposedAsStructure
 *
 * @date	April 28,1998
 * @version	1.1
*/
public class COM_ExposedAs_GroupAttribute extends AttributeObject implements DecodeDisplayInterface,
		java.io.Serializable{
/**
 * Byte value for this attributes flags.
 */
	protected byte[] flags = new byte[2];
/**
 * This is the number of CExposedAs structures contained in this attribute.
 */
	protected byte[] nExposedAs = new byte[2];
/**
 * Vector containing the decoded Exposed As Structures.
 */
	protected Vector exposedAsStructures = new Vector();
/**
 * This method is a member of the DecodeDisplayInterface and is used to 
 * return the object name and the attribute name.  Subclasses of this object
 * should over ride this method to provide more detailed information.
 *
 * @see com.ms.tools.decode.DecodeDisplayInterface
 *
 * @returns String name representing this object and its attribute type.
*/
public String getName(){
	return "COM Exposed As Group";
}
/**
 * Required by the DecodeDisplayInterface, returns any Vectors, or subobjects
 * which will need to be displayed as a subobject in a GUI.  The Hashtable is
 * indexed by a String that is to be used as a display name, the contents
 * will be either a Vector of Strings/Objects or a Decode Object.
 * 
 * @returns Hashtable of subobjects/structures to be displayed as seperate
 *				nodes in the GUI.
*/
public Hashtable getChildren(){
	Hashtable val =  super.getChildren();
	Vector att = exposedAsStructures;
	for (int i=0;i<att.size();i++)
		val.put(((DecodeDisplayInterface)att.elementAt(i)).getName()+i,
				att.elementAt(i));
	return val;
}
/**
 * After the initial decoding phase is complete, where all of the bytes have
 * been moved to their appropriate places, this method retrieves any 
 * referenced Constant Pool objects and stores the information with the 
 * object containing the refernece.
 * 
 * @param Vector data - These are the ConstantObjects that were created from the 
 *						constant pool structures in the class file.
*/
public void link(Vector data){
	super.link(data);
}
/**
 * This is the workhorse for the object, it decodes an instance of itself starting at the
 * specified byte index into the stream provided.  This was created as a static method
 * rather than a constructor, as there may be times when this method returns a more
 * specialized object which is a subclass of itself.
 *
 * @param byte[] stream -  entire bytes of the class that is being decoded
 * @param int index - this is the index into the stream for the starting byte.
 * @param Vector constantPool - these are the constantPool objects, any index into the
 *								constant pool will be resolved by retrieving the object 
 *								at the required instance from this Vector.
 *
 * @returns Object - This is the decoded object which is a populated instance of this 
 *					class.
*/
public static Object getInstance(byte[] stream, int index,Vector constantPool){
        COM_ExposedAs_GroupAttribute xconst = new COM_ExposedAs_GroupAttribute();
		xconst.flags[0]=stream[index++];
		xconst.flags[1]=stream[index++];
		xconst.addEntry("Flags",index-2,2,""+Decode.getIntegerValue(xconst.flags),
				 xconst.flags);
		xconst.nExposedAs[0]=stream[index++];
		xconst.nExposedAs[1]=stream[index++];
		xconst.addEntry("Number of Exposed As Structures",index-2,2,""+Decode.getIntegerValue(xconst.nExposedAs),
				 xconst.nExposedAs);
		for (int i=0;i<Decode.getIntegerValue(xconst.nExposedAs);i++){
			xconst.exposedAsStructures.addElement(
				CExposedAsStructure.getInstance(stream,index,constantPool)
			);
			index=((DecodeObject)xconst.exposedAsStructures.lastElement()).getByteStopIndex()+1;
		}
	    return xconst;
}
/**
 * Generic method for writting debug information to the screen.  By having all
 * methods in this class send their log information here, there is a central
 * point of change for turning debuggin off, or sending it to a differnet 
 * location.
 *
 * @param String data - message to be displayed.
*/
public synchronized void log(String data){
//	System.out.println(data);
}
}