/*
 *
 * sample.decoder.COM_GuidPoolAttribute.java
 *
 * (C) Copyright 1995 - 1999 Microsoft Corporation.  All rights reserved.
 *
 */
package sample.decoder;

import java.util.Vector;
import java.util.Hashtable;
import com.ms.com.Guid;
/**
 * Attribute Name:  COM_GuidPool
 * Attribute Level: Class
 *
 * Restrictions:
 *   The class access flags must be a subset of:
 *      ACC_PUBLIC
 *      ACC_FINAL
 *      ACC_INTERFACE
 *      ACC_ABSTRACT
 *
 *
 * This attribute stores the IID's and CLSID's that are referenced by index
 * from the MethodPool. Valid indices range from 0 inclusive to nGuids
 * exclusive. The index 0xffff is reserved to indicate the "null" index.
 * Hence, the GuidPool can contain a maximum of 65535 guids.
 *
 * Attribute elements:
 *
 *      WORD nGuids
 *
 *         This field indicates the number of GUID's defined in this
 *         table.
 *
 *
 *      GUID aGuid[nGuids]
 *
 *         This array stores the actual GUID representations.
 *========================================================================
 *
 * typedef struct {
 *     WORD        nGuids;         // # of guids in table
 *     // GUID     aGuid[nGuids];
 * } MCGP_Attr;
 *
 * support for attribute COM_GuidPool (struct MCGP_Attr)
 * 
 * SCOPE : CLASS
 * 
 */
public class COM_GuidPoolAttribute extends AttributeObject
    implements DecodeDisplayInterface, java.io.Serializable{
    
	private int     m_nGuids;
    private Guid[]  m_aGuids;
/**
 * After the initial decoding phase is complete, where all of the bytes have
 * been moved to their appropriate places, this method retrieves any 
 * referenced Constant Pool objects and stores the information with the 
 * object containing the refernece.
 *
 * @param Vector data - These are the ConstantObjects that were created from the 
 *						constant pool structures in the class file.
 */
	public void link( Vector data ){
        super.link( data );
	}
/**
 * This method is a member of the DecodeDisplayInterface and is used to 
 * return the object name and the attribute name.  Subclasses of this object
 * should over ride this method to provide more detailed information.
 *
 * @see sample.decoder.DecodeDisplayInterface
 *
 * @returns String name representing this object and its attribute type.
 */
	public String getName(){
	    return "COM_GuidPool";
    }
/**
 * Default method for determining a display String which adequatly 
 * represents this object and its contents.  Subclasses of this object 
 * should override this method to provide more detailed information.
 *
 * @returns String representing this object.
 */
    public String toString(){
	    return getName();
	}
/**
 *       typedef struct {
 *           WORD        nGuids;         // # of guids in table
 *           // GUID     aGuid[nGuids];
 *       } MCGP_Attr;
 */
    int readAttribute( byte[] stream, int index, Vector constantPool ){
        // first word is number of guids in table
		try{
	        m_nGuids = (stream[ index++ ] << 8) | stream[ index++ ];
	        // now read guids
			addEntry("Number of GUIDs",index-2,2,
				 ""+m_nGuids,new byte[]{new Integer(m_nGuids).byteValue()});
			m_aGuids=new Guid[m_nGuids];
			for( int iGuid = 0; iGuid < m_nGuids; iGuid++ ){
				int     d1 = (stream[ index++ ] << 24) |
                                        (stream[ index++ ] << 16) |
                                        (stream[ index++ ] << 8) |
                                        stream[ index++ ];
				short   d2 = (short)((stream[ index++ ] << 8) |
                                                stream[ index++ ]),
                    d3 = (short)((stream[ index++ ] << 8) |
                                                stream[ index++ ]);
				byte    d4 = stream[ index++ ],
					    d5 = stream[ index++ ],
						d6 = stream[ index++ ],
						d7 = stream[ index++ ],
						d8 = stream[ index++ ],
						d9 = stream[ index++ ],
						d10 = stream[ index++ ],
						d11 = stream[ index++ ];
				m_aGuids[ iGuid ] = new Guid( d1, d2, d3, d4, d5, d6, d7, d8, d9, d10, d11 );
				addEntry("GUID",index-17,16,m_aGuids[iGuid].toString(),
				 m_aGuids[iGuid].toByteArray());
			}
		}catch(Exception ii){
			System.out.println("COM_GuidPoolAttribute: Exception "+ii);
		}
        return index;
    }
/**
 * This is the workhorse for the object, it decodes an instance of itself starting at the
 * specified byte index into the stream provided.  This was created as a static method
 * rather than a constructor, as there may be times when this method returns a more
 * specialized object which is a subclass of itself.
 *
 * @param byte[] stream -  entire bytes of the class that is being decoded
 * @param int index - this is the index into the stream for the starting byte.
 * @param Vector constantPool - these are the constantPool objects, any index into the
 *								constant pool will be resolved by retrieving the object 
 *								at the required instance from this Vector.
 *
 * @returns Object - This is the decoded object which is a populated instance of this 
 *					class.
 */
	public static Object getInstance( byte[] stream, int index, Vector constantPool ) { 
        COM_GuidPoolAttribute attr = new COM_GuidPoolAttribute();
        index = attr.readAttribute( stream, index, constantPool );
        return attr;
    }
}