/*
 *
 * sample.decoder.COM_MapsToAttribute.java
 *
 * (C) Copyright 1995 - 1999 Microsoft Corporation.  All rights reserved.
 *
*/
package sample.decoder;

import java.util.Vector;
import java.util.Hashtable;
/**
 * The COM MapsTo attribute provides type information about each field in a COM class.
 * This atribute is generated by the com.struct, com.structmap, dllStruct and 
 * dll.structmap directives.  This attribute is part of the Java/COM integration subsystem 
 * and is also used by J/Direct.
 * 
 * SCOPE : Field
 * 
 * @see sample.decoder.CTypeDescStructure
 *
 * @date	April 28,1998
 * @version	1.0
*/
public class COM_MapsToAttribute extends AttributeObject implements DecodeDisplayInterface,
		java.io.Serializable{
	
	protected byte[] flags = new byte[2];
	protected byte[] wPad = new byte[2];
	protected byte[] dwOffset = new byte[4];
	protected CTypeDescStructure typeDesc = null;
/**
 * This method is a member of the DecodeDisplayInterface and is used to 
 * return the object name and the attribute name.  Subclasses of this object
 * should over ride this method to provide more detailed information.
 *
 * @see sample.decoder.DecodeDisplayInterface
 *
 * @returns String name representing this object and its attribute type.
*/
public String getName(){
	return "COM_MapsTo";
}
/**
 * Required by the DecodeDisplayInterface, returns any Vectors, or subobjects
 * which will need to be displayed as a subobject in a GUI.  The Hashtable is
 * indexed by a String that is to be used as a display name, the contents
 * will be either a Vector of Strings/Objects or a Decode Object.
 *
 * @returns Hashtable of subobjects/structures to be displayed as seperate
 *				nodes in the GUI.
*/
public Hashtable getChildren(){
	Hashtable val = super.getChildren();
	val.put(((DecodeDisplayInterface)typeDesc).getName(),typeDesc);
	return val;
}
/**
 * After the initial decoding phase is complete, where all of the bytes have
 * been moved to their appropriate places, this method retrieves any 
 * referenced Constant Pool objects and stores the information with the 
 * object containing the refernece.
 *
 * @param Vector data - These are the ConstantObjects that were created from the 
 *						constant pool structures in the class file.
*/
public void link(Vector data){
	super.link(data);
}
/**
 * This is the workhorse for the object, it decodes an instance of itself starting at the
 * specified byte index into the stream provided.  This was created as a static method
 * rather than a constructor, as there may be times when this method returns a more
 * specialized object which is a subclass of itself.
 *
 * @param byte[] stream -  entire bytes of the class that is being decoded
 * @param int index - this is the index into the stream for the starting byte.
 * @param Vector constantPool - these are the constantPool objects, any index into the
 *								constant pool will be resolved by retrieving the object 
 *								at the required instance from this Vector.
 *
 * @returns Object - This is the decoded object which is a populated instance of this 
 *					class.
*/
public static Object getInstance(byte[] stream, int index,Vector constantPool){
        COM_MapsToAttribute xconst = new COM_MapsToAttribute();
		xconst.flags[0]=stream[index++];
		xconst.flags[1]=stream[index++];
		xconst.addEntry("Flags",index-2,2,""+Decode.getIntegerValue(xconst.flags),
				 xconst.flags);
		xconst.wPad[0]=stream[index++];
		xconst.wPad[1]=stream[index++];
		xconst.addEntry("Pad",index-2,2,""+Decode.getIntegerValue(xconst.wPad),
				 xconst.wPad);
		xconst.dwOffset[0]=stream[index++];
		xconst.dwOffset[1]=stream[index++];
		xconst.dwOffset[2]=stream[index++];
		xconst.dwOffset[3]=stream[index++];
		xconst.addEntry("Offset",index-4,4,""+Decode.getIntegerValue(xconst.dwOffset),
				 xconst.dwOffset);
		xconst.typeDesc = (CTypeDescStructure)CTypeDescStructure.getInstance(stream,index,constantPool);
        return xconst;
}
/**
 * Generic method for writting debug information to the screen.  By having all
 * methods in this class send their log information here, there is a central
 * point of change for turning debuggin off, or sending it to a differnet 
 * location.
 *
 * @param String data - message to be displayed.
*/
public synchronized void log(String data){
//	System.out.println(data);
}
}