/*
 *
 * sample.decoder.COM_MethodPoolAttribute.java
 *
 * (C) Copyright 1995 - 1999 Microsoft Corporation.  All rights reserved.
 *
*/
package sample.decoder;

import java.util.Vector;
import java.util.Hashtable;
/**
 * The COM Method Pool attribute stores the method signatures of the COM methods
 * that are invoked by and implemented by the Java methods in the Java class file.
 * These signatures are referenced by index by other attributes in the class.  
 * This attribute is generated by the com.method and com.parameters directive and
 * is part of the Java/COM integration subsystem.
 * 
 * SCOPE : Class
 * 
 * @see sample.decoder.CCustomMethodStructure
 * @see sample.decoder.CDispMethodStructure
 * @see sample.decoder.CFuncDescStructure
 * @see sample.decoder.CSpecialArgumentEntryStructure
 * @see sample.decoder.CTypeDescStructure
 * @see sample.decoder.CVarTypeDescStructure
 * @see sample.decoder.CVtblMethodStructure
 * 
 * @date	April 28,1998
 * @version	1.1
*/
public class COM_MethodPoolAttribute extends AttributeObject implements DecodeDisplayInterface,
		java.io.Serializable{
	protected byte[] nFuncs = new byte[2];
	protected Vector funcs = new Vector();
/**
 * This method is a member of the DecodeDisplayInterface and is used to 
 * return the object name and the attribute name.  Subclasses of this object
 * should over ride this method to provide more detailed information.
 *
 * @see sample.decoder.DecodeDisplayInterface
 *
 * @returns String name representing this object and its attribute type.
*/
public String getName(){
	return "COM Method Pool";
}
/**
 * Required by the DecodeDisplayInterface, returns any Vectors, or subobjects
 * which will need to be displayed as a subobject in a GUI.  The Hashtable is
 * indexed by a String that is to be used as a display name, the contents
 * will be either a Vector of Strings/Objects or a Decode Object.
 *
 * @returns Hashtable of subobjects/structures to be displayed as seperate
 *				nodes in the GUI.
*/
public Hashtable getChildren(){
	Hashtable val = super.getChildren();
	for (int i=0;i<funcs.size();i++)
		val.put(((DecodeDisplayInterface)funcs.elementAt(i)).getName()+i,
				funcs.elementAt(i));
	return val;
}
/**
 * After the initial decoding phase is complete, where all of the bytes have
 * been moved to their appropriate places, this method retrieves any 
 * referenced Constant Pool objects and stores the information with the 
 * object containing the refernece.
 *
 * @param Vector data - These are the ConstantObjects that were created from the 
 *						constant pool structures in the class file.
*/
public void link(Vector data){
	super.link(data);
}
/**
 * This is the workhorse for the object, it decodes an instance of itself starting at the
 * specified byte index into the stream provided.  This was created as a static method
 * rather than a constructor, as there may be times when this method returns a more
 * specialized object which is a subclass of itself.
 *
 * @param byte[] stream -  entire bytes of the class that is being decoded
 * @param int index - this is the index into the stream for the starting byte.
 * @param Vector constantPool - these are the constantPool objects, any index into the
 *								constant pool will be resolved by retrieving the object 
 *								at the required instance from this Vector.
 *
 * @returns Object - This is the decoded object which is a populated instance of this 
 *					class.
*/
public static Object getInstance(byte[] stream, int index,Vector constantPool){
        COM_MethodPoolAttribute xconst = new COM_MethodPoolAttribute();
		xconst.nFuncs[0]=stream[index++];
		xconst.nFuncs[1]=stream[index++];
		xconst.addEntry("Number of Functions",index-2,2,""+Decode.getIntegerValue(xconst.nFuncs),
						xconst.nFuncs);
		for (int i=0;i<Decode.getIntegerValue(xconst.nFuncs);i++){
			xconst.funcs.addElement(CFuncDescStructure.getInstance(stream,index,constantPool));	
			index=((DecodeObject)xconst.funcs.lastElement()).getByteStopIndex()+1;	
		}
        return xconst;
}
/**
 * Generic method for writting debug information to the screen.  By having all
 * methods in this class send their log information here, there is a central
 * point of change for turning debuggin off, or sending it to a differnet 
 * location.
 *
 * @param String data - message to be displayed.
*/
public synchronized void log(String data){
	//System.out.println(data);
}
}