/*
 *
 * sample.decoder.COM_ProxiesToAttribute.java
 *
 * (C) Copyright 1995 - 1999 Microsoft Corporation.  All rights reserved.
 *
*/
package sample.decoder;

import java.util.Vector;
import java.util.Hashtable;
/**
 * The COM Proxies To attribute exposes a COM method to Java and describes, for a
 * given method or field of a Java-callable proxy, exactly which COM member function
 * it maps to in the underlying COM object.  This attribute is generated by the java
 * compilerwhen it sees a class containing a Java method that implements an interface
 * tagged with the com.interface directive.  This attribute is generated for the 
 * implementing method and is part of the Java/COM integration subsystem.
 * 
 * SCOPE : Method or Field
 * 
 * @version 1.0 
 */
public class COM_ProxiesToAttribute extends AttributeObject
    implements DecodeDisplayInterface, java.io.Serializable{

	private int m_nMCP_Flags;
    private int m_nMethodPoolIndex;
/**
 * After the initial decoding phase is complete, where all of the bytes have
 * been moved to their appropriate places, this method retrieves any 
 * referenced Constant Pool objects and stores the information with the 
 * object containing the refernece.
 *
 * @param Vector data - These are the ConstantObjects that were created from the 
 *						constant pool structures in the class file.
 */
    public void link( Vector data ){
        super.link( data );
	}
/**
 * This method is a member of the DecodeDisplayInterface and is used to 
 * return the object name and the attribute name.  Subclasses of this object
 * should over ride this method to provide more detailed information.
 *
 * @see sample.decoder.DecodeDisplayInterface
 *
 * @returns String name representing this object and its attribute type.
 */
    public String getName(){
	    return "COM_ProxiesTo";
    }
/**
 * Default method for determining a display String which adequatly 
 * represents this object and its contents.  Subclasses of this object 
 * should override this method to provide more detailed information.
 *
 * @returns String representing this object.
 */
    public String toString(){
	    return getName();
    }
/**
 * This is the workhorse for the object, it decodes an instance of itself starting at the
 * specified byte index into the stream provided.  This was created as a static method
 * rather than a constructor, as there may be times when this method returns a more
 * specialized object which is a subclass of itself.
 *
 * @param byte[] stream -  entire bytes of the class that is being decoded
 * @param int index - this is the index into the stream for the starting byte.
 * @param Vector constantPool - these are the constantPool objects, any index into the
 *								constant pool will be resolved by retrieving the object 
 *								at the required instance from this Vector.
 *
 * @returns Object - This is the decoded object which is a populated instance of this 
 *					class.
 */
    public static Object getInstance( byte[] stream, int index,Vector constantPool ){
        COM_ProxiesToAttribute  attr = new COM_ProxiesToAttribute();
        attr.m_nMCP_Flags = (stream[ index++ ] << 8) | stream[ index++ ];
		attr.addEntry("MCP Flags",index-2,2,""+attr.m_nMCP_Flags,
					  new byte[]{new Integer(attr.m_nMCP_Flags).byteValue()});
        attr.m_nMethodPoolIndex = (stream[ index++ ] << 8) | stream[ index++ ];
		attr.addEntry("Method Pool Index",index-2,2,""+attr.m_nMethodPoolIndex,
					  new byte[]{new Integer(attr.m_nMethodPoolIndex).byteValue()});
		return attr;
    }
}