/*
 *
 * com.ms.tools.decode.COM_RegisterAttribute.java
 *
 * (C) Copyright 1995 - 1999 Microsoft Corporation.  All rights reserved.
 *
*/
package sample.decoder;

import java.util.Vector;
import java.util.Hashtable;
/**
 * The COM_register attribute stores data for registering a class as a COM coclass.
 * This attribute is generated by the com.register directive and is part of the
 * Java/COM integration subsystem.
 * 
 * SCOPE : Class
 * 
 * @date	June 19,1998
 * @version	1.0
*/
public class COM_RegisterAttribute extends AttributeObject implements DecodeDisplayInterface,
		java.io.Serializable{
	
	protected byte[] flags = new byte[2];
	protected byte[] clsidIndex = new byte[2];
	protected byte[] tlbIndex = new byte[2];
	protected byte[] majorTlb = new byte[2];
	protected byte[] minorTlb = new byte[2];
	protected byte[] progIDIndex = new byte[2];
	protected byte[] descIndex = new byte[2];
/**
 * This method is a member of the DecodeDisplayInterface and is used to 
 * return the object name and the attribute name.  Subclasses of this object
 * should over ride this method to provide more detailed information.
 *
 * @see sample.decoder.DecodeDisplayInterface
 *
 * @returns String name representing this object and its attribute type.
*/
public String getName(){
	return "COM_Register";
}
/**
 * Required by the DecodeDisplayInterface, returns any Vectors, or subobjects
 * which will need to be displayed as a subobject in a GUI.  The Hashtable is
 * indexed by a String that is to be used as a display name, the contents
 * will be either a Vector of Strings/Objects or a Decode Object.
 *
 * @returns Hashtable of subobjects/structures to be displayed as seperate
 *				nodes in the GUI.
*/
public Hashtable getChildren(){
	return super.getChildren();
}
/**
 * This is the workhorse for the object, it decodes an instance of itself starting at the
 * specified byte index into the stream provided.  This was created as a static method
 * rather than a constructor, as there may be times when this method returns a more
 * specialized object which is a subclass of itself.
 *
 * @param byte[] stream -  entire bytes of the class that is being decoded
 * @param int index - this is the index into the stream for the starting byte.
 * @param Vector constantPool - these are the constantPool objects, any index into the
 *								constant pool will be resolved by retrieving the object 
 *								at the required instance from this Vector.
 *
 * @returns Object - This is the decoded object which is a populated instance of this 
 *					class.
*/
public static Object getInstance(byte[] stream, int index,Vector constantPool){
	COM_RegisterAttribute xconst = new COM_RegisterAttribute();
	 try{
     	xconst.flags[0]=stream[index++];
		xconst.flags[1]=stream[index++];
		xconst.addEntry("Flags",index-2,2,""+Decode.getIntegerValue(xconst.flags),
						xconst.flags);
		xconst.clsidIndex[0]=stream[index++];
		xconst.clsidIndex[1]=stream[index++];
		xconst.addEntry("CLSID Index",index-2,2,""+Decode.getIntegerValue(xconst.clsidIndex),
						xconst.clsidIndex);
		xconst.tlbIndex[0]=stream[index++];
		xconst.tlbIndex[1]=stream[index++];
		xconst.addEntry("Type Library ID Index",index-2,2,""+Decode.getIntegerValue(xconst.tlbIndex),
						xconst.tlbIndex);
		xconst.majorTlb[0]=stream[index++];
		xconst.majorTlb[1]=stream[index++];
		xconst.addEntry("Major Type Library Version",index-2,2,""+Decode.getIntegerValue(xconst.majorTlb),
						xconst.majorTlb);
		xconst.minorTlb[0]=stream[index++];
		xconst.minorTlb[1]=stream[index++];
		xconst.addEntry("Minor Type Library Version",index-2,2,""+Decode.getIntegerValue(xconst.minorTlb),
						xconst.minorTlb);
		xconst.progIDIndex[0]=stream[index++];
		xconst.progIDIndex[1]=stream[index++];
		xconst.addEntry("Program ID Index",index-2,2,""+constantPool.elementAt(Decode.getIntegerValue(xconst.progIDIndex)).toString(),
						xconst.progIDIndex);
		xconst.descIndex[0]=stream[index++];
		xconst.descIndex[1]=stream[index++];
		xconst.addEntry("Description Index",index-2,2,""+constantPool.elementAt(Decode.getIntegerValue(xconst.descIndex)).toString(),
						xconst.descIndex);
	 }catch(Exception ii){
		 System.out.println("COM_Safety: Exception "+ii);
	 }
		return xconst;
}
/**
 * Generic method for writting debug information to the screen.  By having all
 * methods in this class send their log information here, there is a central
 * point of change for turning debuggin off, or sending it to a differnet 
 * location.
 *
 * @param String data - message to be displayed.
*/
public synchronized void log(String data){
//	System.out.println(data);
}
}