/*
 *
 * sample.decoder.COM_SafetyAttribute.java
 *
 * (C) Copyright 1995 - 1999 Microsoft Corporation.  All rights reserved.
 *
*/
package sample.decoder;

import java.util.Vector;
import java.util.Hashtable;
/**
 * The COM_Safety attribute indicates whether it is safe to call methods on this class.
 * This attribute indicates that a COM class is safe to use by other classes and 
 * Microsoft VM will allow untrusted classes to link with COM classes specified with
 * this attribute.  A COM class can normally only be used from trusted classes.  This
 * attribute is generated by the safe parameter of the com.class, com.struct and 
 * dll.struct directives, and is part of the Java/COM integration subsystem.
 * 
 * SCOPE : Class
 *
 * @date	April 28,1998
 * @version	1.0
*/
public class COM_SafetyAttribute extends AttributeObject implements DecodeDisplayInterface,
		java.io.Serializable{
/**
 * This byte contains the data portion of this attribute
 */
	protected byte[] data = null;
/**
 * This method is a member of the DecodeDisplayInterface and is used to 
 * return the object name and the attribute name.  Subclasses of this object
 * should over ride this method to provide more detailed information.
 *
 * @see sample.decoder.DecodeDisplayInterface
 *
 * @returns String name representing this object and its attribute type.
*/
public String getName(){
	return "COM_Safety";
}
/**
 * This is the workhorse for the object, it decodes an instance of itself starting at the
 * specified byte index into the stream provided.  This was created as a static method
 * rather than a constructor, as there may be times when this method returns a more
 * specialized object which is a subclass of itself.
 *
 * @param byte[] stream -  entire bytes of the class that is being decoded
 * @param int index - this is the index into the stream for the starting byte.
 * @param Vector constantPool - these are the constantPool objects, any index into the
 *								constant pool will be resolved by retrieving the object 
 *								at the required instance from this Vector.
 *
 * @returns Object - This is the decoded object which is a populated instance of this 
 *					class.
*/
public static Object getInstance(byte[] stream, int index,Vector constantPool){
	COM_SafetyAttribute xconst = new COM_SafetyAttribute();
	 try{
     	xconst.data=new byte[2];
		xconst.data[0]=stream[index++];
		xconst.data[1]=stream[index++];
		xconst.addEntry("Data",index-2,2,""+Decode.getIntegerValue(xconst.data),
						xconst.data);
	 }catch(Exception ii){
		 System.out.println("COM_Safety: Exception "+ii);
	 }
		return xconst;
}
/**
 * Generic method for writting debug information to the screen.  By having all
 * methods in this class send their log information here, there is a central
 * point of change for turning debuggin off, or sending it to a differnet 
 * location.
 *
 * @param String data - message to be displayed.
*/
public synchronized void log(String data){
//	System.out.println(data);
}
}