/*
 *
 * sample.decoder.CSpecialArgumentEntryStructure.java
 *
 * (C) Copyright 1995 - 1999 Microsoft Corporation.  All rights reserved.
 *
*/
package sample.decoder;

import java.util.Vector;
import java.util.Hashtable;
/**
 * A structure used to describe the arguments to a custom-marshalled method.
 * 
 * @see sample.decoder.COM_MethodPoolAttribute
 *
 * @date	April 28,1998
 * @version	1.1
*/
public class CSpecialArgumentEntryStructure extends DecodeObject implements DecodeDisplayInterface,
		java.io.Serializable{
/**
 * These are the bytes which represent the flags for the CExposedAs Structure.
 */
	protected byte[] flags = new byte[1];
/**
 * Two bytes representing the option name.
 */
	protected byte[] optName = new byte[2];
/**
 * Represents the type number.
 */
	protected byte[] type = new byte[1];
/**
 * This method is a member of the DecodeDisplayInterface and is used to 
 * return the object name and the attribute name.  Subclasses of this object
 * should over ride this method to provide more detailed information.
 *
 * @see sample.decoder.DecodeDisplayInterface
 *
 * @returns String name representing this object and its attribute type.
*/
public String getName(){
	return "CSpecialArgumentEntry";
}
/**
 * Required by the DecodeDisplayInterface, returns any Vectors, or subobjects
 * which will need to be displayed as a subobject in a GUI.  The Hashtable is
 * indexed by a String that is to be used as a display name, the contents
 * will be either a Vector of Strings/Objects or a Decode Object.
 *
 * @returns Hashtable of subobjects/structures to be displayed as seperate
 *				nodes in the GUI.
*/
public Hashtable getChildren(){
	return new Hashtable();
}
/**
 * This is the workhorse for the object, it decodes an instance of itself starting at the
 * specified byte index into the stream provided.  This was created as a static method
 * rather than a constructor, as there may be times when this method returns a more
 * specialized object which is a subclass of itself.
 *
 * @param byte[] stream -  entire bytes of the class that is being decoded
 * @param int index - this is the index into the stream for the starting byte.
 * @param Vector constantPool - these are the constantPool objects, any index into the
 *								constant pool will be resolved by retrieving the object 
 *								at the required instance from this Vector.
 *
 * @returns Object - This is the decoded object which is a populated instance of this 
 *					class.
*/
public static Object getInstance(byte[] stream, int index,Vector constantPool){
        CSpecialArgumentEntryStructure xconst = new CSpecialArgumentEntryStructure();
		xconst.setByteStartIndex(index);
		xconst.type[0]=stream[index++];
		xconst.addEntry("Type",index-2,1,""+Decode.getIntegerValue(xconst.type),
						xconst.type);
		xconst.optName[0]=stream[index++];
		xconst.optName[1]=stream[index++];
		xconst.addEntry("Option Name",index-2,2,""+Decode.getIntegerValue(xconst.optName),
						xconst.optName);
		xconst.flags[0]=stream[index++];
	xconst.addEntry("Flags",index-1,1,""+Decode.getIntegerValue(xconst.flags),
						xconst.flags);
		xconst.setByteStopIndex(index-1);
        return xconst;
}
/**
 * Generic method for writting debug information to the screen.  By having all
 * methods in this class send their log information here, there is a central
 * point of change for turning debuggin off, or sending it to a differnet 
 * location.
 *
 * @param String data - message to be displayed.
*/
public synchronized void log(String data){
//	System.out.println(data);
}
}