/*
 *
 * sample.decoder.CTypeDescStructure.java
 *
 * (C) Copyright 1995 - 1999 Microsoft Corporation.  All rights reserved.
 *
*/
package sample.decoder;

import java.util.Vector;
import java.util.Hashtable;
/**
 * A structure used to describe the type of a single field in a class or 
 * a single argument and return type of a V-table based method.
 *
 * @see sample.decoder.COM_MethodPoolAttribute
 * @see sample.decoder.COM_MapsToAttribute
 * 
 * @date	April 28,1998
 * @version	1.1
*/
public class CTypeDescStructure extends DecodeObject implements DecodeDisplayInterface,
		java.io.Serializable{
/**
 * These are the bytes which represent the flags for the CExposedAs Structure.
 */
	protected byte[] flags = new byte[1];
/**
 * Represents the type number.
 */
	protected byte[] type = new byte[1];
/**
 * The other two bytes of data that are contained.
 */
    protected byte[] data = new byte[2];
/**
 * This method is a member of the DecodeDisplayInterface and is used to 
 * return the object name and the attribute name.  Subclasses of this object
 * should over ride this method to provide more detailed information.
 *
 * @see sample.decoder.DecodeDisplayInterface
 *
 * @returns String name representing this object and its attribute type.
*/
public String getName(){
	return "CTypeDesc";
}
/**
 * Required by the DecodeDisplayInterface, returns any Vectors, or subobjects
 * which will need to be displayed as a subobject in a GUI.  The Hashtable is
 * indexed by a String that is to be used as a display name, the contents
 * will be either a Vector of Strings/Objects or a Decode Object.
 *
 * @returns Hashtable of subobjects/structures to be displayed as seperate
 *				nodes in the GUI.
*/
public Hashtable getChildren(){
	return new Hashtable();
}
/**
 * Decodes the data bit to determine if IID_IS or SIZE_IS has been set for
 * the parameter.
 * 
 * @param int type - represents the type of the param
 * @param int flags - represents the flags of the param
 * @param byte data - bit to determine if IID_IS or SIZE_IS is set. 
 */
public String decodeData(int type, int flags, byte[] data){
	if (type==0x0d){
		if ((flags&0x10)==0x10){
			//return "IID Index";
			return "IID_IS set : "+data[0];
		}
		else{
			return "IID Index : "+Decode.getIntegerValue(data);
		}
	}else if (type==0x0f){
		if ((flags&0x10)==0x10){
			return "SIZE_IS set : "+data[0];
		}else
			return "array size = "+Decode.getIntegerValue(data);
	}else if (type==0x0c){
		return "ConstantPool Index : "+Decode.getIntegerValue(data);
	}else if((type==0x16)||(type==0x17)){
		return "array size = "+Decode.getIntegerValue(data);
	}else if (type==0x14)
		return "Var Type : "+this.decodeType(Decode.getIntegerValue(data));
	return "";
}
/**
 * Converts the flags from a binary to a human readable form.
 * 
 * @param int hexType - hex value specifying the type of the param.
 * @param int hexFlags - hex value of the flags to decode.
 * 
 * @return String - human readable form of the flags.
 */
public String decodeFlags(int hexType, int hexFlags){
	String retVal="";
			if (((hexFlags&0x01)==0x01)&&
				((hexFlags&0x02)==0x02))
				retVal+="[TDF_INOUT] ";
			else if ((hexFlags&0x01)==0x01)
				retVal+="[TDF_IN] ";
			else if ((hexFlags&0x02)==0x02)
				retVal+="[TDF_OUT]";
	if (hexType==0x0d){
		if (((hexFlags&0x08)==0x08)&&((hexFlags&0x04)==0x04))
			retVal+="[TDF_THREADMODELMASK] ";
		else if ((hexFlags&0x04)==0x04)
			retVal+="[TDF_AUTOMARSHAL] ";
		else if ((hexFlags&0x08)==0x08)
			retVal+="[TDF_NOMARSHAL] ";
	} else {
		if ((hexFlags&0x04)==0x04)
			retVal+="[TDF_LCID] ";
		if ((hexFlags&0x08)==0x08)
			retVal+="[TDF_NOMARSHAL] ";
	}
	if ((hexType==0x0d)||(hexType==0x0f)){
		if ((hexFlags&0x10)==0x10){
			if (hexType==0x0d)
				retVal+="[TDF_USES_IID_IS] ";
			else if (hexType==0x0f)
				retVal+="[TDF_USES_SIZE_IS] ";
		}
		if (((hexType&0x40)==0x40)&&((hexType&0x80)==0x80))
			retVal+="[TDF_USERMASK] ";
		else if ((hexType&0x80)==0x80)
			retVal+="[TDF_USER1] ";
		else if ((hexType&0x40)==0x40)
			retVal+="[TDF_USER2] ";
	}
	return retVal;
}
/**
 * This decodes the hex values for the type, returning a human readable
 * string.  The hex values will be  one of the valid TD_* types.
 * 
 * 
 * Valid types are:
 * 
 *		TD_VOID     0x00
 *		TD_I1       0x01    //signed 8-bit
 *		TD_I2       0x02    //signed 16-bit
 *		TD_I4       0x03    //signed 32-bit
 *		TD_I8       0x04    //signed 64-bit
 *		TD_U1       0x05    //unsgnd 8-bit
 *		TD_U2       0x06    //unsgnd 16-bit
 *		TD_U4       0x07    //unsgnd 32-bit
 *		TD_U8       0x08    //unsgnd 64-bit
 *		TD_R4       0x09    //32-bit float
 *		TD_R8       0x0a    //64-bit float
 *		TD_PTR      0x0b    //ptr to data
 *		TD_STRUCT   0x0c    //byval struct (requires size)
 *		TD_INTF     0x0d    //IUnknown* derivative
 *		TD_JSTR     0x0e    //JString
 *		TD_JARR     0x0f    //JArray
 *		TD_VARIANT  0x10    //Variant
 *		TD_CUSTOM   0x11    //custom type
 *		TD_CUSTOMBYREF 0x12 //custom type byref
 *		TD_CUSTOMBYVAL 0x13 //custom type byval
 *		TD_SAFEARRAY 0x14      //SafeArray
 *		TD_SYSCHAR  0x15       //Unicode char or ansi char, depending on platform setting
 *		TD_SYSFIXEDSTRING 0x16 //Fixed ansi/unicode string (for use inside structs: SizeIndex == #ofchars, including null terminator)
 *		TD_FIXEDARRAY 0x17     //Fixed size array of scalares (for use inside structs: SizeIndex == # of elements)
 *		TD_OBJECT   0x18    //Field type of "L<something>". Treat as embedded struct if JCDW; treat as COM ifc if Object or Ifc.
 * 
 * @param int - bytes to be decoded.
 * 
 * @return String - Human readable string that describes the type.
 */
protected String decodeType(int hexValue){
	if (hexValue==0x00)
		return "TD_VOID";
	if (hexValue==0x01)
		return "TD_1";
	if (hexValue==0x02)
		return "TD_2";
	if (hexValue==0x03)
		return "TD_4";
	if (hexValue==0x04)
		return "TD_8";
	if (hexValue==0x05)
		return "TD_U1";
	if (hexValue==0x06)
		return "TD_U2";
	if (hexValue==0x07)
		return "TD_U4";
	if (hexValue==0x08)
		return "TD_U8";
	if (hexValue==0x09)
		return "TD_R4";
	if (hexValue==0x0a)
		return "TD_R8";
	if (hexValue==0x0b)
		return "TD_PTR";
	if (hexValue==0x0c)
		return "TD_STRUCT";
	if (hexValue==0x0d)
		return "TD_INTF";
	if (hexValue==0x0e)
		return "TD_JSTR";
	if (hexValue==0x0f)
		return "TD_JARR";
	if (hexValue==0x10)
		return "TD_VARIANT";
	if (hexValue==0x11)
		return "TD_CUSTOM";
	if (hexValue==0x12)
		return "TD_CUSTOMBYREF";
	if (hexValue==0x13)
		return "TD_CUSTOMBYVAL";
	if (hexValue==0x14)
		return "TD_SAFEARRAY";
	if (hexValue==0x15)
		return "TD_SYSCHAR";
	if (hexValue==0x16)
		return "TD_SYSFIXEDSTRING";
	if (hexValue==0x17)
		return "TD_FIXEDARRAY";
	if (hexValue==0x18)
		return "TD_OBJECT";
	else return "UNABLE TO DECODE";
}
/**
 * This is the workhorse for the object, it decodes an instance of itself starting at the
 * specified byte index into the stream provided.  This was created as a static method
 * rather than a constructor, as there may be times when this method returns a more
 * specialized object which is a subclass of itself.
 *
 * @param byte[] stream -  entire bytes of the class that is being decoded
 * @param int index - this is the index into the stream for the starting byte.
 * @param Vector constantPool - these are the constantPool objects, any index into the
 *								constant pool will be resolved by retrieving the object 
 *								at the required instance from this Vector.
 *
 * @returns Object - This is the decoded object which is a populated instance of this 
 *					class.
*/
public static Object getInstance(byte[] stream, int index,Vector constantPool){
        CTypeDescStructure xconst = new CTypeDescStructure();
		xconst.setByteStartIndex(index);
		xconst.type[0]=stream[index++];
        xconst.addEntry("Type",index-1,1,""+xconst.decodeType(Decode.getIntegerValue(xconst.type)),
						xconst.type);
	 	xconst.flags[0]=stream[index++];
		xconst.addEntry("Flags",index-1,1,""+xconst.decodeFlags(
							Decode.getIntegerValue(xconst.type),
							Decode.getIntegerValue(xconst.flags)),
						xconst.flags);
	    xconst.data[0]=stream[index++];
        xconst.data[1]=stream[index++];
        xconst.addEntry("Data",index-2,2,""+xconst.decodeData(
								Decode.getIntegerValue(xconst.type),
								Decode.getIntegerValue(xconst.flags),
								xconst.data),
						xconst.data);
        xconst.setByteStopIndex(index-1); 
        return xconst;
}
/**
 * Generic method for writting debug information to the screen.  By having all
 * methods in this class send their log information here, there is a central
 * point of change for turning debuggin off, or sending it to a differnet 
 * location.
 *
 * @param String data - message to be displayed.
*/
public synchronized void log(String data){
//	System.out.println(data);
}
}