/*
 *
 * sample.decoder.CVtblMethodStructure
 *
 * (C) Copyright 1995 - 1999 Microsoft Corporation.  All rights reserved.
 *
*/
package sample.decoder;

import java.util.Vector;
import java.util.Hashtable;
/**
 * A structure used to describe a V-table method.
 *
 * @date	April 28,1998
 * @version	1.1
*/
public class CVtblMethodStructure extends DecodeObject implements DecodeDisplayInterface,
		java.io.Serializable{
	
	protected byte[] iIDIndex = new byte[2];
	protected byte[] vtblIndex = new byte[2];
	protected byte[] nRetValParameter = new byte[2];
	protected byte[] nArguments = new byte[2];
	protected CTypeDescStructure retType = null;
	protected Vector params = new Vector();
/**
 * This method is a member of the DecodeDisplayInterface and is used to 
 * return the object name and the attribute name.  Subclasses of this object
 * should over ride this method to provide more detailed information.
 *
 * @see sample.decoder.DecodeDisplayInterface
 *
 * @returns String name representing this object and its attribute type.
*/
public String getName(){
	return "CVtblMethod";
}
/**
 * Required by the DecodeDisplayInterface, returns any Vectors, or subobjects
 * which will need to be displayed as a subobject in a GUI.  The Hashtable is
 * indexed by a String that is to be used as a display name, the contents
 * will be either a Vector of Strings/Objects or a Decode Object.
 *
 * @returns Hashtable of subobjects/structures to be displayed as seperate
 *				nodes in the GUI.
*/
public Hashtable getChildren(){
	Hashtable val = new Hashtable();
	val.put(((DecodeDisplayInterface)retType).getName()+" Return Value",retType);
	System.out.println("Parameter (type descs) length is "+params.size());
	for (int i=0;i<params.size();i++)
		val.put(((DecodeDisplayInterface)params.elementAt(i)).getName()+" Arg"+i,
				params.elementAt(i));
	return val;
}
/**
 * After the initial decoding phase is complete, where all of the bytes have
 * been moved to their appropriate places, this method retrieves any 
 * referenced Constant Pool objects and stores the information with the 
 * object containing the refernece.
 * 
 * @param Vector data - These are the ConstantObjects that were created from the 
 *						constant pool structures in the class file.
*/
public void link(Vector data){
}
/**
 * This is the workhorse for the object, it decodes an instance of itself starting at the
 * specified byte index into the stream provided.  This was created as a static method
 * rather than a constructor, as there may be times when this method returns a more
 * specialized object which is a subclass of itself.
 *
 * @param byte[] stream -  entire bytes of the class that is being decoded
 * @param int index - this is the index into the stream for the starting byte.
 * @param Vector constantPool - these are the constantPool objects, any index into the
 *								constant pool will be resolved by retrieving the object 
 *								at the required instance from this Vector.
 *
 * @returns Object - This is the decoded object which is a populated instance of this 
 *					class.
*/
public static Object getInstance(byte[] stream, int index,Vector constantPool){
        CVtblMethodStructure xconst = new CVtblMethodStructure();
		xconst.setByteStartIndex(index);
		xconst.iIDIndex[0]=stream[index++];
		xconst.iIDIndex[1]=stream[index++];
		xconst.addEntry("Interface ID Index",index-2,2,""+Decode.getIntegerValue(xconst.iIDIndex),
						xconst.iIDIndex);
		xconst.vtblIndex[0]=stream[index++];
		xconst.vtblIndex[1]=stream[index++];
		xconst.addEntry("VTBL Index",index-2,2,""+Decode.getIntegerValue(xconst.vtblIndex),
						xconst.vtblIndex);
		xconst.nArguments[0]=stream[index++];
		xconst.nArguments[1]=stream[index++];
		xconst.addEntry("Number of Arguments",index-2,2,""+Decode.getIntegerValue(xconst.nArguments),
						xconst.nArguments);
		xconst.nRetValParameter[0]=stream[index++];
		xconst.nRetValParameter[1]=stream[index++];
		xconst.addEntry("Return Value Parameter",index-2,2,""+Decode.getIntegerValue(xconst.nRetValParameter),
						xconst.nRetValParameter);
		xconst.retType=(CTypeDescStructure)CTypeDescStructure.getInstance(stream,index,constantPool);
		index=((DecodeObject)xconst.retType).getByteStopIndex()+1;
		for (int i=0;i<Decode.getIntegerValue(xconst.nArguments);i++){
			xconst.params.addElement(CTypeDescStructure.getInstance(stream,index,constantPool));
			index=((DecodeObject)xconst.params.lastElement()).getByteStopIndex()+1;	
		}
		xconst.setByteStopIndex(index-1);
        return xconst;
}
/**
 * Generic method for writting debug information to the screen.  By having all
 * methods in this class send their log information here, there is a central
 * point of change for turning debuggin off, or sending it to a differnet 
 * location.
 *
 * @param String data - message to be displayed.
*/
public synchronized void log(String data){
//	System.out.println(data);
}
}