/*
 *
 * sample.decoder.CodeAttribute.java
 *
 * (C) Copyright 1995 - 1999 Microsoft Corporation.  All rights reserved.
 *
*/
package sample.decoder;

import java.util.Vector;
import java.util.Hashtable;
/**
 * The Code Attribute is a specialized attribute typically associated with methods. This
 * attribute also contains other attributes that are related to code.  Attributes which
 * do not have a specialized object for decoding and holding their information will be
 * discopvered as generic AttributeObject.
 * 
 * SCOPE : Method
 * 
 * @see sample.decoder.LineNumberTableAttribute
 * @see sample.decoder.LocalVariableTableAttribute
 * 
 * @date	April 28,1998
 * @version	1.2
*/
public class CodeAttribute extends AttributeObject implements DecodeDisplayInterface,
			java.io.Serializable{
/**
 * This is the byte code which represents the actual code for this method.  There 
 * currently is no mechanism for decoding these code btes into source.
*/
	protected byte[] code ;
/**
 * These are the two bytes which represent the maximum stack value for this code.
*/
	protected byte[] max_stack = new byte[2];
/**
 * The 2 bytes for the macimum loacs for this code.
*/
	protected byte[] max_locals = new byte[2];
/**
 * 4 bytes which represent the length in bytes of the code for this object.
*/
	protected byte[] code_length = new byte[4];
/** 
 * Length in bytes of the exception table.  It is help in a two byte array.
*/
	protected byte[] exception_table_length = new byte[2];
/**
 * These are the two bytes which represent the number of attributes for this code.
*/
	protected byte[] attributes_count = new byte[2];
/**
 * This is the exception table, which is comprised of 2 byte indexs into the constant 
 * pool for the class objects which represent the table.
*/
	protected byte[] exception_table;
/**
 * These are the decoded attributes for this code.  The Vector contains instances of 
 * AttributeObject or its subclasses.
*/
	protected Vector attributes=null;
/**
 * Vector containinf all of the Exception Table Structures.
 */
	protected Vector exceptions=new Vector();
/**
 * Access method for getting the bytes representing the code for this Code Attribute.
 *
 * @returns byte[]  - code for this method.
*/
public byte[] getCode(){
	return code;
}
/**
 * Determines the integer value of the two bytes representing the maximum stack.
 *
 * @returns int - value for maximum stack.
*/
public int getMaxStack(){
	return Decode.getIntegerValue(max_stack);
}
/**
 * Determines the integer value of hte two bytes representing the maximum locals.
 *
 * @returns int - value for maximum locals.
*/
public int getMaxLocals(){
	return Decode.getIntegerValue(max_locals);
}
/**
 * This is the number of bytes that represent the code for this Code Attribute.
 *
 * @returns int - length of code attribute bytes.
*/
public int getCodeLength(){
	return Decode.getIntegerValue(code_length);
}
/**
 * This is the number of exceptions in the exception table.
 *
 * @returns int - number of exceptions.
*/
public int getExceptionTableLength(){
	return Decode.getIntegerValue(exception_table_length );
}
/**
 * This is the number of attributes that this Code Attribute has.
 *
 * @returns int - the number of attributes for this Code Attribute.
*/
public int getAttributesCount(){
	return Decode.getIntegerValue(attributes_count);
}
/**
 * These are the bytes that represent the exception table.  Currently these bytes are
 * NOT decoded in any way.
 *
 * @returns byte[] - bytes which comprise the excepetion table.
*/
public byte[] getExceptionTable(){
	return exception_table;
}
/**
 * These are the attributes within the Code Attribute that is succently being decoded.
 * 
 * @returns Vector - These will be AttributeObjects or subclasses of AttributeObject
*/
public Vector getAttributes(){
	return attributes;
}
/**
 * Default method for determining a display String which adequatly 
 * represents this object and its contents.  Subclasses of this object 
 * should override this method to provide more detailed information.
 *
 * @returns String representing this object.
*/
public String toString(){
		return "Code";
}
/**
 * This method is a member of the DecodeDisplayInterface and is used to 
 * return the object name and the attribute name.  Subclasses of this object
 * should over ride this method to provide more detailed information.
 *
 * @see com.ms.tools.decode.DecodeDisplayInterface
 *
 * @returns String name representing this object and its attribute type.
*/
public String getName(){
	return "Code";
}
/**
 * Required by the DecodeDisplayInterface, returns any Vectors, or subobjects
 * which will need to be displayed as a subobject in a GUI.  The Hashtable is
 * indexed by a String that is to be used as a display name, the contents
 * will be either a Vector of Strings/Objects or a Decode Object.
 *
 * @returns Hashtable of subobjects/structures to be displayed as seperate
 *				nodes in the GUI.
*/
public Hashtable getChildren(){
	Hashtable val =super.getChildren();
	Vector att = getAttributes();
	for (int i=0;i<att.size();i++)
		val.put(((DecodeDisplayInterface)att.elementAt(i)).getName()+i,
				att.elementAt(i));
	Vector ex = exceptions;
	for (int i=0;i<ex.size();i++)
		val.put(((DecodeDisplayInterface)ex.elementAt(i)).getName()+i,
				ex.elementAt(i));
	return val;
}
/**
 * This is the workhorse for the object, it decodes an instance of itself starting at the
 * specified byte index into the stream provided.  This was created as a static method
 * rather than a constructor, as there may be times when this method returns a more
 * specialized object which is a subclass of itself.
 *
 * @param byte[] stream -  entire bytes of the class that is being decoded
 * @param int index - this is the index into the stream for the starting byte.
 * @param Vector constantPool - these are the constantPool objects, any index into the
 *								constant pool will be resolved by retrieving the object 
 *								at the required instance from this Vector.
 *
 * @returns Object - This is the decoded object which is a populated instance of this 
 *					class.
*/
public static Object getInstance(byte[] stream, int index,Vector constantPool){
	try{
	int start = index;
	CodeAttribute xconst = new CodeAttribute();
	xconst.max_stack[0]=stream[index++];
	xconst.max_stack[1]=stream[index++];
	xconst.addEntry("Max Stack",index-2,2,""+Decode.getIntegerValue(xconst.max_stack),
			xconst.max_stack);
	xconst.max_locals[0]=stream[index++];
	xconst.max_locals[1]=stream[index++];
	xconst.addEntry("Max Locals",index-2,2,""+Decode.getIntegerValue(xconst.max_locals),
			xconst.max_locals);
	xconst.code_length[0]=stream[index++];
	xconst.code_length[1]=stream[index++];
	xconst.code_length[2]=stream[index++];
	xconst.code_length[3]=stream[index++];
	xconst.addEntry("Code Length",index-4,4,""+Decode.getIntegerValue(xconst.code_length),
			xconst.code_length);
    xconst.code=new byte[Decode.getIntegerValue(xconst.code_length)];
    xconst.log("CodeAttribute: length of code is "+xconst.code.length);
	for (int i=0;i<xconst.code.length;i++)
		xconst.code[i]=stream[index++];
	xconst.addEntry("Code",index-1-xconst.code.length,xconst.code.length,
			"",xconst.code);
	xconst.exception_table_length[0]=stream[index++];
	xconst.exception_table_length[1]=stream[index++];
	xconst.addEntry("Exception Table Length",index-2,2,""+Decode.getIntegerValue(xconst.exception_table_length),
					xconst.exception_table_length);
	for (int i=0;i<Decode.getIntegerValue(xconst.exception_table_length);i++){
		xconst.exceptions.addElement(ExceptionTableStructure.getInstance(stream,index,constantPool));
		index=((DecodeObject)xconst.exceptions.lastElement()).getByteStopIndex()+1;
	}
	xconst.attributes_count[0]=stream[index++];
	xconst.attributes_count[1]=stream[index++];
	xconst.addEntry("Attributes Count",index-2,2,""+Decode.getIntegerValue(xconst.attributes_count),
					xconst.attributes_count);
    AttributeObject aa = new AttributeObject(constantPool);
    xconst.attributes=aa.createObjects(stream,
                Decode.getIntegerValue(xconst.attributes_count),index);
	return xconst;
	}catch(Exception ii){
		System.out.println("CodeAttribute: Exception "+ii);
		return null;
	}
}
/**
 * Generic method for writting debug information to the screen.  By having all
 * methods in this class send their log information here, there is a central
 * point of change for turning debuggin off, or sending it to a differnet 
 * location.
 *
 * @param String data - message to be displayed.
*/
public synchronized void log(String data){
//	System.out.println(data);
}
}