/*
 *
 * sample.decoder.ConditionalAttribute.java
 *
 * (C) Copyright 1995 - 1999 Microsoft Corporation.  All rights reserved.
 *
*/
package sample.decoder;

import java.util.Vector;
import java.util.Hashtable;
/**
 * The Conditional attribute indicates that the method should be conditionally be
 * called based on an expression.
 * 
 * SCOPE : Method
 * 
 * @date	April 28,1998
 * @version	1.1
*/
public class ConditionalAttribute extends AttributeObject implements DecodeDisplayInterface,
		java.io.Serializable{
/**
 * This is the decoded constant_utf8 structure from the constant pool which was indexed
 * by the sourcefile_index bytes.
*/
	protected ConstantUtfObject expression = null;
/**
 * These are 2 bytes which represent an index into the constantpool for a constant+utf8
 * structure which is the source file String.
*/
	protected byte[] expression_index = new byte[2];
/**
 * Determines the integer value of the bytes that represent the expression index.
 * 
 * @returns int - integer value representing an index into the Constant Pool
 */
public int getExpressionIndex(){
	return Decode.getIntegerValue(expression_index);
}
/**
 * This is the String value from the constant_utf8 structure that represents the constant
 * object containing the String with the source file name.
 *
 * @returns String - source file name
*/
public String getExpression(){
	return expression.toString();
}
/**
 * This method is a member of the DecodeDisplayInterface and is used to 
 * return the object name and the attribute name.  Subclasses of this object
 * should over ride this method to provide more detailed information.
 *
 * @see com.ms.tools.decode.DecodeDisplayInterface
 *
 * @returns String name representing this object and its attribute type.
*/
public String getName(){
	return "Conditional";
}
/**
 * This is the workhorse for the object, it decodes an instance of itself starting at the
 * specified byte index into the stream provided.  This was created as a static method
 * rather than a constructor, as there may be times when this method returns a more
 * specialized object which is a subclass of itself.
 *
 * @param byte[] stream -  entire bytes of the class that is being decoded
 * @param int index - this is the index into the stream for the starting byte.
 * @param Vector constantPool - these are the constantPool objects, any index into the
 *								constant pool will be resolved by retrieving the object 
 *								at the required instance from this Vector.
 *
 * @returns Object - This is the decoded object which is a populated instance of this 
 *					class.
*/
public static Object getInstance(byte[] stream, int index,Vector constantPool){
        ConditionalAttribute xconst = new ConditionalAttribute();
        xconst.expression_index[0]=stream[index++];
        xconst.expression_index[1]=stream[index++];
		xconst.addEntry("Expression Index",index-2,2,constantPool.elementAt(
				Decode.getIntegerValue(xconst.expression_index)).toString(),
				xconst.expression_index);
		xconst.expression=(ConstantUtfObject)constantPool.elementAt(
				Decode.getIntegerValue(xconst.expression_index));
        return xconst;
}
/**
 * Generic method for writting debug information to the screen.  By having all
 * methods in this class send their log information here, there is a central
 * point of change for turning debuggin off, or sending it to a differnet 
 * location.
 *
 * @param String data - message to be displayed.
*/
public synchronized void log(String data){
//	System.out.println(data);
}
}