/*
 *
 * sample.decoder.ConstantClassObject.java
 *
 * (C) Copyright 1995 - 1999 Microsoft Corporation.  All rights reserved.
 *
*/
package sample.decoder;
import java.util.Vector;
import java.util.Hashtable;
/**
 * This is an object which represents the constant pool strucuture for a class or 
 * interface.  It is generated by the ConstantObject during processing of the constant
 * pool bytes when decoding a given class file.
 *
 * @see sample.decoder.ConstantObject
 *
 * @date	April 28,1998
 * @version	1.1
*/
public class ConstantClassObject extends ConstantObject implements DecodeDisplayInterface, 
			java.io.Serializable{
/**
 * These are the two bytes which indicate the index into the constant pool where the 
 * constant_utf8_info structure is that contains the name of the class this represents.
*/
	private byte[] name_index=new byte[2];
/**
 * This is the ConstantUtfObject which represents the constant_utf8_info strucuture from 
 * the constant pool that contains the class name.
*/
	private ConstantUtfObject name = null;
/**
 * After the initial decoding phase is complete, where all of the bytes have
 * been moved to their appropriate places, this method retrieves any 
 * referenced Constant Pool objects and stores the information with the 
 * object containing the refernece.
 * 
 * @param Vector data - These are the ConstantObjects that were created from the 
 *						constant pool structures in the class file.
*/
public void link(Vector data){
	super.link(data);
    addEntry("Name Index",getByteStartIndex()+3,2,
		data.elementAt(Decode.getIntegerValue(name_index)).toString(),
		name_index);
		name=(ConstantUtfObject)data.
                elementAt(Decode.getIntegerValue(name_index));
}
/**
 * This method is a member of the DecodeDisplayInterface and is used to 
 * return the object name and the attribute name.  Subclasses of this object
 * should over ride this method to provide more detailed information.
 *
 * @see com.ms.tools.decode.DecodeDisplayInterface
 *
 * @returns String name representing this object and its attribute type.
*/
public String getName(){
	//System.out.println("ConstantClass Name "+name+" "+Decode.getIntegerValue(name_index));
	//if (name==null) return "<<CLASS NAME WAS NULL>>";
	return poolIndex+" : Class - "+name.getString();
}
/**
 * This is the integer representation of the 2 bytes that indicate the index into the
 * constant pool where the name is stored.
 *
 * @returns int - index into the constant pool for a ConstantUtfobject.
*/
public int getNameIndex(){
		return Decode.getIntegerValue(name_index);
}
/**
 * Default method for determining a display String which adequatly 
 * represents this object and its contents.  Subclasses of this object 
 * should override this method to provide more detailed information.
 *
 * @returns String representing this object.
*/
public String toString(){
	return "ConstantClassObject";
}
/**
 * Returns the classname that this object represents.
 * 
 * @return String - fuly qualified classname with dots (i.e java.lang.String)
 */
public String getClassName(){
	return this.name.getString().replace('/','.');	
}
/**
 * Required by the DecodeDisplayInterface, returns any Vectors, or subobjects
 * which will need to be displayed as a subobject in a GUI.  The Hashtable is
 * indexed by a String that is to be used as a display name, the contents
 * will be either a Vector of Strings/Objects or a Decode Object.
 *
 * @returns Hashtable of subobjects/structures to be displayed as seperate
 *				nodes in the GUI.
*/
public Hashtable getChildren(){
	return super.getChildren();
}
/**
 * This is the workhorse for the object, it decodes an instance of itself starting at the
 * specified byte index into the stream provided.  This was created as a static method
 * rather than a constructor, as there may be times when this method returns a more
 * specialized object which is a subclass of itself.
 *
 * @param byte[] stream -  entire bytes of the class that is being decoded
 * @param int index - this is the index into the stream for the starting byte.
 *
 * @returns ConstantObject - this is a ConstantObject representing the current Constant
 *							pool structure.
*/
public static Object getInstance(byte[] stream, int index){
    ConstantClassObject xconst = new ConstantClassObject();
    xconst.log("ConstantClassObject: length "+stream.length+" index "+index);    
	xconst.setByteStartIndex(index);
    xconst.tag=stream[index++];
	xconst.addEntry("Tag",index-2,1,xconst.tag+" - Class",
					new byte[]{(byte)xconst.tag});
    xconst.name_index[0]=stream[index++];
    xconst.name_index[1]=stream[index];
	xconst.setByteStopIndex(index);
    xconst.log("ConstantClassObject: returning");
	return xconst;
}
/**
 * Generic method for writting debug information to the screen.  By having all
 * methods in this class send their log information here, there is a central
 * point of change for turning debuggin off, or sending it to a differnet 
 * location.
 *
 * @param String data - message to be displayed.
*/
public synchronized void log(String data){
//	System.out.println(data);
}
}