/*
 *
 * sample.decoder.ConstantDoubleObject.java
 *
 * (C) Copyright 1995 - 1999 Microsoft Corporation.  All rights reserved.
 *
*/
package sample.decoder;

import java.util.Vector;
import java.util.Hashtable;
/**
 * This is an object which represents the constant pool strucuture for a double
 * It is generated by the ConstantObject during processing of the constant
 * pool bytes when decoding a given class file.
 *
 * @see sample.decoder.ConstantObject
 *
 * @date	April 28,1998
 * @version	1.2
*/
public class ConstantDoubleObject extends ConstantObject implements DecodeDisplayInterface,
		java.io.Serializable{
/**
 * These are the 8 bytes that will decode to the double number that this structure was
 * intended to represent.
*/
	protected byte[] bytes = new byte[8];
/**
 * Accessor method for retrieving the double number that this object represents.  It is
 * decoded from the 8 bytes contained within the constant_double_info structure.
 *
 * @returns double - this is the double stored in this constant object.
*/
public double getDouble(){
	return Decode.getDoubleValue(bytes);
}
/**
 * Default method for determining a display String which adequatly 
 * represents this object and its contents.  Subclasses of this object 
 * should override this method to provide more detailed information.
 *
 * @returns String representing this object.
*/
public String toString(){
	return ""+getDouble();
}
/**
 * This method is a member of the DecodeDisplayInterface and is used to 
 * return the object name and the attribute name.  Subclasses of this object
 * should over ride this method to provide more detailed information.
 *
 * @see sample.decoder.DecodeDisplayInterface
 *
 * @returns String name representing this object and its attribute type.
*/
public String getName(){
	return poolIndex+" : Double - "+getDouble();
}
/**
 * Required by the DecodeDisplayInterface, returns any Vectors, or subobjects
 * which will need to be displayed as a subobject in a GUI.  The Hashtable is
 * indexed by a String that is to be used as a display name, the contents
 * will be either a Vector of Strings/Objects or a Decode Object.
 *
 * @returns Hashtable of subobjects/structures to be displayed as seperate
 *				nodes in the GUI.
*/
public Hashtable getChildren(){
	return super.getChildren();
}
/**
 * After the initial decoding phase is complete, where all of the bytes have
 * been moved to their appropriate places, this method retrieves any 
 * referenced Constant Pool objects and stores the information with the 
 * object containing the refernece.
 *
 * @param Vector data - These are the ConstantObjects that were created from the 
 *						constant pool structures in the class file.
*/
public void link(Vector d){
	super.link(d);
}
/**
 * This is the workhorse for the object, it decodes an instance of itself starting at the
 * specified byte index into the stream provided.  This was created as a static method
 * rather than a constructor, as there may be times when this method returns a more
 * specialized object which is a subclass of itself.
 *
 * @param byte[] stream -  entire bytes of the class that is being decoded
 * @param int index - this is the index into the stream for the starting byte.
 *
 * @returns ConstantObject - this is a ConstantObject representing the current Constant
 *							pool structure.
*/
public static ConstantObject getInstance(byte[] stream, int index){
        ConstantDoubleObject xconst = new ConstantDoubleObject();
        xconst.setByteStartIndex(index);
        xconst.tag=stream[index++];
		xconst.addEntry("Tag",index-2,1,xconst.tag+" - Double",
						new byte[]{(byte)xconst.tag});
        for (int i=0;i<xconst.bytes.length;i++)
                xconst.bytes[i]=stream[index++];
		xconst.addEntry("Data",xconst.getByteStartIndex()+1,index-xconst.getByteStartIndex(),
						""+(new Double(xconst.getDouble())).toString(),xconst.bytes);
        xconst.setByteStopIndex(index-1);
        return xconst;
}
}