/*
 *
 * sample.decoder.ConstantLongObject.java
 *
 * (C) Copyright 1995 - 1999 Microsoft Corporation.  All rights reserved.
 *
*/
package sample.decoder;

import java.util.Vector;
import java.util.Hashtable;
/**
 * This is an object which represents the constant pool strucuture for a long
 * It is generated by the ConstantObject during processing of the constant
 * pool bytes when decoding a given class file.
 *
 * @see sample.decoder.ConstantObject
 *
 * @date	April 28,1998
 * @version	1.1
*/
public class ConstantLongObject extends ConstantObject implements DecodeDisplayInterface,
			java.io.Serializable{
/**
 * These are the 8 bytes that will decode to the long number that this structure was
 * intended to represent.
*/
	protected byte[] bytes = new byte[8];
/**
 * Accessor method for retrieving the long number that this object represents.  It is
 * decoded from the 8 bytes contained within the constant_long_info structure.
 *
 * @returns long - this is the long stored in this constant object.
*/
public long getLong(){
	return Decode.getLongValue(bytes);
}
/**
 * Default method for determining a display String which adequatly 
 * represents this object and its contents.  Subclasses of this object 
 * should override this method to provide more detailed information.
 *
 * @returns String representing this object.
*/
public String toString(){
	return getName();
}
/**
 * This method is a member of the DecodeDisplayInterface and is used to 
 * return the object name and the attribute name.  Subclasses of this object
 * should over ride this method to provide more detailed information.
 *
 * @see sample.decoder.DecodeDisplayInterface
 *
 * @returns String name representing this object and its attribute type.
*/
public String getName(){
	return poolIndex+" : Long - "+getLong();
}
/**
 * After the initial decoding phase is complete, where all of the bytes have
 * been moved to their appropriate places, this method retrieves any 
 * referenced Constant Pool objects and stores the information with the 
 * object containing the refernece.
 *
 * @param Vector data - These are the ConstantObjects that were created from the 
 *						constant pool structures in the class file.
*/
public void link(Vector d){
	super.link(d);
}
/**
 * This is the workhorse for the object, it decodes an instance of itself starting at the
 * specified byte index into the stream provided.  This was created as a static method
 * rather than a constructor, as there may be times when this method returns a more
 * specialized object which is a subclass of itself.
 *
 * @param byte[] stream -  entire bytes of the class that is being decoded
 * @param int index - this is the index into the stream for the starting byte.
 *
 * @returns ConstantObject - this is a ConstantObject representing the current Constant
 *							pool structure.
*/
public static ConstantObject getInstance(byte[] stream, int index){
        ConstantLongObject xconst = new ConstantLongObject();
        xconst.setByteStartIndex(index);
        xconst.tag=stream[index++];
		xconst.addEntry("Tag",index-2,1,xconst.tag+" - Long",
						new byte[]{(byte)xconst.tag});
        for (int i=0;i<xconst.bytes.length;i++)
                xconst.bytes[i]=stream[index++];
		xconst.addEntry("Long",xconst.getByteStartIndex()+1,
						xconst.bytes.length,""+xconst.getLong(),xconst.bytes);
        xconst.setByteStopIndex(index-1);
		return xconst;
}
}