/*
 *
 * sample.decoder.ConstantMethodObject.java
 *
 * (C) Copyright 1995 - 1999 Microsoft Corporation.  All rights reserved.
 *
*/
package sample.decoder;

import java.util.Vector;
import java.util.Hashtable;
/**
 * This is an object which represents the constant pool strucuture for a method.
 * It is generated by the ConstantObject during processing of the constant
 * pool bytes when decoding a given class file.
 *
 * @see sample.decoder.ConstantObject
 *
 * @date	April 28,1998
 * @version	1.2
*/
public class ConstantMethodObject extends ConstantObject implements java.io.Serializable{
/**
 * This is the index into the constant pool where the constant_class_info structure is
 * that represents the class for this method.
*/
	protected byte[] class_index = new byte[2];
/**
 * Index into the constant pool where the constant_name_and_type_info structure is
 * that represents the name and type for the method.
*/
	protected byte[] name_and_type_index = new byte[2];
/**
 * This is the decoded constant_class_info structure that was indexed by class_index
*/
	protected ConstantClassObject className = null;
/**
 * This is the decoded constant_name-and_type_info structure that was indexed by
 * name-and_type_index
*/
	protected ConstantNameAndTypeObject nameType = null;
/**
 * Accessor method that converts the 2 bytes which represent the index into the constant
 * pool table for the constant_class_structure to an integer value.
 *
 * @returns int -  integer representation of the constant pool index class_index
*/
public int getClassIndex(){
	return Decode.getIntegerValue(class_index);
}
/**
 * After the initial decoding phase is complete, where all of the bytes have
 * been moved to their appropriate places, this method retrieves any 
 * referenced Constant Pool objects and stores the information with the 
 * object containing the refernece.
 *
 * @param Vector data - These are the ConstantObjects that were created from the 
 *						constant pool structures in the class file.
*/
public void link(Vector data){
	super.link(data);
	addEntry("Class Index",getByteStartIndex()+1,2,data.elementAt(Decode.getIntegerValue(class_index)).toString(),
			 class_index);
	addEntry("Name and Type Index",getByteStartIndex()+3,2,data.elementAt(Decode.getIntegerValue(name_and_type_index)).toString(),
			 name_and_type_index);
	className=(ConstantClassObject)data.elementAt(Decode.getIntegerValue(class_index));
	nameType=(ConstantNameAndTypeObject)data.elementAt(Decode.getIntegerValue(name_and_type_index));
}
/**
 * Accessor method that converts the 2 bytes which represent the index into the constant
 * pool table for the constant_name_and_type structure to an integer value.
 *
 * @returns int -  integer representation of the constant pool index name_and_type_index
*/
public int getNameAndTypeIndex(){
	return Decode.getIntegerValue(name_and_type_index);
}
/**
 * This is an acesor method that returns the decoded ConstantClassObject that was 
 * indicated by the class_index bytes.
 *
 * @returns ConstantClassObject - decoded constant_class_info structure for this method.
*/
public ConstantClassObject getConstantClassObject(){
	return className;
}
/**
 * This is an acesor method that returns the decoded ConstantNameAndTypeObject that was 
 * indicated by the name_and_type_index bytes.
 *
 * @returns ConstantNameAndTypeObject - decoded constant_name_and-type_info structure 
 *										for this method.
*/
public ConstantNameAndTypeObject getConstantNameAndTypeObject(){
	return nameType;
}
/**
 * This is the String from the ConstantClassObject which represents the Class name for
 * this method.
 *
 * @returns String- This is the methods class type.
*/
public String getClassName(){
	return className.toString();
}
/**
 * This is the String which represents the name and type from the 
 * ConstantNameAndTypeObject.
 *
 * @returns String - String representing the name and type for this method.
*/
public String getNameAndType(){
	return nameType.getName();
}
/**
 * This is the String which represents the descriptor from the ConstantNameAndTypeObject.
 *
 * @returns String - String representing the name and type descriptor.
*/
public String getDescriptor(){
	return nameType.getDescriptor();
}
/**
 * Default method for determining a display String which adequatly 
 * represents this object and its contents.  Subclasses of this object 
 * should override this method to provide more detailed information.
 *
 * @returns String representing this object.
*/
public String toString(){
	return getName();
}
/**
 * This method is a member of the DecodeDisplayInterface and is used to 
 * return the object name and the attribute name.  Subclasses of this object
 * should over ride this method to provide more detailed information.
 *
 * @see com.ms.tools.decode.DecodeDisplayInterface
 *
 * @returns String name representing this object and its attribute type.
*/
public String getName(){
	return poolIndex+" : Method - "+getDescriptor();
}
/**
 * This is the workhorse for the object, it decodes an instance of itself starting at the
 * specified byte index into the stream provided.  This was created as a static method
 * rather than a constructor, as there may be times when this method returns a more
 * specialized object which is a subclass of itself.
 *
 * @param byte[] stream -  entire bytes of the class that is being decoded
 * @param int index - this is the index into the stream for the starting byte.
 *
 * @returns ConstantObject - this is a ConstantObject representing the current Constant
 *							pool structure.
*/
public static ConstantObject getInstance(byte[] stream, int index){
	ConstantMethodObject xconst = new ConstantMethodObject();
	xconst.setByteStartIndex(index);
	xconst.tag=stream[index++];
	xconst.addEntry("Tag",index-2,1,xconst.tag+" - Method",
					new byte[]{(byte)xconst.tag});
	xconst.class_index[0]=stream[index++];
	xconst.class_index[1]=stream[index++];
	xconst.name_and_type_index[0]=stream[index++];
	xconst.name_and_type_index[1]=stream[index];
	xconst.setByteStopIndex(index);
	return xconst;
}
}