/*
 *
 * sample.decoder.ConstantNameAndTypeObject.java
 *
 * (C) Copyright 1995 - 1999 Microsoft Corporation.  All rights reserved.
 *
*/
package sample.decoder;

import java.util.Vector;
import java.util.Hashtable;
/**
 * This is an object which represents the constant pool strucuture for a name_and_type.
 * It is generated by the ConstantObject during processing of the constant
 * pool bytes when decoding a given class file.
 *
 * @see sample.decoder.ConstantObject
 *
 * @date	April 28,1998
 * @version	1.2
*/
public class ConstantNameAndTypeObject extends ConstantObject implements DecodeDisplayInterface,
			java.io.Serializable{
/**
 * These are 2 bytes which represent an index into the constantpool for a constant+utf8
 * structure which is a field name or a method name.
*/
	protected byte[] name_index = new byte[2];
/**
 * These are 2 bytes which represent an index into the constantpool for a constant+utf8
 * structure which is a field descriptor or a method descriptor.
*/
	protected byte[] descriptor_index=new byte[2];
/**
 * This is the decoded constant_utf8 structure from the constant pool which was indexed
 * by the name_index bytes.
*/
	protected ConstantUtfObject name = null;
/**
 * This is the decoded constant_utf8 structure from the constant pool which was indexed
 * by the name_index bytes.
*/
	protected ConstantUtfObject descriptor = null;
/**
 *  This is the String value from the constant_utf8 structure that represents the constant
 * field or method name
 *
 * @returns String - field or method name
*/
public String getConstantName(){
//	return name.toString();
	return "";
}
/**
 * This method is a member of the DecodeDisplayInterface and is used to 
 * return the object name and the attribute name.  Subclasses of this object
 * should over ride this method to provide more detailed information.
 *
 * @see com.ms.tools.decode.DecodeDisplayInterface
 *
 * @returns String name representing this object and its attribute type.
*/
public String getName(){
	return poolIndex+" : Name And Type - "+getConstantName()+", "+this.getDescriptor();
}
/**
 * After the initial decoding phase is complete, where all of the bytes have
 * been moved to their appropriate places, this method retrieves any 
 * referenced Constant Pool objects and stores the information with the 
 * object containing the refernece.
 *
 * @param Vector data - These are the ConstantObjects that were created from the 
 *						constant pool structures in the class file.
*/
public void link(Vector data){
	super.link(data);
	addEntry("Name Index",getByteStartIndex()+1,2,data.elementAt(Decode.getIntegerValue(name_index)).toString(),
			 name_index);
	addEntry("Descriptor Index",getByteStartIndex()+3,2,data.elementAt(Decode.getIntegerValue(descriptor_index)).toString(),
			 descriptor_index);
	name=(ConstantUtfObject)data.elementAt(Decode.getIntegerValue(name_index));
	descriptor=(ConstantUtfObject)data.elementAt(Decode.getIntegerValue(descriptor_index));
}
/**
 *  This is the String value from the constant_utf8 structure that represents the constant
 * field or method descriptor
 *
 * @returns String - field or method descriptor
*/
public String getDescriptor(){
//	return descriptor.toString();
	return "";
}
/**
 * Accessor method that converts the 2 bytes which represent the index into the constant
 * pool table for the constant_utf8 structure to an integer value.
 *
 * @returns int -  integer representation of the constant pool name_index
*/
public int getNameIndex(){
	return Decode.getIntegerValue(name_index);
}
/**
 * Accessor method that converts the 2 bytes which represent the index into the constant
 * pool table for the constant_utf8 structure to an integer value.
 *
 * @returns int -  integer representation of the constant pool descriptor_index
*/
public int getDescriptorIndex(){
	return Decode.getIntegerValue(descriptor_index);
}
/**
 * Default method for determining a display String which adequatly 
 * represents this object and its contents.  Subclasses of this object 
 * should override this method to provide more detailed information.
 *
 * @returns String representing this object.
*/
public String toString(){
	return getName();
}
/**
 * This is the workhorse for the object, it decodes an instance of itself starting at the
 * specified byte index into the stream provided.  This was created as a static method
 * rather than a constructor, as there may be times when this method returns a more
 * specialized object which is a subclass of itself.
 *
 * @param byte[] stream -  entire bytes of the class that is being decoded
 * @param int index - this is the index into the stream for the starting byte.
 *
 * @returns ConstantObject - this is a ConstantObject representing the current Constant
 *							pool structure.
*/
public static ConstantObject getInstance(byte[] stream, int index){
        ConstantNameAndTypeObject xconst = new ConstantNameAndTypeObject();
        xconst.setByteStartIndex(index);
        xconst.tag=stream[index++];
		xconst.addEntry("Tag",index-2,1,xconst.tag+" - Name and Type",
						new byte[]{(byte)xconst.tag});
		xconst.name_index[0]=stream[index++];
		xconst.name_index[1]=stream[index++];
		xconst.descriptor_index[0]=stream[index++];
		xconst.descriptor_index[1]=stream[index++];
        xconst.setByteStopIndex(index-1);
		return xconst;
}
}