/*
 *
 * sample.decoder.ConstantObject.java
 *
 * (C) Copyright 1995 - 1999 Microsoft Corporation.  All rights reserved.
 *
*/
package sample.decoder;

import java.util.Vector;
import java.util.Hashtable;
/**
 * As there are no provisions in the virtual machine specification for adding new 
 * constants into the constat pool, all possible constant strucutres are known. This 
 * means that only specialized instances (subclasses of ConstantObject) will be 
 * returned from this class.
 *
 * @see sample.decoder.ConstantClassObject
 * @see sample.decoder.ConstantDoubleObject
 * @see sample.decoder.ConstantFieldObject
 * @see sample.decoder.ConstantFloatObject
 * @see sample.decoder.ConstantIntegerObject
 * @see sample.decoder.ConstantInterfaceObject
 * @see sample.decoder.ConstantLongObject
 * @see sample.decoder.ConstantMethodObject
 * @see sample.decoder.ConstantStringObject
 * @see sample.decoder.ConstantUtfObject
 * @see sample.decoder.ConstantValueObject
 *
 * @date	April 28,1998
 * @version	1.1
*/
public class ConstantObject extends DecodeObject implements DecodeDisplayInterface,
		java.io.Serializable{
/**
 * Associated with each constant object is a 1 byte tag which indicates what type of 
 * constant_structure is being decoded.  This is the integer value of that byte and
 * the String is the name of the constant_structure.
*/
	public static final int CONSTANT_Class = 7;
/**
 * Associated with each constant object is a 1 byte tag which indicates what type of 
 * constant_structure is being decoded.  This is the integer value of that byte and
 * the String is the name of the constant_structure.
*/
	public static final int CONSTANT_Fieldref = 9;
/**
 * Associated with each constant object is a 1 byte tag which indicates what type of 
 * constant_structure is being decoded.  This is the integer value of that byte and
 * the String is the name of the constant_structure.
*/	
	public static final int CONSTANT_Methodref = 10;
/**
 * Associated with each constant object is a 1 byte tag which indicates what type of 
 * constant_structure is being decoded.  This is the integer value of that byte and
 * the String is the name of the constant_structure.
*/	
	public static final int CONSTANT_InterfaceMethodref = 11;
/**
 * Associated with each constant object is a 1 byte tag which indicates what type of 
 * constant_structure is being decoded.  This is the integer value of that byte and
 * the String is the name of the constant_structure.
*/	
	public static final int CONSTANT_String = 8;
/**
 * Associated with each constant object is a 1 byte tag which indicates what type of 
 * constant_structure is being decoded.  This is the integer value of that byte and
 * the String is the name of the constant_structure.
*/	
	public static final int CONSTANT_Integer = 3;
/**
 * Associated with each constant object is a 1 byte tag which indicates what type of 
 * constant_structure is being decoded.  This is the integer value of that byte and
 * the String is the name of the constant_structure.
*/	
	public static final int CONSTANT_Float = 4;
/**
 * Associated with each constant object is a 1 byte tag which indicates what type of 
 * constant_structure is being decoded.  This is the integer value of that byte and
 * the String is the name of the constant_structure.
*/	
	public static final int CONSTANT_Long = 5;
/**
 * Associated with each constant object is a 1 byte tag which indicates what type of 
 * constant_structure is being decoded.  This is the integer value of that byte and
 * the String is the name of the constant_structure.
*/	
	public static final int CONSTANT_Double = 6;
/**
 * Associated with each constant object is a 1 byte tag which indicates what type of 
 * constant_structure is being decoded.  This is the integer value of that byte and
 * the String is the name of the constant_structure.
*/	
	public static final int CONSTANT_UNKNOWN = 13;
/**
 * Associated with each constant object is a 1 byte tag which indicates what type of 
 * constant_structure is being decoded.  This is the integer value of that byte and
 * the String is the name of the constant_structure.
*/	
	public static final int CONSTANT_NameAndType = 12;
/**
 * As there are some constants (long and double) which take up two constant 
 * pool entries, it is necessary to skip the counter.  This boolean indicates
 * a double wide constant pool entry.
 */
	private boolean wide=false;
/**
 * Associated with each constant object is a 1 byte tag which indicates what type of 
 * constant_structure is being decoded.  This is the integer value of that byte and
 * the String is the name of the constant_structure.
*/	
	public static final int CONSTANT_Utf8 = 1;
/**
 * This is the ConstantObjects index into the Constant Pool
 */
	int poolIndex=0;
/**
 * This method is a member of the DecodeDisplayInterface and is used to 
 * return the object name and the attribute name.  Subclasses of this object
 * should over ride this method to provide more detailed information.
 *
 * @see sample.decoder.DecodeDisplayInterface
 *
 * @returns String name representing this object and its attribute type.
*/
public String getName(){
	return "";}

	public String toString(){return "";}

/**
 * Required by the DecodeDisplayInterface, returns any Vectors, or subobjects
 * which will need to be displayed as a subobject in a GUI.  The Hashtable is
 * indexed by a String that is to be used as a display name, the contents
 * will be either a Vector of Strings/Objects or a Decode Object.
 *
 * @returns Hashtable of subobjects/structures to be displayed as seperate
 *				nodes in the GUI.
*/
public Hashtable getChildren(){
	return new Hashtable();
}
/**
 * After the initial decoding phase is complete, where all of the bytes have
 * been moved to their appropriate places, this method retrieves any 
 * referenced Constant Pool objects and stores the information with the 
 * object containing the refernece.
 *
 * @param Vector data - These are the ConstantObjects that were created from the 
 *						constant pool structures in the class file.
*/
public void link(Vector data){
}
/**
 * Creates a Vector of the speficied number of constants from the stream it
 * is passed starting at the indicated index into the stream.
 *
 * @param byte[] classBytes - the bytes representing the class to decode.
 * @param int indexs -  this is the number of attributes expected.
 * @param int startByte - the index into the classBytes to start decoding.
 *
 * @returns Vector containing ConstantObjects or its subclasses.
*/
public Vector createObjects(byte[] classBytes,int indexs,int startByte)throws Exception{
	Vector list = new Vector();
	ConstantObject reserved = new ConstantObject();
    reserved.index=0;
    reserved.setByteStopIndex(startByte);
	list.addElement(reserved);
    for (int i=1;i<indexs;i++){
		//log("ConstantObject: about to retrieve constant number "+i+" of "+
		//	(indexs-1));
		//log("ConstantObject: last object "+list.lastElement().getClass().getName());
		Object newton=nextObject(classBytes, 
			((DecodeObject)list.elementAt(i-1)).getByteStopIndex()+1);
        ((DecodeObject)newton).index=i;
		//log("ConstantObject: retrieved next constant");
        list.addElement(newton);
		if (getWide()){
			wide=false;
			list.addElement(newton);
			i++;
		}
	}
	for (int j=0;j<list.size();j++){
		((ConstantObject)list.elementAt(j)).link(list);
		((ConstantObject)list.elementAt(j)).poolIndex=j;
	}
	log("ConstantObject : Done decoding, returning Vector");
	return list;
}
private boolean getWide(){
	return wide;
}
/**
 * Generic method for writting debug information to the screen.  By having all
 * methods in this class send their log information here, there is a central
 * point of change for turning debuggin off, or sending it to a differnet 
 * location.
 *
 * @param String data - message to be displayed.
*/
public synchronized void log(String data){
//		System.out.println(data);
}
/**
 * Determines the next ConstantObject or its subclass in the given stream
 * by starting to look for a constant at the given index.  
 *
 * @param byte[] classBytes - the class as bytes to be decoded.
 * @param int index - the index into the byte stream to start looking.
 *
 * @returns Object - the ConstantObject for the next constant in the stream.
*/
public Object nextObject(byte[] stream, int startByte)throws Exception{
	if (stream[startByte]==CONSTANT_Class){ 
		log("ConstantObject: ConstantClassObject found");
		return ConstantClassObject.getInstance(stream, startByte);
    }
	if (stream[startByte]==CONSTANT_Fieldref) 
			return ConstantFieldObject.getInstance(stream, startByte);
    if (stream[startByte]==CONSTANT_Methodref) 
			return ConstantMethodObject.getInstance(stream,startByte);
	if (stream[startByte]==CONSTANT_InterfaceMethodref) 
			return ConstantInterfaceObject.getInstance(stream,startByte);
    if (stream[startByte]==CONSTANT_String) 
			return ConstantStringObject.getInstance(stream,startByte);
	if (stream[startByte]==CONSTANT_Integer) 
			return ConstantIntegerObject.getInstance(stream,startByte);
    if (stream[startByte]==CONSTANT_Float) 
		return ConstantFloatObject.getInstance(stream,startByte);
	if (stream[startByte]==CONSTANT_Long){
		wide=true;
		return ConstantLongObject.getInstance(stream, startByte);
	}
	if (stream[startByte]==CONSTANT_Double){ 
		wide=true;
		return ConstantDoubleObject.getInstance(stream,startByte);
	}
	if (stream[startByte]==CONSTANT_NameAndType) 
			return ConstantNameAndTypeObject.getInstance(stream, startByte);
    if (stream[startByte]==CONSTANT_Utf8) 
			return ConstantUtfObject.getInstance(stream,startByte);
	if (stream[startByte]==00){
		System.out.println("Unknown Constant Object");
		ConstantObject aa = new ConstantObject();
		byte[] bb = new byte[100];
		for (int t=0;t<100;t++)
			bb[t]=stream[startByte+t];
		System.out.println("possible string is "+(new String(bb))+" "+Decode.getByteString(bb));
		aa.setByteStopIndex(startByte-1);
		return aa;
	}
    throw new Exception("ConstantObject: unable to determine tag "+stream[startByte]+" at "+startByte+"!");
}
}