/*
 *
 * sample.decoder.ConstantStringObject
 *
 * (C) Copyright 1995 - 1999 Microsoft Corporation.  All rights reserved.
 *
*/
package sample.decoder;

import java.util.Vector;
import java.util.Hashtable;
/**
 * This is an object which represents the constant pool strucuture for a string.
 * It is generated by the ConstantObject during processing of the constant
 * pool bytes when decoding a given class file.
 *
 * @see sample.decoder.ConstantObject
 *
 * @date	April 28,1998
 * @version	1.2
*/
public class ConstantStringObject extends ConstantObject implements 
				DecodeDisplayInterface, java.io.Serializable{
/**
 * These are 2 bytes which represent an index into the constantpool for a constant+utf8
 * structure which is the string.
*/
	protected byte[] string_index = new byte[2];
/**
 * This is the decoded constant_utf8 structure from the constant pool which was indexed
 * by the string_index bytes.
*/
	protected ConstantUtfObject string = null;
/**
 * After the initial decoding phase is complete, where all of the bytes have
 * been moved to their appropriate places, this method retrieves any 
 * referenced Constant Pool objects and stores the information with the 
 * object containing the refernece.
 *
 * @param Vector data - These are the ConstantObjects that were created from the 
 *						constant pool structures in the class file.
*/
public void link(Vector data){
	super.link(data);
	addEntry("String",getByteStartIndex()+1,2,data.elementAt(Decode.getIntegerValue(string_index)).toString(),
			 string_index);
	string=(ConstantUtfObject)data.elementAt(Decode.getIntegerValue(string_index));
}
/**
 *  This is the String value from the constant_utf8 structure that represents the constant
 * string.
 *
 * @returns String - string value
*/
public String getString(){
	return string.toString();
}
/**
 * Accessor method that converts the 2 bytes which represent the index into the constant
 * pool table for the constant_utf8 structure to an integer value.
 *
 * @returns int -  integer representation of the constant pool string_index
*/
public int getStringIndex(){
	return Decode.getIntegerValue(string_index);
}
/**
 * Default method for determining a display String which adequatly 
 * represents this object and its contents.  Subclasses of this object 
 * should override this method to provide more detailed information.
 *
 * @returns String representing this object.
*/
public String toString(){
	return getString();
}
/**
 * This method is a member of the DecodeDisplayInterface and is used to 
 * return the object name and the attribute name.  Subclasses of this object
 * should over ride this method to provide more detailed information.
 *
 * @see com.ms.tools.decode.DecodeDisplayInterface
 *
 * @returns String name representing this object and its attribute type.
*/
public String getName(){
	return poolIndex+" : String - "+string.getString();
}
/**
 * This is the workhorse for the object, it decodes an instance of itself starting at the
 * specified byte index into the stream provided.  This was created as a static method
 * rather than a constructor, as there may be times when this method returns a more
 * specialized object which is a subclass of itself.
 *
 * @param byte[] stream -  entire bytes of the class that is being decoded
 * @param int index - this is the index into the stream for the starting byte.
 *
 * @returns ConstantObject - this is a ConstantObject representing the current Constant
 *							pool structure.
*/
public static ConstantObject getInstance(byte[] stream, int index){
        ConstantStringObject xconst = new ConstantStringObject();
        xconst.setByteStartIndex(index);
        xconst.tag=stream[index++];
		xconst.addEntry("Tag",
						index-2,1,
						xconst.tag+" - String",
						new byte[]{(byte)xconst.tag});
        xconst.string_index[0]=stream[index++];
        xconst.string_index[1]=stream[index];
		xconst.setByteStopIndex(index);
        return xconst;
}
}