/*
 *
 * sample.decoder.ConstantUtfObject
 *
 * (C) Copyright 1995 - 1999 Microsoft Corporation.  All rights reserved.
 *
*/
package sample.decoder;

import java.util.Vector;
import java.util.Hashtable;
/**
 * This is an object which represents the constant pool strucuture for a utf8 string.
 * It is generated by the ConstantObject during processing of the constant
 * pool bytes when decoding a given class file.
 *
 * @see sample.decoder.ConstantObject
 *
 * @date	April 28,1998
 * @version	1.2
*/
public class ConstantUtfObject extends ConstantObject implements java.io.Serializable{
/**
 * These are 2 bytes which represent number of sequential bytes after these which are the
 * Utf8 string.
*/
	protected byte[] length = new byte[2];
/**
 * Bytes which represent the Utf8 String.
*/
	protected byte[] bytes;
/**
 * This constructor is for creating an empty instance that will hold values, it is not
 * to be used for decoding class files.
*/
public ConstantUtfObject(){}
/**
 * This constructor is intended to be used where a ConstantUtfObject is manually 
 * created with an intended String value.  The String passed as a parameter will set the
 * appropriate length and bytes values.
 *
 * @param String -  this is the String to be used to set the ConstantUtf attributes.
*/
public ConstantUtfObject(String i){
	length[0]=0;
	length[1]=(new Integer(i.length())).byteValue();
	bytes=i.getBytes();
}
/**
 * This is the workhorse for the object, it decodes an instance of itself starting at the
 * specified byte index into the stream provided.  This was created as a static method
 * rather than a constructor, as there may be times when this method returns a more
 * specialized object which is a subclass of itself.
 *
 * @param byte[] stream -  entire bytes of the class that is being decoded
 * @param int index - this is the index into the stream for the starting byte.
 *
 * @returns ConstantObject - this is a ConstantObject representing the current Constant
 *							pool structure.
*/
public static ConstantObject getInstance(byte[] stream, int index){
        ConstantUtfObject xconst = new ConstantUtfObject();
        xconst.setByteStartIndex(index);
        xconst.tag=stream[index++];
		xconst.addEntry("Tag",
						index-2,1,
						xconst.tag+" - Utf8",
						new byte[]{(byte)xconst.tag});
        xconst.length[0]=stream[index++];
		xconst.length[1]=stream[index++];
		xconst.addEntry("Length",index-2,2,""+Decode.getIntegerValue(xconst.length),
						xconst.length);
		xconst.bytes = new byte[Decode.getIntegerValue(xconst.length)];
        for (int i=0;i<xconst.bytes.length;i++)
                xconst.bytes[i]=stream[index++];
		xconst.addEntry("UTF8 String",xconst.getByteStartIndex()+3,
						index-xconst.getByteStartIndex()+2,
						xconst.getString(),xconst.bytes);
        xconst.setByteStopIndex(index-1);
        return xconst;
}
/**
 * Default method for determining a display String which adequatly 
 * represents this object and its contents.  Subclasses of this object 
 * should override this method to provide more detailed information.
 *
 * @returns String representing this object.
*/
public String toString(){
	return getString();
}
/**
 * This method is a member of the DecodeDisplayInterface and is used to 
 * return the object name and the attribute name.  Subclasses of this object
 * should over ride this method to provide more detailed information.
 *
 * @see com.ms.tools.decode.DecodeDisplayInterface
 *
 * @returns String name representing this object and its attribute type.
*/
public String getName(){
	return poolIndex+" : Utf8 - "+getString();
}
/**
 * Accessor method that converts the 2 bytes which represent the length in bytes of this
 * utf8 String.  This value is returned as an integer.
 *
 * @returns int -  integer representation of the length of this utf8 string.
*/
public int getLength(){
	return Decode.getIntegerValue(length);
}
/**
 * String which is represented by the bytes contained within this constant_utf8 structure.
 *
 * @returns String - java String object made from this objects bytes.
*/
public String getString(){
	return new String(bytes);
}
}