/*
 *
 * sample.decoder.ConstantValueAttribute.java
 *
 * (C) Copyright 1995 - 1999 Microsoft Corporation.  All rights reserved.
 *
*/
package sample.decoder;

import java.util.Vector;
import java.util.Hashtable;
/**
 * The Constant value Attribute is a specialized attribute typically associated with 
 * fields.  
 *
 * SCOPE : Field
 * 
 * @date	April 28,1998
 * @version	1.2
*/
public class ConstantValueAttribute extends AttributeObject implements DecodeDisplayInterface,
		java.io.Serializable{
/**
 * The 2 bytes represent an index into the constant pool where a consant object exists
 * of the type:
 *
 *	CONSTANT_Long	CONSTANT_Float	CONSTANT_Double	CONSTANT_Integer	CONSTANT_String
*/
	protected byte[] constantvalue_index = new byte[2];
/**
 * This is the decoded constant structure from the constant pool which was indexed
 * by the constantvalue_index bytes.
*/
	protected ConstantObject constantValue = null;
/**
 * After the initial decoding phase is complete, where all of the bytes have
 * been moved to their appropriate places, this method retrieves any 
 * referenced Constant Pool objects and stores the information with the 
 * object containing the refernece.
 * 
 * @param Vector data - These are the ConstantObjects that were created from the 
 *						constant pool structures in the class file.
*/
public void link(Vector data){
	super.link(data);
	constantValue=(ConstantObject)data.elementAt(
					Decode.getIntegerValue(constantvalue_index));
}
/**
 * Accessor method returning the decoded constant object that was referenced by the 
 * constantvalue_index, and stored in the consantValue field.
 *
 * @returns ConstantObject - represents the constant value that this attribute holds.
*/
public ConstantObject getConstantObject(){
	return constantValue;
}
/**
 * This simply returns the String value of the constantValue object
 *
 * @returns String - representing the decoded constant_value structure.
*/
public String getConstantValue(){
	return constantValue.toString();
}
/**
 * Default method for determining a display String which adequatly 
 * represents this object and its contents.  Subclasses of this object 
 * should override this method to provide more detailed information.
 *
 * @returns String representing this object.
*/
public String toString(){
	return getName();
}
/**
 * Accessor method that converts the 2 bytes which represent the index into the constant
 * pool table for the constant structure to an integer value.
 *
 * @returns int -  integer representation of the constant pool consantvalue_index
*/
public int getConstantValueIndex(){
	return Decode.getIntegerValue(constantvalue_index);
}
/**
 * This method is a member of the DecodeDisplayInterface and is used to 
 * return the object name and the attribute name.  Subclasses of this object
 * should over ride this method to provide more detailed information.
 *
 * @see sample.decoder.DecodeDisplayInterface
 *
 * @returns String name representing this object and its attribute type.
*/
public String getName(){
	return "ConstantValue";
}
/**
 * This is the workhorse for the object, it decodes an instance of itself starting at the
 * specified byte index into the stream provided.  This was created as a static method
 * rather than a constructor, as there may be times when this method returns a more
 * specialized object which is a subclass of itself.
 *
 * @param byte[] stream -  entire bytes of the class that is being decoded
 * @param int index - this is the index into the stream for the starting byte.
 *
 * @returns ConstantObject - this is a ConstantObject representing the current Constant
 *							pool structure.
*/
public static Object getInstance(byte[] stream, int index,Vector constantPool){
        ConstantValueAttribute xconst = new ConstantValueAttribute();
        xconst.constantvalue_index[0]=stream[index++];
        xconst.constantvalue_index[1]=stream[index++];
		xconst.addEntry("Constant Value",index-2,2,
			constantPool.elementAt(Decode.getIntegerValue(xconst.constantvalue_index)).toString(),
			xconst.constantvalue_index);
        xconst.contents = new byte[8];
        for (int i=0;i<xconst.contents.length;i++){
			xconst.contents[i]=stream[index++];
		}
        return xconst;
}
}