/*
 *
 * sample.decoder.Decode.java
 *
 * (C) Copyright 1995 - 1999 Microsoft Corporation.  All rights reserved.
 *
*/
package sample.decoder;

import java.util.Vector;
/**
 * This class is a toolkit of static methods which have been useful in decoding and
 * displaying the information from .class files.  There are methods for returning
 * numberic values from byte arrays, and for displaying byte arrays as hex strings.
 * Although static and synchronized methods are slower than regular object methods, 
 * I have done this so that this class can be used from multiple threads and outside
 * the scope of the Class File Decoder Tool.
 *
 * @date	April 28,1998
 * @version	1.1
*/
public class Decode implements java.io.Serializable{
	/**
	 * This is a collection of all available classpath Loaders that may be used to 
	 * acquire classes.
	 */
	protected static Vector classpathLoaders = null;
/**
 * The byte array is converted to a hexidecimal display string which seperates the
 * hex value of each byte with a space.  No line feeds are added.
 *
 * @param byte[] b - the array of bytes to be represented as a String.
 *
 * @returns String - represents the byte array taken as a hex string.
*/
public static synchronized String getByteString(byte[] b){
	String data = "";
	if (b!=null)
		for (int i=0;i<b.length;i++){
			int integ = (new Integer(b[i])).intValue();
			data+=Integer.toHexString(getIntegerValue(b[i]))+" ";
		}
	return data;
}
/**
 * The byte array is converted to an integer value.  This method operates by converting
 * each of the bytes to bits, (assuming that the rightmost bit is the lowest), and then
 * converts the array of bits to an integer value.
 *
 * @param byte[] b - the bytes to be converted into an integer.
 *
 * @returns int - integer representation of the byte[]
*/
public static synchronized int getIntegerValue(byte[] b){
        String totalBits = "";
        for (int loop=0;loop<b.length;loop++)
                totalBits+=getBits(b[loop]);        
        int i = getIntegerValue(totalBits);
		if ((i==2147483647)&&(b[0]==(byte)0x80))
			return -2147483648;
		if (i<0)
			return ++i;
		return i;
}
/**
 * The byte array is converted to an float value.  This method operates by converting
 * each of the bytes to bits, (assuming that the rightmost bit is the lowest), and then
 * converts the array of bits to an float value.
 *
 * @param byte[] b - the bytes to be converted into an float.
 *
 * @returns float - float representation of the byte[]
*/
public static synchronized float getFloatValue(byte[] bytes){
	int b = getIntegerValue(bytes);															
	return new Integer(b).floatValue();
}
/**
 * The byte array is converted to an double value.  This method operates by converting
 * each of the bytes to bits, (assuming that the rightmost bit is the lowest), and then
 * converts the array of bits to an double value.
 *
 * @param byte[] b - the bytes to be converted into an double.
 *
 * @returns double - double representation of the byte[]
*/
public static synchronized double getDoubleValue(byte[] bytes){
	long b = getLongValue(bytes);
	return new Long(b).doubleValue();
}
/**
 * The byte array is converted to an long value.  This method operates by converting
 * each of the bytes to bits, (assuming that the rightmost bit is the lowest), and then
 * converts the array of bits to an long value.
 *
 * @param byte[] b - the bytes to be converted into an long.
 *
 * @returns long - long representation of the byte[]
*/
public static synchronized long getLongValue(byte[] b){
		String totalBits = "";
		for (int loop=0;loop<b.length;loop++)
			totalBits+=getBits(b[loop]);        
		long i= getLongValue(totalBits);
		if ((i==9223372036854775807L)&&(b[0]==(byte)0x80))
			return -9223372036854775808L;
		if (i<0)
			return ++i;
		return i;

}
/**
 * This method is nearly exactly the same as the math.pow method, except that it takes
 * integers instead of doubles as parameters.  This method reduces the amount of code
 * required to convert bits to integers.
 *
 * @param int i - this is the number that is to be raised to the power of the second
 *					parameter (j);
 * @param int j - this is the power.
 *
 * @returns int - result of the i^j operation
*/
public static synchronized int pow(int i, int j){
               return (new Double(
                        Math.pow((new Integer(i)).doubleValue()
                        ,(new Integer(j)).doubleValue()))).intValue();
}
/**
 * The byte is converted to a bit String, assuming that the rightmost bit is the lowest. 
 *
 * @param byte data - the byte to be converted into a bit String.
 *
 * @returns String - bit representation of the byte
*/
public static synchronized String getBits(byte data){
        String bits = "";
        int num = (new Byte(data)).intValue();
        if (num<0) num=num+256;
        for (int i=7;i>=0;i--){
                int power = (new Double(
                        Math.pow((new Integer(2)).doubleValue()
                        ,(new Integer(i)).doubleValue()))).intValue();
                if ((power&num)==power) bits+="1";
                else bits+="0";
        }
        return bits;
}
/**
 * The String of bits, assuming that the rightmost bit is the lowest, converts each bit
 * in the String to an integer value and sums them all.
 *
 * @param String bits - the bits to be converted into an integer.
 *
 * @returns int - integer representation of the byte[]
*/
public static synchronized int getIntegerValue(String bits){
        int total=0;
        for (int i=0;i<bits.length();i++)   {
                total+=(new Integer(bits.substring(bits.length()-i-1,
                        bits.length()-i))).intValue()*
                (new Double(
                        Math.pow((new Integer(2)).doubleValue()
                        ,(new Integer(i)).doubleValue()))).intValue();
          }
        return total;
}
/**
 * The String of bits, assuming that the rightmost bit is the lowest, converts each bit
 * in the String to an long value and sums them all.
 *
 * @param String bits - the bits to be converted into an long.
 *
 * @returns long - long representation of the byte[]
*/
public static synchronized long getLongValue(String bits){
        long total=0;
        for (int i=0;i<bits.length();i++)   {
                total+=(new Integer(bits.substring(bits.length()-i-1,
                        bits.length()-i))).longValue()*
                (new Double(
                        Math.pow((new Integer(2)).doubleValue()
                        ,(new Integer(i)).doubleValue()))).longValue();
          }
        return total;
}
/**
 * The byte is converted to an integer value.  This method operates by converting
 * the byte to bits, (assuming that the rightmost bit is the lowest), and then
 * converts the array of bits to an integer value.
 *
 * @param byte b - the bytes to be converted into an integer.
 *
 * @returns int - integer representation of the byte
*/
public static synchronized int getIntegerValue(byte c){
	byte[] a = new byte[1];
        a[0]=c;
	return getIntegerValue(a);
}
/**
 * Creates the source, or human readable, form of the given java class file
 * descriptor.  The raw discriptor is passed as a parameter and the sourceified
 * version is returned, complete with [] for arrays.
 * 
 * @param String data - raw descriptor
 * 
 * @return String - source view of descriptor
 */
public static synchronized String getDescriptor(String data){
	if (data.indexOf(")")!=-1)
		data=data.substring(data.indexOf(")")+1,data.length());
	String result ="";
	boolean init = true;
	while (data.length()!=0){
		if (!init) result+=", ";
		init=false;
		String code = "";
		boolean done=false;
		int idd = 0;
		boolean object = false;
		while ((!done)&&(idd!=data.length())){
			String chunk=data.substring(idd,idd+1);
			if (chunk.equals("[")){}
			else if (chunk.equals(";")) done=true;
			else if (chunk.equals("L")) object=true;
			else if ((!chunk.equals("["))&&(!object)) done=true;
			else if (chunk.equals(";")) object=false;
			else if (!object) done=true;
			code+=chunk;
			idd++;
		}
	data=data.substring(idd,data.length());
	int arrayCount = 0;
	while (code.charAt(0)=='['){
		arrayCount++;
		code=code.substring(1,code.length());
	}
	if (code.length()!=1)
		if(code.indexOf("L")==-1)
			throw new RuntimeException
				(("Incorrect length for descriptor type "+code+" "+arrayCount));
	String array="";
	for (int i=0;i<arrayCount;i++) array+="[]";
	if (code.equals("B")) return "byte"+array;
	else if (code.equals("C")) result+= "char"+array;
	else if (code.equals("D")) result+= "double"+array;
	else if (code.equals("F")) result+= "float"+array;
	else if (code.equals("I")) result+= "int"+array;
	else if (code.equals("J")) result+= "long"+array;
	else if (code.equals("S")) result+= "short"+array;
	else if (code.equals("Z")) result+= "boolean"+array;
	else if (code.equals("V")) result+= "void"+array;
	else if (code.indexOf("L")!=-1)
		result+= (code.substring(1,code.length()-1)).replace('/','.')+array;	
	else return data;
	}
	return result;
}
/**
 * As many class file structures use the '/' notation for representing class names,
 * this method replaces the /'s with .'s.
 * 
 * @param String d - raw class name
 * 
 * @return String - source view of the class name.
 */
	public static synchronized String getNameFromDot(String d){
		d.replace('/','.');
		return d;
}
/**
 * Reads the specified class file from the classpath and returns the
 * bytes.  The Class name must be specified like this:
 * 
 *				java.lang.String = java/lang/String.class
 *  
 * @param String filename - Name of the class to load.
 * 
 * @return byte[] - bytes that represent this class, or null if couldn't be loaded.
 */
	public synchronized static byte[] getClassFromPath(String filename){
		if (classpathLoaders==null)
			initClassPathLoaders();
		for (int i=0;i<classpathLoaders.size();i++){
			byte[] b = 
			((ClasspathLoader)classpathLoaders.elementAt(i)).getClassFromPath(filename);
			if (b!=null)
				return b;
		}
		return null;
	}
/**
 * Uses reflection to load all possible classpath Loaders from the current system.
 * Each loader will be used from highest order to lowest when attempting to locate the
 * bytes for a particular class.  All Classes with the name 
 *				
 *			sample.decoder.ClasspathLoaderX   where X is an integer, starting from 0
 * 
 * will be loaded.  The higher integer numbers will be used first.
 */
	protected static void initClassPathLoaders(){
		classpathLoaders = new Vector();
		try{
			boolean done=false;
			int i=0;
			while (true){//when a class cannot be found, the Exception will kick us out
				Class c = Class.forName("sample.decoder.ClasspathLoader"+i++);
					classpathLoaders.insertElementAt(c.newInstance(),0);
			}//end while
		}catch(Exception ii){
		//	System.out.println("Exception intializing classpath loaders "+ii);
		}
	}
/**
 * Given a fully qualified class name this method returns the 'package' part. That
 * is, for java.lang.String it would return java.lang.  For classes without 
 * packages it returns the String "<NOT IN A PACKAGE>"
 * 
 * @param String fullname - fully qualified class name.
 * 
 * @return String - the package portion of the class name.
 */
	public static synchronized String getPackageName(String fullname){
		if (fullname.indexOf(".")==-1)
			return "<NOT IN A PACKAGE>";
		else
			return fullname.substring(0,fullname.lastIndexOf("."));
	}
/**
 * Given a fully qualified class name, this method returns only the class name, 
 * removing the package portion.  Given java.lang.String, it will return String.
 * If a name is given such as Integer, Integer will be returned.
 * 
 * @param String fullname - Fully qualified class name.
 * 
 * @return String - Class portion of the name.
 */
	public static synchronized String getClassName(String fullname){
		if (fullname.indexOf(".")==-1)
			return fullname;
		else
			return fullname.substring(
				fullname.lastIndexOf(".")+1,fullname.length());
	}
/**
 * This is a simple mail method to test the byte -> primative converters for
 * Integer, Float, Long and Double.
 */
	public static void main(String args[]){
		//Test boundary of Integer primative
		System.out.println("Integer "+0x00000000+" "+getIntegerValue(new byte[]{
																	 (byte)0x00,
																	 (byte)0x00,
																	 (byte)0x00,
																	 (byte)0x00}));
		System.out.println("Integer "+0x7fffffff+" "+getIntegerValue(new byte[]{
																	 (byte)0x7f,
																	 (byte)0xff,
																	 (byte)0xff,
																	 (byte)0xff}));
		System.out.println("Integer "+0x80000000+" "+getIntegerValue(new byte[]{
																	 (byte)0x80,
																	 (byte)0x00,
																	 (byte)0x00,
																	 (byte)0x00}));
		System.out.println("Integer "+0x80000001+" "+getIntegerValue(new byte[]{
																	 (byte)0x80,
																	 (byte)0x00,
																	 (byte)0x00,
																	 (byte)0x01}));
		System.out.println("Integer "+0xFFFFFFFF+" "+getIntegerValue(new byte[]{
																	 (byte)0xFF,
																	 (byte)0xFF,
																	 (byte)0xFF,
																	 (byte)0xFF}));
		//Test boundary of Float primative
		System.out.println("Float "+(float)0x00000000+" "+getFloatValue(new byte[]{
																	 (byte)0x00,
																	 (byte)0x00,
																	 (byte)0x00,
																	 (byte)0x00}));
		System.out.println("Float "+(float)0x7fffffff+" "+getFloatValue(new byte[]{
																	 (byte)0x7f,
																	 (byte)0xff,
																	 (byte)0xff,
																	 (byte)0xff}));
		System.out.println("Float "+(float)0x80000000+" "+getFloatValue(new byte[]{
																	 (byte)0x80,
																	 (byte)0x00,
																	 (byte)0x00,
																	 (byte)0x00}));
		System.out.println("Float "+(float)0x80000001+" "+getFloatValue(new byte[]{
																	 (byte)0x80,
																	 (byte)0x00,
																	 (byte)0x00,
																	 (byte)0x01}));
		System.out.println("Float "+(float)0xFFFFFFFF+" "+getFloatValue(new byte[]{
																	 (byte)0xFF,
																	 (byte)0xFF,
																	 (byte)0xFF,
																	 (byte)0xFF}));
		//Test boundary of Long primative
		System.out.println("Long "+0x0000000000000000L+" "+
						   getLongValue(new byte[]{
									 (byte)0x00,
									 (byte)0x00,
									 (byte)0x00,
									 (byte)0x00,
									 (byte)0x00,
									 (byte)0x00,
									 (byte)0x00,
									 (byte)0x00}));
		System.out.println("Long "+0x7fffffffffffffffL+" "+
						   getLongValue(new byte[]{
									 (byte)0x7f,
									 (byte)0xff,
									 (byte)0xff,
									 (byte)0xff,
									 (byte)0xff,
									 (byte)0xff,
									 (byte)0xff,
									 (byte)0xff}));
		System.out.println("Long "+0x8000000000000000L+" "+
						   getLongValue(new byte[]{
									 (byte)0x80,
									 (byte)0x00,
									 (byte)0x00,
									 (byte)0x00,
									 (byte)0x00,
									 (byte)0x00,
									 (byte)0x00,
									 (byte)0x00}));
		System.out.println("Long "+0x8000000000000001L+" "+
						   getLongValue(new byte[]{
									 (byte)0x80,
									 (byte)0x00,
									 (byte)0x00,
									 (byte)0x00,
									 (byte)0x00,
									 (byte)0x00,
									 (byte)0x00,
									 (byte)0x01}));
		System.out.println("Long "+0xFFFFFFFFFFFFFFFFL+" "+
						   getLongValue(new byte[]{
									 (byte)0xFF,
									 (byte)0xFF,
									 (byte)0xFF,
									 (byte)0xFF,
									 (byte)0xFF,
									 (byte)0xFF,
									 (byte)0xFF,
									 (byte)0xFF}));
			//Test boundary of Double primative
		System.out.println("Double "+(double)0x0000000000000000L+" "+
						   getDoubleValue(new byte[]{
									 (byte)0x00,
									 (byte)0x00,
									 (byte)0x00,
									 (byte)0x00,
									 (byte)0x00,
									 (byte)0x00,
									 (byte)0x00,
									 (byte)0x00}));
		System.out.println("Double "+(double)0x7fffffffffffffffL+" "+
						   getDoubleValue(new byte[]{
									 (byte)0x7f,
									 (byte)0xff,
									 (byte)0xff,
									 (byte)0xff,
									 (byte)0xff,
									 (byte)0xff,
									 (byte)0xff,
									 (byte)0xff}));
		System.out.println("Double "+(double)0x8000000000000000L+" "+
						   getDoubleValue(new byte[]{
									 (byte)0x80,
									 (byte)0x00,
									 (byte)0x00,
									 (byte)0x00,
									 (byte)0x00,
									 (byte)0x00,
									 (byte)0x00,
									 (byte)0x00}));
		System.out.println("Double "+(double)0x8000000000000001L+" "+
						   getDoubleValue(new byte[]{
									 (byte)0x80,
									 (byte)0x00,
									 (byte)0x00,
									 (byte)0x00,
									 (byte)0x00,
									 (byte)0x00,
									 (byte)0x00,
									 (byte)0x01}));
		System.out.println("Double "+(double)0xFFFFFFFFFFFFFFFFL+" "+
						   getDoubleValue(new byte[]{
									 (byte)0xFF,
									 (byte)0xFF,
									 (byte)0xFF,
									 (byte)0xFF,
									 (byte)0xFF,
									 (byte)0xFF,
									 (byte)0xFF,
									 (byte)0xFF}));
	}
}