/*
 *
 * sample.decoder.DelegateAttribute.java
 *
 * (C) Copyright 1995 - 1999 Microsoft Corporation.  All rights reserved.
 *
*/
package sample.decoder;

import java.util.Vector;
import java.util.Hashtable;
/**
 * The Delegate attribute specifies that a class is a delegate type and will be used for 
 * method invocations.  When applied to a method, this attribute indicates that the
 * method is a delegate type and will be treated as an invocation method by the 
 * microsoft VM.  This attribute is generated by the delegate keyword.  
 *
 * SCOPE : Method
 * 
 * @date	April 28,1998
 * @version	1.1
*/
public class DelegateAttribute extends AttributeObject implements DecodeDisplayInterface,
		java.io.Serializable{
/**
 * This byte contains the data portion of this attribute
 */
	protected byte[] data = null;
/**
 * Object refered to by the data portion of the delegate attribute.
 */
	protected Object object = null;
/**
 * This method is a member of the DecodeDisplayInterface and is used to 
 * return the object name and the attribute name.  Subclasses of this object
 * should over ride this method to provide more detailed information.
 *
 * @see sample.decoder.DecodeDisplayInterface
 *
 * @returns String name representing this object and its attribute type.
*/
public String getName(){
	return "Delegate";
}
/**
 * After the initial decoding phase is complete, where all of the bytes have
 * been moved to their appropriate places, this method retrieves any 
 * referenced Constant Pool objects and stores the information with the 
 * object containing the refernece.
 * 
 * @param Vector data - These are the ConstantObjects that were created from the 
 *						constant pool structures in the class file.
*/
public void link(Vector d){
	super.link(d);
	if (data!=null)object=d.elementAt(Decode.getIntegerValue(data));
}
/**
 * This is the workhorse for the object, it decodes an instance of itself starting at the
 * specified byte index into the stream provided.  This was created as a static method
 * rather than a constructor, as there may be times when this method returns a more
 * specialized object which is a subclass of itself.
 *
 * @param byte[] stream -  entire bytes of the class that is being decoded
 * @param int index - this is the index into the stream for the starting byte.
 * @param Vector constantPool - these are the constantPool objects, any index into the
 *								constant pool will be resolved by retrieving the object 
 *								at the required instance from this Vector.
 *
 * @returns Object - This is the decoded object which is a populated instance of this 
 *					class.
*/
public static Object getInstance(byte[] stream, int index,Vector constantPool){
        DelegateAttribute xconst = new DelegateAttribute();
      	byte[] temp = new byte[]{
						 stream[index-2],
						 stream[index-1]};
		if (Decode.getIntegerValue(temp)!=0){
			xconst.data=new byte[2];
			xconst.data[0]=stream[index++];
			xconst.data[1]=stream[index++];
			xconst.addEntry("Data",index-2,2,""+Decode.getIntegerValue(xconst.data),xconst.data);
		}
        return xconst;
}
/**
 * Generic method for writting debug information to the screen.  By having all
 * methods in this class send their log information here, there is a central
 * point of change for turning debuggin off, or sending it to a differnet 
 * location.
 *
 * @param String data - message to be displayed.
*/
public synchronized void log(String data){
//	System.out.println(data);
}
}