/*
 *
 * sample.decoder.Entry.java
 *
 * (C) Copyright 1995 - 1999 Microsoft Corporation.  All rights reserved.
 *
 */
package sample.decoder;
/**
 * This object was designed to hold all of the properties that have been decoded
 * from a given Class File sub object.  That is, a CodeAttribute will have an 
 * Entry object associated with each of its properties, with each entry object
 * containing all relevant information to fully describe that property.  The Entry
 * objects are typically stored within the DecodeObject.
 * 
 * @see sample.decoder.DecodeObject
 *
 * @date	April 28,1998
 * @version	1.1
*/
public class Entry{
	/**
	 * Contains a description of the property that is described by this entry.
	 */
	private String description = null;
	/**
	 * The byte offset of this property from the start of the classfile.
	 */
	private int offset = 0;
	/**
	 * The number of bytes that this property occupies in the Class File.
	 */
	private int length = 0;
	/**
	 * An interpreted meaning of the raw bytes that represent this property.
	 */
	private String value = null;
	/**
	 * Actual raw bytes that comprise the property described by this Entry.
	 */
	private byte[] bytes = null;
/**
 * Sets the description associated with this property.
 * 
 * @param String d - Contains the description.
 */
	public void setDescription(String d){
		description=d;
	}
/**
 * Retrieves the description associated with this entry.
 * 
 * @return String - contains the description.
 */
	public String getDescription(){
		return description;
	}
/**
 * Configures the offset for this Entry Object.
 * 
 * @param int i - the byte offset of this offset.
 */
	public void setOffset(int i){
		offset=i;
	}
/**
 * Returns the byte offset of this property from the start of the Class File.
 * 
 * @return int - byte offset. 
 */
	public int getOffset(){
		return offset;
	}
/**
 * Sets the number of bytes that this property occupies.
 * 
 * @param int i - number of bytes that this property occupies in the classfile.
 */
	public void setLength(int i){
		length=i;
	}
/**
 * Returns the number of bytes that this property occupies in the class file.
 * 
 * @return int - number of bytes.
 */
	public int getLength(){
		return length;
	}
/**
 * Sets a human readable interpretation for the bytes that represent this property.
 * 
 * @param String s - human readable interpretation of the bytes.
 */
	public void setValue(String s){
		value=s;
	}
/**
 * Returns the human readable description of the byte values.
 * 
 * @return String - Description of the value of this Property.
 */
	public String getValue(){
		return value;
	}
/** 
 * Sets the raw bytes that represent this property.
 * 
 * @param byte[] b - raw bytes for this property.
 */
	public void setBytes(byte[] b){
		bytes=b;
	}
/**
 * Retrieves the raw bytes that represent this property.
 * 
 * @return byte[] - raw bytes.
 */
	public byte[] getBytes(){
		return bytes;
	}
}