/*
 *
 * sample.decoder.ExceptionAttribute.java
 *
 * (C) Copyright 1995 - 1999 Microsoft Corporation.  All rights reserved.
 *
*/
package sample.decoder;

import java.util.Vector;
import java.util.Hashtable;
/**
 * The Exception Attribute is a specialized attribute typically associated with 
 * methods.  It indicates which checked exceptions a method may throw.  There may
 * be exactly one Exception attribute in each method structure.
 * 
 * SCOPE : Method
 *
 * @date	April 28,1998
 * @version	1.2
*/
public class ExceptionsAttribute extends AttributeObject implements DecodeDisplayInterface,
			java.io.Serializable{
/**
 * 2 bytes which represent the number of exceptions associtated with this object.
*/
	protected byte[] number_of_exceptions = new byte[2];
/**
 * This is a table of 2 byte entries which are indexs into the constant pool table for 
 * constant_class_strcutres containing the names of the exception classes for this 
 * attribute.
*/
	protected byte[] exception_index_table;
/**
 * These are the decoded ConstantClassObjects which are indicated by the indexs in the
 * exception table.
*/
	protected Vector exceptions=null;
/**
 * After the initial decoding phase is complete, where all of the bytes have
 * been moved to their appropriate places, this method retrieves any 
 * referenced Constant Pool objects and stores the information with the 
 * object containing the refernece.
 * 
 * @param Vector data - These are the ConstantObjects that were created from the 
 *						constant pool structures in the class file.
*/
public void link(Vector data){
	super.link(data);
	exceptions=new Vector();
	byte[] temp = new byte[2];
	for (int i=0;i<Decode.getIntegerValue(number_of_exceptions);i++){
		temp[0]=exception_index_table[i*2];
		temp[1]=exception_index_table[i*2+1];
		exceptions.addElement(data.elementAt(Decode.getIntegerValue(temp)));
	}
}
/** 
 * Integer representation of the number of exceptions contained within this object.
 * this will also be the number of entries into the exception table.
 *
 * @returns int - number of exceptions / exception table entries.
*/
public int getNumberOfExceptions(){
	return Decode.getIntegerValue(number_of_exceptions);
}
/**
 * Retrieves the decoded contant_class_structures referenced by the exception table.
 *
 * @returns Vector - contains ConstantClassObjects.
*/
public Vector getExceptions(){
	return exceptions;
}
/**
 * This method is a member of the DecodeDisplayInterface and is used to 
 * return the object name and the attribute name.  Subclasses of this object
 * should over ride this method to provide more detailed information.
 *
 * @see com.ms.tools.decode.DecodeDisplayInterface
 *
 * @returns String name representing this object and its attribute type.
*/
public String getName(){
	return "Exceptions";
}
/**
 * Required by the DecodeDisplayInterface, returns any Vectors, or subobjects
 * which will need to be displayed as a subobject in a GUI.  The Hashtable is
 * indexed by a String that is to be used as a display name, the contents
 * will be either a Vector of Strings/Objects or a Decode Object.
 *
 * @returns Hashtable of subobjects/structures to be displayed as seperate
 *				nodes in the GUI.
*/
public Hashtable getChildren(){
	Hashtable val=super.getChildren();
	Vector att = getExceptions();
	for (int i=0;i<att.size();i++)
		val.put(((DecodeDisplayInterface)att.elementAt(i)).getName()+i,
				att.elementAt(i));
	return val;
}
/**
 * Default method for determining a display String which adequatly 
 * represents this object and its contents.  Subclasses of this object 
 * should override this method to provide more detailed information.
 *
 * @returns String representing this object.
*/
public String toString(){
	return exceptions.toString();
}
/**
 * This is the workhorse for the object, it decodes an instance of itself starting at the
 * specified byte index into the stream provided.  This was created as a static method
 * rather than a constructor, as there may be times when this method returns a more
 * specialized object which is a subclass of itself.
 *
 * @param byte[] stream -  entire bytes of the class that is being decoded
 * @param int index - this is the index into the stream for the starting byte.
 * @param Vector constantPool - these are the constantPool objects, any index into the
 *								constant pool will be resolved by retrieving the object 
 *								at the required instance from this Vector.
 *
 * @returns Object - This is the decoded object which is a populated instance of this 
 *					class.
*/
public static Object getInstance(byte[] stream, int index,Vector constantPool){
        ExceptionsAttribute xconst = new ExceptionsAttribute();
      	xconst.number_of_exceptions[0]=stream[index++];
		xconst.number_of_exceptions[1]=stream[index++];
		xconst.addEntry("Number of Exceptions",index-2,2,""+Decode.getIntegerValue(xconst.number_of_exceptions),
						xconst.number_of_exceptions);
        xconst.exception_index_table=new byte[2*Decode.getIntegerValue(xconst.number_of_exceptions)];
        for (int i=0;i<xconst.exception_index_table.length;i++)
                xconst.exception_index_table[i]=stream[index++];
		xconst.exceptions=new Vector();
	byte[] temp = new byte[2];
	for (int i=0;i<Decode.getIntegerValue(xconst.number_of_exceptions);i++){
		temp[0]=xconst.exception_index_table[i*2];
		temp[1]=xconst.exception_index_table[i*2+1];
		xconst.exceptions.addElement(constantPool.elementAt(Decode.getIntegerValue(temp)));
	}
        return xconst;
}
}