/*
 *
 * sample.decoder.FieldObject.java
 *
 * (C) Copyright 1995 - 1999 Microsoft Corporation.  All rights reserved.
 *
*/
package sample.decoder;

import java.util.Vector;
import java.util.Hashtable;
/**
 * As there are no provisions in the virtual machine specification for adding new 
 * fields, the only possible field strucutre is that decoded by this object.
 *
 * @date	April 28,1998
 * @version	1.1
*/
public class FieldObject extends DecodeObject implements DecodeDisplayInterface, 
			java.io.Serializable{
/**
 * Denotes the flag name and the associated integer value..
*/
	public static final int ACC_PUBLIC = 1;
/**
 * Denotes the flag name and the associated integer value..
*/
	public static final int ACC_PRIVATE = 2;
/**
 * Denotes the flag name and the associated integer value..
*/
    public static final int ACC_PROTECTED = 4;
/**
 * Denotes the flag name and the associated integer value..
*/
    public static final int ACC_STATIC = 8;
/**
 * Denotes the flag name and the associated integer value..
*/
    public static final int ACC_FINAL = 16;
/**
 * Denotes the flag name and the associated integer value..
*/
    public static final int ACC_VOLATILE = 64;
/**
 * Denotes the flag name and the associated integer value..
*/
    public static final int ACC_TRANSIENT = 128;
/**
 * This is a temporary Vector of Constant pool objects used for retrieving
 * any Constant Pool objects that may be indexed by an Attribute object or
 * a subobject of an Attribute.
 *
 * @see sample.decoder.ConstantObject
*/
    private transient Vector constantPool = null;
/** 
 * Contains the 2 bytes representing the access flags for the class decoded.
*/
    protected byte[] access_flags=new byte[2];
/** 
 * Bytes that represent the name_index into the ConstantPool.  The
 * index will point to a Constant_Utf8 Object containing a String that 
 * specifies the field name.
*/
    protected byte[] name_index = new byte[2];
/**
 * This is the decoded constant_utf8 structure from the constant pool which was indexed
 * by the name_index bytes.
*/
    private ConstantUtfObject name = null;
/**
 * This is the decoded constant_utf8 structure from the constant pool which was indexed
 * by the name_index bytes.
*/
	private ConstantUtfObject descriptor = null;
/**
 * These are 2 bytes which represent an index into the constantpool for a constant+utf8
 * structure which is a field descriptor.
*/
	protected byte[] descriptor_index = new byte[2];
/**
 * 2 bytes representing the number of attributes associated with this field.  This does 
 * not indicate the number of bytes long the attributes are.
*/
    protected byte[] attributes_count=new byte[2];
/**
 * This is a Vector containing the decoded AttributeObjects (or a subclass of 
 * Attribute Object).
 *
 * @see sample.decoder.AttributeObject
*/
    protected Vector attributes =  null;
/**
 * Constructor for initializing the instance that will be used for decoding the field
 * structures.  The constant pool is passed as a parameter so that all ConstantObjects 
 * required can be retrieved.
 *
 * @param Vector - This is a vector containing all of the decoded ConstantPool entries.
*/
public FieldObject(Vector pool){
	constantPool=pool;
}
/**
 * As individual instances for of Field object are decoded from the class byte array, 
 * container FieldObjects need to be instantiated.  This constructor allows instances to
 * be created that will only hold data, and not decode from a byte array.
*/
public FieldObject(){
}
/**
 * After the initial decoding phase is complete, where all of the bytes have
 * been moved to their appropriate places, this method retrieves any 
 * referenced Constant Pool objects and stores the information with the 
 * object containing the refernece.
 *
 * @param Vector data - These are the ConstantObjects that were created from the 
 *						constant pool structures in the class file.
*/
protected void link(Vector consta){
	name=(ConstantUtfObject)consta.elementAt(Decode.getIntegerValue(name_index));
	descriptor=(ConstantUtfObject)consta.elementAt(Decode.getIntegerValue(descriptor_index));
}
/**
 * This is the number of attributes that this field has.
 *
 * @returns int - the number of attributes for this field.
*/
public int getAttributesCount(){
		return Decode.getIntegerValue(attributes_count);
}
/**
 *  This is the String value from the constant_utf8 structure that represents the field
 * name.
 *
 * @returns String - string value
*/
public String getFieldName(){
		return name.toString();
}
/**
 * This is the String value from the constant_utf8 structure that represents the constant
 * field descriptor
 *
 * @returns String - field descriptor
*/
public String getDescriptor(){	
		return descriptor.toString();
}
/**
 * These are the attributes within the field that is succently being decoded.
 * 
 * @returns Vector - These will be AttributeObjects or subclasses of AttributeObject
*/
public Vector getAttributes(){
		return attributes;
}
/**
 * Default method for determining a display String which adequatly 
 * represents this object and its contents.  Subclasses of this object 
 * should override this method to provide more detailed information.
 *
 * @returns String representing this object.
*/
public String toString(){
	String a="\n       com.ms.tools.decode.FieldObject\n\n";
	String b=" Access Flags integer value : "+getAccessFlag()+"\n";
	String c=" Access Flags : "+getAccessFlags()+"\n";
	String d=" Field Name : "+getFieldName()+"\n";
	String e=" Descriptor : "+getDescriptor()+"\n";
	String f=" Attributes Count : "+getAttributesCount()+"\n";
	String g=" Attributes : "+getAttributes()+"\n";
	return a+b+c+d+e+f+g;
}
/**
 * This method is a member of the DecodeDisplayInterface and is used to 
 * return the object name and the attribute name.  Subclasses of this object
 * should over ride this method to provide more detailed information.
 *
 * @see sample.decoder.DecodeDisplayInterface
 *
 * @returns String name representing this object and its attribute type.
*/
public String getName(){
	return getFieldName();
}
/**
 * This method creates a 'source like' String that describes this Field.
 * The String will contain modifiers, type, inital value (if applicable).
 * 
 * @return String - String describing this class using source syntax.
 */
public String getDescription(){
	String result = "";
	Vector flags = getAccessFlags();
	for (int i=0;i<flags.size();i++)
		result+=(String)flags.elementAt(i)+" ";
	result+=Decode.getDescriptor(
			getDescriptor())+" ";
	result+=getFieldName()+" ";
	Vector attr = getAttributes();
	for (int i=0;i<attr.size();i++)
		if (attr.elementAt(i) instanceof ConstantValueAttribute){
			result+=" = "+((DecodeDisplayInterface)attr.elementAt(i)).getName();	
		}
	return result;									
}
/**
 * Determines which ranges of Strings in the description are actually the names of
 * other java Objects.  These character ranges can be used to mark the original 
 * Description String as hyperlinks or change the color when there are java objects
 * present.
 * 
 * @return Vector - contains java.awt.Point objects which hold the start and end
 *					position of all the names of java objects in the Description 
 *					String.
 */
	public Vector getObjectLinks(){
		Vector points = new Vector();
		String data = getDescription();
		String desc = getDescriptor();
		desc.trim();
		if (!isPrimative(desc)){
			desc = Decode.getDescriptor(desc);
			points.addElement(new java.awt.Point(data.indexOf(desc),
										data.indexOf(desc)+desc.length()));
		}
		for (int i=0;i<points.size();i++){
			java.awt.Point p = (java.awt.Point)points.elementAt(i);
		}
		return points;
}
/**
 * Required by the DecodeDisplayInterface, returns any Vectors, or subobjects
 * which will need to be displayed as a subobject in a GUI.  The Hashtable is
 * indexed by a String that is to be used as a display name, the contents
 * will be either a Vector of Strings/Objects or a Decode Object.
 *
 * @returns Hashtable of subobjects/structures to be displayed as seperate
 *				nodes in the GUI.
*/
public Hashtable getChildren(){
	Hashtable val = new Hashtable();
	Vector att = getAttributes();
	for (int i=0;i<att.size();i++)
		val.put(((DecodeDisplayInterface)att.elementAt(i)).getName(),
				att.elementAt(i));
	return val;
}
/**
 * These are the attributes within the field that is succently being decoded.
 * 
 * @returns Vector - These will be AttributeObjects or subclasses of AttributeObject
*/
public int getAccessFlag(){
	return Decode.getIntegerValue(access_flags);
}
/**
 * Constructs a Vector of Strings which represent the access flags for this class. Access
 * Strings could be:
 *
 *	ACC_PUBLIC			ACC_PRIVATE			ACC_PROTECTED		ACC_STATIC
 *	ACC_FINAL			ACC_VOLATILE		ACC_TRANSIENT
 *
 * @returns Vector of the above mentioned Strings which apply to this class.
*/
public Vector getAccessFlags(){
	Vector result = new Vector();
	if ((getAccessFlag()&Decode.pow(2,0))==Decode.pow(2,0))
		result.addElement("public");
	if ((getAccessFlag()&Decode.pow(2,1))==Decode.pow(2,1))
		result.addElement("private");
	if ((getAccessFlag()&Decode.pow(2,2))==Decode.pow(2,2))
		result.addElement("protected");
	if ((getAccessFlag()&Decode.pow(2,3))==Decode.pow(2,3))
		result.addElement("static");
	if ((getAccessFlag()&Decode.pow(2,4))==Decode.pow(2,4))
		result.addElement("final");
	if ((getAccessFlag()&Decode.pow(2,6))==Decode.pow(2,6))
		result.addElement("volatile");
	if ((getAccessFlag()&Decode.pow(2,7))==Decode.pow(2,7))
		result.addElement("transient");
	return result;	               
}

/**
 * Creates a Vector of the speficied number of fields from the stream it
 * is passed starting at the indicated index into the stream.
 *
 * @param byte[] classBytes - the bytes representing the class to decode.
 * @param int indexs -  this is the number of fields expected.
 * @param int startByte - the index into the classBytes to start decoding.
 *
 * @returns Vector containing FieldObjects or its subclasses.
*/
public Vector createObjects(byte[] classBytes,int indexs,int startByte)throws Exception{
	log("FieldObject: looking for "+indexs);
	Vector list = new Vector();
    for (int i=0;i<indexs;i++){
		Object newton=null;
        if (list.size()!=0)
			newton=nextObject(classBytes, 
				((DecodeObject)list.elementAt(i-1)).getByteStopIndex()+1);
		else
			newton=nextObject(classBytes, startByte); 
        ((DecodeObject)newton).index=i+1;
        list.addElement(newton);
	}
	return list;
}
/**
 * Determines the next FieldObject or its subclass in the given stream
 * by starting to look for a field structure at the given index.  
 *
 * @param byte[] classBytes - the class as bytes to be decoded.
 * @param int index - the index into the byte stream to start looking.
 *
 * @returns Object - the FieldObject for the next constant in the stream.
*/
protected Object nextObject(byte[] classBytes, int index)throws Exception{
	log("FieldObject: getting next");
	FieldObject newFound = new FieldObject();
    newFound.setByteStartIndex(index);
    newFound.access_flags[0]=classBytes[index++];
    newFound.access_flags[1]=classBytes[index++];
	newFound.addEntry("Access Flags",index-2,2,newFound.getAccessFlagsString(),
		newFound.access_flags);
    newFound.name_index[0]=classBytes[index++];
    newFound.name_index[1]=classBytes[index++];
	newFound.addEntry("Name Index",index-2,2,
		constantPool.elementAt(Decode.getIntegerValue(newFound.name_index)).toString(),
		newFound.name_index);
    newFound.descriptor_index[0]=classBytes[index++];
    newFound.descriptor_index[1]=classBytes[index++];
    newFound.addEntry("Descriptor Index",index-2,2,
		constantPool.elementAt(Decode.getIntegerValue(newFound.descriptor_index)).toString(),
		newFound.descriptor_index);
	newFound.attributes_count[0]=classBytes[index++];
    newFound.attributes_count[1]=classBytes[index++];
    newFound.addEntry("Attribute Count",index-2,2,
		""+Decode.getIntegerValue(newFound.attributes_count),newFound.attributes_count);
	AttributeObject aa = new AttributeObject(constantPool);
    newFound.attributes=aa.createObjects(classBytes, 
			Decode.getIntegerValue(newFound.attributes_count),index);
	if (Decode.getIntegerValue(newFound.attributes_count)!=0){
		newFound.setByteStopIndex(((DecodeObject)newFound.attributes.lastElement()).
				getByteStopIndex());
		log("FieldObject: had Attribute");
	}else newFound.setByteStopIndex(index-1);
    log("FieldObject: done");
	newFound.link(constantPool);
	return newFound;
}
/**
 * Generic method for writting debug information to the screen.  By having all
 * methods in this class send their log information here, there is a central
 * point of change for turning debuggin off, or sending it to a differnet 
 * location.
 *
 * @param String data - message to be displayed.
*/
public synchronized void log(String i){
//		System.out.println(i);
}
}