/*
 *
 * sample.InnerClassesAttribute.java
 *
 * (C) Copyright 1995 - 1999 Microsoft Corporation.  All rights reserved.
 *
*/
package sample.decoder;

import java.util.Vector;
import java.util.Hashtable;
/**
 *
 *
 *   InnerClasses_attribute {
 *     u2 attribute_name_index;
 *     u4 attribute_length;
 *     u2 number_of_classes;
 *     {
 *       u2 inner_class_info_index;   // CONSTANT_Class_info index
 *       u2 outer_class_info_index;   // CONSTANT_Class_info index
 *       u2 inner_name_index;         // CONSTANT_Utf8_info index
 *       u2 inner_class_access_flags; // access_flags bitmask
 *     } classes[number_of_classes]
 *   }
 * 
 *
 * SCOPE : Field, Method 
 * 
 * @see sample.decoder.InnerClassesStructure
 * 
 * @date	April 28,1998
 * @version	1.2
*/
public class InnerClassesAttribute extends AttributeObject implements DecodeDisplayInterface,
		java.io.Serializable{
	/**
	 * holds the bytes which represent the number of inner class structures
	 */
	protected byte[] number_of_classes = new byte[2];
	/**
	 * Vector containing the decoded inner class structures
	 */
	protected Vector innerClassesStructure = new Vector();
/**
 * This method is a member of the DecodeDisplayInterface and is used to 
 * return the object name and the attribute name.  Subclasses of this object
 * should over ride this method to provide more detailed information.
 *
 * @see sample.decoder.DecodeDisplayInterface
 *
 * @returns String name representing this object and its attribute type.
*/
public String getName(){
	return "InnerClasses";
}
/**
 * Required by the DecodeDisplayInterface, returns any Vectors, or subobjects
 * which will need to be displayed as a subobject in a GUI.  The Hashtable is
 * indexed by a String that is to be used as a display name, the contents
 * will be either a Vector of Strings/Objects or a Decode Object.
 *
 * @returns Hashtable of subobjects/structures to be displayed as seperate
 *				nodes in the GUI.
*/
public Hashtable getChildren(){
	Hashtable val = super.getChildren();
	for (int i=0;i<innerClassesStructure.size();i++)
		val.put(((DecodeDisplayInterface)innerClassesStructure.elementAt(i)).getName()+i,
				innerClassesStructure.elementAt(i));
	return val;
}
/**
 * After the initial decoding phase is complete, where all of the bytes have
 * been moved to their appropriate places, this method retrieves any 
 * referenced Constant Pool objects and stores the information with the 
 * object containing the refernece.
 * 
 * @param Vector data - These are the ConstantObjects that were created from the 
 *						constant pool structures in the class file.
*/
public void link(Vector data){
	super.link(data);
}
/**
 * This is the workhorse for the object, it decodes an instance of itself starting at the
 * specified byte index into the stream provided.  This was created as a static method
 * rather than a constructor, as there may be times when this method returns a more
 * specialized object which is a subclass of itself.
 *
 * @param byte[] stream -  entire bytes of the class that is being decoded
 * @param int index - this is the index into the stream for the starting byte.
 * @param Vector constantPool - these are the constantPool objects, any index into the
 *								constant pool will be resolved by retrieving the object 
 *								at the required instance from this Vector.
 *
 * @returns Object - This is the decoded object which is a populated instance of this 
 *					class.
*/
public static Object getInstance(byte[] stream, int index,Vector constantPool){
        InnerClassesAttribute xconst = new InnerClassesAttribute();
		xconst.number_of_classes[0]=stream[index++];
		xconst.number_of_classes[1]=stream[index++];
		xconst.addEntry("Number of Classes",
						index-2,2,
						""+Decode.getIntegerValue(xconst.number_of_classes),
						xconst.number_of_classes);
		for (int i=0;i<Decode.getIntegerValue(xconst.number_of_classes);i++){
			xconst.innerClassesStructure.addElement(
				InnerClassesStructure.getInstance(stream,index,constantPool)
			);
			index=((DecodeObject)xconst.innerClassesStructure.lastElement()).getByteStopIndex()+1;
		}
		return xconst;
}
/**
 * Generic method for writting debug information to the screen.  By having all
 * methods in this class send their log information here, there is a central
 * point of change for turning debuggin off, or sending it to a differnet 
 * location.
 *
 * @param String data - message to be displayed.
*/
public synchronized void log(String data){
//	System.out.println(data);
}
}