/*
 *
 * sample.decoder.InnerClassesStructure.java
 *
 * (C) Copyright 1995 - 1999 Microsoft Corporation.  All rights reserved.
 *
*/
package sample.decoder;

import java.util.Vector;
import java.util.Hashtable;
/**
 *       u2 inner_class_info_index;   // CONSTANT_Class_info index
 *       u2 outer_class_info_index;   // CONSTANT_Class_info index
 *       u2 inner_name_index;         // CONSTANT_Utf8_info index
 *       u2 inner_class_access_flags; // access_flags bitmask
 *
 * @see sample.decoder.InnerClassesAttribute 
 * 
 * @date	April 28,1998
 * @version	1.2
*/
public class InnerClassesStructure extends DecodeObject implements DecodeDisplayInterface,
		java.io.Serializable{
	/**
	* These are the bytes which represent the inner class access flags for the 
	* Inner Class decoded in this structure.
	*/
	protected byte[] inner_class_access_flags = new byte[2];
	/**
	 * Index into the constant pool for a ConstantClass Object that describes the inner
	 * class.
	 */
	protected byte[] inner_class_info_index = new byte[2];
	/**
	 * Index into the constant pool for a ConstantClass Object that describes the
	 * outer class.
	 */
	protected byte[] outer_class_info_index = new byte[2];
	/**
	 * Index into the constant pool for a Constant Utf8 object that contains the 
	 * inner class name.
	 */
	protected byte[] inner_name_index = new byte[2];
/**
 * This method is a member of the DecodeDisplayInterface and is used to 
 * return the object name and the attribute name.  Subclasses of this object
 * should over ride this method to provide more detailed information.
 *
 * @see sample.decoder.DecodeDisplayInterface
 *
 * @returns String name representing this object and its attribute type.
*/
public String getName(){
	return "InnerClasses";
}
/**
 * Required by the DecodeDisplayInterface, returns any Vectors, or subobjects
 * which will need to be displayed as a subobject in a GUI.  The Hashtable is
 * indexed by a String that is to be used as a display name, the contents
 * will be either a Vector of Strings/Objects or a Decode Object.
 * 
 * @returns Hashtable of subobjects/structures to be displayed as seperate
 *				nodes in the GUI.
*/
public Hashtable getChildren(){
	return new Hashtable();
}
/**
 * After the initial decoding phase is complete, where all of the bytes have
 * been moved to their appropriate places, this method retrieves any 
 * referenced Constant Pool objects and stores the information with the 
 * object containing the refernece.
 * 
 * @param Vector data - These are the ConstantObjects that were created from the 
 *						constant pool structures in the class file.
*/
public void link(Vector data){
}
/**
 * This is the workhorse for the object, it decodes an instance of itself starting at the
 * specified byte index into the stream provided.  This was created as a static method
 * rather than a constructor, as there may be times when this method returns a more
 * specialized object which is a subclass of itself.
 *
 * @param byte[] stream -  entire bytes of the class that is being decoded
 * @param int index - this is the index into the stream for the starting byte.
 * @param Vector constantPool - these are the constantPool objects, any index into the
 *								constant pool will be resolved by retrieving the object 
 *								at the required instance from this Vector.
 *
 * @returns Object - This is the decoded object which is a populated instance of this 
 *					class.
*/
public static Object getInstance(byte[] stream, int index,Vector constantPool){
        InnerClassesStructure xconst = new InnerClassesStructure();
		xconst.setByteStartIndex(index);
		xconst.inner_class_info_index[0]=stream[index++];
		xconst.inner_class_info_index[1]=stream[index++];
		xconst.addEntry("Inner Class Info Index",
						index-2,2,
						Decode.getIntegerValue(xconst.inner_class_info_index)+
						" - "+((ConstantClassObject)constantPool.elementAt(Decode.getIntegerValue(
						xconst.inner_class_info_index))).getClassName(),
						xconst.inner_class_info_index);
		xconst.outer_class_info_index[0]=stream[index++];
		xconst.outer_class_info_index[1]=stream[index++];
		int outerIndex = Decode.getIntegerValue(xconst.outer_class_info_index);
		if (outerIndex!=0)
			xconst.addEntry("Outer Class Info Index",
						index-2,2,
						Decode.getIntegerValue(xconst.outer_class_info_index)+
						" - "+((ConstantClassObject)constantPool.elementAt(outerIndex)).getClassName(),
						xconst.outer_class_info_index);
		else 
			xconst.addEntry("Outer Class Info Index",
						index-2,2,
						Decode.getIntegerValue(xconst.outer_class_info_index)+
						"",
						xconst.outer_class_info_index);
		xconst.inner_name_index[0]=stream[index++];
		xconst.inner_name_index[1]=stream[index++];
		int innerName = Decode.getIntegerValue(xconst.inner_name_index);
		if (innerName!=0)
			xconst.addEntry("Inner Name Index",
						index-2,2,
						Decode.getIntegerValue(xconst.inner_name_index)+
						" - "+((ConstantUtfObject)constantPool.elementAt(innerName)).getString(),
						xconst.inner_name_index);
		else 
			xconst.addEntry("Inner Name Index",
						index-2,2,
						Decode.getIntegerValue(xconst.inner_name_index)+
						"",
						xconst.inner_name_index);			
		xconst.inner_class_access_flags[0]=stream[index++];
		xconst.inner_class_access_flags[1]=stream[index++];
		xconst.addEntry("Inner Class Access Flags",
						index-2,2,
						""+Decode.getIntegerValue(xconst.inner_class_access_flags),
						xconst.inner_class_access_flags);
		xconst.setByteStopIndex(index-1);
        return xconst;
}
/**
 * Generic method for writting debug information to the screen.  By having all
 * methods in this class send their log information here, there is a central
 * point of change for turning debuggin off, or sending it to a differnet 
 * location.
 *
 * @param String data - message to be displayed.
*/
public synchronized void log(String data){
	//System.out.println(data);
}
}