/*
 *
 * sample.decoder.LineNumberTableStructure.java
 *
 * (C) Copyright 1995 - 1999 Microsoft Corporation.  All rights reserved.
 *
*/
package sample.decoder;

import java.util.Vector;
import java.util.Hashtable;
/**
 * Each Line Number Table structure indicates that the line number in the original
 * java source file changes at a given point in the code array (Method Code Attribte).
 *
 * @date	April 28,1998
 * @version	1.1
*/
public class LineNumberTableStructure extends DecodeObject implements DecodeDisplayInterface,
		java.io.Serializable{
	/**
	 * This is a valid index into the Methods CodeArray at which the code for this line
	 * in the java source starts.
	 */
	protected byte[] start_pc = new byte[2];
	/**
	 * This is the corresponding line number in the original Java source.
	 */
	protected byte[] line_number = new byte[2];
/**
 * This method is a member of the DecodeDisplayInterface and is used to 
 * return the object name and the attribute name.  Subclasses of this object
 * should over ride this method to provide more detailed information.
 *
 * @see sample.decoder.DecodeDisplayInterface
 *
 * @returns String name representing this object and its attribute type.
*/
public String getName(){
	return "Line Number Table";
}
/**
 * Required by the DecodeDisplayInterface, returns any Vectors, or subobjects
 * which will need to be displayed as a subobject in a GUI.  The Hashtable is
 * indexed by a String that is to be used as a display name, the contents
 * will be either a Vector of Strings/Objects or a Decode Object.
 *
 * @returns Hashtable of subobjects/structures to be displayed as seperate
 *				nodes in the GUI.
*/
public Hashtable getChildren(){
	return new Hashtable();
}
/**
 * This is the workhorse for the object, it decodes an instance of itself starting at the
 * specified byte index into the stream provided.  This was created as a static method
 * rather than a constructor, as there may be times when this method returns a more
 * specialized object which is a subclass of itself.
 *
 * @param byte[] stream -  entire bytes of the class that is being decoded
 * @param int index - this is the index into the stream for the starting byte.
 * @param Vector constantPool - these are the constantPool objects, any index into the
 *								constant pool will be resolved by retrieving the object 
 *								at the required instance from this Vector.
 *
 * @returns Object - This is the decoded object which is a populated instance of this 
 *					class.
*/
public static Object getInstance(byte[] stream, int index,Vector constantPool){
        LineNumberTableStructure xconst = new LineNumberTableStructure();
		xconst.setByteStartIndex(index);
		xconst.start_pc[0]=stream[index++];
		xconst.start_pc[1]=stream[index++];
		xconst.addEntry("Start PC",index-2,2,""+Decode.getIntegerValue(xconst.start_pc),
						xconst.start_pc);
		xconst.line_number[0]=stream[index++];
		xconst.line_number[1]=stream[index++];
		xconst.addEntry("Line Number",index-2,2,""+Decode.getIntegerValue(xconst.line_number),
						xconst.line_number);
		xconst.setByteStopIndex(index-1);
        return xconst;
}
/**
 * Generic method for writting debug information to the screen.  By having all
 * methods in this class send their log information here, there is a central
 * point of change for turning debuggin off, or sending it to a differnet 
 * location.
 *
 * @param String data - message to be displayed.
*/
public synchronized void log(String data){
//	System.out.println(data);
}
}