/*
 *
 * sample.decoder.LocalVariableTableAttribute.java
 *
 * (C) Copyright 1995 - 1999 Microsoft Corporation.  All rights reserved.
 *
*/
package sample.decoder;

import java.util.Vector;
import java.util.Hashtable;
/**
 * The Local Variable Table Attribute is a specialized attribute typically associated 
 * with methods.  
 *
 * @see sample.decoder.CodeAttribte
 * @see sample.decoder.LocalVariableTableStructure
 * 
 * @date	April 28,1998
 * @version	1.2
*/
public class LocalVariableTableAttribute extends AttributeObject implements 
		java.io.Serializable{
/**
 * 2 bytes representing the length in entries of the local variable table.  This does not 
 * indicate the number of bytes long the table is.
*/
	protected byte[] local_variable_table_length = new byte[2];
/**
 * This is a Vector which contains all of the structures in the local_variable_table.
*/
	protected Vector localVariables = new Vector();
/**
 * This is the length of the local variable table represented as teh number of table 
 * entries.  The actual table length will be 10 multiplied by this value.
 *
 * @returns int - local variable table entries.
*/
public int getLocalVariableTableLength(){
	return Decode.getIntegerValue(local_variable_table_length);
}
/**
 * Currently this returns the bytes within the table, as there is no methods or objects
 * for decoding and representing the table information.
 *
 * @returns byte[] - bytes which represent the contents of this table
*/
public Vector getLocalVariableTable(){
	return localVariables;
}
/**
 * This method is a member of the DecodeDisplayInterface and is used to 
 * return the object name and the attribute name.  Subclasses of this object
 * should over ride this method to provide more detailed information.
 *
 * @see com.ms.tools.decode.DecodeDisplayInterface
 *
 * @returns String name representing this object and its attribute type.
*/
public String getName(){
	return "Local Variable Table";
}
/**
 * Required by the DecodeDisplayInterface, returns any Vectors, or subobjects
 * which will need to be displayed as a subobject in a GUI.  The Hashtable is
 * indexed by a String that is to be used as a display name, the contents
 * will be either a Vector of Strings/Objects or a Decode Object.
 *
 * @returns Hashtable of subobjects/structures to be displayed as seperate
 *				nodes in the GUI.
*/
public Hashtable getChildren(){
	Hashtable val = super.getChildren();
	Vector att = localVariables;
	for (int i=0;i<att.size();i++)
		val.put(((DecodeDisplayInterface)att.elementAt(i)).getName()+""+i,
				att.elementAt(i));
	return val;
}
/**
 * This is the workhorse for the object, it decodes an instance of itself starting at the
 * specified byte index into the stream provided.  This was created as a static method
 * rather than a constructor, as there may be times when this method returns a more
 * specialized object which is a subclass of itself.
 *
 * @param byte[] stream -  entire bytes of the class that is being decoded
 * @param int index - this is the index into the stream for the starting byte.
 * @param Vector constantPool - these are the constantPool objects, any index into the
 *								constant pool will be resolved by retrieving the object 
 *								at the required instance from this Vector.
 *
 * @returns Object - This is the decoded object which is a populated instance of this 
 *					class.
*/
public static Object getInstance(byte[] stream, int index,Vector constantPool){
        LocalVariableTableAttribute xconst = new LocalVariableTableAttribute();
		xconst.local_variable_table_length[0]=stream[index++];
		xconst.local_variable_table_length[1]=stream[index++];
		xconst.addEntry("Local Variable Table Length",index-2,2,
						""+Decode.getIntegerValue(xconst.local_variable_table_length),
						xconst.local_variable_table_length);
         for (int i=0;i<Decode.getIntegerValue(xconst.local_variable_table_length);i++){
			 xconst.localVariables.addElement(LocalVariableTableStructure.getInstance(stream,index,constantPool));
			index=((DecodeObject)xconst.localVariables.lastElement()).getByteStopIndex()+1;
		}
		return xconst;
}
}