/*
 *
 * sample.decoder.CExposedAsStructure.java
 *
 * (C) Copyright 1995 - 1999 Microsoft Corporation.  All rights reserved.
 *
*/
package sample.decoder;

import java.util.Vector;
import java.util.Hashtable;
/**
 * Each structure indicates a range of CodeAttribute, code array offsets within which 
 * a local variable has a value.  It also indicates the index into the local 
 * variables of the current frame at which that local variable can be found
 *
 * @date	April 28,1998
 * @version	1.1
*/
public class LocalVariableTableStructure extends DecodeObject implements DecodeDisplayInterface,
		java.io.Serializable{
	/**
	 * valid index into the Code Attribute , code array of an opcode instruction
	 */
	protected byte[] start_pc = new byte[2];
	/**
	 * start_pc+length must be a valid index into the code array, pointing to an 
	 * opcode instruction - or the first byte beyond the code array.
	 */
	protected byte[] length = new byte[2];
	/**
	 * valid index intot he constant pool to a Constant Utf8 object which represents
	 * this variables name.
	 */
	protected byte[] name_index = new byte[2];
	/**
	 * Valid index into the Constant Pool to a ConstantUtf8 object that represents the
	 * variables descriptor (same as a Field descriptor).
	 */
	protected byte[] descriptor_index = new byte[2];
	/**
	 * This local variables index in this methods list of local variables.
	 */
	protected byte[] index = new byte[2];
	/**
	 * Reference to the decoded ConstantUtf8 object indicated by the descriptor_index.
	 */
	protected ConstantUtfObject descriptor = null;
	/**
	 * Reference to the decoded ConstantUtf8 object indicated by the name_index.
	 */
	protected ConstantUtfObject name = null;
/**
 * This method is a member of the DecodeDisplayInterface and is used to 
 * return the object name and the attribute name.  Subclasses of this object
 * should over ride this method to provide more detailed information.
 *
 * @see sample.decoder.DecodeDisplayInterface
 *
 * @returns String name representing this object and its attribute type.
*/
public String getName(){
	return "Local Variable Table";
}
/**
 * Required by the DecodeDisplayInterface, returns any Vectors, or subobjects
 * which will need to be displayed as a subobject in a GUI.  The Hashtable is
 * indexed by a String that is to be used as a display name, the contents
 * will be either a Vector of Strings/Objects or a Decode Object.
 *
 * @returns Hashtable of subobjects/structures to be displayed as seperate
 *				nodes in the GUI.
*/
public Hashtable getChildren(){
	return new Hashtable();
}
/**
 * After the initial decoding phase is complete, where all of the bytes have
 * been moved to their appropriate places, this method retrieves any 
 * referenced Constant Pool objects and stores the information with the 
 * object containing the refernece.
 * 
 * @param Vector data - These are the ConstantObjects that were created from the 
 *						constant pool structures in the class file.
*/
public void link(Vector data){
	if (Decode.getIntegerValue(descriptor_index)!=0)
		descriptor=(ConstantUtfObject)data.elementAt(Decode.getIntegerValue(descriptor_index));
	if (Decode.getIntegerValue(name_index)!=0)
		name=(ConstantUtfObject)data.elementAt(Decode.getIntegerValue(name_index));
}
/**
 * This is the workhorse for the object, it decodes an instance of itself starting at the
 * specified byte index into the stream provided.  This was created as a static method
 * rather than a constructor, as there may be times when this method returns a more
 * specialized object which is a subclass of itself.
 *
 * @param byte[] stream -  entire bytes of the class that is being decoded
 * @param int index - this is the index into the stream for the starting byte.
 * @param Vector constantPool - these are the constantPool objects, any index into the
 *								constant pool will be resolved by retrieving the object 
 *								at the required instance from this Vector.
 *
 * @returns Object - This is the decoded object which is a populated instance of this 
 *					class.
*/
public static Object getInstance(byte[] stream, int index,Vector constantPool){
        LocalVariableTableStructure xconst = new LocalVariableTableStructure();
		xconst.setByteStartIndex(index);
		xconst.start_pc[0]=stream[index++];
		xconst.start_pc[1]=stream[index++];
		xconst.addEntry("Start PC",index-2,2,""+Decode.getIntegerValue(xconst.start_pc),
						xconst.start_pc);
		xconst.length[0]=stream[index++];
		xconst.length[1]=stream[index++];
		xconst.addEntry("Length",index-2,2,""+Decode.getIntegerValue(xconst.length),
						xconst.length);
		xconst.name_index[0]=stream[index++];
		xconst.name_index[1]=stream[index++];
		xconst.addEntry("Name Index",index-2,2,constantPool.elementAt(Decode.getIntegerValue(xconst.name_index)).toString(),
						xconst.name_index);
		xconst.descriptor_index[0]=stream[index++];
		xconst.descriptor_index[1]=stream[index++];
		xconst.addEntry("Descriptor Index",index-2,2,constantPool.elementAt(Decode.getIntegerValue(xconst.descriptor_index)).toString(),
						xconst.descriptor_index);
		xconst.index[0]=stream[index++];
		xconst.index[1]=stream[index++];
		xconst.addEntry("Index",index-2,2,""+Decode.getIntegerValue(xconst.index),xconst.index);	
		xconst.setByteStopIndex(index-1);
		xconst.link(constantPool);
        return xconst;
}
/**
 * Generic method for writting debug information to the screen.  By having all
 * methods in this class send their log information here, there is a central
 * point of change for turning debuggin off, or sending it to a differnet 
 * location.
 *
 * @param String data - message to be displayed.
*/
public synchronized void log(String data){
}
}