/*
 *
 * sample.decoder.MethodObject.java
 *
 * (C) Copyright 1995 - 1999 Microsoft Corporation.  All rights reserved.
 *
*/
package sample.decoder;

import java.util.Vector;
import java.util.Hashtable;
/**
 * As there are no provisions in the virtual machine specification for adding new 
 * methods, the only possible method strucutre is that decoded by this object.
 *
 * @date	April 28,1998
 * @version	1.1
*/
public class MethodObject extends DecodeObject implements DecodeDisplayInterface,
		java.io.Serializable{
/**
 * This is a temporary Vector of Constant pool objects used for retrieving
 * any Constant Pool objects that may be indexed by an Attribute object or
 * a subobject of an Attribute.
 *
 * @see sample.decoder.ConstantObject
*/
	private transient  Vector constantPool = null;
/** 
 * Contains the 2 bytes representing the access flags for the class decoded.
*/
	protected byte[] access_flags=new byte[2];
/** 
 * Bytes that represent the name_index into the ConstantPool.  The
 * index will point to a Constant_Utf8 Object containing a String that 
 * specifies the method name.
*/
	protected byte[] name_index = new byte[2];
/**
 * These are 2 bytes which represent an index into the constantpool for a constant+utf8
 * structure which is a method descriptor.
*/
	protected byte[] descriptor_index = new byte[2];
/**
 * 2 bytes representing the number of attributes associated with this method.  This does 
 * not indicate the number of bytes long the attributes are.
*/
	protected byte[] attributes_count=new byte[2];
/**
 * This is a Vector containing the decoded AttributeObjects (or a subclass of 
 * Attribute Object).
 *
 * @see sample.decoder.AttributeObject
*/
	protected Vector attributes =  null;
/**
 * This is the decoded constant_utf8 structure from the constant pool which was indexed
 * by the name_index bytes.
*/
	protected ConstantUtfObject name = null;
/**
 * This is the decoded constant_utf8 structure from the constant pool which was indexed
 * by the name_index bytes.
*/
	protected ConstantUtfObject descriptor = null;
/**
 * Constructor for initializing the instance that will be used for decoding the method
 * structures.  The constant pool is passed as a parameter so that all ConstantObjects 
 * required can be retrieved.
 *
 * @param Vector - This is a vector containing all of the decoded ConstantPool entries.
*/
public MethodObject(Vector pool){
        constantPool=pool;
}
/**
 * As individual instances for of Method object are decoded from the class byte array, 
 * container MethodObjects need to be instantiated.  This constructor allows instances to
 * be created that will only hold data, and not decode from a byte array.
*/
public MethodObject(){}
/**
 * After the initial decoding phase is complete, where all of the bytes have
 * been moved to their appropriate places, this method retrieves any 
 * referenced Constant Pool objects and stores the information with the 
 * object containing the refernece.
 *
 * @param Vector data - These are the ConstantObjects that were created from the 
 *						constant pool structures in the class file.
*/
protected void link(Vector consta){
	name=(ConstantUtfObject)consta.elementAt(Decode.getIntegerValue(name_index));
	descriptor=(ConstantUtfObject)consta.elementAt(
		Decode.getIntegerValue(descriptor_index));
}
/**
 * This is the number of attributes that this method has.
 *
 * @returns int - the number of attributes for this method.
*/
public int getAttributesCount(){
		return Decode.getIntegerValue(attributes_count);
}
/**
 * This is the String value from the constant_utf8 structure that represents the method
 * name.
 *
 * @returns String - string value
*/
public String getMethodName(){
		return name.toString();
}
/**
 * This is the String value from the constant_utf8 structure that represents the constant
 * method descriptor
 *
 * @returns String - method descriptor
*/
public String getDescriptor(){	
		return descriptor.toString();
}
/**
 * These are the attributes within the method that is succently being decoded.
 * 
 * @returns Vector - These will be AttributeObjects or subclasses of AttributeObject
*/
public Vector getAttributes(){
		return attributes;
}
/**
 * Default method for determining a display String which adequatly 
 * represents this object and its contents.  Subclasses of this object 
 * should override this method to provide more detailed information.
 *
 * @returns String representing this object.
*/
public String toString(){
	String a="\n       com.ms.tools.decode.MethodObject\n\n";
	String b=" Access Flags integer value : "+getAccessFlag()+"\n";
	String c=" Access Flags : "+getAccessFlags()+"\n";
	String d=" Method Name : "+getMethodName()+"\n";
	String e=" Descriptor : "+getDescriptor()+"\n";
	String f=" Attributes Count : "+getAttributesCount()+"\n";
	String g=" Attributes : "+getAttributes()+"\n";
	return a+b+c+d+e+f+g;
}
/**
 * This method is a member of the DecodeDisplayInterface and is used to 
 * return the object name and the attribute name.  Subclasses of this object
 * should over ride this method to provide more detailed information.
 *
 * @see sample.decoder.DecodeDisplayInterface
 *
 * @returns String name representing this object and its attribute type.
*/
public String getName(){
	return getMethodName();
}
/**
 * Required by the DecodeDisplayInterface, returns any Vectors, or subobjects
 * which will need to be displayed as a subobject in a GUI.  The Hashtable is
 * indexed by a String that is to be used as a display name, the contents
 * will be either a Vector of Strings/Objects or a Decode Object.
 *
 * @returns Hashtable of subobjects/structures to be displayed as seperate
 *				nodes in the GUI.
*/
public Hashtable getChildren(){
	Hashtable val = new Hashtable();
	Vector att = getAttributes();
	for (int i=0;i<att.size();i++)
		val.put(((DecodeDisplayInterface)att.elementAt(i)).getName(),
				att.elementAt(i));
	return val;
}
/**
 * This method creates a 'source like' String that describes this Method.
 * The String will contain modifiers, return type, method name, and parameters.
 * 
 * @return String - String describing this class using source syntax.
 */
public String getDescription(){
		String result = "";
		Vector flags = getAccessFlags();
		String desc = getDescriptor();
		String retType= desc.substring(desc.indexOf(")")+1,
									   desc.length());
		for (int i=0;i<flags.size();i++)
			result+=(String)flags.elementAt(i)+" ";
		result+=Decode.getDescriptor(retType)+" ";
		result+=getMethodName()+" ";
		result+="(";
		Vector params = parseParameters();
		for (int i=0;i<params.size();i++){
			result+=", ";
			if (i==0)
				result=result.substring(0,result.length()-2);
			result+=Decode.getDescriptor((String)params.elementAt(i));
		}
			  result+=")";
	return result;
}
/**
 * Tokenizes the parameter string.
 * 
 * @param String data - String representing the parameters that this method takes.
 * @return Vector - each element is a parameter.
 */
	public Vector parseParameters(){
		String params = getDescriptor().substring(
									getDescriptor().indexOf("(")+1,
									getDescriptor().indexOf(")"));
		Vector list = new Vector();
		int start=0;
		int stop = params.length();
		while (start!=stop){
			int index = 0;		
			while (params.charAt(start+index)=='[')
				index++;
			String el = params.substring(start,++index+start);
			if (!isPrimative(el)){
				while (params.charAt(start+index-1)!=';')
					el+=params.charAt(start+index++);
			}
			list.addElement(el);
			start+=index;
		}
		return list;
	}
/**
 * Determines which ranges of Strings in the description are actually the names of
 * other java Objects.  These character ranges can be used to mark the original 
 * Description String as hyperlinks or change the color when there are java objects
 * present.
 * 
 * @return Vector - contains java.awt.Point objects which hold the start and end
 *					position of all the names of java objects in the Description 
 *					String.
 */
	public Vector getObjectLinks(){
		Vector points = new Vector();
		String data = getDescription();
		String desc = getDescriptor();
		Vector params = parseParameters();
		String retType= desc.substring(desc.indexOf(")")+1,desc.length());
		if (!isPrimative(retType)){
			desc = Decode.getDescriptor(desc);
			points.addElement(new java.awt.Point(data.lastIndexOf(desc),
									data.lastIndexOf(desc)+desc.length()));
		}
		for (int i=0;i<params.size();i++){
			String disc = Decode.getDescriptor((String)params.elementAt(i));
			int start = data.indexOf(disc); 
			if (!isPrimative((String)params.elementAt(i)))
				points.addElement(new java.awt.Point(start,start+disc.length()));
		}
		return points;
	}
/**
 * These are the attributes within the method that is succently being decoded.
 * 
 * @returns Vector - These will be AttributeObjects or subclasses of AttributeObject
*/
public int getAccessFlag(){
	return Decode.getIntegerValue(access_flags);
}
/**
 * Constructs a Vector of Strings which represent the access flags for this class. Access
 * Strings could be:
 *
 *	ACC_PUBLIC			ACC_PRIVATE			ACC_PROTECTED		ACC_STATIC
 *	ACC_FINAL			ACC_SYNCHRONIZED	ACC_NATIVE			ACC_ABSTRACT
 *
 * @returns Vector of the above mentioned Strings which apply to this class.
*/
public Vector getAccessFlags(){
	Vector result = new Vector();
	if ((getAccessFlag()&Decode.pow(2,0))==Decode.pow(2,0))
		result.addElement("public");
	if ((getAccessFlag()&Decode.pow(2,1))==Decode.pow(2,1))
		result.addElement("private");
	if ((getAccessFlag()&Decode.pow(2,2))==Decode.pow(2,2))
		result.addElement("protected");
	if ((getAccessFlag()&Decode.pow(2,3))==Decode.pow(2,3))
		result.addElement("static");
	if ((getAccessFlag()&Decode.pow(2,4))==Decode.pow(2,4))
		result.addElement("final");
	if ((getAccessFlag()&Decode.pow(2,5))==Decode.pow(2,5))
		result.addElement("synchronized");
	if ((getAccessFlag()&Decode.pow(2,8))==Decode.pow(2,8))
		result.addElement("native");
	if ((getAccessFlag()&Decode.pow(2,9))==Decode.pow(2,9))
		result.addElement("abstract");
	return result;	               
}
/**
 * Creates a Vector of the speficied number of methods from the stream it
 * is passed starting at the indicated index into the stream.
 *
 * @param byte[] classBytes - the bytes representing the class to decode.
 * @param int indexs -  this is the number of methods expected.
 * @param int startByte - the index into the classBytes to start decoding.
 *
 * @returns Vector containing MethodObjects or its subclasses.
*/
public Vector createObjects(byte[] classBytes,int indexs,int startByte)throws Exception{
	log("MethodObject: Decoding "+indexs+" methods");
	Vector list = new Vector();
    int stop =-1;
    int start = -1;
   for (int i=0;i<indexs;i++){           
		MethodObject newton=null;
        if (list.size()!=0){
                newton=(MethodObject)nextObject(classBytes, 
					((MethodObject)list.elementAt(i-1)).
					getByteStopIndex()+1);
        }else
                newton=(MethodObject)nextObject(classBytes, startByte); 
        newton.index=i+1;
		newton.link(constantPool);
        list.addElement(newton);
		log("MethodObject: method "+i+" complete...");
	}
	return list;
}
/**
 * Determines the next methodObject or its subclass in the given stream
 * by starting to look for a method at the given index.  
 *
 * @param byte[] classBytes - the class as bytes to be decoded.
 * @param int index - the index into the byte stream to start looking.
 *
 * @returns Object - the ConstantObject for the next constant in the stream.
*/
private Object nextObject(byte[] classBytes, int index)throws Exception{
	MethodObject newFound = new MethodObject();
    newFound.setByteStartIndex(index);
    newFound.access_flags[0]=classBytes[index++];
    newFound.access_flags[1]=classBytes[index++];
	newFound.addEntry("Access Flags",index-2,2,newFound.getAccessFlagsString(),
		newFound.access_flags);
    newFound.name_index[0]=classBytes[index++];
    newFound.name_index[1]=classBytes[index++];
	newFound.addEntry("Name Index",index-2,2,
		constantPool.elementAt(Decode.getIntegerValue(newFound.name_index)).toString(),
		newFound.name_index);
    newFound.descriptor_index[0]=classBytes[index++];
    newFound.descriptor_index[1]=classBytes[index++];
    newFound.addEntry("Descriptor Index",index-2,2,
		constantPool.elementAt(Decode.getIntegerValue(newFound.descriptor_index)).toString(),
		newFound.descriptor_index);
	newFound.attributes_count[0]=classBytes[index++];
    newFound.attributes_count[1]=classBytes[index++];
    newFound.addEntry("Attribute Count",index-2,2,
		""+Decode.getIntegerValue(newFound.attributes_count),newFound.attributes_count);
	AttributeObject aa = new AttributeObject(constantPool);
  	newFound.attributes=aa.createObjects(classBytes,
                Decode.getIntegerValue(newFound.attributes_count),index);
    if (newFound.attributes.size()!=0){
		newFound.setByteStopIndex((
			(AttributeObject)newFound.attributes.lastElement()).
            getByteStopIndex());
	}
    else newFound.setByteStopIndex(index-1);
    return newFound;
}
/**
 * Generic method for writting debug information to the screen.  By having all
 * methods in this class send their log information here, there is a central
 * point of change for turning debuggin off, or sending it to a differnet 
 * location.
 *
 * @param String data - message to be displayed.
*/
public synchronized void log(String data){
	//System.out.println(data);
}
}