/*
 *
 * sample.decoder.NAT_LAttribute.java
 *
 * (C) Copyright 1995 - 1999 Microsoft Corporation.  All rights reserved.
 *
*/
package sample.decoder;

import java.util.Vector;
import java.util.Hashtable;
/**
 * The Native Link attribute represents the DLL name and DLL entry point name for a 
 * Win32 API or for other DLLs to be called by a Java method.  This attribute also 
 * specifiesthe calling convention (ANSI,Unicode,OLE, or raw antive) and other
 * semantics.  This attribute is generated by the dll.import directive and supports
 * J/Direct.
 * 
 * SCOPE : Class, Method
 * 
 * @date	April 28,1998
 * @version	1.1
*/
public class NAT_LAttribute extends AttributeObject implements DecodeDisplayInterface,
		java.io.Serializable{
	/**
	 * Byte representing the linktype
	 * 
	 *		0 = Raw, uses Microsoft Raw Native Interface
	 *		1 = Inherits linktype from enclosing class
	 *		2 = ANSI conversion applied to strings, function name
	 *		3 = Unicode conversion applied to strings, function names
	 *		4 = Automatically chooses ANSI/Unicode conversion based on OS
	 *		5 = OLE conversion, BSTR string conversion and HRESULT<>retval mapping
	 */
	protected byte linktype = 0;
	/**
	 * Byte representing the flags.
	 * 
	 *	bit 0: Indicates that the method sets the Win32 error code
	 *	bit 1-7: reserved
	 */
	protected byte flags = 0;
   /**
    * Valid index into the Constant Pool for a ConstantUtf8 structure that contains
    * the dll library name.
    */
   protected byte[] libname_index = new byte[2];
   /**
    * Valid index into the Constant Pool for a ConstantUtf8 structure that contains
    * the dll library entry point.  Identifies the entrypoint for the DLL (index into 
    * CP, 0 to indicate that the method name==entrypoint)
    */
	protected byte[] entryname_index = new byte[2];
	/**
	 * Reference to the ConstantUtf8 Structure pointed to by the libname_index value.
	 */
   protected ConstantUtfObject libname = null;
   /**
    * Reference to the ConstantUtf8 Structure indicated by the entryname_index value.
    */
   protected ConstantUtfObject entryname = null;
/**
 * This method is a member of the DecodeDisplayInterface and is used to 
 * return the object name and the attribute name.  Subclasses of this object
 * should over ride this method to provide more detailed information.
 *
 * @see sample.decoder.DecodeDisplayInterface
 *
 * @returns String name representing this object and its attribute type.
*/
public String getName(){
	return "Native Link Attribute";
}
/**
 * After the initial decoding phase is complete, where all of the bytes have
 * been moved to their appropriate places, this method retrieves any 
 * referenced Constant Pool objects and stores the information with the 
 * object containing the refernece.
 *
 * @param Vector data - These are the ConstantObjects that were created from the 
 *						constant pool structures in the class file.
*/
public void link(Vector data){
	super.link(data);
	libname=(ConstantUtfObject)data.elementAt(Decode.getIntegerValue(libname_index));
	if (Decode.getIntegerValue(entryname_index)!=0)
		entryname=(ConstantUtfObject)data.elementAt(
										Decode.getIntegerValue(entryname_index));
}
/**
 * This is the workhorse for the object, it decodes an instance of itself starting at the
 * specified byte index into the stream provided.  This was created as a static method
 * rather than a constructor, as there may be times when this method returns a more
 * specialized object which is a subclass of itself.
 *
 * @param byte[] stream -  entire bytes of the class that is being decoded
 * @param int index - this is the index into the stream for the starting byte.
 * @param Vector constantPool - these are the constantPool objects, any index into the
 *								constant pool will be resolved by retrieving the object 
 *								at the required instance from this Vector.
 *
 * @returns Object - This is the decoded object which is a populated instance of this 
 *					class.
*/
public static Object getInstance(byte[] stream, int index,Vector constantPool){
        NAT_LAttribute xconst = new NAT_LAttribute();
        xconst.linktype=stream[index++];
		xconst.addEntry("Link Type",index-2,1,""+Decode.getIntegerValue(xconst.linktype),
						new byte[]{xconst.linktype});
        xconst.flags=stream[index++];
        xconst.addEntry("Flags",index-2,1,""+Decode.getIntegerValue(xconst.flags),
						new byte[]{xconst.flags});
		xconst.libname_index[0]=stream[index++];
        xconst.libname_index[1]=stream[index++];
        xconst.addEntry("Library Name",index-3,2,constantPool.elementAt(Decode.getIntegerValue(xconst.libname_index)).toString(),
						xconst.libname_index);
		xconst.entryname_index[0]=stream[index++];
        xconst.entryname_index[1]=stream[index++];
        xconst.addEntry("Entry Name",index-3,2,
						((DecodeDisplayInterface)constantPool.elementAt(
						Decode.getIntegerValue(xconst.entryname_index))).getName(),
						xconst.entryname_index);
		return xconst;
}
}