/*
 *
 * sample.decoder.SourceDirAttribute.java
 *
 * (C) Copyright 1995 - 1999 Microsoft Corporation.  All rights reserved.
 *
*/
package sample.decoder;

import java.util.Vector;
import java.util.Hashtable;
/**
 * Contains a String representing the source directory where the java source file was
 * located.
 * 
 * SCOPE : Class
 * 
 * @date	April 28,1998
 * @version	1.2
*/
public class SourceDirAttribute extends AttributeObject implements DecodeDisplayInterface,
		java.io.Serializable{
/**
 * This is the decoded constant_utf8 structure from the constant pool which was indexed
 * by the sourcefile_index bytes.
*/
	protected ConstantUtfObject sourcedir = null;
/**
 * These are 2 bytes which represent an index into the constantpool for a constant+utf8
 * structure which is the source file String.
 */
	protected byte[] sourcedir_index = new byte[2];
/**
 * Retrieves the integer value of the index into the Constant Pool for the Constant
 * Utf8 object that contains the source directory string.
 */
public int getSourceDirIndex(){
	return Decode.getIntegerValue(sourcedir_index);
}
/**
 * This is the String value from the constant_utf8 structure that represents the constant
 * object containing the String with the source file name.
 *
 * @returns String - source file name
*/
public String getSourceDir(){
	return sourcedir.toString();
}
/**
 * This method is a member of the DecodeDisplayInterface and is used to 
 * return the object name and the attribute name.  Subclasses of this object
 * should over ride this method to provide more detailed information.
 *
 * @see sample.decoder.DecodeDisplayInterface
 *
 * @returns String name representing this object and its attribute type.
*/
public String getName(){
	return "Source Dir";
}
/**
 * After the initial decoding phase is complete, where all of the bytes have
 * been moved to their appropriate places, this method retrieves any 
 * referenced Constant Pool objects and stores the information with the 
 * object containing the refernece.
 * 
 * @param Vector data - These are the ConstantObjects that were created from the 
 *						constant pool structures in the class file.
*/
public void link(Vector data){
	super.link(data);
	sourcedir=(ConstantUtfObject)data.elementAt(Decode.getIntegerValue(sourcedir_index));
}
/**
 * This is the workhorse for the object, it decodes an instance of itself starting at the
 * specified byte index into the stream provided.  This was created as a static method
 * rather than a constructor, as there may be times when this method returns a more
 * specialized object which is a subclass of itself.
 *
 * @param byte[] stream -  entire bytes of the class that is being decoded
 * @param int index - this is the index into the stream for the starting byte.
 * @param Vector constantPool - these are the constantPool objects, any index into the
 *								constant pool will be resolved by retrieving the object 
 *								at the required instance from this Vector.
 *
 * @returns Object - This is the decoded object which is a populated instance of this 
 *					class.
*/
public static Object getInstance(byte[] stream, int index,Vector constantPool){
        SourceDirAttribute xconst = new SourceDirAttribute();
        xconst.sourcedir_index[0]=stream[index++];
        xconst.sourcedir_index[1]=stream[index++];
		xconst.addEntry("Source Directory Index",index-2,2,constantPool.elementAt(Decode.getIntegerValue(xconst.sourcedir_index)).toString(),
						xconst.sourcedir_index);
        return xconst;
}
/**
 * Generic method for writting debug information to the screen.  By having all
 * methods in this class send their log information here, there is a central
 * point of change for turning debuggin off, or sending it to a differnet 
 * location.
 *
 * @param String data - message to be displayed.
*/
public synchronized void log(String data){
//	System.out.println(data);
}
}