/*
 *
 * sample.viewer.ClassDecoder.java
 *
 * (C) Copyright 1995 - 1999 Microsoft Corporation.  All rights reserved.
 *
*/
package sample.viewer;

import sample.decoder.*;
import com.ms.wfc.app.Application;
import com.ms.wfc.core.Event;
import com.ms.wfc.core.Container;
import com.ms.wfc.core.EventHandler;
import com.ms.wfc.core.IResourceManager;
import com.ms.wfc.core.ResourceManager;
import com.ms.wfc.ui.Button;
import com.ms.wfc.ui.*;
import java.util.Vector;
import java.util.Enumeration;
/**
 * This class can take a variable number of parameters on the command
 * line. Program execution begins with the main() method. The class
 * constructor is not invoked unless an object of type 'ClassDecoder'
 * created in the main() method.
 * 
 * @see sample.decoder.ClassObject
 * 
 * @version 1.0a
 */
public class ClassDecoder extends Form implements IFindListener{
	/**
	 * This value will be associated with decoded objects that are Inner Interfaces
	 */
	private int INNER_INTERFACE=0;
	/**
	 * Associated value for undefined decoded objects.
	 */
	private int NONE=1;
	/**
	 * Associated value for an Errored decoded object.
	 */
	private int ERROR=2;
	/**
	 * This value is associated with objects that contain circular references
	 */
	private int CIRCULAR_REFERENCE=3;
	/**
	 * Value associated with Class Objects.
	 */
	private int CLASS=4;
	/**
	 * Value associated with Interface objects.
	 */
	private int INTERFACE=5;
	/**
	 * Value associated with Package objects.
	 */
	private int PACKAGE = 6;
	/**
	 * Value associated with Imported Packages
	 */
	private int IMPORT_PACKAGE=7;
	/**
	 * Value associated with Imported Classes
	 */
	private int IMPORT_CLASS=8;
	/**
	 * value associated with a Super Class
	 */
	private int SUPER_CLASS=9;
	/**
	 * Value associated with an Inherited Method
	 */
	private int INHERITED_METHOD=10;
	/**
	 * Value associated with an Inherited Field
	 */
	private int INHERITED_FIELD=11;
	/**
	 * Value associated with a Package protected method
	 */
	private int PACKAGE_PROTECTED_METHOD=12;
	/**
	 * Value associated with a generic Method
	 */
	private int METHOD=13;
	/**
	 * Value associated with a protected method
	 */
	private int PROTECTED_METHOD=14;
	/**
	 * Value associated with a private method.
	 */
	private int PRIVATE_METHOD=15;
	/**
	 * Value associated with a package protected field
	 */
	private int PACKAGE_PROTECTED_FIELD=16;
	/**
	 * Value associated with a generic field.
	 */
	private int FIELD=17;
	/**
	 * Value associated with a protected field.
	 */
	private int PROTECTED_FIELD=18;
	/**
	 * Value associated with a private field.
	 */
	private int PRIVATE_FIELD=19;
	/**
	 * Value associated with an Inner Class
	 */
	private int INNER_CLASS=20;
	/**
	 * Value associated with a group of Delegates.
	 */
	private int DELEGATES=21;
	/**
	 * Value associated with an initializer method
	 */
	private int INITIALIZER=22;
	/**
	 * Value associated with generic information
	 */
	private int INFORMATION=23;
	/**
	 * Value associated with Super Class Information
	 */
	private int SUPER_CLASS_INFORMATION=24;
	/**
	 * Value associated with Method Information
	 */
	private int METHOD_INFORMATION=25;
	/**
	 * Value associated with Interface Information
	 */
	private int INTERFACE_INFORMATION=26;
	/**
	 * Value associated with Class Information
	 */
	private int CLASS_INFORMATION=27;
	/**
	 * Value associated with Field information
	 */
	private int FIELD_INFORMATION=28;
	/**
	 * Value associated with an overloaded method
	 */
	private int METHOD_OVERLOAD=29;
	/**
	 * Value associated with an Attribute
	 */
	private int ATTRIBUTE=30;
	/**
	 * Value associated with a Structure
	 */
	private int STRUCTURE=31;
	/**
	 * Value associated with a Constant from the constant Pool
	 */
	private int CONSTANT=32;
	/**
	 * Value associated with the Decode application
	 */
	private int DECODE=33;
	/**
	 * Value associated with the Find action.
	 */
	private int FIND=34;
	/**
	 * Value associated with the back action.
	 */
	private int BACK=35;
	/**
	 * Value associated with the forward action.
	 */
	private int FORWARD=36;
	/**
	 * Value associated with the delete action.
	 */
	private int DELETE=37;
	/**
	 * Value associated with information related to a Structure
	 */
	private int STRUCTURE_INFORMATION =this.INTERFACE_INFORMATION;
	/**
	 * Value associated with information related to attributes.
	 */
	private int ATTRIBUTE_INFORMATION =this.SUPER_CLASS_INFORMATION;
	/**
	 * Value associated with information related to Constants
	 */
	private int CONSTANT_INFORMATION=this.INFORMATION;
	/**
	 * Value associated with information related to imports.
	 */
	private int IMPORT_INFORMATION=this.CLASS_INFORMATION;
	/**
	 * This value should be checked by any code that needs to be conditionally 
	 * executed only if the application is NOT currently loading or decoding a 
	 * file.
	 */
	private boolean loading =false;
	/**
	 * Table that uses a RichEdit201 Control as a key, to retrieve its associated
	 * com.ms.wfc.ui.Label.  The labels RichEdits and associated labels are added
	 * to this table, so that the title of the RichEdit can be correctly displayed
	 * when flipping between classes.
	 * 
	 * @see sample.viewer.ClassDecoder#initTree
	 * @see sample.viewer.ClassDecoder#refresh
	 * @see sample.viewer.ClassDecoder#checkIfFileLoaded
	 */
	private java.util.Hashtable sourceLabels = new java.util.Hashtable();
	/**
	 * This Vector holds a list of visible TreeNodes that have already been found, to
	 * allow the find operation to display only newly found items.
	 * 
	 * @see sample.viewer.ClassDecoder#processFindText
	 * @see sample.viewer.ClassDecoder#findNext
	 * @see sample.viewer.ClassDecoder#findMenuAction
	 * @see sample.viewer.ClassDecoder#findNode
	 * @see sample.viewer.ClassDecoder#filterButtonEvent
	 */
	private java.util.Vector findList = new java.util.Vector();
	/**
	 * String containing the text that will be used to find the required node.
	 */
	private String findString = "";
	/**
	 * Dialog that will be displayed if the Find Menu Item is invoked.  This dialog
	 * is created at initialization, and remains hidden when not active.
	 */
	private FindDialog findDialog = new FindDialog(this);
	/**
	 * Stack containing the previously decoded nodes displayed, which will result in 
	 * the stack containing the last Class from which a hyperlink was selected, or the
	 * class immediately before the currently decoded Class.  This stack is used to
	 * determine what Class should be displayed when the Back action happens.
	 * 
	 * The stack will contain the root internal TreeNode for the currently decoded
	 * class.  Typically this will be retrieved useing the getMagicNode method.
	 * 
	 * @see sample.viewer.ClassDecoder.getMagicNode
	 * 
	 * @see sample.viewer.ClassDecoder#processLinkEvent
	 * @see sample.viewer.ClassDecoder#decode
	 * @see sample.viewer.ClassDecoder#removePackage
	 * @see sample.viewer.ClassDecoder#processForward
	 * @see sample.viewer.ClassDecoder#processBack
	 */
	private java.util.Stack backStack = new java.util.Stack();
	/**
	 * Stack containing the decoded nodes which have accumulated from the user 
	 * pressing the back button.  This list will be erased when the user selects a 
	 * hyperlink or decodes a new class.
	 * 
	 * The stack will contain the root internal TreeNode for the currently displayed
	 * class.
	 * 
	 * @see sample.viewer.ClassDecoder.getMagicNode
	 * 
	 * @see sample.viewer.ClassDecoder#processLinkEvent
	 * @see sample.viewer.ClassDecoder#decode
	 * @see sample.viewer.ClassDecoder#removePackage
	 * @see sample.viewer.ClassDecoder#processForward
	 * @see sample.viewer.ClassDecoder#processBack
	 */
	private java.util.Stack forwardStack = new java.util.Stack();
	/**
	 * Maps an internal TreeNode object to its corresponding DecodeObject.  This 
	 * table is used to store the original source of data.
	 * 
	 * @see sample.viewer.ClassDecoder#buildLabels
	 * @see sample.viewer.ClassDecoder#setSourceViewAttributes
	 * @see sample.viewer.ClassDecoder#buildTreeChildren
	 * @see sample.viewer.ClassDecoder#refresh
	 * @see sample.viewer.ClassDecoder#removeNode
	 */
	private java.util.Hashtable nodes = new java.util.Hashtable();
	/**
	 * Stores the internal class node that represents a decoded class as the key, 
	 * with its associated RichEdit201 control containing the source view as the data.
	 * 
	 * @see sample.viewer.ClassDecoder#initTree
	 * @see sample.viewer.ClassDecoder#getRoot
	 * @see sample.viewer.ClassDecoder#refresh
	 * @see sample.viewer.ClassDecoder#getVisibleRoot
	 * @see sample.viewer.ClassDecoder#removeNode
	 * @see sample.viewer.ClassDecoder#getMagicNode
	 */
	private java.util.Hashtable rootTable = new java.util.Hashtable();
	/**
	 * A String comprising of the associated line number in the Classes RichEdit201
	 * source view, along with the id of the RichEdit201 Control is used as the key
	 * in this table.  The data is the internal node that represents the information
	 * used to create the source view line of code.  This table will assist in 
	 * mapping a selection in the source view to a tree node, and ultimately the 
	 * DecodeObject that originated all of the information.
	 * 
	 * @see sample.viewer.ClassDecoder#makeLabels
	 * @see sample.viewer.ClassDecoder#setSourceViewAttributes
	 * @see sample.viewer.ClassDecoder#processLinkEvent
	 * @see sample.viewer.ClassDecoder#selectSource
	 * @see sample.viewer.ClassDecoder#cleanItems
	 */
	private java.util.Hashtable treeItem = new java.util.Hashtable();
	/**
	 * Maintains a Map of which line number in the RichEdit201 controls source view, 
	 * an internal node maps to.  The internal node is used as the key with the data
	 * comprised of an Integer containing the line line number. 
	 * 
	 * @see sample.viewer.ClassDecoder#makelabels
	 * @see sample.viewer.ClassDecoder#updateSourceMap
	 * @see sample.viewer.ClassDecoder#cleanItems
	 */
	private java.util.Hashtable itemTree = new java.util.Hashtable();
	/**
	 * The highlight method uses this variable to store the last area of the current
	 * RichEdit201 control that was highlighted.  Berfore highlighting a new area, 
	 * this information is used to deselect the current region.
	 * 
	 * @see sample.viewer.ClassDecoder#highlight
	 */
	private java.awt.Point sourceSelected = null;
	/**
	 * This is required to remove the previously bolded line in a rich edit control.
	 * As the user may select one line from rich edit a, then the next from rich edit b
	 * it is necessary to track the rich edit with the bolded line so that it can be
	 * undone.
	 * 
	 * @see sample.viewer.ClassDecoder#setLoading
	 * @see sample.viewer.ClassDecoder#highlight
	 * @see sample.viewer.ClassDecoder#removeNode
	 * @see sample.viewer.ClassDecoder#getMagicNode
	 */
	RichEdit201 lastRich = null;
	/**
	 * Stores the currently visible nodes as keys and the internal nodes as data.
	 * This will be used to map visible nodes to internal, because a dynamic 
	 * traversal of the internal tree would not be possible due to naming collisions
	 */
	private java.util.Hashtable outerInternal = new java.util.Hashtable();
/**
 * Default constructor which creates the initial form and displays it.
 */	
	public ClassDecoder(){
		super();
		findDialog.setVisible(false);//leaves the find dialog in a hidden state.
		initForm();		//Required by the VJ++ 6.0 Forms designer
		this.enterClassname.select();//ensures that the edit control has the focus.
	}
/**
 * ClassDecoder overrides dispose so it can clean up the
 * component list.
 */
	public void dispose(){
		super.dispose();
		components.dispose();
	}
/**
 * Updates the table with the given set of DecodeDisplayInterface 
 * parameters.  Does nothing if no data is available.
 * 
 * @param DecodeDisplayInterface data - information to be displayed
 *										in the table.
 */
	private void updateTable(DecodeDisplayInterface data){
		if (data!=null){
			Vector entries = ((DecodeObject)data).getEntries();
			if (entries!=null){
				ListItem[] lie = new ListItem[entries.size()];
				for (int i=0;i<entries.size();i++){
					Entry e = (Entry)entries.elementAt(i);
					lie[i]=new ListItem(e.getDescription(),-1,new String[]{
										new Integer(e.getOffset()).toString(),
										new Integer(e.getLength()).toString(),
										e.getValue(),
										Decode.getByteString(e.getBytes())
										});
				}
				gridLabel.setText(((DecodeObject)data).getTitle());
				DataGrid1.setItems(sortListByOffset(lie));
			}
		}else{
			gridLabel.setText("");
			DataGrid1.setItems(new ListItem[] {});
		}
	}
/**
 * This method sorts the columns in the ListView that display the properties of the
 * currently selected Decode object, by the properties offset from the beginning of the
 * decoded class.  
 * 
 * @param ListItem[] list - Items from the list view that need sorting.
 * 
 * @return ListItem[] - a new ListItem[] that is sorted by offset
 */
	ListItem[] sortListByOffset(ListItem[] list){
		ListItem[] newList = new ListItem[list.length];
		Vector items = new Vector();
		for (int i=0;i<list.length;i++){
			String offset = list[i].getSubItem(0);
			boolean done=false;
			for (int j=0;j<items.size();j++){
				if (Integer.parseInt(offset) <
					Integer.parseInt(((ListItem)items.elementAt(j)).getSubItem(0))){
					done=true;
					items.insertElementAt(list[i],j);
					break;
				}
			}
			if (!done)
				items.addElement(list[i]);
		}
		for (int i=0;i<items.size();i++)
			newList[i]=(ListItem)items.elementAt(i);
		return newList;
	}
/**
 * Event handler for closing the application.  This method simply calls 
 * Application.Exit();
 * 
 * @param Object source - Control which generated this event
 * @param Event e - the Generated event.
 */
	private void exitMenuAction(Object source, Event e) {
		Application.exit();
	}
/**
 * As part of implementing the IFindListener interface, this method is used to hide
 * the Find Dialog when the user selects the Cancel Button.
 */
	public void processFindCancel(){
		findDialog.setVisible(false);
	}
/**
 * As part of implementing the IFindListener interface, this method is called when the
 * user selects the Find or FindNext button.  
 */
	public void processFindNext(){
		this.findNext();
	}
/**
 * As part of implementing the IFindListener interface, this method is called when the
 * user toggles the 'case' option in the Find Dialog.  This should force the find to
 * match the case of the text, or allow any case in the search.  It is currently 
 * not implemented in this class, forcing all searches to be case sensitive.
 * 
 * @param boolean type - set case sensitivity on/true off/false.
 */
	public void processFindCase(boolean type){
	}
/**
 * As part of implementing the IFindListener interface, this method is called when the
 * user toggles the 'Whole' option in the Find Dialog.  This should force the find to
 * match the entire find text, or allow any subset of the find text to be present in 
 * the names returned by the search.  It is currently not implemented in this class, 
 * forcing all searches to allow substrings.
 * 
 * @param boolean type - set whole word find selects on/true, off/false.
 */
	public void processFindWhole(boolean type){
	}
/**
 * As part of implementing the IFindListener interface, this method is called when the
 * user is done specifying the search string in the  Find Dialog.  This method resets 
 * the findList, and sets the findString to the users input.
 *  
 * @param String t - test to search for.
 */
	public void processFindText(String t){
		findString=t;
		findList.removeAllElements();
	}
/**
 * Finds the next visible Node which has a label containing the findString, 
 *  from the entire visible Tree List.
 */
	protected void findNext(){
		TreeNode n = this.findNode(findString);
		if (n!=null){
			classTree.setSelectedNode(n);
			findList.addElement(n);
			classTree.select();
		}else{
			int res = MessageBox.show(this,
				   "Finished searching the document",
				   "Find Window",
				   MessageBox.ICONINFORMATION|MessageBox.OK);
			findList.removeAllElements();
		}
	}
/**
 * This method is invoked whenever the Find menu option is selected, and causes the
 * Find Dialog to be displayed.
 * 
 * @param Object source - Control that sent the event.
 * @param Event e - Event sent to trigger the Find action.
 */
	private void findMenuAction(Object source, Event e) {
		findDialog.setVisible(true);
	//	findString=enterClassname.getText();
		findList.removeAllElements();
	//	findNext();
	}
/**
 * This method is used to turn off any screen components that may have painting 
 * problems during the loading process, and to set a global boolean that indicates
 * the applicaiton is currently loading. The boolean will be used by sensitive
 * event handlers to handle events differently during the loading process.
 * 
 * @param boolean b - Sets the application state to loading/true restores state/false.
 */
	void setLoading(boolean b){
		if (b==true){
			this.DataGrid1.setVisible(false);
			this.classTree.setVisible(false);
			if (this.lastRich!=null)
				lastRich.setVisible(false);
			this.setCursor(Cursor.WAIT);
			this.loading=true;
		}
		else {
			this.setCursor(Cursor.ARROW);
			this.classTree.setVisible(true);
			this.treeLabel.setVisible(true);
			this.DataGrid1.setVisible(true);		
			this.treePanel.setVisible(true);
			this.loading=false;
			refresh();
		}
	}
/**
 * Causes a new class file to be loaded and decoded.  The newly decoded
 * classfile is set as the currect decoded file, with its root node
 * displayed in the table.
 * 
 * @param Object source - Component which generated the event.
 * @param Event e - the generated event.
 */
	private void openMenuAction(Object source, Event e) {
		setLoading(true);
		OpenFileDialog op = new OpenFileDialog();
		op.setMultiselect(false);
		op.setShowReadOnly(false);
		op.setCheckFileExists(true);
		op.setDefaultExt("class");
		op.setDereferenceLinks(true);
		op.setTitle("Loading Class File To Decode");
		updateStatus("Loading Class File To Decode.");
		int dlgResult = op.showDialog();
		if (dlgResult == DialogResult.OK) {
			updateStatus("About to load "+op.getFileName());		
			//CHECK FOR EXISTING CLASS
			if(!checkIfFileLoaded(op.getFileName(),false)){
				addDecodeClass(op.getFileName());
				updateStatus("done");
			}else{//already loaded.
				int res = MessageBox.show(this,"Do you want to load "+
								op.getFileName()+" again?",
						"Reload Class",
						MessageBox.ICONWARNING|MessageBox.YESNO);
				if (res==6){//YES
					addDecodeClass(op.getFileName());
					updateStatus("done");
				}else{
					checkIfFileLoaded(op.getFileName(),true);
				}//LOAD AGAIN, LEAVINGing old.
			}
		}
		setLoading(false);
	}
/**
 * This method constructs the source view by traversing the root node, and creating a
 * line entry in the RichEdit Control whenever this method determines that it is 
 * appropriate.  Currently all types of fields, methods, classes, interfaces will have
 * entries in the source view.
 * 
 * @param TreeNode classRoot - Root Node (representing this class)
 * @param RichEdit201 sView - RichEdit Control that will contain the source view for 
 *							this class.
 */
	protected void buildLabels(TreeNode classRoot, RichEdit201 sView){
		makeLabels(classRoot,
				   (DecodeObject)nodes.get(classRoot),
				   sView);//Add the class to source view
		for (int i=0;i<classRoot.getNodeCount();i++){
			if (classRoot.getNode(i).getNodeCount()>0){//Ensures that there are children
			//This will iterate through all of the folders under the class in the tree.
			//The if statement filters the types that will be in the source view.
				int type=classRoot.getNode(i).getNode(0).getImageIndex();
				if ((type==this.FIELD)||(type==this.PACKAGE_PROTECTED_FIELD)||
					(type==this.PRIVATE_FIELD)||(type==this.PROTECTED_FIELD)||
					(type==this.METHOD)||(type==this.INITIALIZER)||
					(type==this.PACKAGE_PROTECTED_METHOD)||(type==this.PRIVATE_METHOD)||
					(type==this.PROTECTED_METHOD))
					for (int con=0;con<classRoot.getNode(i).getNodeCount();con++){
						makeLabels(classRoot.getNode(i).getNode(con),
								   (DecodeObject)nodes.get(classRoot.getNode(i).getNode(con)),
								   sView);
					}
				}
			}	
	}
/**
 * Uses the description from the currently monitored Class File object
 * to create a String which represents the source view of the Object.
 * The String is added to the RichEdit control for the classfile, with
 * the appropriate hyperlinks and events linked to the text.
 * 
 * @param TreeNode box - The tree Node that represents this description
 * @param DecodeObject - the decoded class file object with the 
 *							description.
 * @param RichEdit201 edit - RichEdit Control that will contain the text
 */
	protected void makeLabels(TreeNode box, DecodeObject data,RichEdit201 edit){
		String info = data.getDescription();
		String cText = edit.getText();
		cText+=info+="\n";
		edit.setText(cText);
		String[] lines = edit.getLines();
		treeItem.put((lines.length-1)+edit.id,box);
		itemTree.put(box,new Integer(lines.length-1));
	}
/**
 * Creates a new ClassObject which will decode the specified
 * class.  The filename must be a fully qualified filename.
 * 
 * @param String fname - file to decode
 */
	protected void addDecodeClass(byte[] fname){
		try{
			ClassObject cobj = new ClassObject(fname);
			updateStatus("Decode successful.");
			initTree(cobj);
		}catch(Exception ii){
			MessageBox.show("Exception decoding class "+ii,"Error Decoding Class");
			updateStatus("Error decoding class.");
		}
	}
/**
 * Creates a new ClassObject which will decode the specified
 * class.  The filename must be a fully qualified filename.
 * 
 * @param String fname - file to decode
 */
	protected void addDecodeClass(String fname){
		try{
			ClassObject cobj = new ClassObject(fname);
			updateStatus("Decode successful.");
			initTree(cobj);
		}catch(Exception ii){
			MessageBox.show("Exception decoding class "+ii,"Error Decoding Class");
			updateStatus("Error decoding class.");
		}
	}
/**
 * Sets the attributes in the rich edit control for the area that
 * have been marked as regular.
 * 
 * @param java.awt.Point p = area that is regular
 * @param RichEdit201 ric = control containing the area. 
 */
	void setRegular(java.awt.Point p,RichEdit201 ric,int data){
		int lineOffset = ric.getLineOffset(data);
		int start=lineOffset+p.x;
		int stop=lineOffset+p.y;
		ric.setSelectionStart(start);
		ric.setSelectionEnd(stop);
		ric.setSelectionStart(start);
		ric.setSelectionEnd(stop);
		ric.setColor(0x00202020);
	}
/**
 * Sets the attributes in the rich edit control for the area that
 * have been marked as special.
 * 
 * @param java.awt.Point p = area that is special
 * @param RichEdit201 ric = control containing the area.
 */
	void setSpecial(java.awt.Point p,RichEdit201 ric,int data){
		int lineOffset = ric.getLineOffset(data);
		int start=lineOffset+p.x;
		int stop=lineOffset+p.y;
		ric.setSelectionStart(start);
		ric.setSelectionEnd(stop);
		ric.setSelectionStart(start);
		ric.setSelectionEnd(stop);		
		ric.setColor(0x00f00000);
		ric.setHyperLink();
		ric.setUnderline();
	}
/**
 * Adds the hyperlinks, color, bold, underline to the areas that have
 * been marked.
 * 
 * @param RichEdit201 rich - RichEdit area to modify.
 */ 
	void setSourceViewAttributes(RichEdit201 rich){
		for (int i=0;i<rich.getLines().length;i++){
			TreeNode key = (TreeNode)treeItem.get(""+i+rich.id);
			DecodeObject dob= (DecodeObject)nodes.get(key);
			Vector points = dob.getObjectLinks();
			for (int p =0;p<points.size();p++)
				this.setSpecial((java.awt.Point)points.elementAt(p),
								rich,
								i);
		}
	}
/**
 * This method receives events whenever a Hyper Link is selected
 * 
 * @param Object sender - Control that sent this event
 * @param LinkEvent e - The actual Link Event that was sent.
 */
	public void processLinkEvent(Object sender, LinkEvent e){
		if (e.isRMouseUp()){
			updateSourceMap((RichEdit201)sender, 
						 treeItem.get(
					(new Integer(((RichEdit201)sender).getLineFromChar(e.start))).toString()
						 +((RichEdit201)sender).id));
		((RichEdit201)sender).select(e.start,e.end);
		String text = ((RichEdit201)sender).getSelText();
		this.setLoading(true);
		if (!checkIfLoaded(text,true)){
	
			try{
				byte[] b = Decode.getClassFromPath(text);
				TreeNode magic = this.getMagicNode();
				if (magic!=null){
					backStack.push(magic);
					forwardStack.removeAllElements();
				}
				addDecodeClass(b);
			}catch(Exception ii){
				System.out.println(("processLinkEvent : Exception loading class "+ii));
			}
		}
		this.setLoading(false);

		}
	}
/**
 * Handles decode messages from a popup menu.
 * 
 * @param Object source - MenuItem that was selected.
 * @param Event e - event notification. 
 */
	private void decode(Object source, Event e){
		String text = classTree.getSelectedNode().getText();
		this.setLoading(true);
		try{
			byte[] b = Decode.getClassFromPath(text);
			TreeNode magic = this.getMagicNode();
			if (magic!=null){
				backStack.push(magic);
				forwardStack.removeAllElements();
			}
			addDecodeClass(b);
		}catch(Exception ii){
			System.out.println("processLinkEvent : Exception loading class "+ii);
		}
		this.setLoading(false);
	}
/**
 * Forces the RichEdit control to NEVER have the Focus.  This will
 * eliminate the highlighted text from showing, and will not allow
 * the edit cursor to be available.  The user will not be able to 
 * highlight any text.
 * 
 * @param Object source - source of the FocusEvent
 * @param Event e - the onFocusEvent
 */
	private void editEnter(Object source,Event e){
			this.classTree.select();
	}
/**
 * Searches for a root node contained within the internalTree that has the same
 * name as the given String.  This node is returned if found.
 * 
 * @param String name - root name
 * 
 * @return TreeNode - Node with the given name, or null.
 */
	TreeNode getTreeRootFor(String name){
		TreeNode[] roots = internalTree.getNodes();
		if (roots==null) return null;
		for (int i=0;i<roots.length;i++)
			if (roots[i].getText().equals(name))
				return roots[i];
		return null;
	}
/**
 * Creates a new empty SourceView panel that will contain the decoded field, method and
 * class source like information.
 * 
 * @return RichEdit201 - new RichEdit201 component with correct settings.
 */
	RichEdit201 getSourcePanel(){
		RichEdit201 panel = new RichEdit201();
		panel.setDock(ControlDock.FILL);
		panel.setFont(Font.DEFAULT_GUI);
		panel.setForeColor(Color.WINDOWTEXT);
		panel.addOnMouseDown(new MouseEventHandler(this.selectSource));
		panel.setSize(new Point(296, 308));
		panel.setTabIndex(3);
		panel.addOnLink(new LinkEventHandler(this.processLinkEvent));
		panel.setReadOnly(true);
		panel.setScrollBars(RichEditScrollBars.BOTH);
		panel.setWordWrap(false);
		panel.setAcceptsTab(false);
		panel.setAllowDrop(false);
		panel.setAcceptsReturn(false);
		panel.setAutoSize(true);
		panel.setAutoWordSelection(false);
		panel.setCursor(Cursor.ARROW);
		panel.setBackColor(Color.CONTROLLIGHTLIGHT);
		panel.setDock(ControlDock.FILL);
		panel.setSize(new Point(296, 277));
		panel.setTabIndex(2);
		panel.setVisible(false);
		panel.addOnGotFocus(new EventHandler(this.editEnter));
		return panel;
	}
/**
 * Adds the Methods, Fields, Attributes and class contents to the
 * tree, using the given ClassObject as a data source.
 * 
 * @param ClassObject obj - Decoded class to be displayed.
 */
	private void initTree(ClassObject obj){
		RichEdit201 panel = getSourcePanel();
		if (obj.getFileName()!=null)
			this.richLabel.setText("Source View - "+obj.getFileName());//checks for filename	
		else
			this.richLabel.setText("Source View - "+obj.getName());// uses class name
		sourceLabels.put(panel,richLabel.getText());
		updateStatus("Adding new root for class");
		TreeNode packageRoot = getTreeRootFor(Decode.getPackageName(obj.getName()));
		if (packageRoot==null){//Check to see if the package root already exists.
			packageRoot =	new TreeNode(
						Decode.getPackageName(obj.getName()),
						this.PACKAGE,
						this.PACKAGE
						);
			internalTree.addNode(packageRoot);
		}
		int type = 0;
		if (obj.getName().indexOf("$")!=-1){//is inner class or interface
			if (obj.isInterface())
				type=this.INNER_INTERFACE;
			else
				type=this.INNER_CLASS;
		}else{//is regular java class or interface.
			if (obj.isInterface())
				type = this.INTERFACE;
			else
				type=this.CLASS;
		}
		TreeNode root = new TreeNode(Decode.getClassName(obj.getName())
									 ,type,type);
		nodes.put(root,obj);
		updateStatus("About to create children");
		buildTreeChildren(root, obj,panel);
		buildImports(root,obj.getConstantPool(),panel);
		buildInterfaces(root,obj.getInterfaces(),panel);
		buildSuper(root,obj.getSuperClass(),panel);
		sortTreeNode(root);
		buildLabels(root,panel);//Fills in the link object for the source view.
		packageRoot.addNode(root);
		setSourceViewAttributes(panel);//Creates the hyperlinks.
		updateStatus("Completed building children");
		rootTable.put(root,panel);	
		panel2.add(panel);
		this.syncFilter(true);//ensure that the correct settings for package and class
								//filtering are enabled.
		filterTree(root);
	}
/**
 * Finds the node which contains the selected text 
 * 
 * @see sample.viewer.ClassDecoder#findNode
 * 
 * @param String text = text for node name.
 * 
 * @return TreeNode - returns the node which has a label containing the specified text
 *						or null, if none were found.
 */
	protected TreeNode findNode(String text){
		for (int i=0;i<classTree.getNodeCount();i++){
			TreeNode n = findNode(text,classTree.getNode(i));
			if (n!=null)//Don't need to check in findList because findNode(TN,S) does
				return n;
		}
		return null;
	}
/**
 * This method will search for the given String in the specified node, and all of its
 * children.  The first match found is returned, if none is found, null is returned.
 * 
 * @param String text - text for node name.
 * @param TreeNode root - root to start the search from.  
 */
	TreeNode findNode(String text, TreeNode root){
		if ((root.getText().indexOf(text)!=-1)&&(!findList.contains(root)))
			return root;
		for (int i=0;i<root.getNodeCount();i++){
			TreeNode n = root.getNode(i);
			if (n.getText().indexOf(text)!=-1){
				if (!findList.contains(n)){
					findList.addElement(n);
					return n;
				}
			}
		}
		for(int i=0;i<root.getNodeCount();i++){
			TreeNode n = findNode(text,root.getNode(i));
			if (n!=null)//Don't need to check if already found... 
				return n;
		}
		return null;
	}
/**
 * Sorts the children nodes of this tree in ascending alphabetical order.
 * 
 * @param TreeNode node - tree to be sorted.
 */
	void sortTreeNode(TreeNode node){
		// Iterate through all children, organizing alphabetically
		Vector sort = new Vector();
		for (int i=0;i<node.getNodeCount();i++){
			TreeNode n = node.getNode(i);
			for (int j=0;j<sort.size();j++){
				if (n.getText().compareTo(((TreeNode)sort.elementAt(j)).getText())<0){
					sort.insertElementAt(n,j);
					break;
				}
			}
			if (!sort.contains(n)){
					sort.addElement(n);
				}
			if (sort.size()==0)
				sort.addElement(n);
		}
		//recursively sort all sub children.
		for (int i=0;i<sort.size();i++){
			TreeNode n = (TreeNode)sort.elementAt(i);
			sortTreeNode(n);
			node.setNode(i,n);
		}
	}
/**
 * Given the Decode object which can be any subclass of DecodeObject, this function
 * determines the classification for the object, and returns the associated image
 * index.  This will be used to add associate the correct icon with this object.
 * 
 * @param Object o - subclass of the DecodeObject to be identified.
 * 
 * @return int - image index which corresponds to the correct classification.
 */
	int getImageIndex(Object o){
		if (o instanceof FieldObject){
			FieldObject f = (FieldObject)o;
			Vector flags = f.getAccessFlags();
			if (flags.size()==0)
				return this.PACKAGE_PROTECTED_FIELD;
			for (int i=0;i<flags.size();i++){
				if (((String)flags.elementAt(i)).equals("protected"))
					return this.PROTECTED_FIELD;
				if (((String)flags.elementAt(i)).equals("public"))
					return this.FIELD;
				if (((String)flags.elementAt(i)).equals("private"))
					return this.PRIVATE_FIELD;
			}
			return this.PACKAGE_PROTECTED_FIELD;
		}
		if (o instanceof MethodObject){
			MethodObject f = (MethodObject)o;
			Vector flags = f.getAccessFlags();
			if (f.getName().indexOf("<")!=-1)
				return this.INITIALIZER;
			if (flags.size()==0)
				return this.PACKAGE_PROTECTED_METHOD;
			for (int i=0;i<flags.size();i++){
				if (((String)flags.elementAt(i)).equals("protected"))
					return this.PROTECTED_METHOD;
				if (((String)flags.elementAt(i)).equals("public"))
					return this.METHOD;
				if (((String)flags.elementAt(i)).equals("private"))
					return this.PRIVATE_METHOD;
			}
			return this.PACKAGE_PROTECTED_METHOD;
		}
		if (o.getClass().getName().indexOf("Structure")!=-1)
			return this.STRUCTURE;
		if (o instanceof AttributeObject)
			return this.ATTRIBUTE;
		if (o instanceof ConstantObject)
			return this.CONSTANT;
	//This needs to return the specific folder icon for its contents.
		if (o instanceof Vector){
			Object oo = (((Vector)o).firstElement());
			if (oo.getClass().getName().indexOf("Structure")!=-1)
				return this.STRUCTURE_INFORMATION;
			if (oo instanceof FieldObject)
				return this.FIELD_INFORMATION;
			if (oo instanceof MethodObject)
				return this.METHOD_INFORMATION;
			if (oo instanceof ConstantObject)
				return this.CONSTANT_INFORMATION;
			if (oo instanceof AttributeObject)
				return this.ATTRIBUTE_INFORMATION;
		}
		return this.NONE;	
	}
/**
 * This method adds a container node for the imports, and a child node for each 
 * String in the Vector.  Each child will be the name of a class that was used
 * somewhere in the decoded class, and is therefore determined to be an import.
 * 
 * @param TreeNode root - TreeNode that will receive the import node as a child. 
 * @param Vector data - Vector of Strings containing import class names.
 * @param RichEdit201 panel - The RichEdit201 control that holds the source view.
 */
	private void buildImports(TreeNode root,Vector data,RichEdit201 panel){
		if ((data!=null)&&(data.size()>0)){
			TreeNode t = new TreeNode("Imports",
									  this.IMPORT_INFORMATION,
									  this.IMPORT_INFORMATION);
			root.addNode(t);
			for(int i=0;i<data.size();i++){
				ConstantObject el = (ConstantObject)data.elementAt(i);
				if (el instanceof ConstantClassObject){
					String name = ((ConstantClassObject)el).getClassName();
					TreeNode n = new TreeNode(name,this.IMPORT_CLASS,this.IMPORT_CLASS);
					t.addNode(n);
				}
			}
		}
	}
/**
 * Adds the Strings in the vector to the specified root node, as interfaces.  The 
 * panel is updated with the appropriate information.
 * 
 * @param TreeNode root - node to receive interfaces as children.
 * @param Vector data - contains Strings that represent the interface names.
 * @param RichEdit201 panel - The associated source view for the decoded class
 */
	private void buildInterfaces(TreeNode root,Vector data,RichEdit201 panel){
		if ((data!=null)&&(data.size()>0)){
			for(int i=0;i<data.size();i++){
				ConstantClassObject el = (ConstantClassObject)data.elementAt(i);
				String name = el.getClassName();
				TreeNode n = new TreeNode(name,this.INTERFACE,this.INTERFACE);
				root.addNode(n);
			}
		}
	}	
/**
 * Adds the String to the root node, as a Super Class.  The 
 * panel is updated with the appropriate information.
 * 
 * @param TreeNode root - node to receive interfaces as children.
 * @param Vector data - contains Strings that represent the interface names.
 * @param RichEdit201 panel - The associated source view for the decoded class
 */
	private void buildSuper(TreeNode root,String data,RichEdit201 panel){
		if (data!=null){
			TreeNode t = new TreeNode(data,this.SUPER_CLASS,this.SUPER_CLASS);
			
			root.addNode(t);
		}
	}
/**
 * Recursive method for traversing the decoded class object, and
 * creating the tree nodes/Source view for the data.
 * 
 * @param TreeNode root - root node which will have children added
 * @param Object data - Object to be examined for ndoes, and source
 * @param ListBox panel - List that will receive the source.
 */
	private void buildTreeChildren(TreeNode root, Object data,RichEdit201 panel){
		if (data instanceof Vector){
			for (int j=0;j<((Vector)data).size();j++){
				Object obj = ((Vector)data).elementAt(j);
				if (obj instanceof DecodeDisplayInterface){
					//Need to determine correct image index.
					int image = getImageIndex(obj);
					TreeNode child = new TreeNode(
						((DecodeDisplayInterface)obj).getName(),
						image,
						image);
					if ((obj instanceof FieldObject)||(obj instanceof MethodObject)){
						String description = ((DecodeObject)obj).getDescription();
					}
					root.addNode(child);
					if (!(obj instanceof Vector))
						nodes.put(child,obj);
					buildTreeChildren(child,(DecodeDisplayInterface)obj,panel);
				}
			}
		}else if (data instanceof DecodeDisplayInterface)
			for (Enumeration i = ((DecodeDisplayInterface)data).getChildren().keys();i.hasMoreElements();){
				String key = (String)i.nextElement();
				Object ch = ((DecodeDisplayInterface)data).getChildren().get(key);
				if ((!(ch instanceof Vector))||
					((ch instanceof Vector)&&(((Vector)ch).size()>0))){
					TreeNode child = new TreeNode(key);
					if (!(ch instanceof Vector)){
						nodes.put(child,ch);
					}
					if ((ch instanceof FieldObject)||(ch instanceof MethodObject)){
						String description = ((DecodeObject)ch).getDescription();
					}
					int image=getImageIndex(ch);
					child.setImageIndex(image);
					child.setSelectedImageIndex(image);		
					root.addNode(child);
					buildTreeChildren(child,((DecodeDisplayInterface)data).getChildren().get(key),panel);
				}
			}
		}
/**
 * Adds the given text to the status bar at the bottom of the application.
 * The text will remain there until it is overwritten.
 * 
 * @param String info - text to be displayed on the status bar.
 */
	private void updateStatus(String info){
		statusBar1.setText(info);
	}
/**
 * Determines the root node for this displayed class.  This will typically be a Class
 * node which will have children for all attributes for the class.
 * 
 * @param TreeNode t - Some node in the tree
 * 
 * @return TreeNode - the Class Node that is the root for the given child.
 */
	TreeNode getRoot(TreeNode t){
		boolean done=false;
		while (!done){
			if(t==null)
				done=true;
			else if ((t.getImageIndex()==this.CLASS)||(t.getImageIndex()==this.INNER_CLASS))
				return t;
			else if (rootTable.get(t)!=null)
				done=true;
			else if (t.getParent()==null)
				done=true;
			else if (!done)
				t=t.getParent();
		}
		return t;
	}
/**
 * Determines what java package (absolute root node) is appropriate for the given node.
 * 
 * @param Node from an unknown package.
 * 
 * @return TreeNode - TreeNode representing the package for the given node, or null.
 */
	TreeNode getPackage(TreeNode t){
		while (t!=null){
			if(t.getImageIndex()==this.PACKAGE)
				return t;
			t=t.getParent();
		}
		return null;		
	}
/**
 * Handles the generated events when a tree node is selected.  It causes
 * the table to be updated with the information represented by the tree
 * node (if it is available), and selects the applicable line in the 
 * source view.
 * 
 * @param Object source - Control which generated the event.
 * @param TreeViewCancelEvent e - The generated event.
 */
	private void nodeSelected(Object source, TreeViewEvent e) {
			refresh();
	}
/**
 * This method filters the tree, taking into consideration the settings in the menus
 * and buttons for what the user wants to see.  After the filtering is complete, the
 * selected node will be the best fit of the visible nodes, to the internal node
 * taken as a parameter.
 * 
 * @param TreeNode internalRoot - node to be used in determining the best visible node
 *									after the filtering.
 */
	void filterTree(TreeNode internalRoot){
		classTree.removeAll();
		buildDisplayTree(classTree,internalTree);
		refresh();
		TreeNode t = internalRoot;
		while (isUserHidden(t.getImageIndex()))
			t=t.getParent();
		TreeNode outer = getMappedNode(t);
		classTree.setSelectedNode(getMappedNode(outer));
		classTree.select();
		refresh();
	}
/**
 * Creates the displayed tree based on the users filter options, and the internal
 * tree containing all information from the decoded class
 * 
 * @param Object source - Component requesting a tree restructure
 * @param Event e - the event causing the restructure.
 */
	void filterTree(Object source, Event e){
		TreeNode t = getInternalNode(classTree.getSelectedNode());
		if (t==null)
			t=internalTree.getNode(0);
		if (t!=null)
			filterTree(t);
	}
/**
 * Recursive method for traversing the internal tree and building the display tree
 * based on the user selections. This method is required to establish the root nodes
 * of the displayed panel.
 * 
 * @param TreeView view - View that will be displayed with the filtered nodes.
 * @param TreeNode root - the root of the node that will be displayed.
 */
	void buildDisplayTree(TreeView view, TreeView intern){
		TreeNode retVal = null;
		view.removeAll();  
		outerInternal.clear();
		TreeNode[] roots = intern.getNodes();
		for (int rInt=0;rInt<roots.length;rInt++){
			TreeNode n = roots[rInt];
			if (n==null) return;
			int index = n.getImageIndex();
			if (!this.isUserHidden(index)){//checks only the root nodes. (packages)
				TreeNode root = new TreeNode();
				root.setText(n.getText());
				root.setImageIndex(n.getImageIndex());
				root.setSelectedImageIndex(n.getSelectedImageIndex());
				view.addNode(root);//root here is a clone of the internal root
				this.outerInternal.put(root,n);		
				Vector rootLists = new Vector();//may not be necessary now.
				for (int i=0;i<n.getNodeCount();i++)
					rootLists.addElement(n.getNode(i));
				for (int i=0;i<rootLists.size();i++){
					buildDisplayTree((TreeNode)rootLists.elementAt(i),
											  root,6);
				}
				//checks all children, adding to itself if valid
				retVal=root;//This should be a class node ONLY.
				if (root.getImageIndex()==this.PACKAGE)
					retVal=root.getNode(0);
			}else{//This means that the user wants to hide the internal root
				TreeNode holder = new TreeNode();
				buildDisplayTree(n,holder,5);
				for (int i=0;i<holder.getNodeCount();i++)
					classTree.addNode(holder.getNode(i));//this will always be clones
				if (holder.getNodeCount()>0)
					retVal=holder.getNode(0);
			}
		}
		classTree.setSelectedNode(retVal);
	}
/**
 * This handles the filtering / addition of children nodes.
 * 
 * @param TreeNode node - the TreeNode on the display panel which needs filtering.
 */
	void buildDisplayTree(TreeNode node,TreeNode root,int t){
		TreeNode newRoot = root;
		if ((!isUserHidden(node.getImageIndex()))||
			(((node.getImageIndex()==this.INTERFACE)||
			  (node.getImageIndex()==this.INNER_INTERFACE))&&
			  (node.getNodeCount()>0))){
			newRoot=new TreeNode();
			newRoot.setImageIndex(node.getImageIndex());
			newRoot.setSelectedImageIndex(node.getSelectedImageIndex());
			newRoot.setText(node.getText());
			this.outerInternal.put(newRoot,node);
			root.addNode(newRoot);
		}
		for (int i=0;i<node.getNodeCount();i++)
			buildDisplayTree(node.getNode(i),newRoot,4);
	}
/**
 * Repaints the screen, setting the correct selection for tree node, list view and
 * source view.
 */
	void refresh(){
		TreeNode base = classTree.getSelectedNode();
		TreeNode internalBase=null;
		TreeNode internalRoot=null;
		TreeNode root=null;
		if (base!=null){ //such as after removing all visible nodes.	
			internalBase = getInternalNode(base);
			root = getRoot(base);
			internalRoot = getRoot(internalBase);
		}else{
			return;
		}
			RichEdit201 rich = null;
				rich = (RichEdit201)rootTable.get(internalRoot);
			if (rich!=null){
				if (!((Control)rich).getVisible()){
					for(Enumeration i = rootTable.elements();i.hasMoreElements();){
						((Control)i.nextElement()).setVisible(false);
					}
					((Control)rich).setVisible(true);
					this.treeLabel.setText(
						((DecodeDisplayInterface)nodes.get(internalRoot)).getName()
					);
					this.richLabel.setText((String)sourceLabels.get(rich));
				}	
				updateSourceMap(rich,internalBase);
			}else{//Rich edit was not found for the node.  This likely means that a package
				// was selected, or there are no visible nodes.
			for(Enumeration i = rootTable.elements();i.hasMoreElements();)
				((Control)i.nextElement()).setVisible(false);
			this.richLabel.setText("Source View");
			TreeNode tn = classTree.getSelectedNode();
			if (tn!=null)
				this.treeLabel.setText("Package - "+tn.getText());
			else 
				this.treeLabel.setText("");
		}
			if (nodes.get(internalBase)!=null)
				updateTable((DecodeDisplayInterface)nodes.get(internalBase));
			else updateTable(null);
	}
/**
 * This method will highlight the selected area if the mouse selects
 * a portion of text in the source view.
 * 
 * @param RichEdit201 rich - rich edit text area where the selection
 *								was made.
 * @param Object node - the object associated with the selected area. 
 */
	synchronized void updateSourceMap(RichEdit201 rich, Object node){
		if (!this.loading){
		Integer ind =new Integer(0);//initial state
		TreeNode n = (TreeNode)node;
		boolean done=false;
		while (!done){//This will ensure that the appropriate source view is always
					//highlighted when its children tree nodes are selected.
			if (n==null) done=true;
			else {
				ind =(Integer)itemTree.get(n);
				if (ind==null)
					n=n.getParent();
				else done=true;
			}
		}
		if (ind!=null){
			int index=ind.intValue();
			int start = rich.getText().indexOf(rich.getLines()[index]);
			int stop = start+rich.getLines()[index].length();
			highlight(rich.getLineOffset(index),
						rich.getLineOffset(index)+rich.getLines()[index].length(),
						rich);
		}else
			highlight(0,0,rich);
		}
	}
/**
 * Highlights the text in the iven rich edit control betweeen the specified points.
 * Any previous rich edit highlights are undone, even if they are not on the same
 * rich edit control.
 * 
 * @param int start - character index to start bolding.
 * @param int end - character index of where to stop bolding
 * @param RichEdit201 rich - Control to receive the bold.
 */
	void highlight(int start,int end,RichEdit201 rich){
		if ((sourceSelected!=null)&&(lastRich!=null)){
			lastRich.setSelectionStart(sourceSelected.x);
			lastRich.setSelectionEnd(sourceSelected.y);
			lastRich.setSelectionStart(sourceSelected.x);
			lastRich.setSelectionEnd(sourceSelected.y);
			lastRich.setUnBold();
		}
			rich.setSelectionStart(start);
			rich.setSelectionEnd(end);
			rich.setSelectionStart(start);
			rich.setSelectionEnd(end);
			rich.setBold();
			sourceSelected= new java.awt.Point(start,end);
			rich.select();
			lastRich=rich;
	}
/**
 * This is the event handler for selections in the Source View.
 * 
 * @param Object source- This will be the RichEdit control where the 
 *						mouse event originated.
 * @param MouseEvent m - The generated mouse event.
 */
	private void selectSource(Object source, MouseEvent m) {
		RichEdit201 rich = (RichEdit201)source;
		int p = ((RichEdit)source).getSelectionEnd();
		int index = rich.getLineFromChar(p);
		Object obo = this.treeItem.get(index+((RichEdit201)source).id);
		if (obo instanceof TreeNode){	
			TreeNode ext = getMappedNode((TreeNode)obo);//returns null if class is filtered
			if (ext!=null){
				classTree.setSelectedNode(ext);
				//call the highlight thing itself.
				//This is where the failure occures when flipping between classes
				this.classTree.select();
			}else{
				updateSourceMap(rich,(TreeNode)obo);
				updateTable((DecodeDisplayInterface)nodes.get((TreeNode)obo));
			}
		}else{
			highlight(rich.getLineOffset(index),
					  index+rich.getLines()[index].length(),rich);
		}
	}
/**
 * Searches the already decoded Classes to see if the specified class is there.  It uses
 * the boolean parameter to determine whether this node should be selected if found.
 * 
 * @param String cname - fully qualified name of the class to find.
 * @param boolean selectIt - Spefifies if the class should be selected.
 * 
 * @return boolean - true if the class exists, else false
 */
	boolean checkIfLoaded(String cname,boolean selectIt){
		for (int pkgs=0;pkgs<internalTree.getNodeCount();pkgs++){
			if (internalTree.getNode(pkgs).getText().equals(Decode.getPackageName(cname)))
				for (int cls=0;cls<internalTree.getNode(pkgs).getNodeCount();cls++){
					if (internalTree.getNode(pkgs).getNode(cls).getText().equals(Decode.getClassName(cname))){
						if (selectIt){
							classTree.setSelectedNode(getMappedNode(internalTree.getNode(pkgs).getNode(cls)));
							refresh();
						}
						return true;
					}
				}
		}
		return false;
	}
/**
 * Checks the already decoded classes to see if the file specified was used.  This is
 * accomplished by iterating through the sourceLabels used for the Source View, to see
 * if the filename is displayed.  If this clas is already decoded, it may be selected
 * as the currently viewed.
 * 
 * @param String fname - filename of Classfile.
 * @param boolean selectIt - true/ the tree will have this as the currently selected
 *							class.
 */
	boolean checkIfFileLoaded(String fname, boolean selectIt){
		for (Enumeration rLabels = sourceLabels.keys();rLabels.hasMoreElements();){
			Object key = rLabels.nextElement();
			String storedF = (String)sourceLabels.get(key);
			if (storedF.indexOf(fname)!=-1){//Found the decoded class.
				if (selectIt){
					classTree.setSelectedNode(getVisibleRoot((RichEdit201)key));			
					refresh();
				}
				return true;
			}
		}
	return false;	
	}
/**
 * Determines what the visible root node is for the passed Source View.  The rootTable
 * is analysed to determine the appropriate internal node, this node is then used to
 * find the best fit of the visible nodes.
 * 
 * @param RichEdit201 sView - SourceView which needs the associated root node.
 * 
 * @return TreeNode - The visible Class Root node for the given source view. 
 */
	TreeNode getVisibleRoot(RichEdit201 sView){
		for (Enumeration roots = rootTable.keys();roots.hasMoreElements();){
			Object key = roots.nextElement();
			RichEdit201 storedF = (RichEdit201)rootTable.get(key);
			if (storedF==sView){//Found the root Node
				return getMappedNode((TreeNode)key);
			}
		}
	return null;			
	}
/**
 * Handles the open from classpath menu action, causing the class specified in the
 * toolbar edit control to be loaded from the classpath if it is available.  If this
 * fails, a dialog box will pop up.
 * 
 * @param Object source - Control which sent this event.
 * @param Event e - The event sent to trigger this action.
 */
	private void openFromClasspath(Object source, Event e) {
		if (enterClassname.getText()!=null){
		if (!checkIfLoaded(enterClassname.getText(),true)){
			this.setLoading(true);
			byte[] b = Decode.getClassFromPath(enterClassname.getText());
			if (b!=null){
				try{
				addDecodeClass(b);
				}catch(Exception ii){
					System.out.println("openfrom CLasspath exception "+ii);
				}
			}else
				MessageBox.show("Cannot find ["+enterClassname.getText()+"] in Classpath");
		}
		this.setLoading(false);
		}
	}
/**
 * Handles all actions associated with buttons from any of the applications 
 * toolbars.  
 * 
 * @param Object source - Control which sent the event, this will typically be the 
 *						toolbar containing the button.
 * @param ToolBarButtonClickEvent e - Event resulting from a single click.
 */
	private void filterButtonEvent(Object source, ToolBarButtonClickEvent e) {
		int index = e.button.getImageIndex();
		if ((index==this.PACKAGE)||(index==this.STRUCTURE)||(index==this.CLASS)||
			(index==this.IMPORT_CLASS)||(index==this.ATTRIBUTE)||
			(index==this.SUPER_CLASS)||(index==this.METHOD)||(index==this.CONSTANT)||
			(index==this.FIELD)||(index==this.INTERFACE)){
			// The node to be used as the root should either be the currently selected 
			// nodes equivalent in the internal tree, or the root from knowing the 
			// currently displayed RichEdit201 control
		
			filterTree(null,null);
			this.syncFilter(false);
		}
		else if (index==this.DECODE){
			this.openFromClasspath(null,null);
		}
		else if (index==this.FIND){
			if (!findString.equals(enterClassname.getText())){
				findString=enterClassname.getText();
				findList.removeAllElements();
			}
			findNext();
			classTree.select();
		}
		else if (index==this.BACK){
			this.processBack(null,null);
		}
		else if (index==this.FORWARD){
			this.processForward(null,null);
		}
		else if (index==this.DELETE){
			this.removeNode(null,null);
		}
	}
/**
 * Determines if the given index has been specified as filtered by the user.  This is
 * checked, by comparing the given index with the state of the toolbar pushbuttons.
 * 
 * @param int index -  state to check.
 * 
 * @return boolean - whether this state is requested as filtered.
 */
	boolean isUserHidden(int index){
		if ((index==this.IMPORT_CLASS)||
				 (index==this.IMPORT_INFORMATION)) 
			return this.imprt.getPushed();
		else if (index==this.SUPER_CLASS) return this.sclass.getPushed();
		else if ((index==this.ATTRIBUTE)||
				 (index==this.ATTRIBUTE_INFORMATION)) 
			return this.attribute.getPushed();
		else if ((index==this.INTERFACE)||
				 (index==this.INNER_INTERFACE)) 
			return this.interf.getPushed();
		else if ((index==this.CONSTANT)||
				 (index==this.CONSTANT_INFORMATION)) 
			return this.constants.getPushed();
		else if ((index==this.METHOD)||
				 (index==this.PRIVATE_METHOD)||
				 (index==this.INITIALIZER)||
				 (index==this.PROTECTED_METHOD)||
				 (index==this.PACKAGE_PROTECTED_METHOD)||
				 (index==this.METHOD_INFORMATION)) 
			return this.mthd.getPushed();
		else if ((index==this.FIELD)||
				 (index==this.PRIVATE_FIELD)||
				 (index==this.PROTECTED_FIELD)||
				 (index==this.PACKAGE_PROTECTED_FIELD)||
				 (index==this.FIELD_INFORMATION)) 
			return this.field.getPushed();
		else if ((index==this.STRUCTURE)||
				 (index==this.STRUCTURE_INFORMATION)) 
			return this.structure.getPushed();
		else return false; //Default return value...
	}
/**
 * Handles any keyboard events that may be generated for this application.  These
 * events are mapped to bindings, and fire off the appropriate actions.
 * 
 * @param Object source - source of the keyboard event
 * @param KeyEvent e - the actual keyboard event containing information about the keys
 *						that were pressed.
 */
	private void processKeyBinding(Object source, KeyEvent e){
		if (e.getKeyCode()==114){//Handle F3 event.
			this.findNext();
		}
	}
/**
 * Handles the remove event, eradicating the currently selected node from the 
 * application.  A dialog box is presented, giving the user a chance to back out.
 * 
 * @param Object source - Component requesting the remove
 * @param Event e - the generated remove event.
 */
	private void removeNode(Object source, Event e){
		TreeNode n = classTree.getSelectedNode();
		int res = MessageBox.show(this,"Do you want to remove "+n.getText()+" from the view?",
						"Removing Class",
						MessageBox.ICONWARNING|MessageBox.YESNO);
		if (res==6){//YES
			TreeNode root = getRoot(n);
					removeNode(root,true);
		}
		TreeNode tn = classTree.getSelectedNode();
		if ((tn!=null)&&(tn.getImageIndex()==this.PACKAGE)){
			if (classTree.getSelectedNode().getNodeCount()==0){
				classTree.getSelectedNode().remove();
			}
		}
	}
/**
 * Removes a package from the listed view.  This is different than simply removing a
 * node, as each package will contain multiple class nodes, each of which will have
 * associated label, richedit, and data objects associated with it.  Removing the 
 * pacakge as a sinlge node will cause all of these things to be left behind.
 * 
 * @param TreeNode root - the TreeNode which is a Package. (determined by image index)
 */
	void removePackage(TreeNode root,boolean resetNode){
		this.treeLabel.setText("");	
		for (int i=0;i<root.getNodeCount();i++){//remove each class from the package
											//to ensure that the internal 'stuff'
											//is properly eradicated
			removeNode(root.getNode(i),false);
		}
		getInternalNode(root).remove();//If there is no internal root, something is wrong
		root.remove();//remove the package itself.
		if (resetNode){
			if (classTree.getNodeCount()>0){
				if (classTree.getNode(0).getNodeCount()>0)
					classTree.setSelectedNode(classTree.getNode(0).getNode(0));
				else
					classTree.setSelectedNode(classTree.getNode(0));
			}
		}
		this.backStack.removeAllElements();
		this.forwardStack.removeAllElements();
	}
/**
 * Removes the references between line numbers and internal nodes from the 
 * itemTree and treeItem tables.  As there will be no references in either table
 * for nodes that do not have information in the Source View, we must handle the case
 * where removing a node is passed that does not exist.
 * 
 * @param TreeNode n - Node to clear from itemTree and treeItem
 */
	void cleanItems(TreeNode n,String id){
		for (int i=0;i<n.getNodeCount();i++)
			cleanItems(n.getNode(i),id);
		Integer x = (Integer)itemTree.get(n);
		itemTree.remove(n);
		if (x!=null)treeItem.remove(x.toString()+id); //Need to add RichEdit201 ID....
	}
/**
 * Removes the given node from the tree, erasing all relevent information from the 
 * entire application.
 * 
 * @param TreeNode root - node to be eradicated.
 */
	  void removeNode(TreeNode root,boolean resetNode){
			if (root==null) return;
			TreeNode internal = getInternalNode(root);
			TreeNode internalRoot = getRoot(internal);
			TreeNode myParent = root.getParent();
			if (root.getImageIndex()==this.PACKAGE)//This could cause loops for contained
													//packages
				removePackage(root,true); 
			else{
				Object o = rootTable.get(internalRoot);
				root.remove();
				internalRoot.remove();
				nodes.remove(internalRoot);
				panel2.remove((Control)o);//REMOVES RICHEDIT FROM DISPLAY
				cleanItems(internalRoot,((RichEdit201)o).id);
				
				rootTable.remove(internalRoot);//REMOVES MAP BETWEEN ROOT AND RICHEDIT
				lastRich=null;
				if (resetNode){
					if (myParent!=null){
						if (myParent.getNodeCount()>0)
							classTree.setSelectedNode(myParent.getNode(0));
						else
							classTree.setSelectedNode(myParent);
					}else{
						if (classTree.getNodeCount()>0)
							//this doesn't work.
								classTree.setSelectedNode(classTree.getNode(0).getNode(0));
					}
				}
				findList.removeAllElements();
				findString="";
			}	
			this.backStack.removeAllElements();
			this.forwardStack.removeAllElements();
	  }
/**
 * Handles a refresh request from the user.
 * 
 * @param Object source - container requesting the refresh.
 * @param Event e - the refresh event.
 */
	private void processRefresh(Object source, Event e){
		refresh();
	}
/**
 * Determines the TreeNode that is used to represent the displayed class.  The 
 * result of this method will be used to store the representation of the displayed
 * class through recording this tree node.
 * 
 * @return TreeNode - tree node that will be used to recall this decoded class.
 */
	TreeNode getMagicNode(){
		//lastrich etc.
		for (Enumeration i = rootTable.keys();i.hasMoreElements();){
			Object key = i.nextElement();
			if (rootTable.get(key)==this.lastRich)
				return (TreeNode)key;
		}
		return null;
	}
/**
 * Determines the internal node that mirrors the given node.
 * 
 * @param TreeNode ext - external node which needs an internal map.
 * 
 * @return TreeNode - internal node 
 */
	TreeNode getInternalNode(TreeNode ext){
		return (TreeNode)this.outerInternal.get(ext);
	}
/**
 * Attempts to find the 'brother' node that exists in the node tree specified, or below
 * which has exactly the same display name as the original.  Typically the first node
 * will be a visible node, and the second will be one from the internal tree.  A node
 * will cannot be returned as a match if it is of a type currenty filtered by the users
 * menu or button options.
 * 
 * @param TreeNode lookfor - original node that needs a match
 * @param TreeNode patch - root node of a tree to analyze for a match.
 * 
 * @return TreeNode - resulting match or null if none were found. 
 */
	TreeNode search(TreeNode lookfor, TreeNode patch){
		if ((!this.isUserHidden(patch.getImageIndex()))||
			(((patch.getImageIndex()==this.INTERFACE)||
			  (patch.getImageIndex()==this.INNER_INTERFACE))
			 &&(patch.getNodeCount()>0))){
			if (patch.getText()==lookfor.getText())
				return patch;
			for (int i=0;i<patch.getNodeCount();i++){
				TreeNode tester = search(lookfor,patch.getNode(i));
				if (tester!=null)
					return tester;
			}
		}else{
			for (int i=0;i<patch.getNodeCount();i++){
				TreeNode close = search(lookfor,patch.getNode(i));
				if (close!=null) 
					return close;
			}
		}
		return null;		
	}
/**
 * Given an internal node, this method returns its first child that is not filtered
 * by the user.
 * 
 * @param TreeNode internal - internal node from which to search.
 * 
 * @return TreeNode - a visible child node, or null if none were found.
 */
	TreeNode getFirstVisible(TreeNode internal){
	//Find a child of this root node that is visible.  Any will do, 
		if ((!this.isUserHidden(internal.getImageIndex())||
			 (((internal.getImageIndex()==this.INTERFACE)||
			   (internal.getImageIndex()==this.INNER_INTERFACE))&&
			   (internal.getNodeCount()>0)))){
			//must get from classTree root.
			TreeNode[] view = classTree.getNodes();
			for (int i=0;i<view.length;i++){
				if (internal.getText().equals(view[i].getText()))
					return view[i];
			}
			return null;
		}else {
			for (int i=0;i<internal.getNodeCount();i++){
				TreeNode test = getFirstVisible(internal.getNode(i));
				if (test!=null)
					return test;
			}
			return null;
		}
	}
/**
 * Determines the closest displayed node to the given internal node.  This will be
 * used to set the display during forward and back actions.
 * 
 * @param TreeNode internal - internal tree node to map.
 * 
 * @return TreeNode - closest mapped displayed node.
 */
	TreeNode getMappedNode(TreeNode internal){
		if (internal.getParent()==null){//internal node is a root	
			//Start checking from all external roots.
			TreeNode[] roots = classTree.getNodes();
			for (int i=0;i<roots.length;i++){
				TreeNode map = search(internal,roots[i]);
				if (map!=null)
					return map;
			}
			return getFirstVisible(internal);
		}else{//internal node is NOT a root.	
			//start checking from my parents external root
			TreeNode parentRoot = getMappedNode(internal.getParent());
			if (parentRoot!=null){ //return null;//Screwup.
				TreeNode cand = search(internal,parentRoot);
				if (cand!=null)
					return cand;
				return getFirstVisible(internal);
			}
			return null;
		}
	}
/**
 * Handles events related to the 'forward' action.  This will set the currently 
 * displayed decoded class to the previously decoded class.  It is useful for
 * going back down the path of hyperlinks.
 * 
 * @param Object source - the Control which sent this event.
 * @param Event e - event sent to trigger this action.
 */
	private void processForward(Object source, Event e){
		//can use lastRich for the rich edit panel.
		if (forwardStack.size()>0){
			//May be using the wrong node in the stacks and for displaying...see back
			TreeNode root = getMagicNode();
			if (root!=null){
				backStack.push(root);
				classTree.setSelectedNode(getMappedNode((TreeNode)forwardStack.pop()));
				refresh();
			}
		}
	}
/**
 * Handles a 'back' request event from the user.  It sets the currently viewed class
 * to the one previously decoded.
 * 
 * @param Object source - component requesting the previous class.
 * @param Event e - the Back event.
 */
	private void processBack(Object source, Event e){
		if (backStack.size()>0){
			TreeNode root = getMagicNode();
			if (root!=null){
				forwardStack.push(root);
				//Must determine the equivalent displayed node AFTER the current node
				//is retrieved, as the filter may have changed.
				TreeNode backNode = getMappedNode((TreeNode)backStack.pop());
				classTree.setSelectedNode(backNode);
				refresh();
			}
		}
	}
/**
 * Adds the appropriate menu items for an undecoded class to the given Context Menu
 * 
 * @param ContextMenu c - Context Menu to recieve the Menu Items.
 */
	protected void addClassNamePopup(ContextMenu c){
		MenuItem cn = new MenuItem("Decode");
		cn.addOnClick(new EventHandler(this.decode));
		c.add(cn);
	}
/**
 * Adds any popup menu items that are specific to Class, Field, or Method Objects to 
 * the given ContextMenu.
 * 
 * @param ContextMenu c - ContextMenu to receive the items.
 */
	protected void addClassFieldMethodPopup(ContextMenu c){
		MenuItem a1 = new MenuItem("InsertAttribute...");
		a1.setEnabled(false);
		c.add(a1);
	}
/**
 * Adds any popup menu items that are specific to Class Object to the given 
 * ContextMenu.
 * 
 * @param ContextMenu c - ContextMenu to receive the items.
 */
	protected void addClassPopup(ContextMenu c){
		MenuItem m = new MenuItem("Remove");
		m.addOnClick(new EventHandler(this.removeNode));
		c.add(m);
	}
/**
 * Adds any popup menu items that are specific to Package Objects to 
 * the given ContextMenu.
 * 
 * @param ContextMenu c - ContextMenu to receive the items.
 */
	protected void addPackagePopup(ContextMenu c){
		MenuItem m = new MenuItem("List Classes");
		m.setEnabled(false);
		MenuItem m2 = new MenuItem("Print...");
		c.add(m);
		m2.setEnabled(false);
		c.add(m2);
	}
/**
 * Adds any popup menu items that are common to all Objects to 
 * the given ContextMenu.
 * 
 * @param Contextmenu c - ContextMenu to receive the items.
 */
	protected void addPopularPopup(ContextMenu c){
		MenuItem m = new MenuItem("Cut");
		m.setEnabled(false);
		c.add(m);
		MenuItem m2 = new MenuItem("Copy");
		m2.setEnabled(false);
		c.add(m2);
		MenuItem m3 = new MenuItem("Paste");
		m3.setEnabled(false);
		c.add(m3);
		MenuItem m4 = new MenuItem("Print");
		m4.setEnabled(false);
		c.add(m4);
		MenuItem m1 = new MenuItem("Edit");
		m1.setEnabled(false);
		c.add(m1);
	
	}
/**
 * On a popup menu request, this method determines which menu items are required -
 * populating the menu item appropriately.
 * 
 * @param Object source - Context menu that will be displayed.
 * @param Event e - Generated event for this action.
 */
	private void processTreePopup(Object source, Event e){
		((ContextMenu)source).clear();
		int type = classTree.getSelectedNode().getImageIndex();
		if ((type ==this.SUPER_CLASS)||(type==this.IMPORT_CLASS)||
			(((type==this.INTERFACE)||(type==this.INNER_INTERFACE))&&
			 (classTree.getSelectedNode().getNodeCount()==0)))
			addClassNamePopup((ContextMenu)source);
		else{
			if ((type==this.METHOD)||(type==this.INITIALIZER)||
				(type==this.PRIVATE_METHOD)||(type==this.PROTECTED_METHOD)||
				(type==this.FIELD)||(type==this.PRIVATE_FIELD)||
				(type==this.PROTECTED_FIELD)||(type==this.CLASS)||
				(type==this.PACKAGE_PROTECTED_FIELD)||
				(type==this.PACKAGE_PROTECTED_METHOD))
				addClassFieldMethodPopup((ContextMenu)source);
			if ((type==this.CLASS)||(type==this.PACKAGE)||
				(type==this.INNER_CLASS)||
				(((type==this.INTERFACE)||(type==this.INNER_INTERFACE))&&
			 (classTree.getSelectedNode().getNodeCount()!=0)))
				addClassPopup((ContextMenu)source);
			if (type==this.PACKAGE)
				addPackagePopup((ContextMenu)source);
			if ((type==this.METHOD)||(type==this.INITIALIZER)||
				(type==this.PRIVATE_METHOD)||(type==this.PROTECTED_METHOD)||
				(type==this.FIELD)||(type==this.PRIVATE_FIELD)||
				(type==this.PROTECTED_FIELD)||(type==this.CLASS)||
				(type==this.ATTRIBUTE)||(type==this.CONSTANT)||
				(type==this.STRUCTURE)||(type==this.PACKAGE_PROTECTED_FIELD)||
				(type==this.PACKAGE_PROTECTED_METHOD)||
				(type==this.INTERFACE)||(type==this.INNER_CLASS)||
				(type==this.INNER_INTERFACE))
				addPopularPopup((ContextMenu)source);
				}
	}
/**
 * Syncs the state of the state of the Menu and the buttons
 * 
 * @param boolean withMenu - syncs buttons with menu if true, else menu with buttons
 */
	void syncFilter(boolean withMenu){
		if (withMenu){
			mthd.setPushed(methodItem.getChecked());
			sclass.setPushed(sclassItem.getChecked());
			imprt.setPushed(importItem.getChecked());
			attribute.setPushed(attributeItem.getChecked());
			interf.setPushed(interfaceItem.getChecked());
			constants.setPushed(constantItem.getChecked());
			field.setPushed(fieldItem.getChecked());
			structure.setPushed(structureItem.getChecked());
		}else{
			methodItem.setChecked(mthd.getPushed());
			sclassItem.setChecked(sclass.getPushed());
			importItem.setChecked(imprt.getPushed());
			attributeItem.setChecked(attribute.getPushed());
			interfaceItem.setChecked(interf.getPushed());
			constantItem.setChecked(constants.getPushed());
			fieldItem.setChecked(field.getPushed());
			structureItem.setChecked(structure.getPushed());
		}
	}
/**
 * Handles action events generated from selecting any of the menu items in the Filter
 * menu.  This method syncs the users selected options with the current state of the
 * filter buttons in the toolbar.
 * 
 * @param Object source - Control which sent the event.
 * @param Event e - Event which caused this action.
 */
	private void processFilterMenu(Object source, Event e){
		((MenuItem)source).setChecked(!((MenuItem)source).getChecked());
		syncFilter(true);
		filterTree(null,null);
	}
	/**
	 * NOTE: The following code is required by the Visual J++ form
	 * designer.  It can be modified using the form editor.  Do not
	 * modify it using the code editor.
	 */
	Container components = new Container();
	StatusBar statusBar1 = new StatusBar();
	RebarBand rebarBand1 = new RebarBand();
	Panel panel2 = new Panel();
	Splitter splitter1 = new Splitter();
	Panel panel1 = new Panel();
	MainMenu mainMenu1 = new MainMenu();
	MenuItem menuItem1 = new MenuItem();
	MenuItem menuItem2 = new MenuItem();
	MenuItem menuItem4 = new MenuItem();
	MenuItem menuItem5 = new MenuItem();
	Panel treePanel = new Panel();
	Splitter split = new Splitter();
	Panel panel3 = new Panel();
	Label treeLabel = new Label();
	Label gridLabel = new Label();
	Label richLabel = new Label();
	TreeView classTree = new TreeView();
	TreeView internalTree = new TreeView();
	ListView DataGrid1 = new ListView();
	ColumnHeader Description = new ColumnHeader();
	ColumnHeader Offset = new ColumnHeader();
	ColumnHeader Length = new ColumnHeader();
	ColumnHeader Value = new ColumnHeader();
	ColumnHeader Bytes = new ColumnHeader();
	ImageList imageList1 = new ImageList();
	MenuItem menuItem11 = new MenuItem();
	MenuItem fromFile = new MenuItem();
	Edit enterClassname = new Edit();
	Button button1 = new Button();
	Rebar rebar1 = new Rebar();
	Panel panel4 = new Panel();
	ToolBar toolBar1 = new ToolBar();
	ToolBarButton attribute = new ToolBarButton();
	ToolBarButton mthd = new ToolBarButton();
	ToolBarButton field = new ToolBarButton();
	ToolBarButton structure = new ToolBarButton();
	ToolBarButton imprt = new ToolBarButton();
	ToolBarButton sclass = new ToolBarButton();
	ToolBarButton constants = new ToolBarButton();
	ToolBarButton interf = new ToolBarButton();
	Label label1 = new Label();
	MenuItem menuItem12 = new MenuItem();
	MenuItem remove = new MenuItem();
	MenuItem menuItem13 = new MenuItem();
	MenuItem menuItem14 = new MenuItem();
	MenuItem menuItem15 = new MenuItem();
	MenuItem menuItem16 = new MenuItem();
	protected ContextMenu treePopup = new ContextMenu();
	ToolBar actionBar = new ToolBar();
	ToolBarButton decodeButton = new ToolBarButton();
	MenuItem menuItem17 = new MenuItem();
	ToolBarButton forward = new ToolBarButton();
	ToolBarButton find = new ToolBarButton();
	ToolBar toolBar2 = new ToolBar();
	ToolBarButton deleteB = new ToolBarButton();
	ToolBarButton back = new ToolBarButton();
	MenuItem importItem = new MenuItem();
	MenuItem sclassItem = new MenuItem();
	MenuItem attributeItem = new MenuItem();
	MenuItem interfaceItem = new MenuItem();
	MenuItem constantItem = new MenuItem();
	MenuItem methodItem = new MenuItem();
	MenuItem fieldItem = new MenuItem();
	MenuItem structureItem = new MenuItem();

	private void initForm()
	{
		// NOTE:  This form is storing resource information in an
		// external file.  Do not modify the string parameter to any
		// resources.getObject() function call. For example, do not
		// modify "foo1_location" in the following line of code
		// even if the name of the Foo object changes: 
		//   foo1.setLocation((Point)resources.getObject("foo1_location"));

		IResourceManager resources = new ResourceManager(this, "ClassDecoder");
		statusBar1.setBackColor(Color.CONTROL);
		statusBar1.setLocation((Point)resources.getObject("statusBar1_location"));
		statusBar1.setSize((Point)resources.getObject("statusBar1_size"));
		statusBar1.setTabIndex(resources.getInt("statusBar1_tabIndex"));
		statusBar1.setText(resources.getString("statusBar1_text"));

		panel2.setBackColor(Color.ACTIVEBORDER);
		panel2.setDock(ControlDock.BOTTOM);
		panel2.setLocation((Point)resources.getObject("panel2_location"));
		panel2.setSize((Point)resources.getObject("panel2_size"));
		panel2.setTabIndex(resources.getInt("panel2_tabIndex"));
		panel2.setText(resources.getString("panel2_text"));

		splitter1.setBackColor(Color.ACTIVEBORDER);
		splitter1.setCursor(Cursor.HSPLIT);
		splitter1.setDock(ControlDock.BOTTOM);
		splitter1.setLocation((Point)resources.getObject("splitter1_location"));
		splitter1.setSize((Point)resources.getObject("splitter1_size"));
		splitter1.setTabIndex(resources.getInt("splitter1_tabIndex"));
		splitter1.setTabStop(false);

		panel1.setBackColor(Color.ACTIVEBORDER);
		panel1.setDock(ControlDock.FILL);
		panel1.setSize((Point)resources.getObject("panel1_size"));
		panel1.setTabIndex(resources.getInt("panel1_tabIndex"));
		panel1.setText(resources.getString("panel1_text"));

		menuItem4.setText(resources.getString("menuItem4_text"));
		menuItem4.addOnClick(new EventHandler(this.exitMenuAction));

		treePanel.setDock(ControlDock.LEFT);
		treePanel.setLocation((Point)resources.getObject("treePanel_location"));
		treePanel.setSize((Point)resources.getObject("treePanel_size"));
		treePanel.setTabIndex(resources.getInt("treePanel_tabIndex"));
		treePanel.setText(resources.getString("treePanel_text"));

		split.setDock(ControlDock.LEFT);
		split.setLocation((Point)resources.getObject("split_location"));
		split.setSize((Point)resources.getObject("split_size"));
		split.setTabIndex(resources.getInt("split_tabIndex"));
		split.setTabStop(false);

		panel3.setBackColor(Color.ACTIVEBORDER);
		panel3.setDock(ControlDock.FILL);
		panel3.setLocation((Point)resources.getObject("panel3_location"));
		panel3.setSize((Point)resources.getObject("panel3_size"));
		panel3.setTabIndex(resources.getInt("panel3_tabIndex"));
		panel3.setText(resources.getString("panel3_text"));

		treeLabel.setDock(ControlDock.TOP);
		treeLabel.setSize((Point)resources.getObject("treeLabel_size"));
		treeLabel.setTabIndex(resources.getInt("treeLabel_tabIndex"));
		treeLabel.setTabStop(false);
		treeLabel.setText(resources.getString("treeLabel_text"));
		treeLabel.setBorderStyle(BorderStyle.FIXED_3D);

		gridLabel.setDock(ControlDock.TOP);
		gridLabel.setSize((Point)resources.getObject("gridLabel_size"));
		gridLabel.setTabIndex(resources.getInt("gridLabel_tabIndex"));
		gridLabel.setTabStop(false);
		gridLabel.setText(resources.getString("gridLabel_text"));
		gridLabel.setBorderStyle(BorderStyle.FIXED_3D);

		richLabel.setDock(ControlDock.TOP);
		richLabel.setSize((Point)resources.getObject("richLabel_size"));
		richLabel.setTabIndex(resources.getInt("richLabel_tabIndex"));
		richLabel.setTabStop(false);
		richLabel.setText(resources.getString("richLabel_text"));
		richLabel.setBorderStyle(BorderStyle.FIXED_3D);

		internalTree.setSize((Point)resources.getObject("internalTree_size"));
		internalTree.setTabIndex(resources.getInt("internalTree_tabIndex"));
		internalTree.setTabStop(false);
		internalTree.setText(resources.getString("internalTree_text"));
		internalTree.setVisible(resources.getBoolean("internalTree_visible"));
		internalTree.setIndent(19);

		Description.setText("Description");
		Description.setWidth(120);

		Offset.setText("Offset");

		Length.setText("Length");

		Value.setText("Value");
		Value.setWidth(170);

		Bytes.setText("Bytes");
		Bytes.setWidth(100);

		DataGrid1.setDock(ControlDock.FILL);
		DataGrid1.setLocation((Point)resources.getObject("DataGrid1_location"));
		DataGrid1.setSize((Point)resources.getObject("DataGrid1_size"));
		DataGrid1.setTabIndex(resources.getInt("DataGrid1_tabIndex"));
		DataGrid1.setText(resources.getString("DataGrid1_text"));
		DataGrid1.setAllowColumnReorder(true);
		DataGrid1.setFullRowSelect(true);
		DataGrid1.setHideSelection(false);
		DataGrid1.setHoverSelection(true);
		DataGrid1.setLabelWrap(false);
		DataGrid1.setView(ViewEnum.REPORT);
		DataGrid1.setColumns(new ColumnHeader[] {
							 Description, 
							 Offset, 
							 Length, 
							 Value, 
							 Bytes});

		imageList1.setMaskColor(Color.CONTROL);
		imageList1.setBackColor(Color.WHITE);
		imageList1.setImageStream((ImageListStreamer)resources.getObject("imageList1_imageStream"));
		/* @designTimeOnly imageList1.setLocation((Point)resources.getObject("imageList1_location")); */

		menuItem11.setShortcut(Shortcut.CTRL_SHIFT_C);
		menuItem11.setText(resources.getString("menuItem11_text"));
		menuItem11.addOnClick(new EventHandler(this.openFromClasspath));

		fromFile.setDefault(resources.getBoolean("fromFile_default"));
		fromFile.setShortcut(Shortcut.CTRL_SHIFT_O);
		fromFile.setText(resources.getString("fromFile_text"));
		fromFile.addOnClick(new EventHandler(this.openMenuAction));

		menuItem2.setMenuItems(new MenuItem[] {
							   menuItem11, 
							   fromFile});
		menuItem2.setText(resources.getString("menuItem2_text"));
		menuItem2.addOnClick(new EventHandler(this.openMenuAction));

		menuItem1.setMenuItems(new MenuItem[] {
							   menuItem2, 
							   menuItem4});
		menuItem1.setText(resources.getString("menuItem1_text"));

		enterClassname.setLocation((Point)resources.getObject("enterClassname_location"));
		enterClassname.setSize((Point)resources.getObject("enterClassname_size"));
		enterClassname.setTabIndex(resources.getInt("enterClassname_tabIndex"));
		enterClassname.setText(resources.getString("enterClassname_text"));
		enterClassname.setAcceptsReturn(false);

		button1.setSize((Point)resources.getObject("button1_size"));
		button1.setTabIndex(resources.getInt("button1_tabIndex"));
		button1.setTabStop(false);
		button1.setText(resources.getString("button1_text"));
		button1.addOnClick(new EventHandler(this.openFromClasspath));

		panel4.setBackColor(Color.ACTIVEBORDER);
		panel4.setDock(ControlDock.FILL);
		panel4.setLocation((Point)resources.getObject("panel4_location"));
		panel4.setSize((Point)resources.getObject("panel4_size"));
		panel4.setTabIndex(resources.getInt("panel4_tabIndex"));
		panel4.setText(resources.getString("panel4_text"));

		rebarBand1.setAllowVertical(false);
		rebarBand1.setBackColor(Color.ACTIVEBORDER);
		rebarBand1.setBandBreak(true);
		rebarBand1.setChildControl(panel4);
		rebarBand1.setChildEdge(false);
		rebarBand1.setForeColor(Color.LIGHTGRAY);
		rebarBand1.setIdealWidth(75);
		rebarBand1.setInitialHeight(25);
		rebarBand1.setMaxInitialHeight(108);
		rebarBand1.setMinChildHeight(24);
		rebarBand1.setWidth(75);

		rebar1.setBackColor(Color.ACTIVEBORDER);
		rebar1.setSize((Point)resources.getObject("rebar1_size"));
		rebar1.setTabIndex(resources.getInt("rebar1_tabIndex"));
		rebar1.setText(resources.getString("rebar1_text"));
		rebar1.setBands(new RebarBand[] {
						rebarBand1});
		rebar1.setFixedOrder(true);
		rebar1.setImageList(imageList1);

		attribute.setImageIndex(30);
		attribute.setStyle(ToolBarButtonStyle.TOGGLEBUTTON);
		attribute.setToolTipText(resources.getString("attribute_toolTipText"));

		mthd.setImageIndex(13);
		mthd.setStyle(ToolBarButtonStyle.TOGGLEBUTTON);
		mthd.setToolTipText(resources.getString("mthd_toolTipText"));

		field.setImageIndex(17);
		field.setStyle(ToolBarButtonStyle.TOGGLEBUTTON);
		field.setToolTipText(resources.getString("field_toolTipText"));

		structure.setImageIndex(31);
		structure.setStyle(ToolBarButtonStyle.TOGGLEBUTTON);
		structure.setToolTipText(resources.getString("structure_toolTipText"));

		imprt.setImageIndex(8);
		imprt.setStyle(ToolBarButtonStyle.TOGGLEBUTTON);
		imprt.setToolTipText(resources.getString("imprt_toolTipText"));

		sclass.setImageIndex(9);
		sclass.setStyle(ToolBarButtonStyle.TOGGLEBUTTON);
		sclass.setToolTipText(resources.getString("sclass_toolTipText"));

		constants.setImageIndex(32);
		constants.setStyle(ToolBarButtonStyle.TOGGLEBUTTON);
		constants.setToolTipText(resources.getString("constants_toolTipText"));

		interf.setImageIndex(5);
		interf.setStyle(ToolBarButtonStyle.TOGGLEBUTTON);
		interf.setToolTipText(resources.getString("interf_toolTipText"));

		toolBar1.setDock(ControlDock.NONE);
		toolBar1.setSize((Point)resources.getObject("toolBar1_size"));
		toolBar1.setTabIndex(resources.getInt("toolBar1_tabIndex"));
		toolBar1.setAppearance(ToolBarAppearance.FLAT);
		toolBar1.setButtons(new ToolBarButton[] {
							imprt, 
							sclass, 
							attribute, 
							interf, 
							constants, 
							mthd, 
							field, 
							structure});
		toolBar1.setDivider(false);
		toolBar1.setDropDownArrows(true);
		toolBar1.setImageList(imageList1);
		toolBar1.setShowToolTips(true);
		toolBar1.addOnButtonClick(new ToolBarButtonClickEventHandler(this.filterButtonEvent));

		label1.setBackColor(Color.ACTIVEBORDER);
		label1.setFont((Font)resources.getObject("label1_font"));
		label1.setForeColor(Color.CONTROLTEXT);
		label1.setLocation((Point)resources.getObject("label1_location"));
		label1.setSize((Point)resources.getObject("label1_size"));
		label1.setTabIndex(resources.getInt("label1_tabIndex"));
		label1.setTabStop(false);
		label1.setText(resources.getString("label1_text"));

		menuItem12.setShortcut(Shortcut.CTRL_F);
		menuItem12.setText(resources.getString("menuItem12_text"));
		menuItem12.addOnClick(new EventHandler(this.findMenuAction));

		remove.setShortcut(Shortcut.CTRL_R);
		remove.setText(resources.getString("remove_text"));

		menuItem13.setShortcut(Shortcut.F5);
		menuItem13.setText(resources.getString("menuItem13_text"));
		menuItem13.addOnClick(new EventHandler(this.processRefresh));

		menuItem5.setMenuItems(new MenuItem[] {
							   menuItem12, 
							   menuItem13});
		menuItem5.setText(resources.getString("menuItem5_text"));

		menuItem15.setShortcut(Shortcut.CTRL_B);
		menuItem15.setText(resources.getString("menuItem15_text"));
		menuItem15.addOnClick(new EventHandler(this.processBack));

		menuItem16.setShortcut(Shortcut.CTRL_G);
		menuItem16.setText(resources.getString("menuItem16_text"));
		menuItem16.addOnClick(new EventHandler(this.processForward));

		menuItem14.setMenuItems(new MenuItem[] {
								menuItem15, 
								menuItem16, 
								remove});
		menuItem14.setText(resources.getString("menuItem14_text"));

		treePopup.addOnPopup(new EventHandler(this.processTreePopup));
		/* @designTimeOnly treePopup.setLocation((Point)resources.getObject("treePopup_location")); */

		classTree.setContextMenu(treePopup);
		classTree.setDock(ControlDock.FILL);
		classTree.setLocation((Point)resources.getObject("classTree_location"));
		classTree.setSize((Point)resources.getObject("classTree_size"));
		classTree.setTabIndex(resources.getInt("classTree_tabIndex"));
		classTree.setText(resources.getString("classTree_text"));
		classTree.setImageIndex(3);
		classTree.setImageList(imageList1);
		classTree.setIndent(19);
		classTree.setSelectedImageIndex(3);
		classTree.addOnKeyUp(new KeyEventHandler(this.processKeyBinding));
		classTree.addOnAfterSelect(new TreeViewEventHandler(this.nodeSelected));

		decodeButton.setImageIndex(33);
		decodeButton.setToolTipText(resources.getString("decodeButton_toolTipText"));

		forward.setImageIndex(36);
		forward.setToolTipText(resources.getString("forward_toolTipText"));

		find.setImageIndex(34);
		find.setToolTipText(resources.getString("find_toolTipText"));

		actionBar.setDock(ControlDock.NONE);
		actionBar.setLocation((Point)resources.getObject("actionBar_location"));
		actionBar.setSize((Point)resources.getObject("actionBar_size"));
		actionBar.setTabIndex(resources.getInt("actionBar_tabIndex"));
		actionBar.setAppearance(ToolBarAppearance.FLAT);
		actionBar.setButtons(new ToolBarButton[] {
							 decodeButton, 
							 find});
		actionBar.setDivider(false);
		actionBar.setDropDownArrows(true);
		actionBar.setImageList(imageList1);
		actionBar.setWrappable(false);
		actionBar.setShowToolTips(true);
		actionBar.addOnButtonClick(new ToolBarButtonClickEventHandler(this.filterButtonEvent));

		deleteB.setImageIndex(37);
		deleteB.setToolTipText(resources.getString("deleteB_toolTipText"));

		back.setImageIndex(35);
		back.setToolTipText(resources.getString("back_toolTipText"));

		toolBar2.setDock(ControlDock.NONE);
		toolBar2.setLocation((Point)resources.getObject("toolBar2_location"));
		toolBar2.setSize((Point)resources.getObject("toolBar2_size"));
		toolBar2.setTabIndex(resources.getInt("toolBar2_tabIndex"));
		toolBar2.setAppearance(ToolBarAppearance.FLAT);
		toolBar2.setAutoSize(false);
		toolBar2.setButtons(new ToolBarButton[] {
							deleteB, 
							back, 
							forward});
		toolBar2.setDivider(false);
		toolBar2.setDropDownArrows(true);
		toolBar2.setImageList(imageList1);
		toolBar2.setWrappable(false);
		toolBar2.setShowToolTips(true);
		toolBar2.addOnButtonClick(new ToolBarButtonClickEventHandler(this.filterButtonEvent));

		importItem.setText(resources.getString("importItem_text"));
		importItem.addOnClick(new EventHandler(this.processFilterMenu));

		sclassItem.setText(resources.getString("sclassItem_text"));
		sclassItem.addOnClick(new EventHandler(this.processFilterMenu));

		attributeItem.setText(resources.getString("attributeItem_text"));
		attributeItem.addOnClick(new EventHandler(this.processFilterMenu));

		interfaceItem.setText(resources.getString("interfaceItem_text"));
		interfaceItem.addOnClick(new EventHandler(this.processFilterMenu));

		constantItem.setText(resources.getString("constantItem_text"));
		constantItem.addOnClick(new EventHandler(this.processFilterMenu));

		methodItem.setText(resources.getString("methodItem_text"));
		methodItem.addOnClick(new EventHandler(this.processFilterMenu));

		fieldItem.setText(resources.getString("fieldItem_text"));
		fieldItem.addOnClick(new EventHandler(this.processFilterMenu));

		structureItem.setText(resources.getString("structureItem_text"));
		structureItem.addOnClick(new EventHandler(this.processFilterMenu));

		menuItem17.setMenuItems(new MenuItem[] {
								importItem, 
								sclassItem, 
								attributeItem, 
								interfaceItem, 
								constantItem, 
								methodItem, 
								fieldItem, 
								structureItem});
		menuItem17.setRadioCheck(true);
		menuItem17.setText(resources.getString("menuItem17_text"));

		mainMenu1.setMenuItems(new MenuItem[] {
							   menuItem1, 
							   menuItem5, 
							   menuItem14, 
							   menuItem17});
		/* @designTimeOnly mainMenu1.setLocation((Point)resources.getObject("mainMenu1_location")); */

		this.setText(resources.getString("this_text"));
		this.setAcceptButton(button1);
		this.setAutoScaleBaseSize(new Point(5, 13));
		this.setClientSize((Point)resources.getObject("this_clientSize"));
		this.setIcon((Icon)resources.getObject("this_icon"));
		this.setIsMDIContainer(true);
		this.setMenu(mainMenu1);
		this.addOnKeyUp(new KeyEventHandler(this.processKeyBinding));
		/* @designTimeOnly this.setLocalizable(true); */

		this.setNewControls(new Control[] {
							button1, 
							panel1, 
							splitter1, 
							panel2, 
							statusBar1, 
							internalTree});
		panel2.setNewControls(new Control[] {
							  richLabel});
		panel1.setNewControls(new Control[] {
							  rebar1, 
							  treePanel, 
							  split, 
							  panel3});
		treePanel.setNewControls(new Control[] {
								 treeLabel, 
								 classTree});
		panel3.setNewControls(new Control[] {
							  gridLabel, 
							  DataGrid1});
		rebar1.setNewControls(new Control[] {
							  panel4});
		panel4.setNewControls(new Control[] {
							  actionBar, 
							  toolBar1, 
							  toolBar2, 
							  label1, 
							  enterClassname});
	}

	/**
	 * The main entry point for the application. 
	 *
	 * @param args Array of parameters passed to the application
	 * via the command line.
	 */
	public static void main(String args[]){
		Application.run(new ClassDecoder());
	}
}