/*
 *
 * sample.viewer.FindDialog.java
 *
 * (C) Copyright 1995 - 1999 Microsoft Corporation.  All rights reserved.
 *
*/
package sample.viewer;

import com.ms.wfc.app.*;
import com.ms.wfc.core.*;
import com.ms.wfc.ui.*;
import com.ms.wfc.html.*;

/**
 * Implements a Find Dialog that sends a specified application the user input through
 * the IFindListener interface.  Currently only one listener is allowed.
 * 
 * This class can take a variable number of parameters on the command
 * line. Program execution begins with the main() method. The class
 * constructor is not invoked unless an object of type 'FindDialog'
 * created in the main() method.
 * 
 * @see sample.viewer.IFindListener
 * 
 * @version 1.0a
 */
public class FindDialog extends Form{
	/**
	 * Application that will be relayed the information that the user enters into this
	 * form. 
	 */
	private IFindListener finder = null;
/**
 * This constructor forces the application to pass the object which will be receiving 
 * the information from this form.
 * 
 * @param IFindListener find - Object that will process the users input.
 */	
	public FindDialog(IFindListener find){
		super();
		finder = find;
		// Required for Visual J++ Form Designer support
		initForm();		
	}
/**
 * FindDialog overrides dispose so it can clean up the
 * component list.
 */
	public void dispose(){
		super.dispose();
		components.dispose();
	}
/**
 * Handles the FindNext event, causing the listening application to be updated.
 * 
 * @param Object source - Control which sent the event.
 * @param Event e - Event which caused this action.
 */
	private void processFindNext(Object source, Event e){
		finder.processFindNext();
	}
/**
 * Handles the Cancelt event, causing the listening application to be updated.
 * 
 * @param Object source - Control which sent the event.
 * @param Event e - Event which caused this action.
 */
	private void processCancel(Object source, Event e){
		finder.processFindCancel();
	}
/**
 * Handles the event related to toggling the 'whole word' option.  This will cause
 * the listening application to be updated with the state of the 'whole' checkbox.
 * 
 * @param Object source - Control which sent the event.
 * @param Event e - Event which caused this action.
 */
	private void processWholeStateChange(Object source, Event e){
		boolean state=true;
		finder.processFindWhole(whole.getChecked());
	}
/**
 * Handles the event related to toggling the 'case' option.  This will cause
 * the listening application to be updated with the state of the 'case' checkbox.
 * 
 * @param Object source - Control which sent the event.
 * @param Event e - Event which caused this action.
 */
	private void processCaseStateChange(Object source, Event e){
		boolean state = false;
		finder.processFindCase(this.checkBox2.getChecked());
	}
/**
 * Handles the TextChanged event, causing the listening application to be updated
 * with the contents of the Edit Control.
 * 
 * @param Object source - Control which sent the event.
 * @param Event e - Event which caused this action.
 */
	private void processTextChanged(Object source, Event e){
		finder.processFindText(((Edit)source).getText());
	}
/**
 * Handles the Key Binding events, causing the listening application to be updated
 * with the keys associated action.
 * 
 * @param Object source - Control which sent the event.
 * @param Event e - Event which caused this action.
 */
	private void processKeyBinding(Object source, KeyEvent e){
			if (e.getKeyCode()==114){//Handle F3 event.
				finder.processFindNext();
			}else if ((e.getControl())&&(e.getKeyCode()==67))//CTRL-C
				finder.processFindCancel();
			else if ((e.getAlt())&&(e.getKeyCode()==67))//ALT-C
				finder.processFindCase(this.checkBox2.getChecked());
			else if ((e.getAlt())&&(e.getKeyCode()==87))//ALT-W
				finder.processFindCase(this.whole.getChecked());
			else if ((e.getAlt())&&(e.getKeyCode()==70))//ALT-F
				finder.processFindNext();
	}
	/**
	 * NOTE: The following code is required by the Visual J++ form
	 * designer.  It can be modified using the form editor.  Do not
	 * modify it using the code editor.
	 */
	Container components = new Container();
	CheckBox whole = new CheckBox();
	CheckBox checkBox2 = new CheckBox();
	Edit edit1 = new Edit();
	Label label1 = new Label();
	Button findNext = new Button();
	Button cancel = new Button();

	private void initForm()
	{
		// NOTE:  This form is storing resource information in an
		// external file.  Do not modify the string parameter to any
		// resources.getObject() function call. For example, do not
		// modify "foo1_location" in the following line of code
		// even if the name of the Foo object changes: 
		//   foo1.setLocation((Point)resources.getObject("foo1_location"));

		IResourceManager resources = new ResourceManager(this, "FindDialog");
		whole.setLocation((Point)resources.getObject("whole_location"));
		whole.setSize((Point)resources.getObject("whole_size"));
		whole.setTabIndex(resources.getInt("whole_tabIndex"));
		whole.setText(resources.getString("whole_text"));
		whole.setVisible(resources.getBoolean("whole_visible"));
		whole.addOnKeyUp(new KeyEventHandler(this.processKeyBinding));
		whole.addOnCheckStateChanged(new EventHandler(this.processWholeStateChange));

		checkBox2.setLocation((Point)resources.getObject("checkBox2_location"));
		checkBox2.setSize((Point)resources.getObject("checkBox2_size"));
		checkBox2.setTabIndex(resources.getInt("checkBox2_tabIndex"));
		checkBox2.setText(resources.getString("checkBox2_text"));
		checkBox2.setVisible(resources.getBoolean("checkBox2_visible"));
		checkBox2.addOnKeyUp(new KeyEventHandler(this.processKeyBinding));
		checkBox2.addOnCheckStateChanged(new EventHandler(this.processCaseStateChange));

		edit1.setAllowDrop(true);
		edit1.setLocation((Point)resources.getObject("edit1_location"));
		edit1.setSize((Point)resources.getObject("edit1_size"));
		edit1.setTabIndex(resources.getInt("edit1_tabIndex"));
		edit1.setText(resources.getString("edit1_text"));
		edit1.addOnTextChanged(new EventHandler(this.processTextChanged));

		label1.setLocation((Point)resources.getObject("label1_location"));
		label1.setSize((Point)resources.getObject("label1_size"));
		label1.setTabIndex(resources.getInt("label1_tabIndex"));
		label1.setTabStop(false);
		label1.setText(resources.getString("label1_text"));

		findNext.setLocation((Point)resources.getObject("findNext_location"));
		findNext.setSize((Point)resources.getObject("findNext_size"));
		findNext.setTabIndex(resources.getInt("findNext_tabIndex"));
		findNext.setText(resources.getString("findNext_text"));
		findNext.addOnClick(new EventHandler(this.processFindNext));
		findNext.addOnKeyUp(new KeyEventHandler(this.processKeyBinding));

		cancel.setLocation((Point)resources.getObject("cancel_location"));
		cancel.setSize((Point)resources.getObject("cancel_size"));
		cancel.setTabIndex(resources.getInt("cancel_tabIndex"));
		cancel.setText(resources.getString("cancel_text"));
		cancel.addOnClick(new EventHandler(this.processCancel));
		cancel.addOnKeyUp(new KeyEventHandler(this.processKeyBinding));

		this.setAllowDrop(true);
		this.setText(resources.getString("this_text"));
		this.setAutoScaleBaseSize(new Point(5, 13));
		this.setCancelButton(cancel);
		this.setClientSize((Point)resources.getObject("this_clientSize"));
		this.setKeyPreview(true);
		this.setMaximizeBox(resources.getBoolean("this_maximizeBox"));
		this.setMinimizeBox(resources.getBoolean("this_minimizeBox"));
		this.setShowInTaskbar(false);
		this.setStartPosition(FormStartPosition.CENTER_SCREEN);
		this.setTopMost(true);
		this.addOnKeyUp(new KeyEventHandler(this.processKeyBinding));
		/* @designTimeOnly this.setLocalizable(true); */

		this.setNewControls(new Control[] {
							cancel, 
							findNext, 
							label1, 
							edit1, 
							checkBox2, 
							whole});
	}
}