/*
 *
 * sample.viewer.LinkEvent.java
 *
 * (C) Copyright 1995 - 1999 Microsoft Corporation.  All rights reserved.
 *
*/
package sample.viewer;
/**
 * This event is created when an EN_LINK notification is received by the RichEdir201
 * Control.  The contents of the ENLINK structure are used to create this object, 
 * which also contains some helper methods for determining state.
 * 
 * @see sample.viewer.ENLINK
 * @see sample.viewer.LinkEventHandler
 * @see sample.viewer.RichEdit201
 * 
 * @version 1.0a
 */
public class LinkEvent extends com.ms.wfc.core.Event{
	/**
	 * This is the start of the area which has been marked as a link, and is the 
	 * subject of this event.
	 */
	public int start=-1;
	/**
	 * The end point of the selected area that has been marked as a link.
	 */
	public int end = -1;
	/**
	 * Win32 Message ID.
	 */
	public int id=0;
/**
 * This method analyzes the Win32 Message ID (id) to determine if this event was sent
 * as a result of the user performing a left mouse button double click
 * 
 * @return boolean - true if a left mouse button double click occured.
 */
	public boolean isLDoubleClick(){
		if (id==com.ms.win32.win.WM_LBUTTONDBLCLK)
			return true;
		else
			return false;
	}
/**
 * This method analyzes the Win32 Message ID (id) to determine if this event was sent
 * as a result of the user performing a right mouse button double click
 * 
 * @return boolean - true if a right mouse button double click occured.
 */
	public boolean isRDoubleClick(){
		if (id==com.ms.win32.win.WM_RBUTTONDBLCLK)
			return true;
		else
			return false;
	}
/**
 * This method analyzes the Win32 Message ID (id) to determine if this event was sent
 * as a result of the user performing a left mouse button down happened.
 * 
 * @return boolean - true if a left mouse button down occured.
 */	
	public boolean isLMouseDown(){
		if (id==com.ms.win32.win.WM_LBUTTONDOWN)
			return true;
		else
			return false;
	}
/**
 * This method analyzes the Win32 Message ID (id) to determine if this event was sent
 * as a result of the user performing a right mouse button down happened.
 * 
 * @return boolean - true if a right mouse button down occured.
 */
	public boolean isRMouseDown(){
		if (id==com.ms.win32.win.WM_RBUTTONDOWN)
			return true;
		else
			return false;
	}
/**
 * This method analyzes the Win32 Message ID (id) to determine if this event was sent
 * as a result of the user performing a left mouse button up happened.
 * 
 * @return boolean - true if a left mouse button up occured.
 */
	public boolean isLMouseUp(){
		if (id==com.ms.win32.win.WM_LBUTTONUP)
			return true;
		else
			return false;
	}
/**
 * This method analyzes the Win32 Message ID (id) to determine if this event was sent
 * as a result of the user performing a right mouse button up happened.
 * 
 * @return boolean - true if a right mouse button up occured.
 */
	public boolean isRMouseUp(){
		if (id==com.ms.win32.win.WM_RBUTTONUP)
			return true;
		else
			return false;
	}
/**
 * This method analyzes the Win32 Message ID (id) to determine if this event was sent
 * as a result of a mouse movement.
 * 
 * @return boolean - true if a mouse movement occured.
 */
	public boolean isMouseMove(){
		if (id==com.ms.win32.win.WM_MOUSEMOVE)
			return true;
		else
			return false;
	}
/**
 * Overrides java.lang.Object toString method to produce meaningful information 
 * about the range of text that this event covers, and what type of windows message
 * was sent.
 * 
 * @return String - String describing this event.
 */
	public String toString(){
		String p = "["+id+"] "+"("+start+","+ end+") ";	
		if (isLDoubleClick())p+="LDouble Click ";
		if (isRDoubleClick())p+="RDouble Click ";
		if (isLMouseDown())p+="LMouse Down ";
		if (isRMouseDown())p+="RMouse Down ";
		if (isLMouseUp())p+="LMouse Up ";
		if (isRMouseUp())p+="RMouse Up ";
		if (isMouseMove())p+="MouseMove ";
		return p;
	}
}