/*
 * sample.win32.Advapi32.java
 *
 * (C) Copyright 1995 - 1999 Microsoft Corporation.  All rights reserved.
 *
 */
package sample.win32;
/**
 * This is a 'wrapper' class that contains J/Direct calls to the appropriate 
 * Crypto API functions required to get the encrypt/decrypt, verify/sign streams
 * working.  Rather than exposing the functions exactly as they are in the Win32
 * API, javatized funtions which throw com.ms.dll.Win32Exception (instead of returning 
 * booleans) are used.  Mapping of data from a value to a pointer is also taken 
 * care of by these java methods.  All of the methods exposed by this class are 
 * from the advapi32.dll.
 * 
 * @see sample.crypto.DecryptStream
 * @see sample.crypto.EncryptStream
 * @see sample.crypto.SignStream
 * @see sample.crypto.VerifyStream
 * 
 * @dll.import("advapi32.dll",setLastError)
 */
public class Advapi32{
/**
 * The CryptAcquireContext function is used to acquire a handle to a particular key 
 * container within a particular CSP. This returned handle can then be used to make 
 * calls to the selected CSP.
 * 
 * This function performs two operations. It first attempts to find a CSP with the 
 * characteristics described in the dwProvType and pszProvider parameters. If the 
 * CSP is found, then the function attempts to find a key container within the CSP 
 * matching the name specified by the pszContainer parameter.
 * 
 * This function can also be used to create and destroy key containers, depending 
 * on the value of the dwFlags parameter.
 * 
 * @param int phProv - Address to which the function copies a handle to the CSP. 
 * 
 * @param String pszContainer - Key container name. This is a zero-terminated string
 *      that identifies the key container to the CSP. This name is independent of 
 *      the method used to store the keys. Some CSPs will store their key containers
 *      internally (in hardware), some will use the system registry, and others will
 *      use the file system. pszContainer is to be set to NULL when dwFlags is set 
 *      to CRYPT_VERIFYCONTEXT. 
 * 
 *      If this parameter is NULL, then a default key container name will be used. 
 *      For example, if the Microsoft Base Cryptographic Provider is being used, 
 *      then the current user's logon name will be used as the name of the key 
 *      container. Other CSPs may also have default key containers that can be 
 *      acquired in this way. 
 * 
 *      An application can obtain the name of the acquired key container at a later 
 *      time by reading the PP_CONTAINER CSP parameter with the CryptGetProvParam 
 *      function. 
 * 
 * @param String pszProvider - Provider name. This is a zero-terminated string that 
 *      specifies the CSP to be used. If this parameter is NULL then the user default
 *      provider is used.
 * 
 *      An application can obtain the name of the acquired CSP at a later time by 
 *      reading the PP_NAME CSP parameter with the getProvParam function. 
 *
 *  @param int dwProvType - Type of provider to acquire. The following provider types
 *      are predefined, and are discussed in detail in Interfacing with a 
 *      Cryptographic Service Provider (CSP). 
 * 
 *      PROV_RSA_FULL 
 *      PROV_RSA_SIG 
 *      PROV_DSS 
 *      PROV_DSS_DH 
 *      PROV_DH_SCHANNEL 
 *      PROV_FORTEZZA 
 *      PROV_MS_EXCHANGE 
 *      PROV_RSA_SCHANNEL
 *      PROV_SSL 
 * 
 * @param dwFlags - Flag values. This parameter is normally set to zero, but some 
 *      applications will set one or more of the following flags:
 * 
 *      CRYPT_VERIFYCONTEXT.        If this flag is set, then the application will 
 *                                  have no access to the private keys and the 
 *                                  pszContainer parameter must be set to NULL.
 * 
 *                                  This option is intended to be used with 
 *                                  applications that will not be using private keys.
 * 
 *                                  When CryptAcquireContext is called, many CSPs 
 *                                  will require input from the owning user before 
 *                                  granting access to the private keys in the key 
 *                                  container. For example, the private keys may be 
 *                                  encrypted, requiring a password from the user 
 *                                  before they can be used. However, if the 
 *                                  CRYPT_VERIFYCONTEXT flag is specified, access to
 *                                  the private keys is not required and the user 
 *                                  interface can be bypassed. 
 * 
 *      CRYPT_NEWKEYSET.            If this flag is set, then a new key container 
 *                                  will be created with the name specified by 
 *                                  pszContainer. If pszContainer is NULL, then a 
 *                                  key container with the default name will be 
 *                                  created. See CRYPT_MACHINE_KEYSET for 
 *                                  information on combining flags. 
 * 
 *      CRYPT_MACHINE_KEYSET.       By default, keys are stored in the 
 *                                  HKEY_CURRENT_USER portion of the registry. The 
 *                                  CRYPT_MACHINE_KEYSET flag can be combined with 
 *                                  all of the other flags, indicating that the 
 *                                  location for the key of interest is HKEY_LOCAL_
 *                                  MACHINE. When combined with the CRYPT_NEWKEYSET 
 *                                  flag, the CRYPT_MACHINE_KEYSET flag is useful 
 *                                  when access is being performed from a service or
 *                                  user account that did not log on interactively. 
 *                                  This combination enables access to user specific
 *                                  keys under HKEY_LOCAL_MACHINE. 
 * 
 *                                  Note  When key containers are created, most CSPs
 *                                  will not automatically create any public/private
 *                                  key pairs. These keys must be created as a 
 *                                  separate step with the CryptGenKey function. 
 * 
 *      CRYPT_DELETEKEYSET.         If this flag is set, then the key container 
 *                                  specified by pszContainer is deleted. If 
 *                                  pszContainer is NULL, then the key container 
 *                                  with the default name is deleted. All key pairs 
 *                                  in the key container are also destroyed.
 * 
 *                                  When the CRYPT_DELETEKEYSET flag is set, the 
 *                                  value returned in phProv is undefined and, thus,
 *                                  the releaseContext function need not be 
 *                                  called afterwards. 
 * 
 * @return int - This is the handle to the CSP.
 * 
 * @exception java.lang.com.ms.dll.Win32Exception - If it does not succeed this exception is
 *          thrown which contins the error code from GetLastError.
 * 
 * The following table lists the error codes most commonly thrown.
 * 
 *      Error code                      Description 
 *
 *      ERROR_INVALID_PARAMETER     One of the parameters contains an invalid value.
 *                                  This is most often an illegal pointer. 
 * 
 *      ERROR_NOT_ENOUGH_MEMORY     The operating system ran out of memory during 
 *                                  the operation. 
 * 
 *      NTE_BAD_FLAGS               The dwFlags parameter has an illegal value. 
 * 
 *      NTE_BAD_KEYSET              The registry entry for the key container could 
 *                                  not be opened and may not exist. 
 * 
 *      NTE_BAD_KEYSET_PARAM        The pszContainer or pszProvider parameter is set
 *                                  to an illegal value.
 * 
 *      NTE_BAD_PROV_TYPE           The value of the dwProvType parameter is out of 
 *                                  range. All provider types must be from 1 to 999,
 *                                  inclusive. 
 * 
 *      NTE_BAD_SIGNATURE           The provider DLL signature could not be verified.
 *                                  Either the DLL or the digital signature has been
 *                                  tampered with.
 * 
 *      NTE_EXISTS                  The dwFlags parameter is CRYPT_NEWKEYSET, but 
 *                                  the key container already exists. 
 * 
 *      NTE_KEYSET_ENTRY_BAD        The registry entry for the pszContainer key 
 *                                  container was found (in the HKEY_CURRENT_USER 
 *                                  window), but is corrupt. See System 
 *                                  Administration for details about the CryptoAPI 
 *                                  registry usage. 
 * 
 *      NTE_KEYSET_NOT_DEF          No registry entry exists in the 
 *                                  HKEY_CURRENT_USER window for the key container 
 *                                  specified by pszContainer. 
 * 
 *      NTE_NO_MEMORY               The CSP ran out of memory during the operation. 
 * 
 *      NTE_PROV_DLL_NOT_FOUND      The provider DLL file does not exist or is not 
 *                                  on the current path. 
 * 
 *      NTE_PROV_TYPE_ENTRY_BAD     The registry entry for the provider type 
 *                                  specified by dwProvType is corrupt. This error 
 *                                  may relate to either the user default CSP list 
 *                                  or the machine default CSP list. See System 
 *                                  Administration for details about the CryptoAPI 
 *                                  registry usage. 
 * 
 *      NTE_PROV_TYPE_NO_MATCH      The provider type specified by dwProvType does 
 *                                  not match the provider type found in the 
 *                                  registry. Note that this error can only occur 
 *                                  when pszProvider specifies an actual CSP name. 
 * 
 *      NTE_PROV_TYPE_NOT_DEF       No registry entry exists for the provider type 
 *                                  specified by dwProvType. 
 * 
 *      NTE_PROVIDER_DLL_FAIL       The provider DLL file could not be loaded, and 
 *                                  may not exist. If it exists, then the file is 
 *                                  not a valid DLL. 
 * 
 *      NTE_SIGNATURE_FILE_BAD      An error occurred while loading the DLL file 
 *                                  image, prior to verifying its signature. 
 */
    public synchronized static int acquireContext(
                                     String pszContainer,
                                     String pszProvider,
                                     int dwProvType,
                                     int dwFlags){
        int[] phProv = new int[1];  
        if(!CryptAcquireContext(phProv,
                            pszContainer,
                            pszProvider,
                            dwProvType,
                            dwFlags))
            throw new com.ms.dll.Win32Exception(com.ms.dll.DllLib.getLastWin32Error());
        return phProv[0];
    }
/**
 * @see #acquireContext
 */
    protected static native boolean CryptAcquireContext(
                                    int[] phProv,
                                    String pszContainer, 
                                    String pszProvider,  
                                    int dwProvType, 
                                    int dwFlags);
/**
 * The CryptCreateHash function is used to initiate the hashing of a stream of data.
 * It returns to the caller a handle to a CSP hash object. This handle can also be 
 * used in subsequent calls to CryptHashData and CryptHashSessionKey in order to 
 * hash streams of data and session keys.
 * 
 * The Microsoft Base Cryptographic Provider defines the following hashing 
 * algorithms.
 * 
 *      Constant            Description 
 * 
 *      CALG_HMAC HMAC      a keyed hash algorithm  
 *      CALG_MAC            Message Authentication Code 
 *      CALG_MD2 MD2
 *      CALG_MD5 MD5
 *      CALG_SHA US DSA     Secure Hash Algorithm 
 *      CALG_SHA1           Same as CALG_SHA 
 *      CALG_SSL3_SHAMD5    SSL3 client authentication 
 * 
 * The computation of the actual hash is done with the HashData and HashSessionKey 
 * methods. These require a handle to the hash object. After all the data has been 
 * added to the hash object, any of the following operations can be performed: 
 * 
 *  -   The hash value can be retrieved by using getHashParam. 
 *      
 *  -   A session key can be derived by using deriveKey. 
 *      
 *  -   The hash can be signed by using signHash. 
 *      
 *  -   A signature can be verified by using verifySignature. 
 *      
 *  -   After one of the functions from this list has been called, hashData and 
 *          hashSessionKey may not be called.

 * @param int hProv - Handle to the CSP to use. An application obtains this handle 
 *      by using the acquireContext function. 
 * 
 * @param int Algid - Algorithm identifier of the hash algorithm to use. 
 *      The valid values for this parameter will vary, depending on the CSP that is 
 *      used. See the "Remarks" section for the list of default algorithms. 
 * 
 * @param int hKey - If the type of hash algorithm is a keyed hash, such as the HMAC
 *      or MAC algorithm, the key for the hash should be passed in this parameter. 
 *      For nonkeyed algorithms, this parameter should be set to zero.
 * 
 *      For keyed algorithms, the key must be to a block cipher, such as RC2, that 
 *      has a cipher mode of CBC. 
 * 
 * @return int - A handle to the new hash object. 
 * 
 * @exception java.lang.com.ms.dll.Win32Exception - If it fails, this exception is thrown 
 *      with error information from the GetLastError function.
 * 
 *      The following table lists the error codes most commonly thrown. The error 
 *      codes prefaced by "NTE" are generated by the particular CSP you are using.
 * 
 *          Error code              Description 
 * 
 *      ERROR_INVALID_HANDLE        One of the parameters specifies an invalid 
 *                                  handle.
 * 
 *      ERROR_INVALID_PARAMETER     One of the parameters contains an invalid value.
 *                                  This is most often an illegal pointer. 
 * 
 *      ERROR_NOT_ENOUGH_MEMORY     The operating system ran out of memory during 
 *                                  the operation.
 * 
 *      NTE_BAD_ALGID               The Algid parameter specifies an algorithm that 
 *                                  this CSP does not support. 
 * 
 *      NTE_BAD_FLAGS               The dwFlags parameter is nonzero. 
 * 
 *      NTE_BAD_KEY                 A keyed hash algorithm (such as CALG_MAC) is 
 *                                  specified by Algid and the hKey parameter is 
 *                                  either zero or it specifies an invalid key 
 *                                  handle. This error code will also be returned 
 *                                  if the key is to a stream cipher, or if the 
 *                                  cipher mode is anything other than CBC. 
 * 
 *      NTE_NO_MEMORY               The CSP ran out of memory during the operation.
 */
    public synchronized static int createHash(int hProv,
                            int Algid,
                            int hKey,
                            int dwFlags){
        int[] phHash = new int[1];
        if(!CryptCreateHash(hProv,
                            Algid,
                            hKey,
                            dwFlags,
                            phHash))
                throw new com.ms.dll.Win32Exception(com.ms.dll.DllLib.getLastWin32Error());
        return phHash[0];
        
    }
/**
 * @see #createHash
 */
    protected static native boolean CryptCreateHash(
                                    int hProv,
                                    int Algid,
                                    int hKey,
                                    int dwFlags,
                                    int[] phHash);
/**
 * The decrypt function is used to decrypt data that was previously encrypted 
 * by using the CryptEncrypt function. 
 * 
 * Important changes have been made to the CryptoAPI in order to support S/MIME 
 * e-mail interoperability, which affect the handling of enveloped messages.
 * 
 * Due to French import laws, the Microsoft CSPs do not allow encryption when the 
 * Microsoft Windows NT operating system is used in France. Therefore, under this 
 * condition, this function will fail with the error NTE_PERM. 
 * 
 * If data is to be decrypted and hashed simultaneously, a handle to a hash object 
 * can be passed in the hHash parameter. The hash value will be updated with the 
 * decrypted plaintext. This option is useful when simultaneously decrypting and 
 * verifying a signature.
 * 
 * Before calling decrypt, the application should obtain a handle to the hash 
 * object by calling the CryptCreateHash function. After the decryption is complete,
 * the hash value can be obtained (through getHashParam), it can be signed 
 * (through signHash), or it can be used to verify a digital signature (through
 * verifySignature).
 * 
 * When a large amount of data needs to be decrypted, it can be done in sections. 
 * This is done by calling CryptDecrypt repeatedly. The Final parameter should be 
 * set to TRUE only on the last invocation of decrypt, so the decryption engine can 
 * properly finish the decryption process. The following extra actions are performed
 *  when Final is TRUE: 
 * 
 *      - If the key is a block cipher key, the data will be padded to a multiple of
 *          the block size of the cipher. To find the block size of a cipher, use 
 *          getKeyParam to get the KP_BLOCKLEN parameter of the key. 
 * 
 *      - If the cipher is operating in a chaining mode, the next decrypt operation 
 *          will reset the cipher's feedback register to the KP_IV value of the key.
 * 
 *      - If the cipher is a stream cipher, the next decrypt call will reset the 
 *          cipher to its initial state. 
 * 
 * 
 * @param int hKey - A handle to the key to use for the decryption. An application 
 *      obtains this handle by using either the CryptGenKey or CryptImportKey 
 *      function. 
 * 
 *      This key specifies the decryption algorithm that is used.
 * 
 * @param int hHash - A handle to a hash object. This parameter is used only if a 
 *      hash of the data is to be computed. See the "Remarks" section for more 
 *      information.
 * 
 *      If no hash is to be done, this parameter must be zero. 
 * 
 * @param boolean Final - The Boolean value that specifies whether this is the last 
 *      section in a series being decrypted. This will be TRUE if this is the last 
 *      or only block. If it is not, then it will be FALSE. See the "Remarks" 
 *      section for more information. 
 * 
 *  @param byte[] pbData - The buffer holding the data to be decrypted. After the 
 *      decryption has been performed, the plaintext is placed back in this same 
 *      buffer. 
 * 
 *      The number of encrypted bytes in this buffer is specified by pcbData. 
 * 
 * @param int pcbData - Before calling this function, the caller should set this 
 *      parameter to the number of bytes to be decrypted. Upon return, this address 
 *      will contain the number of bytes of plaintext generated.
 * 
 *      When a block cipher is used, this data length must be a multiple of the 
 *      block size, unless this is the final section of data to be decrypted and 
 *      the Final flag is TRUE. 
 * 
 * @return int - the length of the encrypted data (stored in the pbData buffer).
 * 
 * @exception java.lang.com.ms.dll.Win32Exception - If it fails, this exceptoin is thrown
 *      with the errorcode from the GetLastError function.
 * 
 *      The following table lists the error codes most commonly returned by the 
 *      GetLastError function. The error codes prefaced by "NTE" are generated by 
 *      the particular CSP you are using.
 * 
 *      Error code                  Description 
 * 
 *      ERROR_INVALID_HANDLE        One of the parameters specifies an invalid 
 *                                  handle.
 * 
 *      ERROR_INVALID_PARAMETER     One of the parameters contains an invalid value.
 *                                  This is most often an illegal pointer. 
 * 
 *      NTE_BAD_ALGID               The hKey session key specifies an algorithm that
 *                                  this CSP does not support. 
 * 
 *      NTE_BAD_DATA                The data to be decrypted is invalid. For example,
 *                                  when a block cipher is used and the Final flag 
 *                                  is FALSE, the value specified by pcbData must be 
 *                                  a multiple of the block size. This error can 
 *                                  also be returned when the padding is found to be
 *                                  invalid. 
 * 
 *      NTE_BAD_FLAGS               The dwFlags parameter is nonzero. 
 * 
 *      NTE_BAD_HASH                The hHash parameter contains an invalid handle.
 * 
 *      NTE_BAD_KEY                 The hKey parameter does not contain a valid 
 *                                  handle to a key. 
 * 
 *      NTE_BAD_LEN                 The size of the output buffer is too small to 
 *                                  hold the generated plaintext. 
 * 
 *      NTE_BAD_UID                 The CSP context that was specified when the key 
 *                                  was created cannot be found. 
 * 
 *      NTE_DOUBLE_ENCRYPT          The application attempted to decrypt the same 
 *                                  data twice. 
 * 
 *      NTE_FAIL                    The function failed in some unexpected way. 
 * 
 * 
 */
    public static synchronized int decrypt(
                                    int hKey,
                                    int hHash,
                                    boolean Final,
                                    int dwFlags,
                                    byte[] bData,
                                    int cbData){
        int[] pcbData = new int[]{cbData};
        if (!CryptDecrypt(hKey,hHash,Final,dwFlags,bData,pcbData)){
                throw new com.ms.dll.Win32Exception(com.ms.dll.DllLib.getLastWin32Error());

        }
        return pcbData[0];
        
    }
/**
 * @see #decrypt
 */ 
    protected static native boolean CryptDecrypt(
                                    int hKey,
                                    int hHash, 
                                    boolean Final,
                                    int dwFlags,
                                    byte[] bData,
                                    int[] pcbData);
/**
 * The deriveKey function generates cryptographic keys derived from base data. 
 * This function guarantees that all keys generated from the same base data will be 
 * identical, provided the same CSP and algorithms are used. The base data can be a 
 * password or any other user data.
 * 
 * This function is the same as genKey, except that the generated session keys are 
 * derived from base data instead of being random. Another difference is that the 
 * deriveKey function cannot be used to generate public/private key pairs.
 * 
 * A handle to the session key is returned in phKey. This handle can then be used 
 * as needed with any of the other CryptoAPI functions that require key handles.
 * 
 * To generate a key for a symmetric encryption algorithm, use the Algid parameter 
 * to specify the algorithm. The algorithms available will most likely be different 
 * for each CSP. If you are using the Microsoft Base Cryptographic Provider, use one
 * of the following values to specify the algorithm.
 * 
 *      Value               Description 
 * 
 *      CALG_RC2            RC2 block cipher 
 * 
 *      CALG_RC4            RC4 stream cipher 
 * 
 * When keys are generated for symmetric block ciphers, the key by default will be 
 * set up in cipher block chaining (CBC) mode with an initialization vector of zero.
 * This cipher mode provides a good default method for bulk-encrypting data. To 
 * change these parameters, use the CryptSetKeyParam function.
 * 
 * After the CryptDeriveKey function has been called, no more data can be added to 
 * the hash object. The CryptDestroyHash function should be called at this point to 
 * destroy the hash object.
 * 
 * 
 * @param int hProv - A handle to the application's CSP. An application obtains this
 *      handle by using the CryptAcquireContext function. 
 * 
 * @param int Algid - The identifier for the algorithm for which the key is to be 
 *      generated. The valid values for this parameter will vary, depending on the 
 *      CSP that is used. See the "Remarks" section for a list of possible algorithm
 *      identifiers. 
 * 
 * @param int hBaseData - A handle to a hash object that has been fed the exact base
 *      data.  To obtain this handle, an application must first create a hash object
 *      with createHash and then add the base data to the hash object with hashData.
 *      This process is described in detail in Hashes and Digital Signatures. 
 * 
 * @param int dwFlags - The flags specifying the type of key generated. This 
 *      parameter can be zero, or you can specify one or more of the following flags 
 *      by using the bitwise OR operator to combine them: 
 * 
 *      CRYPT_EXPORTABLE.       If this flag is set, then the session key can be 
 *                              transferred out of the CSP into a key blob through 
 *                              the exportKey function. Because keys generally must 
 *                              be exportable, this flag should usually be set.
 * 
 *                              If this flag is not set, then the session key will 
 *                              not be exportable. This means the key will be 
 *                              available only within the current session and only 
 *                              the application that created it will be able to use 
 *                              it. 
 * 
 *                              This flag does not apply to public/private key pairs.
 * 
 *      CRYPT_CREATE_SALT.      Typically, when a session key is made from a hash 
 *                              value, there are a number of leftover bits. For 
 *                              example, if the hash value is 128 bits and the 
 *                              session key is 40 bits, there will be 88 bits left 
 *                              over.
 * 
 *                              If this flag is set, then the key will be assigned a
 *                              salt value based on the unused hash value bits. You 
 *                              can retrieve this salt value by using the 
 *                              getKeyParam function with the dwParam parameter set 
 *                              to KP_SALT. 
 * 
 *                              If this flag is not set, then the key will be given 
 *                              a salt value of zero. 
 * 
 *                              When keys with nonzero salt values are exported (by 
 *                              using eExportKey), the salt value must also be 
 *                              obtained and kept with the key blob. 
 * 
 *      CRYPT_NO_SALT.          Specifies that a no salt value gets allocated for a 
 *                              40-bit symmetric key. For more information, see 
 *                              Microsoft Cryptographic ProvidersRelease Notes. 
 * 
 *      CRYPT_USER_PROTECTED.   If this flag is set, then the user will be notified 
 *                              through a dialog box or another method when certain 
 *                              actions are attempted by using this key. The precise 
 *                              behavior is specified by the CSP being used. Prior 
 *                              to Microsoft Internet Explorer 4.0, Microsoft 
 *                              Cryptographic Providers ignore this flag. Starting 
 *                              with Microsoft Internet Explorer 4.0, Microsoft 
 *                              Providers support this flag value. 
 * 
 *      CRYPT_UPDATE_KEY.       Some CSPs use session keys that are derived from 
 *                              multiple hash values. When this is the case, 
 *                              deriveKey must be called multiple times.
 * 
 *                              If this flag is set, a new session key is not 
 *                              generated. Instead, the key specified by phKey is 
 *                              modified. The precise behavior of this flag is 
 *                              dependent on the type of key being generated and on 
 *                              the particular CSP being used. 
 * 
 *                              The Microsoft Cryptographic Providers ignore this 
 *                              flag.
 * 
 * @return int - The handle of the newly generated key. 
 * 
 * @exception java.lang.com.ms.dll.Win32Exception - If it fails, this exception is thrown
 *      which contains the value from the GetLastError function.
 * 
 *  The following table lists the error codes most commonly returned by the 
 *  GetLastError function. The error codes prefaced by "NTE" are generated by the 
 *  particular CSP you are using.
 * 
 *      Error code                  Description 
 * 
 *      ERROR_INVALID_HANDLE        One of the parameters specifies an invalid handle.
 *
 *      ERROR_INVALID_PARAMETER     One of the parameters contains an invalid value.
 *                                  This is most often an illegal pointer. 
 *
 *      NTE_BAD_ALGID               The Algid parameter specifies an algorithm that 
 *                                  this CSP does not support. 
 * 
 *      NTE_BAD_FLAGS               The dwFlags parameter contains an invalid value. 
 * 
 *      NTE_BAD_HASH                The hBaseData parameter does not contain a valid 
 *                                  handle to a hash object. 
 * 
 *      NTE_BAD_HASH_STATE          An attempt was made to add data to a hash object
 *                                  that is already marked "finished." 
 * 
 *      NTE_BAD_UID                 The hProv parameter does not contain a valid 
 *                                  context handle. 
 * 
 *      NTE_FAIL                    The function failed in some unexpected way. 
 * 
 */
    public static synchronized int deriveKey(
                                    int hProv,
                                    int Algid,
                                    int hBaseData,
                                    int dwFlags){
        int[] phKey = new int[1];
        if (!CryptDeriveKey(hProv,Algid,hBaseData,dwFlags,phKey)){
            throw new com.ms.dll.Win32Exception(com.ms.dll.DllLib.getLastWin32Error());

            
        }
        return phKey[0];
    }
/**
 * @see #deriveKey
 */
    protected static native boolean CryptDeriveKey(
                                    int hProv,
                                    int Algid,
                                    int hBaseData,
                                    int dwFlags,
                                    int[] phKey);
/**
 * The destroyHash function destroys the hash object referenced by the hHash 
 * parameter. After a hash object has been destroyed, it can no longer be used and 
 * its handle is useless from then on.
 * 
 * All hash objects should be destroyed with the CryptDestroyHash function when the 
 * application is finished with them.
 * 
 * When a hash object is destroyed, many CSPs will scrub the memory in the CSP where
 * the hash object was held. The CSP memory is then freed.
 * 
 * There should be a one-to-one correspondence between calls to createHash and 
 * destroyHash.
 * 
 * 
 * @param int hHash - Handle to the hash object to be destroyed. 
 * 
 * @exceptoin java.lang.com.ms.dll.Win32Exception - If it fails, this exception is thrown
 *      with extended error information from the GetLastError function.
 * 
 *      The following table lists the error codes most commonly returned by the 
 *      GetLastError function. The error codes prefaced by "NTE" are generated by 
 *      the particular CSP you are using.
 * 
 *      Error code              Description 
 * 
 *      ERROR_BUSY              The hash object specified by hHash is currently 
 *                              being used by another process. 
 * 
 *      ERROR_INVALID_HANDLE    The hHash parameter specifies an invalid handle. 
 * 
 *      ERROR_INVALID_PARAMETER The hHash parameter contains an invalid value. 
 * 
 *      NTE_BAD_ALGID           The hHash handle specifies an algorithm that this 
 *                              CSP does not support. 
 * 
 *      NTE_BAD_HASH            The hash object specified by the hHash parameter is 
 *                              invalid. 
 * 
 *      NTE_BAD_UID             The CSP context that was specified when the hash 
 *                              object was created cannot be found. 
 */
    public static synchronized void destroyHash(int hHash){
        if (!CryptDestroyHash(hHash)){
            throw new com.ms.dll.Win32Exception(com.ms.dll.DllLib.getLastWin32Error());
        }
    }
/**
 * @see #destroyHash
 */
    protected static native boolean CryptDestroyHash(int hHash);
/**
 * The destroyKey function releases the handle referenced by the hKey parameter.
 * After a key handle has been released, it becomes invalid and cannot be used again.
 * 
 * If the handle refers to a session key, or to a public key that has been imported 
 * into the CSP through importKey, this function destroys the key and frees the 
 * memory that the key occupied. Many CSPs will scrub the memory where the key was 
 * held before freeing it.
 * 
 * On the other hand, if the handle refers to a public/private key pair (obtained 
 * from getUserKey), the underlying key pair is not destroyed by this function. 
 * Only the handle is destroyed.
 * 
 * Keys take up memory in both the operating system's memory space and the CSP's 
 * memory space. Some CSPs will be implemented in hardware with very limited memory 
 * resources. For this reason, it is important that applications destroy all keys 
 * with the destroyKey function when they are finished with them.
 * 
 * 
 * @param int hKey - Handle to the key to be destroyed. 
 * 
 * @exception java.lang.com.ms.dll.Win32Exception - If it fails, is thrown with extended 
 *      error information from the GetLastError function.
 * 
 *      The following table lists the error codes most commonly returned by the 
 *      GetLastError function. The error codes prefaced by "NTE" are generated by 
 *      the particular CSP you are using.
 * 
 *      Error code              Description
 * 
 *      ERROR_INVALID_HANDLE    The hKey parameter specifies an invalid handle.  
 * 
 *      ERROR_INVALID_PARAMETER The hKey parameter contains an invalid value. 
 * 
 *      NTE_BAD_KEY             The hKey parameter does not contain a valid handle 
 *                              to a key.
 * 
 *      NTE_BAD_UID             The CSP context that was specified when the key was 
 *                              created cannot be found. 
 */
    public static synchronized void destroyKey(int hKey){
        if (!CryptDestroyKey(hKey)){
            throw new com.ms.dll.Win32Exception(com.ms.dll.DllLib.getLastWin32Error());
        }
    }
/**
 * @see #destroyKey
 */ 
    protected static native boolean CryptDestroyKey(int hKey);
/**
 * The encrypt function is used to encrypt data. The algorithm used to encrypt 
 * the data is designated by the key held by the CSP module, which is referenced by 
 * the hKey parameter.
 * 
 * Important changes have been made to the CryptoAPI in order to support S/MIME 
 * e-mail interoperability, which affect the handling of enveloped messages. See 
 * the Remarks for CryptMsgOpenToEncode for details.
 * 
 * Due to French import laws, the Microsoft CSPs do not allow encryption when the 
 * Microsoft Windows NT operating system is used in France. Therefore, under this 
 * condition, this function will fail with the error NTE_PERM. 
 * 
 * @param int hKey - A handle to the key to use for the encryption. An application 
 *      obtains this handle by using either the genKey or the importKey function.
 * 
 *      This key specifies the encryption algorithm that is used. 
 * 
 * @param int hHash - A handle to a hash object. This parameter is used only if a 
 *      hash of the data is to be computed at the same time the encryption is being 
 *      performed. See the "Remarks" section for more information. 
 * 
 *      If no hash is to be done, this parameter must be zero. 
 * 
 * @param boolean Final - The Boolean value that specifies whether this is the last 
 *      section in a series being encrypted. This should be TRUE if this is the last
 *      or only block, and FALSE if it is not. See the "Remarks" section for more 
 *      information.
 * 
 * @param int dwFlags - The flag values. This parameter is reserved for future use 
 *      and should always be zero. 
 * 
 * @param byte[] pbData - The buffer holding the data to be encrypted. After the 
 *      encryption has been performed, the encrypted data is placed back in this 
 *      same buffer. 
 * 
 *      The size of this buffer is specified by cbBuffer. The number of bytes of 
 *      data to be encrypted is specified by pcbData. 
 * 
 *      This parameter can be NULL if all you are doing is determining the number of
 *      bytes required for the returned data. 
 * 
 * @param int pcbData - The data length. Before calling this function, the caller 
 *      should set this parameter to the number of bytes to be encrypted. Upon 
 *      return, this address will contain the number of bytes of encrypted data.
 * 
 *      If the buffer specified by pbData is not large enough to hold the data, the 
 *      function returns the ERROR_MORE_DATA error code (through GetLastError) and 
 *      stores the required buffer size, in bytes, into the variable pointed to by 
 *      pcbData. 
 * 
 *      If pbData is NULL, then no error is returned, and the function stores the 
 *      size of the data, in bytes, in the variable pointed to be pcbData. This 
 *      lets an application determine the correct buffer size unambiguously. 
 * 
 *      When a block cipher is used, this data length must be a multiple of the 
 *      block size, unless this is the final section of data to be encrypted and 
 *      the Final flag is TRUE. 
 * 
 * @return int - The number of bytes in the pbData buffer. 
 * 
 *      Note that, depending on the algorithm used, the encrypted text can be 
 *      slightly larger than the original plaintext. In this case, the pbData 
 *      buffer needs to be sized accordingly. 
 * 
 *      As a rule, if a stream cipher is used the ciphertext will be the same size 
 *      as the plaintext. If a block cipher is used, the ciphertext will be up to a 
 *      "block length" larger than the plaintext. 
 * 
 * @exception java.lang.com.ms.dll.Win32Exception - If it fails, this exception is thrown
 *      with extended error information from the GetLastError function.
 * 
 *      The following table lists the error codes most commonly returned by the 
 *      GetLastError function. The error codes prefaced by "NTE" are generated by 
 *      the particular CSP you are using.
 * 
 *      Error code                  Description 
 * 
 *      ERROR_INVALID_HANDLE        One of the parameters specifies an invalid 
 *                                  handle.  
 * 
 *      ERROR_INVALID_PARAMETER     One of the parameters contains an invalid value.
 *                                  This is most often an illegal pointer. 
 * 
 *      NTE_BAD_ALGID               The hKey session key specifies an algorithm that
 *                                  this CSP does not support.
 * 
 *      NTE_BAD_DATA                The data to be encrypted is invalid. For example,
 *                                  when a block cipher is used and the Final flag 
 *                                  is FALSE, the value specified by pcbData must be
 *                                  a multiple of the block size. 
 * 
 *      NTE_BAD_FLAGS               The dwFlags parameter is nonzero. 
 * 
 *      NTE_BAD_HASH                The hHash parameter contains an invalid handle.
 * 
 *      NTE_BAD_HASH_STATE          An attempt was made to add data to a hash object
 *                                  that is already marked "finished." 
 * 
 *      NTE_BAD_KEY                 The hKey parameter does not contain a valid 
 *                                  handle to a key. 
 * 
 *      NTE_BAD_LEN                 The size of the output buffer is too small to 
 *                                  hold the generated ciphertext. 
 * 
 *      NTE_BAD_UID                 The CSP context that was specified when the key 
 *                                  was created cannot be found. 
 * 
 *      NTE_DOUBLE_ENCRYPT          The application attempted to encrypt the same 
 *                                  data twice. 
 * 
 *      NTE_FAIL                    The function failed in some unexpected way. 
 * 
 *      NTE_NO_MEMORY               The CSP ran out of memory during the operation. 
 * 
 * If data is to be hashed and encrypted simultaneously, a handle to a hash object 
 * can be passed in the hHash parameter. The hash value will be updated with the 
 * plaintext passed in. This option is useful when generating signed and encrypted 
 * text.
 * 
 * Before calling encrypt, the application should obtain a handle to the hash 
 * object by calling the CryptCreateHash function. After the encryption is complete, 
 * the hash value can be obtained through the getHashParam function or the hash
 * can be signed using the signHash function.
 * 
 * When a large amount of data needs to be encrypted, it can be done in sections. 
 * This is done by calling encrypt repeatedly. The Final parameter should be 
 * set to TRUE only on the last invocation of encrypt, so the encryption engine can 
 * properly finish the encryption process. The following extra actions are performed
 * when Final is TRUE: 
 * 
 *  -   If the key is a block cipher key, the data will be padded to a multiple of 
 *      the block size of the cipher. To find the block size of a cipher, use 
 *      getKeyParam to get the KP_BLOCKLEN parameter of the key. 
 * 
 *  -   If the cipher is operating in a chaining mode, the next encrypt operation 
 *      will reset the cipher's feedback register to the KP_IV value of the key. 
 * 
 *  -   If the cipher is a stream cipher, the next encrypt will reset the cipher to 
 *      its initial state. 
 */
    public static synchronized int encrypt(
                                    int hKey,
                                    int hHash,
                                    boolean Final,
                                    int dwFlags,
                                    byte[] pbData,
                                    int cbData,
                                    int cbBuffer){
        int[] pcbData = new int[]{cbData};  
        if (!CryptEncrypt(hKey,hHash,Final,dwFlags,pbData,pcbData,cbBuffer)){
            throw new com.ms.dll.Win32Exception(com.ms.dll.DllLib.getLastWin32Error());
        }
        return pcbData[0];
    }
/**
 * @see #encrypt
 */
    protected static native boolean CryptEncrypt(
                                    int hKey,
                                    int hHash,
                                    boolean Final,
                                    int dwFlags, 
                                    byte[] pbData,
                                    int[] pcbData,  
                                    int cbBuffer);
/**
 * This method uses the same J/Direct function as the exportKey method, but specifies
 * the data length as zero, so the amount of space required is returned.
 * 
 * @see #exportKey for parameter types.
 */
    public static synchronized int exportKeyLength(
                                    int hKey,
                                    int hExpKey,
                                    int dwBlobType,
                                    int dwFlags,
                                    byte[] pbData){
        int[] pcbDataLen = new int[1];
        if (!CryptExportKey(hKey,hExpKey,dwBlobType,dwFlags,pbData,pcbDataLen)){
            throw new com.ms.dll.Win32Exception(com.ms.dll.DllLib.getLastWin32Error());
            
        }
        return pcbDataLen[0];
    }                                          
/**
 * The exportKey function is used to export cryptographic keys out of a 
 * cryptographic service provider in a secure manner.
 * 
 * A handle to the key to be exported is passed into the function and the function 
 * returns a key blob to the caller. This key blob can be sent over a nonsecure 
 * transport or stored in a nonsecure storage location. The key blob is useless 
 * until the intended recipient uses the CryptImportKey function on it, which will 
 * then import the key into the recipient's CSP.
 * 
 * @param int hKey - Handle to the key to be exported. 
 * 
 * @param int hExpKey - Handle to a cryptographic key belonging to the destination 
 *      user. The key data within the key blob created is encrypted by using this 
 *      key.  This ensures that only the destination user will be able to make use 
 *      of the key  blob. 
 * 
 *      Most often, this will be the key exchange public key of the destination user.
 *      However, certain protocols in some CSPs require that a session key belonging
 *      to the destination user be used for this purpose. (Neither the Microsoft 
 *      Base Cryptographic Provider or the Microsoft Enhanced Cryptographic Provider
 *      currently support this.) 
 * 
 *      If the key blob type specified by dwBlobType is PUBLICKEYBLOB, then this 
 *      parameter is unused and should be set to zero. 
 * 
 *      If the key blob type specified by dwBlobType is PRIVATEKEYBLOB, then this is
 *      typically a handle to a session key that is to be used to encrypt the key 
 *      blob.  Some CSPs allow this parameter to be zero, in which case the 
 *      application should encrypt the private key blob manually so as to protect it.
 * 
 *      To determine how Microsoft Cryptographic Providers respond to this parameter,
 *      see The Private-Key Blob sections of Microsoft Cryptographic Service 
 *      Providers. 
 * 
 * @param int dwBlobType - Type of key blob to be exported. This must be one of the 
 *      following constants, which are discussed in Exchanging Cryptographic Keys: 
 * 
 *      SIMPLEBLOB      -       used to transport session keys 
 * 
 *      PUBLICKEYBLOB   -       used to transport public keys 
 * 
 *      PRIVATEKEYBLOB  -       used to transport public/private key pairs 
 * 
 *      OPAQUEBLOB      -       used to store session keys in a Schannel CSP. 
 *                              OPAQUEBLOBs are non-transferrable and must be used 
 *                              within the CSP that generated the blob. 
 * 
 * @param int pbData - Pointer to a buffer that receives the key blob. 
 *      This parameter can be NULL to set the size of this information for memory 
 *      allocation purposes. For more information, see Common In/Out Parameter 
 *      Conventions. 
 * 
 * @param int pcbDataLen - Variable that specifies the size, in bytes, of the buffer
 *      pointed to by the pbDataLen parameter. When the function returns, the 
 *      variable pointed to by the pcbData parameter contains the number of bytes 
 *      stored in the buffer. This parameter can be NULL, only if pbDataLen is NULL.
 * 
 *      Note  When processing the data returned in the buffer, applications need to 
 *      use the actual size of the data returned. The actual size may be slightly 
 *      smaller than the size of the buffer specified on input. (On input, buffer 
 *      sizes are usually specified large enough to insure that the largest possible
 *      output data will fit in the buffer.) On output, the variable pointed to by 
 *      this parameter is updated to reflect the actual size of the data copied to 
 *      the buffer. 
 * 
 * @exception java.lang.com.ms.dll.Win32Exception - If it fails, this exception is thrown
 *      with extended error information from the GetLastError function.
 * 
 *      The following table lists the error codes most commonly returned by the 
 *      GetLastError function. The error codes prefaced by "NTE" are generated by 
 *      the particular CSP you are using.
 * 
 *      Error code                  Description 
 * 
 *      ERROR_INVALID_HANDLE        One of the parameters specifies an invalid 
 *                                  handle.  
 * 
 *      ERROR_INVALID_PARAMETER     One of the parameters contains an invalid value.
 *                                  This is most often an illegal pointer. 
 * 
 *      ERROR_MORE_DATA             If the buffer specified by the pbData parameter 
 *                                  is not large enough to hold the returned data, 
 *                                  the function sets the ERROR_MORE_DATA code, and 
 *                                  stores the required buffer size, in bytes, into 
 *                                  the variable pointed to by pcbDataLen. 
 * 
 *      NTE_BAD_FLAGS               The dwFlags parameter is nonzero. 
 * 
 *      NTE_BAD_KEY                 One or both of the keys specified by hKey and 
 *                                  hExpKey are invalid. 
 * 
 *      NTE_BAD_KEY_STATE           You do not have permission to export the key. 
 *                                  That is, when the hKey key was created, the 
 *                                  CRYPT_EXPORTABLE flag was not specified. 
 * 
 *      NTE_BAD_PUBLIC_KEY          The key blob type specified by dwBlobType is 
 *                                  PUBLICKEYBLOB, but hExpKey does not contain a 
 *                                  public key handle. 
 * 
 *      NTE_BAD_TYPE                The dwBlobType parameter specifies an unknown 
 *                                  blob type. 
 * 
 *      NTE_BAD_UID                 The CSP context that was specified when the hKey
 *                                  key was created cannot be found. 
 * 
 *      NTE_NO_KEY                  A session key is being exported and the hExpKey 
 *                                  parameter does not specify a public key. 
 */
    public static synchronized int exportKey(
                                    int hKey,
                                    int hExpKey,
                                    int dwBlobType,
                                    int dwFlags,
                                    byte[] pbData,
                                    int cbDataLen){
        int[] pcbDataLen = new int[]{cbDataLen};
        if (!CryptExportKey(hKey,hExpKey,dwBlobType,dwFlags,pbData,pcbDataLen)){
            throw new com.ms.dll.Win32Exception(com.ms.dll.DllLib.getLastWin32Error());
            
        }
        return pcbDataLen[0];
    }
/**
 * @see #exportKey
 * @see #exportKeyLength
 */
    protected static native boolean CryptExportKey(
                                    int hKey,
                                    int hExpKey,
                                    int dwBlobType,
                                    int dwFlags,
                                    byte[] pbData,
                                    int[] pcbDataLen);
/**
 * The CryptGenKey function generates random cryptographic keys for use with the CSP 
 * module. A handle to the key is returned in phKey. This handle can then be used as
 * needed with any of the other CryptoAPI functions requiring key handles.
 * 
 * The calling application is required to specify the algorithm when calling this 
 * function. Because this algorithm type is kept bundled with the key, the 
 * application does not need to specify the algorithm later when the actual 
 * cryptographic operations are performed.
 * 
 * The maximum size of an RSA signature key has been bumped up to 16,384 bits.
 * 
 * To generate a key to be used with a symmetric encryption algorithm (that is, a 
 * session key), use the Algid parameter to specify the algorithm. The algorithms 
 * available will most likely be different for each CSP. If you are using the 
 * Microsoft Base Cryptographic Provider, one of the following values can be used 
 * to specify the algorithm.
 * 
 *      Value               Description 
 * 
 *      CALG_RC2            RC2 block cipher 
 *      CALG_RC4            RC4 stream cipher 
 * 
 * If you are using a Diffie-Hellman CSP, use one of the following values.
 * 
 *      Value               Description
 * 
 *      CALG_DH_SF          Specifies a "Store and Forward" Diffie-Hellman key.
 *      CALG_DH_EPHEM       Specifies an "Ephemeral" Diffie-Hellman key. 
 * 
 * When keys are generated for symmetric block ciphers, the key by default will be 
 * set up in cipher block chaining (CBC) mode with an initialization vector of zero.
 * This cipher mode provides a good default method for bulk encrypting data. To 
 * change these parameters, use the CryptSetKeyParam function.
 * 
 * In addition to generating keys for symmetric algorithms, the CryptGenKey function 
 * can also generate keys for public-key algorithms. The use of public-key 
 * algorithms is restricted to key exchange and digital signatures. Each CryptoAPI 
 * client generally possesses one key pair for each of these operations. To generate
 * one of these key pairs, set the Algid parameter to one of the following values.
 * 
 *      Value                   Description 
 * 
 *      AT_KEYEXCHANGE          Key exchange
 *      AT_SIGNATURE            Digital signature 
 * 
 * Note  When key specifications AT_KEYEXCHANGE and AT_SIGNATURE are specified for 
 *      the Algid parameter, the algorithm identifiers that are used to generate 
 *      the key depend on the provider used. As a result, for these key 
 *      specifications, the values returned from CryptGetKeyParam (when the KP_ALGID
 *      parameter is specified) depend on the provider used. See ALG_ID to determine 
 *      which algorithm identifier is used by the different providers for the key 
 *      specs AT_KEYEXCHANGE and AT_SIGNATURE.
 * 
 * 
 * @param int hProv - Handle to the application's CSP. An application obtains this 
 *          handle by using the CryptAcquireContext function. 
 * 
 * @param int Algid - Identifier for the algorithm for which the key is to be 
 *          generated. 
 *     
 *          The valid values for this parameter will vary, depending on the CSP 
 *          that is used. See the "Remarks" section for a list of possible algorithm
 *          identifiers. 
 * 
 * @param int dwFlags - Flags specifying the type of key generated. 
 * 
 *          The sizes of RSA signature and key exchange keys may be set when the key
 *          is generated. The key size is set with the upper 16 bits of the dwFlags 
 *          parameter and these 16 bits represent the length of the key (modulus) in 
 *          bits. So if a 2048 bit RSA signature key is to be generated then the 
 *          value 0x08000000 would be combined together with the dwFlags parameter 
 *          in bitwise OR operation. Notice that the upper 16 bits of 0x08000000 is 
 *          0x0800, or 2048 in decimal notation. If none of the upper 16 bits are 
 *          set, then the default key size is generated. If a key larger than the 
 *          max (16,384 signature, 512 key exchange) or smaller than the min (384 
 *          sig and key exchange) is given, the call fails with 
 *          ERROR_INVALID_PARAMETER. For the parameters and limitations of keys 
 *          generated using Microsoft providers, see Microsoft Cryptographic Service
 *          Providers. 
 * 
 *          This parameter can be zero, or you can specify one or more of the 
 *          following flags by using the bitwise OR operator to combine them: 
 * 
 *      CRYPT_EXPORTABLE.           If this flag is set, then the key can be 
 *                                  transferred out of the CSP into a key blob by 
 *                                  using the CryptExportKey function. Because 
 *                                  session keys generally must be exportable, this 
 *                                  flag should usually be set when they are created.
 * 
 *                                  If this flag is not set, then the key will not 
 *                                  be exportable. For a session key, this means 
 *                                  that the key will be available only within the 
 *                                  current session and only the application that 
 *                                  created it will be able to use it. For a 
 *                                  public/private key pair, this means that the 
 *                                  private key cannot be transported or backed up.
 * 
 *                                  This flag applies only to session key and 
 *                                  private key blobs. It does not apply to public 
 *                                  keys, which are always exportable. 
 * 
 *      CRYPT_CREATE_SALT.          If this flag is set, then the key will be 
 *                                  assigned a random salt value automatically. You 
 *                                  can retrieve this salt value by using the 
 *                                  getKeyParam function with the dwParam parameter 
 *                                  set to KP_SALT.
 * 
 *                                  If this flag is not set, then the key will be 
 *                                  given a salt value of zero. 
 * 
 *                                  When keys with nonzero salt values are exported 
 *                                  (through exportKey), then the salt value must 
 *                                  also be obtained and kept with the key blob. 
 * 
 *      CRYPT_NO_SALT.              Specifies that a no salt value gets allocated 
 *                                  for a 40-bit symmetric key. For more information,
 *                                  see Microsoft Cryptographic ProvidersRelease 
 *                                  Notes. 
 * 
 *      CRYPT_USER_PROTECTED.       If this flag is set, then the user will be 
 *                                  notified through a dialog box or another method 
 *                                  when certain actions are attempted using this 
 *                                  key. The precise behavior is specified by the 
 *                                  CSP being used. Prior to Microsoft Internet 
 *                                  Explorer 4.0, Microsoft Cryptographic Providers 
 *                                  ignore this flag. Starting with Microsoft 
 *                                  Internet Explorer 4.0, Microsoft Providers 
 *                                  support this flag value. 
 * 
 *      CRYPT_PREGEN.               Specifies an initial Diffie-Hellman or DSS key 
 *                                  generation. Useful only with Diffie-Hellman/DSS 
 *                                  CSPs. 
 * 
 * @return int - The handle of the newly generated key. 
 * 
 * @exception com.ms.dll.Win32Exception - If it fails, erro number from the 
 *          GetLastError function.
 * 
 *          The following table lists the error codes most commonly returned by the 
 *          GetLastError function. The error codes prefaced by "NTE" are generated 
 *          by the particular CSP you are using.
 * 
 *      Error code                  Description 
 * 
 *      ERROR_INVALID_HANDLE        One of the parameters specifies an invalid 
 *                                  handle.
 * 
 *      ERROR_INVALID_PARAMETER     One of the parameters contains an invalid value. 
 *                                  This is most often an illegal pointer. 
 * 
 *      NTE_BAD_ALGID               The Algid parameter specifies an algorithm that 
 *                                  this CSP does not support. 
 * 
 *      NTE_BAD_FLAGS               The dwFlags parameter contains an invalid value.
 * 
 *      NTE_BAD_UID                 The hProv parameter does not contain a valid 
 *                                  context handle. 
 * 
 *      NTE_FAIL                    The function failed in some unexpected way. 

 */
    public static synchronized int genKey(
                                    int hProv,
                                    int Algid,
                                    int dwFlags){
        int[] phKey = new int[1];
        if (!CryptGenKey(hProv,Algid,dwFlags,phKey)){
            throw new com.ms.dll.Win32Exception(com.ms.dll.DllLib.getLastWin32Error());
        }
        return phKey[0];
    }
/**
 * @see @genKey
 */
    protected static native boolean CryptGenKey(
                                    int hProv,
                                    int Algid,
                                    int dwFlags,
                                    int[] phKey);
    
/**
 * The CryptGetProvParam function retrieves parameters that govern the operations of
 * a CSP.
 * 
 * The parameter number contained in the dwParam parameter can be set to one of the 
 * following values: 
 * 
 *          PP_CONTAINER.       The key container name. When this parameter is 
 *                              specified, the function fills the pbData buffer with 
 *                              the name of the current key container. This name is 
 *                              in the form of a zero-terminated CHAR string.
 * 
 *                              This string is exactly the same as the one passed in
 *                              the pszContainer parameter of the CryptAcquireContext 
 *                              function in order to specify that the current key 
 *                              container be used. This parameter is often read in 
 *                              order to determine the name of the default key 
 *                              container.
 * 
 *          PP_ENUMALGS.        The algorithm information. When this parameter is 
 *                              specified, the function fills the pbData buffer with
 *                              information about one of the algorithms supported by 
 *                              the CSP. The PP_ENUMALGS parameter must be read 
 *                              repeatedly to enumerate all the supported algorithms. 
 *                              The algorithms are not enumerated in any particular 
 *                              order.
 * 
 *                              The first time that the PP_ENUMALGS parameter is 
 *                              read, the CRYPT_FIRST flag should be specified. This
 *                              will ensure that information about the first 
 *                              algorithm in the enumeration list is returned. The 
 *                              PP_ENUMALGS parameter can then be repeatedly read in
 *                              order to retrieve the information about the rest of 
 *                              the algorithms. When the CryptGetProvParam function 
 *                              fails with ERROR_NO_MORE_ITEMS, then the end of the 
 *                              enumeration list has been reached. A code sample 
 *                              illustrating this is located in the "Example" 
 *                              section. 
 * 
 *                              When calling CryptGetProvParam with the PP_ENUMALGS 
 *                              flag in the dwParam parameter the dwBits field 
 *                              should be changed to the length of the default key 
 *                              sizes. 
 * 
 *                              The following code fragment indicates the format of 
 *                              the data that the function returns in the pbData 
 *                              buffer: 
 * 
 *                                      ALG_ID aiAlgid;
 *                                      DWORD dwBits;
 *                                      DWORD dwNameLen;
 *                                      CHAR szName[dwNameLen];
 * 
 * 
 *      The following table defines each of these fields. 
 * 
 *          Field               Description 
 *        
 *          aiAlgid             The algorithm identifier. This is the value that is 
 *                              passed to the genKey, deriveKey, or createHash 
 *                              functions in order to specify that a particular 
 *                              algorithm be used. 
 * 
 *          dwBits              The number of bits in the keys used by the algorithm.
 *                              If this is a hash algorithm, this value indicates 
 *                              the number of bits in the hash values generated by 
 *                              this algorithm.
 * 
 *          dwNameLen           The number of characters in the algorithm name, 
 *                              including the terminating zero. 
 * 
 *          szName              The zero-terminated name of the algorithm. 
 * 
 * 
 *          PP_ENUMCONTAINERS.  The key container names. When this parameter is 
 *                              specified, the function fills the pbData buffer 
 *                              with the name of one of the key containers 
 *                              maintained by the CSP. This name is in the form of a
 *                              zero-terminated CHAR string. The PP_ENUMCONTAINERS 
 *                              parameter must be read repeatedly to enumerate all 
 *                              the container names.
 * 
 *                              These container names are enumerated in the same way 
 *                              as are the CSP's supported algorithms. Refer to the 
 *                              PP_ENUMALGS entry for more information. 
 * 
 *          PP_IMPTYPE.         The CSP implementation type. The pbData buffer will     
 *                              contain a DWORD value that indicates how the CSP is 
 *                              implemented. Possible values are:
 * 
 *                                      CRYPT_IMPL_HARDWARE 
 *                                      CRYPT_IMPL_SOFTWARE 
 *                                      CRYPT_IMPL_MIXED 
 *                                      CRYPT_IMPL_UNKNOWN 
 * 
 *          PP_NAME.            The CSP name. When this parameter is specified, the 
 *                              function fills the pbData buffer with the name of 
 *                              the CSP. This name is in the form of a 
 *                              zero-terminated CHAR string.
 * 
 *                              This string is identical to the one passed in the 
 *                              pszProvider parameter of the acquireContext function 
 *                              in order to specify that the current CSP be used. 
 *                              For example, the Microsoft Base Cryptographic 
 *                              Provider will return "Microsoft Base Cryptographic 
 *                              Provider v1.0" when this parameter is read. 
 * 
 *          PP_VERSION.         The CSP version number. The pbData buffer will 
 *                              contain a DWORD value that indicates the version 
 *                              number of the CSP. The least significant byte 
 *                              contains the minor version number and the next most 
 *                              significant byte the major version number. For 
 *                              example, version 1.0 would be represented here as 
 *                              0x00000100. 
 * 
 * 
 * Version Information for The Microsoft Base Cryptographic Provider and The 
 *      Microsoft Enhanced Cryptographic Provider: 
 * 
 *      To maintain backward compatibility with earlier versions of these providers,
 *      the name retains the "v1.0" designation in later versions. This can be seen 
 *      in the wincrypt.h header file. Version 2.0 of this provider is currently 
 *      shipping. To see which version of the provider you have, make a call to 
 *      getProvParam with the dwParam argument set to PP_VERSION. For example, if 
 *      0x00000200 is returned in pbData, then you have version 2.0. 
 * 
 *      PP_KEYSET_SEC_DESCR. Indicates that the security descriptor is to be 
 *      retrieved for the registry entry where the key set is stored.
 * 
 *      Note  The PP_KEYSET_SEC_DESCR flag is not supported under Microsoft 
 *      Windows 95. 
 * 
 * The dwFlags parameter is used to pass in the SECURITY_INFORMATION bit flags that 
 * indicate the requested security information. The pointer to the security 
 * descriptor is returned in the pbData parameter and the length of the security 
 * descriptor is returned in the pcbData parameter. It may be helpful to look at 
 * the documentation on RegGetKeySecurity and RegSetKeySecurity (WIN32 calls).
 * 
 *      Algorithm Identifiers
 * 
 * When enumerating algorithms (dwParam == PP_ENUMALGS), your application may need 
 * to determine the class of a particular algorithm. For example, you may want to 
 * display a list of encryption algorithms to the user and disregard the rest. This 
 * can be done with the GET_ALG_CLASS(x) macro. This macro takes an algorithm 
 * identifier as an argument and returns a code indicating the general class of 
 * algorithm that the identifier belongs to. Possible return values include: 
 * 
 *          ALG_CLASS_DATA_ENCRYPT 
 *          ALG_CLASS_HASH 
 *          ALG_CLASS_KEY_EXCHANGE 
 *          ALG_CLASS_SIGNATURE 
 * 
 *  The following table lists the algorithms supported by the Microsoft Base 
 *  Cryptographic Provider along with the class of each algorithm.
 * 
 *  Name            Identifier          Class 
 * 
 *  "MD2"           CALG_MD2            ALG_CLASS_HASH 
 *  "MD5"           CALG_MD5            ALG_CLASS_HASH 
 *  "SHA"           CALG_SHA            ALG_CLASS_HASH 
 *  "MAC"           CALG_MAC            ALG_CLASS_HASH 
 *  "RSA_SIGN"      CALG_RSA_SIGN       ALG_CLASS_SIGNATURE 
 *  "RSA_KEYX"      CALG_RSA_KEYX       ALG_CLASS_KEY_EXCHANGE 
 *  "RC2"           CALG_RC2            ALG_CLASS_DATA_ENCRYPT 
 *  "RC4"           CALG_RC4            ALG_CLASS_DATA_ENCRYPT 
 * 
 * If your application does not recognize an algorithm identifier, it is not 
 * recommended that you use the algorithm. Making use of an unknown cryptographic 
 * algorithm can sometimes produce unpredictable results.
 * 
 * 
 * @param int hProv - Handle to the CSP on which to query parameters. 
 * @param int dwParam - Parameter number. See the "Remarks" section for a list of 
 *          valid parameters. 
 * @param byte[] pbData - Pointer to a buffer that receives the specified parameter 
 *          data. The form of this data will vary, depending on the parameter number.
 * 
 *          This parameter can be NULL to set the size of this information for 
 *          memory allocation purposes. For more information, see Common In/Out 
 *          Parameter Conventions. 
 * 
 * @param int pcbData - A variable that specifies the size, in bytes, of the buffer 
 *          pointed to by the pbData parameter. When the function returns, the 
 *          variable pointed to by the pcbData parameter contains the number of 
 *          bytes stored in the buffer. This parameter can be NULL, only if pbData 
 *          is NULL.
 * 
 *          Note that when processing the data returned in the buffer, applications 
 *          need to use the actual size of the data returned. The actual size may be
 *          slightly smaller than the size of the buffer specified on input. (On 
 *          input, buffer sizes are usually specified large enough to insure that 
 *          the largest possible output data will fit in the buffer.) On output, the
 *          variable pointed to by this parameter is updated to reflect the actual 
 *          size of the data copied to the buffer. 
 * 
 *          Note  When one of the enumeration parameters (PP_ENUMALGS or 
 *          PP_ENUMCONTAINERS) is being read, the pcbData parameter works somewhat 
 *          differently. If pbData is NULL or the value pointed to by pcbData is 
 *          too small, the value returned in this parameter is the size of the 
 *          largest item in the enumeration list instead of the size of the item 
 *          currently being read.
 * 
 *          When one of the enumeration parameters is read and the pbData parameter 
 *          is NULL, the CRYPT_FIRST flag must be specified in order for the size 
 *          information to be correctly retrieved. 
 * 
 * @param int dwFlags - Flag values. 
 * 
 *          When one of the enumeration parameters (PP_ENUMALGS or PP_ENUMCONTAINERS)
 *          is being read, then the CRYPT_FIRST flag can be specified. When this 
 *          flag is set, the first item in the enumeration list is returned. If this
 *          flag is not set, then the next item in the list is returned. 
 * 
 *          When the enumeration parameter PP_ENUMCONTAINERS is being read, then the 
 *          CRYPT_MACHINE_KEYSET flag can be specified. This flag can be set if you 
 *          want the enumeration to come from the HKEY_LOCAL_MACHINE portion of the 
 *          registry rather than from the HKEY_CURRENT_USER portion of the registry 
 *          (the default). 
 * 
 *          When the dwParam is PP_KEYSET_SEC_DESCR, the security descriptor on the 
 *          registry key where the keys are stored is being retrieved. For this case,
 *          dwFlags is used to pass in the SECURITY_INFORMATION bit flags (can be 
 *          combined with a bitwise OR operation), as defined in the Win32 
 *          Programmer's Reference, that indicates the requested security 
 *          information. The following is specified there: 
 *  
 *          typedef DWORD SECURITY_INFORMATION;
 * 
 *              Value                       Meaning
 * 
 *              OWNER_SECURITY_INFORMATION  Indicates the owner identifier of the 
 *                                          object is being referenced. 
 * 
 *              GROUP_SECURITY_INFORMATION  Indicates the primary group identifier 
 *                                          of the object is being referenced. 
 * 
 *              DACL_SECURITY_INFORMATION   Indicates the discretionary ACL of the 
 *                                          object is being referenced. 
 * 
 *              SACL_SECURITY_INFORMATION   Indicates the system ACL of the object 
 *                                          is being referenced. 
 * 
 * @exception com.ms.dll.Win32Exception - thrown if there was a problem executing
 *          this function, it contains the error code from the  GetLastError function.
 * 
 *          The following table lists the error codes most commonly returned by the 
 *          GetLastError function. The error codes prefaced by "NTE" are generated
 *          by the particular CSP you are using.
 * 
 *          Error code                  Description 
 * 
 *          ERROR_INVALID_HANDLE        One of the parameters specifies an invalid 
 *                                      handle.
 * 
 *          ERROR_INVALID_PARAMETER     One of the parameters contains an invalid 
 *                                      value. This is most often an illegal pointer.
 * 
 *          ERROR_MORE_DATA             If the buffer specified by the pbData 
 *                                      parameter is not large enough to hold the 
 *                                      returned data, the function sets the 
 *                                      ERROR_MORE_DATA code, and stores the 
 *                                      required buffer size, in bytes, into the 
 *                                      variable pointed to by pcbData. 
 * 
 *          ERROR_NO_MORE_ITEMS         The end of the enumeration list has been 
 *                                      reached. No valid data has been placed in 
 *                                      the pbData buffer. This error is returned 
 *                                      only when dwParam equals PP_ENUMALGS or 
 *                                      PP_ENUMCONTAINERS. 
 * 
 *          NTE_BAD_FLAGS               The dwFlags parameter is nonzero.
 * 
 *          NTE_BAD_TYPE                The dwParam parameter specifies an unknown 
 *                                      parameter number. 
 * 
 *          NTE_BAD_UID                 The CSP context specified by hProv is 
 *                                      invalid. 
 */
    public static synchronized int getProvParam(
                                              int hProv,
                                              int dwParam,
                                              byte[] pbData,
                                              int pcData,
                                              int dwFlags){
        int [] pcbData = new int[1];
        pcbData[0]=pcData;
        if (!CryptGetProvParam(hProv,dwParam,pbData,pcbData,dwFlags)){
            throw new com.ms.dll.Win32Exception(com.ms.dll.DllLib.getLastWin32Error());
        }
        return pcbData[0];
    }
/**
 * @see #getProvParam
 */
    protected static native boolean CryptGetProvParam(
                        int hProv, // in
                        int dwParam,    // in
                        byte[] pbData,     // out
                        int[] pcbData,   // in/out
                        int dwFlags);     // in
    
/**
 * The CryptGetUserKey function retrieves a handle to a permanent user key pair, 
 * such as the user's signature key pair.
 * 
 * @param int hProv - A handle to the application's CSP. An application obtains this
 *          handle by using the CryptAcquireContext function. 
 * @param int dwKeySpec - The specification of the key to retrieve. The following 
 *          keys are retrievable from almost all providers: 
 * 
 *              AT_KEYEXCHANGE.             Exchange key pair. 
 *              AT_SIGNATURE.               Signature key pair. 
 * 
 *          Additionally, some providers allow access to other user-specific keys 
 *          through this function. See the documentation on the specific provider 
 *          for details. 
 * @return int - Handle of the retrieved key.
 * 
 * @exception com.ms.dll.Win32Exception - thrown if there was a problem with the 
 *          cuntion call that returns a win32 error code.
 * 
 *          The following table lists the error codes most commonly returned by the 
 *          GetLastError function. The error codes prefaced by "NTE" are generated 
 *          by the particular CSP you are using.
 * 
 *          Error code                  Description 
 * 
 *          ERROR_INVALID_HANDLE        One of the parameters specifies an invalid 
 *                                      handle.  
 * 
 *          ERROR_INVALID_PARAMETER     One of the parameters contains an invalid 
 *                                      value. This is most often an illegal pointer.
 * 
 *          NTE_BAD_KEY                 The dwKeySpec parameter contains an invalid 
 *                                      value.
 * 
 *          NTE_BAD_UID                 The hProv parameter does not contain a valid
 *                                      context handle. 
 * 
 *          NTE_NO_KEY                  The key requested by the dwKeySpec parameter
 *                                      does not exist. 
 */ 
    public static synchronized int getUserKey(
                                    int hProv,
                                    int dwKeySpec){
        int[] phUserKey = new int[1];
        if (!CryptGetUserKey(hProv,dwKeySpec,phUserKey)){
            throw new com.ms.dll.Win32Exception(com.ms.dll.DllLib.getLastWin32Error());
        }
        return phUserKey[0];
    }
/**
 * @see #getUserKey
 */
    protected static native boolean CryptGetUserKey(
                                    int hProv,
                                    int dwKeySpec,
                                    int[] phUserKey);
/**
 * The CryptHashData function is used by applications to feed data to a specified 
 * hash object. This function and CryptHashSessionKey can be called multiple times 
 * to compute the hash on long streams or on discontinuous streams.
 * 
 * Before calling this function, the CryptCreateHash function must be called to get 
 * a handle to a hash object.
 * 
 * @param int hHash - A handle to the hash object. An application obtains this 
 *          handle by using the CryptCreateHash function. 
 * @param byte[] pbData - The data to be hashed. 
 * @param int cbData - The number of bytes of data to be hashed. This must be zero 
 *          if the CRYPT_USERDATA flag is set. 
 * @param int dwFlags - The flag values. The following values are currently defined:
 * 
 *          CRYPT_USERDATA.             When this flag is set, the CSP will prompt 
 *                                      the user to input some data directly. This 
 *                                      is then added to the hash. The application 
 *                                      is not allowed access to the data. For 
 *                                      example, this flag can be used to allow the 
 *                                      user to enter a PIN into the system. 
 * 
 *                                      The Microsoft Base Cryptographic Provider 
 *                                      ignores this parameter. 
 * 
 * @exception com.ms.dll.Win32Exception - If an error occures during this method
 *          call, it is thrown as an exception.
 * 
 *          The following table lists the error codes most commonly returned by the 
 *          GetLastError function. The error codes prefaced by "NTE" are generated 
 *          by the particular CSP you are using.
 * 
 *          Error code                      Description 
 * 
 *          ERROR_INVALID_HANDLE            One of the parameters specifies an 
 *                                          invalid handle. 
 * 
 *          ERROR_INVALID_PARAMETER         One of the parameters contains an 
 *                                          invalid value. This is most often an 
 *                                          illegal pointer. 
 * 
 *          NTE_BAD_ALGID                   The hHash handle specifies an algorithm 
 *                                          that this CSP does not support. 
 * 
 *          NTE_BAD_FLAGS                   The dwFlags parameter contains an 
 *                                          invalid value. 
 * 
 *          NTE_BAD_HASH                    The hash object specified by the hHash 
 *                                          parameter is invalid. 
 * 
 *          NTE_BAD_HASH_STATE              An attempt was made to add data to a 
 *                                          hash object that is already marked 
 *                                          "finished." 
 * 
 *          NTE_BAD_KEY                     A keyed hash algorithm is being used, 
 *                                          but the session key is no longer valid. 
 *                                          This error will be generated if the 
 *                                          session key is destroyed before the 
 *                                          hashing operation is complete. 
 * 
 *          NTE_BAD_LEN                     The CRYPT_USERDATA flag is set and the 
 *                                          cbData parameter has a nonzero value. 
 * 
 *          NTE_BAD_UID                     The CSP context that was specified when 
 *                                          the hash object was created cannot be 
 *                                          found. 
 * 
 *          NTE_FAIL                        The function failed in some unexpected 
 *                                          way. 
 * 
 *          NTE_NO_MEMORY                   The CSP ran out of memory during the 
 *                                          operation. 
 */
    public static synchronized void hashData(int hHash,
                         byte[] pbData,
                         int cbData,
                         int dwFlags){
        if (!CryptHashData(hHash,pbData,cbData,dwFlags))
            throw new com.ms.dll.Win32Exception(com.ms.dll.DllLib.getLastWin32Error());
    }
/**
 * @see #hashData
 */
    protected static native boolean CryptHashData(
                                    int hHash,
                                    byte[] pbData,
                                    int cbData,
                                    int dwFlags);
/**
 * The CryptImportKey function is used to transfer a cryptographic key from a key 
 * blob to the CSP.
 * 
 * @param int hProv - A handle to the application's CSP. An application obtains this
 *          handle by using the CryptAcquireContext function. 
 * 
 * @param byte[] pbData - The buffer containing the key blob. This key blob was 
 *          generated by the CryptExportKey function, either by this same 
 *          application or by another application running on a different computer. 
 * 
 *          This key blob consists of a standard header followed by the encrypted 
 *          key. 
 * @param int dwDataLen - The length, in bytes, of the key blob. 
 * @param int hPubKey - The meaning of this parameter differs, depending on the CSP 
 *          type and the type of key blob being imported. 
 * 
 *  -   If a signed key blob is being imported, this key is used to validate the 
 *          signature of the key blob. In this case, this parameter should contain a
 *          handle to the key exchange public key of the party that created the key 
 *          blob. 
 * 
 *  -   If the key blob is encrypted with the key exchange key pair (for example, a 
 *          SIMPLEBLOB), then this parameter may be the handle to the key exchange 
 *          key. 
 * 
 * -    If the key blob is encrypted with a session key (for example, an encrypted 
 *          PRIVATEKEYBLOB), then this parameter should contain a handle to this 
 *          session key. 
 * 
 * -    If the key blob is not encrypted (for example, a PUBLICKEYBLOB), then this 
 *          parameter is not used, and should be zero. 
 * 
 * @param int dwFlags - The flag values. This parameter is currently used only when 
 *          a public/private key pair is being imported into the CSP (in the form of
 *          a PRIVATEKEYBLOB). In this case, if the key being imported is eventually
 *          to be re-exported, then the CRYPT_EXPORTABLE flag must be placed in this
 *          parameter. If this flag is not used then calls to CryptExportKey with 
 *          the key handle will fail. 
 * 
 *          CRYPT_NO_SALT. Specifies that a no-salt value gets allocated for a 
 *          40-bit symmetric key. For more information, see Microsoft Cryptographic 
 *          ProvidersRelease Notes. 
 * 
 * @return int - A handle to the key that was imported. 
 * 
 * @exceptino com.ms.dll.Win32Exception - This will be thrown if the function fails.
 * 
 *          The following table lists the error codes most commonly returned by the 
 *          GetLastError function. The error codes prefaced by "NTE" are generated 
 *          by the particular CSP you are using.
 * 
 *          Error code                  Description 
 * 
 *          ERROR_INVALID_HANDLE        One of the parameters specifies an invalid 
 *                                      handle.  
 * 
 *          ERROR_INVALID_PARAMETER     One of the parameters contains an invalid 
 *                                      value. This is most often an illegal pointer.
 * 
 *          NTE_BAD_ALGID               The simple key blob you are trying to import
 *                                      is not encrypted with the expected key 
 *                                      exchange algorithm. 
 * 
 *          NTE_BAD_DATA                Either the algorithm that works with the 
 *                                      public key you are trying to import is not 
 *                                      supported by this CSP, or an attempt was 
 *                                      made to import a session key that was 
 *                                      encrypted with something other than one of 
 *                                      YOUR public keys. 
 * 
 *          NTE_BAD_FLAGS               The dwFlags parameter is nonzero. 
 * 
 *          NTE_BAD_TYPE                The key blob type is not supported by this 
 *                                      CSP and is possibly invalid. 
 * 
 *          NTE_BAD_UID                 The hProv parameter does not contain a valid\
 *                                      context handle. 
 * 
 *          NTE_BAD_VER                 The key blob's version number does not match
 *                                      the CSP version. This usually indicates that
 *                                      the CSP needs to be upgraded.  
 */
    public synchronized static int importKey(
                                    int hProv,
                                    byte[] pbData,
                                    int dwDataLen,
                                    int hPubKey,
                                    int dwFlags){
        int[] phKey = new int[1];
        if (!CryptImportKey(hProv,pbData,dwDataLen,hPubKey,dwFlags,phKey)){
            throw new com.ms.dll.Win32Exception(com.ms.dll.DllLib.getLastWin32Error());
        }
        return phKey[0];
    }
/**
 * @see #importKey
 */
    protected static native boolean CryptImportKey(
                                    int hProv,
                                    byte[] pbData,
                                    int dwDataLen,
                                    int hPubKey,
                                    int dwFlags,
                                    int[] phKey);
/**
 * The releaseContext function is used to release a handle to a CSP and a key 
 * container.
 * 
 * After this function has been called, the session is over, and all existing 
 * session keys and hash objects that were created by using the hProv handle become 
 * invalid. In practice, all of these objects should be destroyed (with the 
 * destroyKey and destroyHash functions) before the releaseContext function is 
 * called.
 * 
 * This should be performed when the application is finished using the CSP. After 
 * this function is called, the CSP handle specified by the hProv parameter will no 
 * longer be valid. Neither the key container nor any key pairs are destroyed by 
 * this function.
 * 
 * @param int hProv - Handle to the application's CSP. This is the handle the 
 *          application obtains by using the CryptAcquireContext function. 
 * @param int dwFlags - Flag values. This parameter is reserved for future use and 
 *          should always be zero. 
 * 
 * @exception com.ms.dll.Win32Exception - thrown if the method fails, contains the 
 *          error code from com.ms.dll.DllLib.getLastWin32Error.
 * 
 *          The following table lists the error codes most commonly returned by the 
 *          GetLastError function. The error codes prefaced by "NTE" are generated 
 *          by the particular CSP you are using.
 * 
 *          Error code                      Description 
 * 
 *          ERROR_BUSY                      The CSP context specified by hProv is 
 *                                          currently being used by another process. 
 * 
 *          ERROR_INVALID_HANDLE            One of the parameters specifies an 
 *                                          invalid handle.  
 * 
 *          ERROR_INVALID_PARAMETER         One of the parameters contains an 
 *                                          invalid value. This is most often an 
 *                                          illegal pointer. 
 * 
 *          NTE_BAD_FLAGS                   The dwFlags parameter is nonzero.
 * 
 *          NTE_BAD_UID                     The hProv parameter does not contain a 
 *                                          valid context handle. 
 */
    public static synchronized void releaseContext(int hProv,int dwFlags){
        if(!CryptReleaseContext(hProv,dwFlags)){
            throw new com.ms.dll.Win32Exception(
                com.ms.dll.DllLib.getLastWin32Error());
        }
    }
/**
 * @see #releaseContext
 */ 
protected static native boolean CryptReleaseContext(
                                    int hProv,
                                    int dwFlags);
/**
 * The CryptSignHash function is used to sign a piece of data. Because all signature
 * algorithms are asymmetric and thus incredibly slow, the CryptoAPI will not let 
 * data be signed directly. Instead, you must first hash the data and then use 
 * signHash to sign the hash value.
 * 
 * Before calling this function, the CryptCreateHash function must be called to get 
 * a handle to a hash object. The CryptHashData or CryptHashSessionKey function is 
 * then used to add the data or session keys to the hash object.
 * 
 * After this function has been completed, the only hash function that can be 
 * called using the hHash handle is the CryptDestroyHash function.
 * 
 * 
 * @param int hHash - Handle to the hash object to be signed. 
 * @param int dwKeySpec - Key pair to use to sign the hash. The following keys can 
 *          be specified. Value Description 
 * 
 *              AT_KEYEXCHANGE          Exchange private key 
 *              AT_SIGNATURE            Signature private key 
 * 
 *          The signature algorithm used is specified when the key pair was 
 *          originally created. 
 * 
 *          The only signature algorithm that the Microsoft Base Cryptographic 
 *          Provider supports is the RSA Public-Key algorithm. 
 * 
 * @param String sDescription - This parameter should no longer be used, and should 
 *          be set to NULL to prevent security vulnerabilities. However, it is still
 *          supported for backward compatibility in the Microsoft Base Cryptographic
 *          Provider. 
 * 
 * @param int dwFlags - Flag values. This parameter is reserved for future use and 
 *          should always be zero.
 * 
 * @param byte[] pbSignature - Pointer to a buffer that receives the signature data.
 *          This parameter can be NULL to set the size of this information for 
 *          memory allocation purposes. For more information, see Common In/Out 
 *          Parameter Conventions. 
 * 
 * @param int pdwSigLen - Pointer to a variable that specifies the size, in bytes, 
 *          of the buffer pointed to by the pdwSigLen parameter. When the function 
 *          returns, the variable pointed to by the pdwSigLen parameter contains the 
 *          number of bytes stored in the buffer. This parameter can be NULL, only 
 *          if pbSignature is NULL. 
 * 
 *          Note that when processing the data returned in the buffer, applications 
 *          need to use the actual size of the data returned. The actual size may 
 *          be slightly smaller than the size of the buffer specified on input. (On 
 *          input, buffer sizes are usually specified large enough to insure that 
 *          the largest possible output data will fit in the buffer.) On output, the 
 *          variable pointed to by this parameter is updated to reflect the actual 
 *          size of the data copied to the buffer. 
 * 
 * @exception com.ms.dll.Win32Exception - thrown if a problem occured during the
 *          method execution.
 * 
 *          The following table lists the error codes most commonly returned by the 
 *          GetLastError function. The error codes prefaced by "NTE" are generated 
 *          by the particular CSP you are using.
 * 
 *          Error code                  Description 
 * 
 *          ERROR_INVALID_HANDLE        One of the parameters specifies an invalid 
 *                                      handle.
 * 
 *          ERROR_INVALID_PARAMETER     One of the parameters contains an invalid 
 *                                      value. This is most often an illegal pointer.
 * 
 *          ERROR_MORE_DATA             If the buffer specified by the pbSignature 
 *                                      parameter is not large enough to hold the 
 *                                      returned data, the function sets the 
 *                                      ERROR_MORE_DATA code and stores the required 
 *                                      buffer size, in bytes, into the variable 
 *                                      pointed to by pdwSigLen. 
 * 
 *          NTE_BAD_ALGID               The hHash handle specifies an algorithm that 
 *                                      this CSP does not support. 
 * 
 *          NTE_BAD_FLAGS               The dwFlags parameter is nonzero. 
 * 
 *          NTE_BAD_HASH                The hash object specified by the hHash 
 *                                      parameter is invalid. 
 * 
 *          NTE_BAD_UID                 The CSP context that was specified when the 
 *                                      hash object was created cannot be found. 
 * 
 *          NTE_NO_KEY                  The private key specified by dwKeySpec does 
 *                                      not exist. 
 * 
 *          NTE_NO_MEMORY               The CSP ran out of memory during the 
 *                                      operation. 
 */
    public static synchronized void signHash(int hHash,
                                             int dwKeySpec,
                                             String sDescription,
                                             int dwFlags,
                                             byte[] pbSignature,
                                             int pdwSignLen){
        int[] pdwSig = new int[]{pdwSignLen};
        if (!CryptSignHash(hHash,dwKeySpec,sDescription,dwFlags,
                           pbSignature,pdwSig)){
            throw new com.ms.dll.Win32Exception(com.ms.dll.DllLib.getLastWin32Error());
        }
    }
/**
 * 
 */
    public static synchronized int getSignHashLength(int hHash, int dwKeySpec){
        int[] pdwSignLen = new int[1];
        if(!CryptSignHash(hHash,
                          dwKeySpec,
                          null,
                          0,
                          null,
                          pdwSignLen)){
            throw new com.ms.dll.Win32Exception(com.ms.dll.DllLib.getLastWin32Error());
        }
        return pdwSignLen[0];
    }
/**
 * @see #signHash
 */
    protected  static native boolean CryptSignHash(
                                    int hHash,
                                    int dwKeySpec,
                                    String sDescription,
                                    int dwFlags,
                                    byte[] pbSignature,
                                    int[] pdwSigLen);
/**
 * The CryptVerifySignature function is used to verify a signature against a hash 
 * object.
 * 
 * Before calling this function, the CryptCreateHash function must be called to get 
 * a handle to a hash object. The CryptHashData and/or CryptHashSessionKey 
 * functions are then used to add the data and/or session keys to the hash object.
 * 
 * After this function has been completed, the only hash function that can be called
 * by using the hHash handle is the CryptDestroyHash function.
 * 
 * @param int hHash - A handle to the hash object to verify against.
 * @param byte[] pbSignature - The address of the signature data to be verified. 
 * @param int dwSigLen - The number of bytes in the pbSignature signature data. 
 * @param int hPubKey - A handle to the public key to use to authenticate the 
 *          signature. This public key must belong to the key pair that was 
 *          originally used to create the digital signature. 
 * @param String sDescription - This parameter should no longer be used, and should 
 *          be set to NULL to prevent security vulnerabilities. However, it is still
 *          supported for backward compatibility in the Microsoft Base Cryptographic
 *          Provider.
 * @param int dwFlags - The flag values. This parameter is reserved for future use 
 *          and should always be zero. 
 * 
 * @exception com.ms.dll.Win32Exception - thrown if a problem occures during this 
 *          method call.
 * 
 *          The following table lists the error codes most commonly returned by the 
 *          GetLastError function. The error codes prefaced by "NTE" are generated 
 *          by the particular CSP you are using.
 * 
 *          Error code                      Description 
 * 
 *          ERROR_INVALID_HANDLE            One of the parameters specifies an 
 *                                          invalid handle. 
 * 
 *          ERROR_INVALID_PARAMETER         One of the parameters contains an 
 *                                          invalid value. This is most often an 
 *                                          illegal pointer. 
 * 
 *          NTE_BAD_FLAGS                   The dwFlags parameter is nonzero.
 *          
 *          NTE_BAD_HASH                    The hash object specified by the hHash 
 *                                          parameter is invalid. 
 * 
 *          NTE_BAD_KEY                     The hPubKey parameter does not contain a 
 *                                          handle to a valid public key. 
 * 
 *          NTE_BAD_SIGNATURE               The signature was not verified. This 
 *                                          could be because the data itself has 
 *                                          changed, the description string did not 
 *                                          match, or the wrong public key was 
 *                                          specified by hPubKey.
 * 
 *                                          This error can also be returned if the 
 *                                          hashing or signature algorithms do not 
 *                                          match the ones used to create the 
 *                                          signature.
 * 
 *          NTE_BAD_UID                     The CSP context that was specified when 
 *                                          the hash object was created cannot be 
 *                                          found. 
 * 
 *          NTE_NO_MEMORY                   The CSP ran out of memory during the 
 *                                          operation. 
 */
    public static synchronized boolean verifySignature(
                                    int hHash,
                                    byte[] pbSignature,
                                    int dwSigLen,
                                    int hPubKey,
                                    String sDescription,
                                    int dwFlags){
        if (!CryptVerifySignature(hHash,pbSignature,dwSigLen,hPubKey,
                                  sDescription,dwFlags)){
            if(com.ms.dll.DllLib.getLastWin32Error()==
               com.ms.win32.win.NTE_BAD_SIGNATURE){
                return false;
            }else{
                throw new com.ms.dll.Win32Exception(com.ms.dll.DllLib.getLastWin32Error());
        }
        }
        return true;    
    }
/**
 * @see #verifySignature
 */ 
    protected static native boolean CryptVerifySignature(
                                    int hHash,
                                    byte[] pbSignature,
                                    int dwSigLen,
                                    int hPubKey,
                                    String sDescription,
                                    int dwFlags);
}